@@ -2,23 +2,17 @@
 preprocessors for signals of numpy array format
 """
 
-from .preproc_manager import PreprocManager
+from .bandpass import BandPass
 from .base import (
     PreProcessor,
     preprocess_multi_lead_signal,
     preprocess_single_lead_signal,
 )
-from .bandpass import BandPass
 from .baseline_remove import BaselineRemove
-from .normalize import (
-    Normalize,
-    MinMaxNormalize,
-    NaiveNormalize,
-    ZScoreNormalize,
-)
+from .normalize import MinMaxNormalize, NaiveNormalize, Normalize, ZScoreNormalize
+from .preproc_manager import PreprocManager
 from .resample import Resample
 
-
 __all__ = [
     "PreprocManager",
     "PreProcessor",
