@@ -4,7 +4,7 @@
 from abc import ABC, abstractmethod
 from itertools import repeat
 from numbers import Real
-from typing import List, NoReturn, Optional, Tuple
+from typing import List, Optional, Tuple
 
 import numpy as np
 
@@ -44,6 +44,7 @@ class PreProcessor(ReprMixin, ABC):
             3d array, which is a tensor of several ECGs, of shape (batch, lead, siglen)
         fs: real number,
             sampling frequency of the ECG signal
+
         """
         raise NotImplementedError
 
@@ -53,7 +54,7 @@ class PreProcessor(ReprMixin, ABC):
         """
         return self.apply(sig, fs)
 
-    def _check_sig(self, sig: np.ndarray) -> NoReturn:
+    def _check_sig(self, sig: np.ndarray) -> None:
         """
         check validity of the signal
 
@@ -64,6 +65,7 @@ class PreProcessor(ReprMixin, ABC):
             1d array, which is a single-lead ECG
             2d array, which is a multi-lead ECG of "lead_first" format
             3d array, which is a tensor of several ECGs, of shape (batch, lead, siglen)
+
         """
         if sig.ndim not in [1, 2, 3]:
             raise ValueError(
@@ -84,7 +86,6 @@ def preprocess_multi_lead_signal(
     filter_order: Optional[int] = None,
 ) -> np.ndarray:
     """
-
     perform preprocessing for multi-lead ecg signal (with units in mV),
     preprocessing may include median filter, bandpass filter, and rpeaks detection, etc.
     also works for single-lead ecg signal (sig_fmt="channel_first")
@@ -119,15 +120,19 @@ def preprocess_multi_lead_signal(
     filtered_ecg: ndarray,
         the array of the processed ecg signal,
         the format of the signal is kept the same with the original signal, i.e. `sig_fmt`
+
     """
+    raw_sig = np.asarray(raw_sig)
+    assert raw_sig.ndim in [2, 3], "multi-lead signal should be 2d or 3d array"
     assert sig_fmt.lower() in [
         "channel_first",
         "lead_first",
         "channel_last",
         "lead_last",
-    ]
+    ], f"multi-lead signal format `{sig_fmt}` not supported"
     if sig_fmt.lower() in ["channel_last", "lead_last"]:
-        filtered_ecg = raw_sig.T
+        # might have a batch dimension at the first axis
+        filtered_ecg = np.moveaxis(raw_sig, -2, -1)
     else:
         filtered_ecg = raw_sig
 
@@ -144,7 +149,7 @@ def preprocess_multi_lead_signal(
 
     # filter signal
     if band_fs:
-        assert band_fs[0] < band_fs[1]
+        assert band_fs[0] < band_fs[1], "Invalid frequency band"
         nyq = 0.5 * fs
         if band_fs[0] <= 0 and band_fs[1] < nyq:
             band = "lowpass"
@@ -156,7 +161,7 @@ def preprocess_multi_lead_signal(
             band = "bandpass"
             frequency = band_fs
         else:
-            raise ValueError("Invalid frequency band")
+            raise AssertionError("Invalid frequency band")
         if filter_type.lower() == "fir":
             filtered_ecg = filter_signal(
                 signal=filtered_ecg,
@@ -177,7 +182,7 @@ def preprocess_multi_lead_signal(
                 or round(0.01 * fs),  # better be determined by the `buttord`
             )
         else:
-            raise ValueError("Unsupported filter type")
+            raise ValueError(f"Unsupported filter type `{filter_type}`")
 
     if sig_fmt.lower() in ["channel_last", "lead_last"]:
         filtered_ecg = filtered_ecg.T
@@ -190,9 +195,10 @@ def preprocess_single_lead_signal(
     fs: Real,
     bl_win: Optional[List[Real]] = None,
     band_fs: Optional[List[Real]] = None,
+    filter_type: str = "butter",
+    filter_order: Optional[int] = None,
 ) -> np.ndarray:
     """
-
     perform preprocessing for single lead ecg signal (with units in mV),
     preprocessing may include median filter, bandpass filter, and rpeaks detection, etc.
 
@@ -212,6 +218,10 @@ def preprocess_single_lead_signal(
         a typical pair is [0.5, 45],
         be careful when detecting paced rhythm,
         if is None or empty, bandpass filtering will not be performed
+    filter_type: str, default "butter",
+        type of the bandpass filter, can be "butter" or "fir"
+    filter_order: int, optional,
+        order of the bandpass filter,
 
     Returns
     -------
@@ -222,8 +232,10 @@ def preprocess_single_lead_signal(
     ----
     bandpass filter uses FIR filters, an alternative can be Butterworth filter,
     e.g. `butter_bandpass_filter` in `utils.utils_singal`
+
     """
-    filtered_ecg = raw_sig
+    filtered_ecg = np.asarray(raw_sig)
+    assert filtered_ecg.ndim == 1, "single-lead signal should be 1d array"
 
     # remove baseline
     if bl_win:
@@ -235,7 +247,7 @@ def preprocess_single_lead_signal(
 
     # filter signal
     if band_fs:
-        assert band_fs[0] < band_fs[1]
+        assert band_fs[0] < band_fs[1], "Invalid frequency band"
         nyq = 0.5 * fs
         if band_fs[0] <= 0 and band_fs[1] < nyq:
             band = "lowpass"
@@ -247,15 +259,27 @@ def preprocess_single_lead_signal(
             band = "bandpass"
             frequency = band_fs
         else:
-            raise ValueError("Invalid frequency band")
-        filtered_ecg = filter_signal(
-            signal=filtered_ecg,
-            ftype="FIR",
-            # ftype="butter",
-            band=band,
-            order=int(0.3 * fs),
-            sampling_rate=fs,
-            frequency=frequency,
-        )["signal"]
+            raise AssertionError("Invalid frequency band")
+        if filter_type.lower() == "fir":
+            filtered_ecg = filter_signal(
+                signal=filtered_ecg,
+                ftype="FIR",
+                # ftype="butter",
+                band=band,
+                order=int(0.3 * fs),
+                sampling_rate=fs,
+                frequency=frequency,
+            )["signal"]
+        elif filter_type.lower() == "butter":
+            filtered_ecg = butter_bandpass_filter(
+                data=filtered_ecg,
+                lowcut=band_fs[0],
+                highcut=band_fs[1],
+                fs=fs,
+                order=filter_order
+                or round(0.01 * fs),  # better be determined by the `buttord`
+            )
+        else:
+            raise ValueError(f"Unsupported filter type `{filter_type}`")
 
     return filtered_ecg
