@@ -1,8 +1,9 @@
 """
 """
 
+import warnings
 from random import sample
-from typing import List, NoReturn, Optional, Tuple
+from typing import List, Optional, Tuple
 
 import numpy as np
 
@@ -20,6 +21,7 @@ __all__ = [
 
 class PreprocManager(ReprMixin):
     """
+    Manager of preprocessors
 
     Examples
     --------
@@ -35,48 +37,48 @@ class PreprocManager(ReprMixin):
         normalize={"method": "min-max"},
     )
     ppm = PreprocManager.from_config(config)
-    sig = torch.rand(12,80000).numpy()
+    sig = torch.randn(12, 80000).numpy()
     sig, fs = ppm(sig, 200)
     ```
+
     """
 
     __name__ = "PreprocManager"
 
     def __init__(
         self, *pps: Optional[Tuple[PreProcessor, ...]], random: bool = False
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         pps: tuple of `PreProcessor`, optional,
             the sequence of preprocessors to be added to the manager
         random: bool, default False,
             whether to apply the augmenters in random order
+
         """
         super().__init__()
         self.random = random
         self._preprocessors = list(pps)
 
-    def _add_bandpass(self, **config: dict) -> NoReturn:
+    def _add_bandpass(self, **config: dict) -> None:
         """ """
         self._preprocessors.append(BandPass(**config))
 
-    def _add_baseline_remove(self, **config: dict) -> NoReturn:
+    def _add_baseline_remove(self, **config: dict) -> None:
         """ """
         self._preprocessors.append(BaselineRemove(**config))
 
-    def _add_normalize(self, **config: dict) -> NoReturn:
+    def _add_normalize(self, **config: dict) -> None:
         """ """
         self._preprocessors.append(Normalize(**config))
 
-    def _add_resample(self, **config: dict) -> NoReturn:
+    def _add_resample(self, **config: dict) -> None:
         """ """
         self._preprocessors.append(Resample(**config))
 
     def __call__(self, sig: np.ndarray, fs: int) -> Tuple[np.ndarray, int]:
         """
-
         Parameters
         ----------
         sig: np.ndarray,
@@ -90,9 +92,11 @@ class PreprocManager(ReprMixin):
             the preprocessed signal
         new_fs: int,
             the sampling frequency of the preprocessed signal
+
         """
         if len(self.preprocessors) == 0:
             # raise ValueError("No preprocessors added to the manager.")
+            # allow dummy (empty) preprocessors
             return sig, fs
         ordering = list(range(len(self.preprocessors)))
         if self.random:
@@ -106,7 +110,6 @@ class PreprocManager(ReprMixin):
     @classmethod
     def from_config(cls, config: dict) -> "PreprocManager":
         """
-
         Parameters
         ----------
         config: dict,
@@ -117,6 +120,7 @@ class PreprocManager(ReprMixin):
         -------
         ppm: PreprocManager,
             a new instance of `PreprocManager`
+
         """
         ppm = cls(random=config.get("random", False))
         _mapping = {
@@ -137,32 +141,53 @@ class PreprocManager(ReprMixin):
                 # just ignore the other items
                 pass
                 # raise ValueError(f"Unknown preprocessor: {pp_name}")
+        if ppm.empty:
+            warnings.warn(
+                "No preprocessors added to the manager. You are using a dummy preprocessor.",
+                RuntimeWarning,
+            )
         return ppm
 
-    def rearrange(self, new_ordering: List[str]) -> NoReturn:
+    def rearrange(self, new_ordering: List[str]) -> None:
         """
-
         Parameters
         ----------
         new_ordering: list of str,
             the new ordering of the preprocessors
+
         """
-        _mapping = {
+        if self.random:
+            warnings.warn(
+                "The preprocessors are applied in random order, "
+                "rearranging the preprocessors will not take effect.",
+                RuntimeWarning,
+            )
+        _mapping = {  # built-in preprocessors
             "Resample": "resample",
             "BandPass": "bandpass",
             "BaselineRemove": "baseline_remove",
             "Normalize": "normalize",
         }
+        _mapping.update({v: k for k, v in _mapping.items()})
         for k in new_ordering:
             if k not in _mapping:
+                # allow custom preprocessors
+                assert k in [
+                    item.__class__.__name__ for item in self._preprocessors
+                ], f"Unknown preprocessor name: `{k}`"
                 _mapping.update({k: k})
+        assert len(new_ordering) == len(
+            set(new_ordering)
+        ), "Duplicate preprocessor names."
+        assert len(new_ordering) == len(
+            self._preprocessors
+        ), "Number of preprocessors mismatch."
         self._preprocessors.sort(
-            key=lambda aug: new_ordering.index(_mapping[aug.__name__])
+            key=lambda item: new_ordering.index(_mapping[item.__class__.__name__])
         )
 
-    def add_(self, pp: PreProcessor, pos: int = -1) -> NoReturn:
+    def add_(self, pp: PreProcessor, pos: int = -1) -> None:
         """
-
         add a (custom) preprocessor to the manager,
         this method is preferred against directly manipulating
         the internal list of preprocessors via `PreprocManager.preprocessors.append(pp)`
@@ -174,6 +199,7 @@ class PreprocManager(ReprMixin):
         pos: int, default -1,
             the position to insert the preprocessor,
             should be >= -1, with -1 the indicator of the end
+
         """
         assert isinstance(pp, PreProcessor)
         assert pp.__class__.__name__ not in [
@@ -191,6 +217,10 @@ class PreprocManager(ReprMixin):
     def preprocessors(self) -> List[PreProcessor]:
         return self._preprocessors
 
+    @property
+    def empty(self) -> bool:
+        return len(self.preprocessors) == 0
+
     def extra_repr_keys(self) -> List[str]:
         """
         return the extra keys for `__repr__`
