@@ -1,8 +1,9 @@
 """
 """
 
+import warnings
 from random import sample
-from typing import Any, List, NoReturn, Optional, Sequence, Tuple, Union
+from typing import Any, List, Optional, Sequence, Tuple, Union
 
 import torch
 from torch import Tensor
@@ -24,7 +25,6 @@ __all__ = [
 
 class AugmenterManager(torch.nn.Module):
     """
-
     The `Module` to manage the augmenters
 
     Examples
@@ -46,48 +46,52 @@ class AugmenterManager(torch.nn.Module):
         stretch_compress={},
     )
     am = AugmenterManager.from_config(config)
-    sig, label, mask = torch.rand(2,12,5000), torch.rand(2,26), torch.rand(2,5000,1)
-    sig, label, mask = am(sig, label, mask)
+    sig = torch.randn(32, 12, 5000)
+    label = torch.randint(0, 2, (32, 26), dtype=torch.float32)
+    mask1 = torch.randint(0, 2, (32, 5000, 3), dtype=torch.float32)
+    mask2 = torch.randint(0, 3, (32, 5000), dtype=torch.long)
+    sig, label, mask1, mask2 = am(sig, label, mask1, mask2)
     ```
+
     """
 
     __name__ = "AugmenterManager"
 
     def __init__(
         self, *augs: Optional[Tuple[Augmenter, ...]], random: bool = False
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         aug: tuple of `Augmenter`, optional,
             the augmenters to be added to the manager
         random: bool, default False,
             whether to apply the augmenters in random order
+
         """
         super().__init__()
         self.random = random
         self._augmenters = list(augs)
 
-    def _add_baseline_wander(self, **config: dict) -> NoReturn:
+    def _add_baseline_wander(self, **config: dict) -> None:
         self._augmenters.append(BaselineWanderAugmenter(**config))
 
-    def _add_label_smooth(self, **config: dict) -> NoReturn:
+    def _add_label_smooth(self, **config: dict) -> None:
         self._augmenters.append(LabelSmooth(**config))
 
-    def _add_mixup(self, **config: dict) -> NoReturn:
+    def _add_mixup(self, **config: dict) -> None:
         self._augmenters.append(Mixup(**config))
 
-    def _add_random_flip(self, **config: dict) -> NoReturn:
+    def _add_random_flip(self, **config: dict) -> None:
         self._augmenters.append(RandomFlip(**config))
 
-    def _add_random_masking(self, **config: dict) -> NoReturn:
+    def _add_random_masking(self, **config: dict) -> None:
         self._augmenters.append(RandomMasking(**config))
 
-    def _add_random_renormalize(self, **config: dict) -> NoReturn:
+    def _add_random_renormalize(self, **config: dict) -> None:
         self._augmenters.append(RandomRenormalize(**config))
 
-    def _add_stretch_compress(self, **config: dict) -> NoReturn:
+    def _add_stretch_compress(self, **config: dict) -> None:
         self._augmenters.append(StretchCompress(**config))
 
     def forward(
@@ -98,7 +102,6 @@ class AugmenterManager(torch.nn.Module):
         **kwargs: Any,
     ) -> Union[Tensor, Tuple[Tensor]]:
         """
-
         Parameters
         ----------
         sig: Tensor,
@@ -112,6 +115,7 @@ class AugmenterManager(torch.nn.Module):
         Returns
         -------
         Tensor(s), the augmented ECGs, labels, and optional extra tensors
+
         """
         if len(self.augmenters) == 0:
             # raise ValueError("No augmenters added to the manager.")
@@ -143,7 +147,6 @@ class AugmenterManager(torch.nn.Module):
     @classmethod
     def from_config(cls, config: dict) -> "AugmenterManager":
         """
-
         Parameters
         ----------
         config: dict,
@@ -154,6 +157,7 @@ class AugmenterManager(torch.nn.Module):
         -------
         am: AugmenterManager,
             a new instance of `AugmenterManager`
+
         """
         am = cls(random=config.get("random", False))
         _mapping = {
@@ -179,24 +183,40 @@ class AugmenterManager(torch.nn.Module):
                 # raise ValueError(f"Unknown augmenter name: {aug_name}")
         return am
 
-    def rearrange(self, new_ordering: List[str]) -> NoReturn:
+    def rearrange(self, new_ordering: List[str]) -> None:
         """
-
         Parameters
         ----------
         new_ordering: list of str,
             the list of augmenter names in the new order
+
         """
-        _mapping = {
+        if self.random:
+            warnings.warn(
+                "The augmenters are applied in random order, "
+                "rearranging the augmenters will not take effect.",
+                RuntimeWarning,
+            )
+        _mapping = {  # built-in augmenters
             "".join([w.capitalize() for w in k.split("_")]): k
             for k in "label_smooth,mixup,random_flip,random_masking,random_renormalize,stretch_compress".split(
                 ","
             )
         }
         _mapping.update({"BaselineWanderAugmenter": "baseline_wander"})
+        _mapping.update({v: k for k, v in _mapping.items()})
         for k in new_ordering:
             if k not in _mapping:
+                # allow custom augmenters
+                assert k in [
+                    am.__class__.__name__ for am in self._augmenters
+                ], f"Unknown augmenter name: `{k}`"
                 _mapping.update({k: k})
+        assert len(new_ordering) == len(set(new_ordering)), "Duplicate augmenter names."
+        assert len(new_ordering) == len(
+            self._augmenters
+        ), "Number of augmenters mismatch."
+
         self._augmenters.sort(
-            key=lambda aug: new_ordering.index(_mapping[aug.__name__])
+            key=lambda aug: new_ordering.index(_mapping[aug.__class__.__name__])
         )
