@@ -1,7 +1,7 @@
 """
 """
 
-from typing import Any, List, NoReturn, Optional, Sequence, Tuple
+from typing import Any, List, Optional, Sequence, Tuple
 
 import torch
 from torch import Tensor
@@ -16,6 +16,15 @@ __all__ = [
 class LabelSmooth(Augmenter):
     """
     Label smoothing augmentation.
+
+    Examples
+    --------
+    ```python
+    ls = LabelSmooth()
+    label = torch.randint(0, 2, (32, 26), dtype=torch.float32)
+    _, label = ls(None, label)
+    ```
+
     """
 
     __name__ = "LabelSmooth"
@@ -27,9 +36,8 @@ class LabelSmooth(Augmenter):
         prob: float = 0.5,
         inplace: bool = True,
         **kwargs: Any
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         fs: int, optional,
@@ -41,6 +49,7 @@ class LabelSmooth(Augmenter):
         inplace: bool, default True,
             if True, the input tensor will be modified inplace
         kwargs: keyword arguments
+
         """
         super().__init__()
         self.fs = fs
@@ -57,7 +66,6 @@ class LabelSmooth(Augmenter):
         **kwargs: Any
     ) -> Tuple[Tensor, ...]:
         """
-
         Parameters
         ----------
         sig: Tensor,
@@ -80,6 +88,7 @@ class LabelSmooth(Augmenter):
             the output label tensor
         extra_tensors: sequence of Tensors, optional,
             if set in the input arguments, unchanged
+
         """
         if not self.inplace:
             label = label.clone()
