@@ -3,7 +3,7 @@
 
 from numbers import Real
 from random import randint
-from typing import Any, List, NoReturn, Optional, Sequence, Tuple, Union
+from typing import Any, List, Optional, Sequence, Tuple, Union
 
 import numpy as np
 import torch
@@ -19,6 +19,16 @@ __all__ = [
 class RandomMasking(Augmenter):
     """
     Randomly mask ECGs with a probability.
+
+    Examples
+    --------
+    ```python
+    rm = RandomMasking(fs=500, prob=0.7)
+    sig = torch.randn(32, 12, 5000)
+    critical_points = [np.arange(250, 5000 - 250, step=400) for _ in range(32)]
+    sig, _ = rm(sig, None, critical_points=critical_points)
+    ```
+
     """
 
     __name__ = "RandomMasking"
@@ -31,9 +41,8 @@ class RandomMasking(Augmenter):
         prob: Union[Sequence[Real], Real] = [0.3, 0.15],
         inplace: bool = True,
         **kwargs: Any
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         fs: int,
@@ -50,6 +59,7 @@ class RandomMasking(Augmenter):
         inplace: bool, default True,
             whether to mask inplace or not
         kwargs: Keyword arguments.
+
         """
         super().__init__()
         self.fs = fs
@@ -74,7 +84,6 @@ class RandomMasking(Augmenter):
         **kwargs: Any
     ) -> Tuple[Tensor, ...]:
         """
-
         Parameters
         ----------
         sig: Tensor,
@@ -98,6 +107,7 @@ class RandomMasking(Augmenter):
             the label tensor of the augmented ECGs, unchanged
         extra_tensors: sequence of Tensors, optional,
             if set in the input arguments, unchanged
+
         """
         batch, lead, siglen = sig.shape
         if not self.inplace:
