@@ -2,7 +2,7 @@
 """
 
 from numbers import Real
-from typing import Any, Iterable, List, NoReturn, Optional, Sequence, Tuple
+from typing import Any, Iterable, List, Optional, Sequence, Tuple
 
 import numpy as np
 from torch import Tensor
@@ -20,6 +20,15 @@ class RandomRenormalize(Augmenter):
     """
     Randomly re-normalize the ECG tensor,
     using the Z-score normalization method.
+
+    Examples
+    --------
+    ```python
+    rrn = RandomRenormalize()
+    sig = torch.randn(32, 12, 5000)
+    sig, _ = rrn(sig, None)
+    ```
+
     """
 
     __name__ = "RandomRenormalize"
@@ -32,9 +41,8 @@ class RandomRenormalize(Augmenter):
         prob: float = 0.5,
         inplace: bool = True,
         **kwargs: Any
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         mean: array_like, default [-0.05,0.1],
@@ -50,6 +58,7 @@ class RandomRenormalize(Augmenter):
         inplace: bool, default True,
             Whether to apply the random re-normalization augmenter in-place.
         kwargs: keyword arguments
+
         """
         super().__init__()
         self.mean = np.array(mean)
@@ -72,7 +81,6 @@ class RandomRenormalize(Augmenter):
         **kwargs: Any
     ) -> Tuple[Tensor, ...]:
         """
-
         Parameters
         ----------
         sig: Tensor,
@@ -93,6 +101,7 @@ class RandomRenormalize(Augmenter):
             the label tensor of the augmented ECGs, unchanged
         extra_tensors: sequence of Tensors, optional,
             if set in the input arguments, unchanged
+
         """
         batch, lead, siglen = sig.shape
         if self.mean.ndim == 2:
@@ -101,6 +110,8 @@ class RandomRenormalize(Augmenter):
             assert self.std.shape[0] == lead
         if not self.inplace:
             sig = sig.clone()
+        if self.prob == 0:
+            return (sig, label, *extra_tensors)
         indices = self.get_indices(self.prob, pop_size=batch)
         if self.per_channel:
             mean = DEFAULTS.RNG.normal(
