@@ -3,13 +3,12 @@
 
 from numbers import Real
 from random import choice, randint
-from typing import Any, List, NoReturn, Optional, Sequence, Tuple, Union
+from typing import Any, List, Optional, Sequence, Tuple, Union
 
 import numpy as np
+import scipy.signal as SS
 import torch
-import torch.multiprocessing as tmp
 import torch.nn.functional as F
-from scipy.signal import resample, resample_poly  # noqa: F401
 from torch import Tensor
 
 from ..cfg import DEFAULTS
@@ -30,20 +29,21 @@ class StretchCompress(Augmenter):
     -------
     ```python
     sc = StretchCompress()
-    sig = torch.ones((32, 12, 5000))
-    lb = torch.ones((32, 5000, 3))
-    mask = torch.ones((32, 5000, 1))
-    sig, lb, mask = sc(sig, lb, mask)
+    sig = torch.randn((32, 12, 5000))
+    labels = torch.randint(0, 2, (32, 5000, 26))
+    label = torch.randint(0, 2, (32, 26), dtype=torch.float32)
+    mask = torch.randint(0, 2, (32, 5000, 3), dtype=torch.float32)
+    sig, label, mask = sc(sig, label, mask)
     ```
+
     """
 
     __name__ = "StretchCompress"
 
     def __init__(
         self, ratio: Real = 6, prob: float = 0.5, inplace: bool = True, **kwargs: Any
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         ratio: real number, default 6,
@@ -55,6 +55,7 @@ class StretchCompress(Augmenter):
         inplace: bool, default True,
             if True, the input ECGs will be modified inplace,
         kwargs: keyword arguments
+
         """
         super().__init__()
         self.prob = prob
@@ -71,7 +72,6 @@ class StretchCompress(Augmenter):
         self, sig: Tensor, *labels: Optional[Sequence[Tensor]], **kwargs: Any
     ) -> Tuple[Tensor, ...]:
         """
-
         Parameters
         ----------
         sig: Tensor,
@@ -89,6 +89,7 @@ class StretchCompress(Augmenter):
             the stretched or compressed ECG tensors
         labels: sequence of Tensors, optional,
             the stretched or compressed label tensors
+
         """
         batch, lead, siglen = sig.shape
         if not self.inplace:
@@ -200,29 +201,44 @@ class StretchCompress(Augmenter):
             the stretched or compressed ECG tensors
         labels: sequence of Tensors, optional,
             the stretched or compressed label tensors
+
         """
         batch, lead, siglen = sig.shape
         if not self.inplace:
             sig = sig.clone()
         if self.prob == 0:
-            return sig
+            if len(labels) == 0:
+                return sig
+            return (sig, *labels)
         indices = self.get_indices(prob=self.prob, pop_size=batch)
-        with tmp.Pool(processes=4) as pool:
-            sig[indices, ...] = torch.as_tensor(
-                pool.starmap(
-                    func=_stretch_compress_one_batch_element,
-                    iterable=[
-                        (
-                            self.ratio,
-                            sig[batch_idx, ...].unsqueeze(0),
-                        )
-                        for batch_idx in indices
-                    ],
-                ),
-                dtype=sig.dtype,
-                device=sig.device,
+        # with tmp.Pool(processes=4) as pool:
+        #     data = pool.starmap(
+        #         func=_stretch_compress_one_batch_element,
+        #         iterable=[
+        #             (
+        #                 self.ratio,
+        #                 sig[batch_idx, ...].unsqueeze(0),
+        #                 *(label[batch_idx, ...].unsqueeze(0) for label in labels)
+        #             )
+        #             for batch_idx in indices
+        #         ],
+        #     )
+        # for batch_idx, (sig_, *labels_) in zip(indices, data):
+        #     sig[batch_idx, ...] = sig_
+        #     for idx, label in enumerate(labels_):
+        #         labels[idx][batch_idx, ...] = label
+        for batch_idx in indices:
+            data = _stretch_compress_one_batch_element(
+                self.ratio,
+                sig[batch_idx, ...].unsqueeze(0),
+                *(label[batch_idx, ...].unsqueeze(0) for label in labels),
             )
-        return sig
+            sig[batch_idx, ...] = data[0]
+            for idx, label in enumerate(data[1:]):
+                labels[idx][batch_idx, ...] = label
+        if len(labels) == 0:
+            return sig
+        return (sig, *labels)
 
     def extra_repr_keys(self) -> List[str]:
         """ """
@@ -236,7 +252,7 @@ class StretchCompress(Augmenter):
 def _stretch_compress_one_batch_element(
     ratio: Real, sig: Tensor, *labels: Sequence[Tensor]
 ) -> Tensor:
-    """finished, NOT checked,
+    """
 
     Parameters
     ----------
@@ -255,12 +271,16 @@ def _stretch_compress_one_batch_element(
         the stretched or compressed ECG tensor
     labels: Tensors, optional, of shapes (label_len, channels)
         the stretched or compressed label tensors
+
     """
     labels = list(labels)
     label_len = []
     n_labels = len(labels)
     siglen = sig.shape[-1]
     for idx in range(n_labels):
+        if labels[idx].ndim < 3:
+            label_len.append(0)
+            continue
         labels[idx] = labels[idx].permute(
             0, 2, 1
         )  # (1, label_len, n_classes) -> (1, n_classes, label_len)
@@ -281,12 +301,14 @@ def _stretch_compress_one_batch_element(
             ..., half_diff_len : siglen + half_diff_len
         ].squeeze(0)
         for idx in range(n_labels):
+            if label_len[idx] == 0:
+                continue
             labels[idx] = F.interpolate(
                 labels[idx],
                 size=new_len,
                 mode="linear",
                 align_corners=True,
-            )[..., half_diff_len : siglen + half_diff_len].squeeze(0)
+            )[..., half_diff_len : siglen + half_diff_len]
     else:  # compress and pad
         sig = F.pad(
             F.interpolate(
@@ -300,6 +322,8 @@ def _stretch_compress_one_batch_element(
             value=0.0,
         ).squeeze(0)
         for idx in range(n_labels):
+            if label_len[idx] == 0:
+                continue
             labels[idx] = F.pad(
                 F.interpolate(
                     labels[idx],
@@ -310,17 +334,20 @@ def _stretch_compress_one_batch_element(
                 pad=(half_diff_len, diff_len - half_diff_len),
                 mode="constant",
                 value=0.0,
-            ).squeeze(0)
+            )
     for idx, (label, ll) in enumerate(zip(labels, label_len)):
+        if ll == 0:
+            labels[idx] = labels[idx].squeeze(0)
+            continue
         if ll != siglen:
             labels[idx] = F.interpolate(
                 label, size=(ll,), mode="linear", align_corners=True
             )
-        labels[idx] = labels[idx].permute(
-            1, 0
+        labels[idx] = (
+            labels[idx].squeeze(0).permute(1, 0)
         )  # (n_classes, label_len) -> (label_len, n_classes)
     if len(labels) > 0:
-        return (sig,) + tuple(labels)
+        return (sig, *labels)
     return sig
 
 
@@ -334,11 +361,12 @@ class StretchCompressOffline(ReprMixin):
     ```python
     sco = StretchCompressOffline()
     seglen = 600
-    sig = torch.rand((12, 60000)).numpy()
+    sig = torch.randn((12, 60000)).numpy()
     labels = torch.ones((60000, 3)).numpy().astype(int)
     masks = torch.ones((60000, 1)).numpy().astype(int)
     segments = sco(600, sig, labels, masks, critical_points=[10000,30000])
     ```
+
     """
 
     __name__ = "StretchCompressOffline"
@@ -349,9 +377,8 @@ class StretchCompressOffline(ReprMixin):
         prob: float = 0.5,
         overlap: float = 0.5,
         critical_overlap: float = 0.85,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         ratio: real number, default 6,
@@ -364,6 +391,7 @@ class StretchCompressOffline(ReprMixin):
             the overlap of offline generated data
         critical_overlap: float, default 0.85,
             the overlap of the critical region of the ECG
+
         """
         self.prob = prob
         assert 0 <= self.prob <= 1, "Probability must be between 0 and 1"
@@ -390,7 +418,6 @@ class StretchCompressOffline(ReprMixin):
         critical_points: Optional[Sequence[int]] = None,
     ) -> List[Tuple[Union[np.ndarray, int], ...]]:
         """
-
         Parameters
         ----------
         seglen: int,
@@ -410,6 +437,7 @@ class StretchCompressOffline(ReprMixin):
         -------
         list of generated segments,
         with segments consists of (seg, label1, label2, ..., start_idx, end_idx)
+
         """
         siglen = sig.shape[1]
         forward_len = int(round(seglen - seglen * self.overlap))
@@ -469,7 +497,6 @@ class StretchCompressOffline(ReprMixin):
         end_idx: Optional[int] = None,
     ) -> Tuple[Union[np.ndarray, int], ...]:
         """
-
         Parameters
         ----------
         seglen: int,
@@ -492,6 +519,7 @@ class StretchCompressOffline(ReprMixin):
         -------
         tuple of generated segment,
         consists of (seg, label1, label2, ..., start_idx, end_idx)
+
         """
         assert not all(
             [start_idx is None, end_idx is None]
@@ -512,7 +540,7 @@ class StretchCompressOffline(ReprMixin):
                 start_idx = max(0, end_idx - new_len)
                 ratio = (end_idx - start_idx) / seglen - 1
             aug_seg = sig[..., start_idx:end_idx]
-            aug_seg = resample(x=aug_seg, num=seglen, axis=1)
+            aug_seg = SS.resample(x=aug_seg, num=seglen, axis=1)
             for lb in labels:
                 dtype = lb.dtype
                 aug_labels.append(
