@@ -1,16 +1,18 @@
 """
 """
 
+import warnings
 from abc import ABC, abstractmethod
-from typing import Any, Callable, List, NoReturn, Optional, Sequence, Union, Dict
+from typing import Any, Callable, List, Optional, Sequence, Union, Dict
 
 import numpy as np
 from torch import Tensor
 
 from ..utils.misc import ReprMixin, add_docstring
-from torch_ecg.utils.utils_data import ECGWaveFormNames
+from ..utils.utils_data import ECGWaveFormNames
 from ..utils.utils_metrics import (  # noqa: F401
     QRS_score,
+    cls_to_bin,
     metrics_from_confusion_matrix,
     confusion_matrix,
     ovr_confusion_matrix,
@@ -32,7 +34,7 @@ class Metrics(ReprMixin, ABC):
     __name__ = "Metrics"
 
     def __call__(self, *args: Any, **kwargs: Any) -> "Metrics":
-        return self.compute()
+        return self.compute(*args, **kwargs)
 
     @abstractmethod
     def compute(self, *args: Any, **kwargs: Any) -> "Metrics":
@@ -43,7 +45,6 @@ class Metrics(ReprMixin, ABC):
 class ClassificationMetrics(Metrics):
     """
     Metrics for the task of classification
-
     """
 
     __name__ = "ClassificationMetrics"
@@ -53,9 +54,8 @@ class ClassificationMetrics(Metrics):
         multi_label: bool = True,
         macro: bool = True,
         extra_metrics: Optional[Callable] = None,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         multi_label: bool,
@@ -81,14 +81,14 @@ class ClassificationMetrics(Metrics):
             k: np.nan
             for k in [
                 "sens",  # sensitivity, recall, hit rate, or true positive rate
-                "spec",  # specificity, selectivity or true negative rate
-                "prec",  # precision or positive predictive value
+                "spec",  # specificity, selectivity, or true negative rate
+                "prec",  # precision, or positive predictive value
                 "npv",  # negative predictive value
                 "jac",  # jaccard index, threat score, or critical success index
                 "acc",  # accuracy
                 "phi",  # phi coefficient, or matthews correlation coefficient
-                "fnr",  # false negative rate, miss rate
-                "fpr",  # false positive rate, fall-out
+                "fnr",  # false negative rate, or miss rate
+                "fpr",  # false positive rate, or fall-out
                 "fdr",  # false discovery rate
                 "for",  # false omission rate
                 "plr",  # positive likelihood ratio
@@ -108,9 +108,8 @@ class ClassificationMetrics(Metrics):
         self._cm = None
         self._cm_ovr = None
 
-    def set_macro(self, macro: bool) -> NoReturn:
+    def set_macro(self, macro: bool) -> None:
         """
-
         Parameters
         ----------
         macro: bool,
@@ -128,9 +127,17 @@ class ClassificationMetrics(Metrics):
         outputs: Union[np.ndarray, Tensor],
         num_classes: Optional[int] = None,
         weights: Optional[np.ndarray] = None,
+        thr: float = 0.5,
     ) -> "ClassificationMetrics":
-        self._cm = confusion_matrix(labels, outputs, num_classes)
-        self._cm_ovr = ovr_confusion_matrix(labels, outputs, num_classes)
+        """ """
+        labels, outputs = cls_to_bin(labels, outputs, num_classes)
+        num_samples, num_classes = np.shape(labels)
+        # probability outputs to binary outputs
+        bin_outputs = np.zeros_like(outputs, dtype=int)
+        bin_outputs[outputs >= thr] = 1
+        bin_outputs[outputs < thr] = 0
+        self._cm = confusion_matrix(labels, bin_outputs, num_classes)
+        self._cm_ovr = ovr_confusion_matrix(labels, bin_outputs, num_classes)
         self._metrics = metrics_from_confusion_matrix(
             labels, outputs, num_classes, weights
         )
@@ -161,6 +168,7 @@ class ClassificationMetrics(Metrics):
         outputs: Union[np.ndarray, Tensor],
         num_classes: Optional[int] = None,
         weights: Optional[np.ndarray] = None,
+        thr: float = 0.5,
     ) -> "ClassificationMetrics":
         return self.compute(labels, outputs, num_classes, weights)
 
@@ -306,6 +314,19 @@ class ClassificationMetrics(Metrics):
     def auprc(self) -> Union[float, np.ndarray]:
         return self._metrics[f"{self.__prefix}auprc"]
 
+    @property
+    def classification_report(self) -> dict:
+        if self.__prefix == "macro_":
+            return {
+                k.replace("macro_", ""): v
+                for k, v in self._metrics.items()
+                if k.startswith("macro_")
+            }
+        else:
+            return {
+                k: v for k, v in self._metrics.items() if not k.startswith("macro_")
+            }
+
     @property
     def extra_metrics(self) -> dict:
         return self._em
@@ -320,7 +341,6 @@ class ClassificationMetrics(Metrics):
 class RPeaksDetectionMetrics(Metrics):
     """
     Metrics for the task of R peaks detection, proposed in CPSC2019
-
     """
 
     __name__ = "RPeaksDetectionMetrics"
@@ -329,9 +349,8 @@ class RPeaksDetectionMetrics(Metrics):
         self,
         thr: float = 0.075,
         extra_metrics: Optional[Callable] = None,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         thr: float, default 0.075,
@@ -417,7 +436,6 @@ class RPeaksDetectionMetrics(Metrics):
 class WaveDelineationMetrics(Metrics):
     """
     Metrics for the task of ECG wave delineation
-
     """
 
     __name__ = "WaveDelineationMetrics"
@@ -427,9 +445,8 @@ class WaveDelineationMetrics(Metrics):
         macro: bool = True,
         tol: float = 0.15,
         extra_metrics: Optional[Callable] = None,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         macro: bool,
@@ -464,9 +481,8 @@ class WaveDelineationMetrics(Metrics):
         }
         self._metrics.update({f"macro_{k}": np.nan for k in self._metrics})
 
-    def set_macro(self, macro: bool) -> NoReturn:
+    def set_macro(self, macro: bool) -> None:
         """
-
         Parameters
         ----------
         macro: bool,
@@ -484,10 +500,10 @@ class WaveDelineationMetrics(Metrics):
         outputs: Union[np.ndarray, Tensor],
         class_map: Dict[str, int],
         fs: int,
+        mask_format: str = "channel_first",
         tol: Optional[float] = None,
     ) -> "WaveDelineationMetrics":
         f"""
-
         compute metrics for the task of ECG wave delineation
         (sensitivity, precision, f1_score, mean error and standard deviation of the mean errors)
         for multiple evaluations
@@ -511,7 +527,7 @@ class WaveDelineationMetrics(Metrics):
             format of the mask, one of the following:
             'channel_last' (alias 'lead_last'), or
             'channel_first' (alias 'lead_first')
-        tol: float, default 0.15,
+        tol: float, optional, defaults to self.tol,
             tolerance for the duration of the waveform,
             with units in seconds
 
@@ -530,7 +546,7 @@ class WaveDelineationMetrics(Metrics):
         truth_masks = labels.numpy() if isinstance(labels, Tensor) else labels
         pred_masks = outputs.numpy() if isinstance(outputs, Tensor) else outputs
         raw_metrics = compute_wave_delineation_metrics(
-            truth_masks, pred_masks, class_map, fs, tol or self.tol
+            truth_masks, pred_masks, class_map, fs, mask_format, tol or self.tol
         )
         self._metrics = {
             metric: {
@@ -551,7 +567,7 @@ class WaveDelineationMetrics(Metrics):
         }
         self._metrics.update(
             {
-                f"macro_{metric}": np.nanmean(self._metrics[metric].values())
+                f"macro_{metric}": np.nanmean(list(self._metrics[metric].values()))
                 for metric in [
                     "sensitivity",
                     "precision",
@@ -562,15 +578,17 @@ class WaveDelineationMetrics(Metrics):
             }
         )
         # sample-wise metrics
+        warnings.simplefilter("ignore")
         clf_mtx = ClassificationMetrics()
-        swm = clf_mtx(
+        swm = clf_mtx.compute(
             truth_masks.reshape((-1)).copy(),
             pred_masks.reshape((-1)).copy(),
-            len(class_map),
+            max(class_map.values()) + 1,
         )
+        warnings.simplefilter("default")
         self._metrics.update(
             {
-                "jaccard": swm._metrics["jac"],
+                "jaccard": {k: swm._metrics["jac"][v] for k, v in class_map.items()},
                 "macro_jaccard": swm._metrics["macro_jac"],
             }
         )
@@ -584,9 +602,10 @@ class WaveDelineationMetrics(Metrics):
         outputs: Union[np.ndarray, Tensor],
         class_map: Dict[str, int],
         fs: int,
+        mask_format: str = "channel_first",
         tol: Optional[float] = None,
     ) -> "WaveDelineationMetrics":
-        return self.compute(labels, outputs, class_map, fs, tol)
+        return self.compute(labels, outputs, class_map, fs, mask_format, tol)
 
     @property
     def sensitivity(self) -> Union[float, Dict[str, float]]:
