@@ -11,15 +11,9 @@ from abc import ABC, abstractmethod
 from collections import OrderedDict, deque
 from copy import deepcopy
 from pathlib import Path
-from typing import Dict, List, NoReturn, Optional, Tuple, Union
+from typing import Dict, List, Optional, Tuple, Union
 
 import numpy as np
-
-try:
-    from tqdm.auto import tqdm
-except ModuleNotFoundError:
-    from tqdm import tqdm
-
 import torch
 import torch_optimizer as extra_optim  # noqa: F401
 from torch import nn, optim
@@ -27,6 +21,7 @@ from torch.nn.parallel import DataParallel as DP  # noqa: F401
 from torch.nn.parallel import DistributedDataParallel as DDP  # noqa: F401
 from torch.utils.data import DataLoader
 from torch.utils.data.dataset import Dataset
+from tqdm.auto import tqdm
 
 from ..augmenters import AugmenterManager
 from ..cfg import CFG, DEFAULTS
@@ -36,8 +31,8 @@ from ..models.loss import (
     FocalLoss,
     MaskedBCEWithLogitsLoss,
 )
-from ..utils.misc import ReprMixin, dict_to_str, dicts_equal, get_date_str
-from ..utils.utils_nn import default_collate_fn as collate_fn  # noqa: F401
+from ..utils.misc import ReprMixin, dict_to_str, dicts_equal, get_date_str, get_kwargs
+from ..utils.utils_nn import default_collate_fn
 from .loggers import LoggerManager
 
 __all__ = [
@@ -64,11 +59,11 @@ class BaseTrainer(ReprMixin, ABC):
         dataset_cls: Dataset,
         model_config: dict,
         train_config: dict,
+        collate_fn: Optional[callable] = None,
         device: Optional[torch.device] = None,
         lazy: bool = False,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         model: Module,
@@ -97,6 +92,9 @@ class BaseTrainer(ReprMixin, ABC):
                 "optimizer": str,
                     "decay": float, optional, depending on the optimizer
                     "momentum": float, optional, depending on the optimizer
+        collate_fn: callable, optional,
+            the collate function for the data loader,
+            defaults to `default_collate_fn`
         device: torch.device, optional,
             the device to be used for training
         lazy: bool, default False,
@@ -119,6 +117,7 @@ class BaseTrainer(ReprMixin, ABC):
         self.dtype = next(self._model.parameters()).dtype
         self.model.to(self.device)
         self.lazy = lazy
+        self.collate_fn = collate_fn or default_collate_fn
 
         self.log_manager = None
         self.augmenter_manager = None
@@ -174,6 +173,8 @@ class BaseTrainer(ReprMixin, ABC):
                 total=self.n_train,
                 desc=f"Epoch {self.epoch}/{self.n_epochs}",
                 unit="signals",
+                dynamic_ncols=True,
+                mininterval=1.0,
             ) as pbar:
                 self.log_manager.epoch_start(self.epoch)
                 # train one epoch
@@ -284,9 +285,8 @@ class BaseTrainer(ReprMixin, ABC):
 
         return self.best_state_dict
 
-    def train_one_epoch(self, pbar: tqdm) -> NoReturn:
+    def train_one_epoch(self, pbar: tqdm) -> None:
         """
-
         train one epoch, and update the progress bar
 
         Parameters
@@ -349,7 +349,6 @@ class BaseTrainer(ReprMixin, ABC):
         """
         batch dimension, usually 0,
         but can be 1 for some models, e.g. RR_LSTM
-
         """
         raise NotImplementedError
 
@@ -373,7 +372,7 @@ class BaseTrainer(ReprMixin, ABC):
             "learning_rate",
         ] + self.extra_required_train_config_fields
 
-    def _validate_train_config(self) -> NoReturn:
+    def _validate_train_config(self) -> None:
         """ """
         for field in self.required_train_config_fields:
             if field not in self.train_config:
@@ -432,7 +431,7 @@ class BaseTrainer(ReprMixin, ABC):
         """
         raise NotImplementedError
 
-    def _update_lr(self, eval_res: Optional[dict] = None) -> NoReturn:
+    def _update_lr(self, eval_res: Optional[dict] = None) -> None:
         """finished, NOT checked,
 
         update learning rate using lr_scheduler, perhaps based on the eval_res
@@ -460,9 +459,8 @@ class BaseTrainer(ReprMixin, ABC):
         ]:
             self.scheduler.step()
 
-    def _setup_from_config(self, train_config: dict) -> NoReturn:
+    def _setup_from_config(self, train_config: dict) -> None:
         """
-
         Parameters
         ----------
         train_config: dict,
@@ -505,13 +503,13 @@ class BaseTrainer(ReprMixin, ABC):
         """ """
         return f"{self._model.__name__}_{self.train_config.optimizer}_LR_{self.lr}_BS_{self.batch_size}"
 
-    def _setup_log_manager(self) -> NoReturn:
+    def _setup_log_manager(self) -> None:
         """ """
         config = {"log_suffix": self.extra_log_suffix()}
         config.update(self.train_config)
         self.log_manager = LoggerManager.from_config(config=config)
 
-    def _setup_directories(self) -> NoReturn:
+    def _setup_directories(self) -> None:
         """ """
         if not self.train_config.get("model_dir", None):
             self._train_config.model_dir = self.train_config.checkpoints
@@ -519,7 +517,7 @@ class BaseTrainer(ReprMixin, ABC):
         self.train_config.checkpoints.mkdir(parents=True, exist_ok=True)
         self.train_config.model_dir.mkdir(parents=True, exist_ok=True)
 
-    def _setup_callbacks(self) -> NoReturn:
+    def _setup_callbacks(self) -> None:
         """ """
         self._train_config.monitor = self.train_config.get("monitor", None)
         if self.train_config.monitor is None:
@@ -541,7 +539,7 @@ class BaseTrainer(ReprMixin, ABC):
                 level=logging.WARNING,
             )
 
-    def _setup_augmenter_manager(self) -> NoReturn:
+    def _setup_augmenter_manager(self) -> None:
         """ """
         self.augmenter_manager = AugmenterManager.from_config(config=self.train_config)
 
@@ -550,9 +548,8 @@ class BaseTrainer(ReprMixin, ABC):
         self,
         train_dataset: Optional[Dataset] = None,
         val_dataset: Optional[Dataset] = None,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         setup the dataloaders for training and validation
 
         Parameters
@@ -577,7 +574,7 @@ class BaseTrainer(ReprMixin, ABC):
             num_workers=num_workers,
             pin_memory=True,
             drop_last=False,
-            collate_fn=collate_fn,
+            collate_fn=self.collate_fn,
         )
         self.val_loader = DataLoader(
             dataset=val_dataset,
@@ -586,7 +583,7 @@ class BaseTrainer(ReprMixin, ABC):
             num_workers=num_workers,
             pin_memory=True,
             drop_last=False,
-            collate_fn=collate_fn,
+            collate_fn=self.collate_fn,
         )
         ```
 
@@ -605,34 +602,51 @@ class BaseTrainer(ReprMixin, ABC):
             return len(self.val_loader.dataset)
         return 0
 
-    def _setup_optimizer(self) -> NoReturn:
+    def _setup_optimizer(self) -> None:
         """ """
         if self.train_config.optimizer.lower() == "adam":
+            optimizer_kwargs = get_kwargs(optim.Adam)
+            optimizer_kwargs.update(
+                {k: self.train_config.get(k, v) for k, v in optimizer_kwargs.items()}
+            )
+            optimizer_kwargs.update(dict(lr=self.lr))
             self.optimizer = optim.Adam(
                 params=self.model.parameters(),
-                lr=self.lr,
+                **optimizer_kwargs,
             )
         elif self.train_config.optimizer.lower() in ["adamw", "adamw_amsgrad"]:
+            optimizer_kwargs = get_kwargs(optim.AdamW)
+            optimizer_kwargs.update(
+                {k: self.train_config.get(k, v) for k, v in optimizer_kwargs.items()}
+            )
+            optimizer_kwargs.update(
+                dict(
+                    lr=self.lr,
+                    amsgrad=self.train_config.optimizer.lower().endswith("amsgrad"),
+                )
+            )
             self.optimizer = optim.AdamW(
                 params=self.model.parameters(),
-                lr=self.lr,
-                weight_decay=self.train_config.decay,
-                eps=1e-08,  # default
-                amsgrad=self.train_config.optimizer.lower().endswith("amsgrad"),
+                **optimizer_kwargs,
             )
         elif self.train_config.optimizer.lower() == "sgd":
+            optimizer_kwargs = get_kwargs(optim.SGD)
+            optimizer_kwargs.update(
+                {k: self.train_config.get(k, v) for k, v in optimizer_kwargs.items()}
+            )
+            optimizer_kwargs.update(dict(lr=self.lr))
             self.optimizer = optim.SGD(
                 params=self.model.parameters(),
-                lr=self.lr,
-                momentum=self.train_config.momentum,
-                weight_decay=self.train_config.decay,
+                **optimizer_kwargs,
             )
         else:
             raise NotImplementedError(
-                f"optimizer `{self.train_config.optimizer}` not implemented!"
+                f"optimizer `{self.train_config.optimizer}` not implemented! "
+                "Please use one of the following: `adam`, `adamw`, `adamw_amsgrad`, `sgd`, "
+                "or override this method to setup your own optimizer."
             )
 
-    def _setup_scheduler(self) -> NoReturn:
+    def _setup_scheduler(self) -> None:
         """ """
         if (
             self.train_config.lr_scheduler is None
@@ -665,12 +679,14 @@ class BaseTrainer(ReprMixin, ABC):
                 steps_per_epoch=len(self.train_loader),
                 # verbose=False,
             )
-        else:
+        else:  # TODO: add linear and linear with warmup schedulers
             raise NotImplementedError(
-                f"lr scheduler `{self.train_config.lr_scheduler.lower()}` not implemented for training"
+                f"lr scheduler `{self.train_config.lr_scheduler.lower()}` not implemented for training! "
+                "Please use one of the following: `none`, `plateau`, `step`, `one_cycle`, "
+                "or override this method to setup your own lr scheduler."
             )
 
-    def _setup_criterion(self) -> NoReturn:
+    def _setup_criterion(self) -> None:
         """ """
         loss_kw = self.train_config.get("loss_kw", {})
         for k, v in loss_kw.items():
@@ -694,12 +710,14 @@ class BaseTrainer(ReprMixin, ABC):
             self.criterion = nn.CrossEntropyLoss(**loss_kw)
         else:
             raise NotImplementedError(
-                f"loss `{self.train_config.loss}` not implemented!"
+                f"loss `{self.train_config.loss}` not implemented! "
+                "Please use one of the following: `BCEWithLogitsLoss`, `BCEWithLogitsWithClassWeightLoss`, "
+                "`BCELoss`, `MaskedBCEWithLogitsLoss`, `MaskedBCEWithLogitsLoss`, `FocalLoss`, "
+                "`AsymmetricLoss`, `CrossEntropyLoss`, or override this method to setup your own criterion."
             )
 
     def _check_model_config_compatability(self, model_config: dict) -> bool:
         """
-
         Parameters
         ----------
         model_config: dict,
@@ -714,7 +732,7 @@ class BaseTrainer(ReprMixin, ABC):
         """
         return dicts_equal(self.model_config, model_config)
 
-    def resume_from_checkpoint(self, checkpoint: Union[str, dict]) -> NoReturn:
+    def resume_from_checkpoint(self, checkpoint: Union[str, dict]) -> None:
         """NOT finished, NOT checked,
 
         resume a training process from a checkpoint
@@ -742,7 +760,7 @@ class BaseTrainer(ReprMixin, ABC):
                 "epoch",
             ]
         ).issubset(ckpt.keys()), insufficient_msg
-        if not self._check_model_config_compatability(self, ckpt["model_config"]):
+        if not self._check_model_config_compatability(ckpt["model_config"]):
             raise ValueError(
                 "model config of the checkpoint is not compatible with the config of the current model"
             )
@@ -751,9 +769,8 @@ class BaseTrainer(ReprMixin, ABC):
         self._setup_from_config(ckpt["train_config"])
         # TODO: resume optimizer, etc.
 
-    def save_checkpoint(self, path: str) -> NoReturn:
+    def save_checkpoint(self, path: str) -> None:
         """
-
         Parameters
         ----------
         path: str,
@@ -773,7 +790,6 @@ class BaseTrainer(ReprMixin, ABC):
 
     def extra_repr_keys(self) -> List[str]:
         """
-
         Returns
         -------
         list of str,
