@@ -262,7 +262,6 @@ def load_weights(
     classes: Sequence[Union[int, str]] = None, return_fmt: str = "np"
 ) -> Union[np.ndarray, pd.DataFrame]:
     """
-
     load the weight matrix of the `classes`
 
     Parameters:
@@ -277,6 +276,7 @@ def load_weights(
     --------
     mat: 2d array or DataFrame,
         the weight matrix of the `classes`
+
     """
     if classes:
         l_nc = [normalize_class(c, ensure_scored=True) for c in classes]
@@ -284,6 +284,7 @@ def load_weights(
         mat = df_weights_abbr.loc[l_nc, l_nc]
     else:
         mat = df_weights_abbr.copy()
+        classes = df_weights_abbr.columns.tolist()
 
     if return_fmt.lower() == "np":
         mat = mat.values
@@ -299,7 +300,6 @@ def load_weights(
 
 def normalize_class(c: Union[str, int], ensure_scored: bool = False) -> str:
     """
-
     normalize the class name to its abbr.,
     facilitating the computation of the `load_weights` function
 
@@ -315,6 +315,7 @@ def normalize_class(c: Union[str, int], ensure_scored: bool = False) -> str:
     --------
     nc: str,
         the abbr. of the class
+
     """
     nc = snomed_ct_code_to_abbr.get(str(c), str(c))
     if ensure_scored and nc not in df_weights_abbr.columns:
@@ -324,7 +325,6 @@ def normalize_class(c: Union[str, int], ensure_scored: bool = False) -> str:
 
 def get_class(snomed_ct_code: Union[str, int]) -> Dict[str, str]:
     """
-
     look up the abbreviation and the full name of an ECG arrhythmia,
     given its SNOMED CT Code
 
@@ -337,6 +337,7 @@ def get_class(snomed_ct_code: Union[str, int]) -> Dict[str, str]:
     --------
     arrhythmia_class: dict,
         containing `abbr` the abbreviation and `fullname` the full name of the arrhythmia
+
     """
     arrhythmia_class = {
         "abbr": snomed_ct_code_to_abbr[str(snomed_ct_code)],
@@ -354,6 +355,7 @@ def get_class_count(
     fmt: str = "a",
 ) -> Dict[str, int]:
     """
+    get the number of classes in the `tranches`
 
     Parameters:
     -----------
@@ -380,6 +382,7 @@ def get_class_count(
     class_count: dict,
         key: class in the format of `fmt`
         value: count of a class in `tranches`
+
     """
     assert threshold >= 0
     tranche_names = CFG(
@@ -444,6 +447,7 @@ def get_class_weight(
     min_weight: Real = 0.5,
 ) -> Dict[str, int]:
     """
+    get the weight of each class in each tranche
 
     Parameters:
     -----------
@@ -473,6 +477,7 @@ def get_class_weight(
     class_weight: dict,
         key: class in the format of `fmt`
         value: weight of a class in `tranches`
+
     """
     class_count = get_class_count(
         tranches=tranches,
@@ -621,6 +626,7 @@ def get_cooccurrence(
     c1: Union[str, int], c2: Union[str, int], ensure_scored: bool = False
 ) -> int:
     """
+    Get the co-occurrence count between two diagnoses.
 
     Parameters:
     -----------
@@ -635,10 +641,11 @@ def get_cooccurrence(
     cooccurrence: int,
         cooccurrence of class `c1` and `c2`, if they are not the same class;
         otherwise the occurrence of the class `c1` (also `c2`)
+
     """
     _c1 = normalize_class(c1, ensure_scored=ensure_scored)
     _c2 = normalize_class(c2, ensure_scored=ensure_scored)
-    cooccurrence = dx_cooccurrence_all.loc[_c1, _c2]
+    cooccurrence = dx_cooccurrence_all.loc[_c1, _c2].item()
     return cooccurrence
 
 
