@@ -1,18 +1,23 @@
 # -*- coding: utf-8 -*-
 """
+The 2nd China Physiological Signal Challenge (CPSC 2019):
+Challenging QRS Detection and Heart Rate Estimation from Single-Lead ECG Recordings
 """
 
 import json
 from numbers import Real
 from pathlib import Path
-from typing import Any, NoReturn, Optional, Sequence, Union
+from typing import Any, Optional, Sequence, Union
 
 import numpy as np
+import pandas as pd
+import scipy.signal as SS
 from scipy.io import loadmat
 
-from ...utils.download import http_get
+from ...cfg import DEFAULTS
 from ...utils.misc import add_docstring
-from ..base import DEFAULT_FIG_SIZE_PER_SEC, CPSCDataBase
+from ..base import DEFAULT_FIG_SIZE_PER_SEC, CPSCDataBase, DataBaseInfo
+
 
 __all__ = [
     "CPSC2019",
@@ -20,76 +25,70 @@ __all__ = [
 ]
 
 
-class CPSC2019(CPSCDataBase):
-    """
-
+_CPSC2019_INFO = DataBaseInfo(
+    title="""
     The 2nd China Physiological Signal Challenge (CPSC 2019):
     Challenging QRS Detection and Heart Rate Estimation from Single-Lead ECG Recordings
-
-    ABOUT CPSC2019
-    --------------
+    """,
+    about="""
     1. Training data consists of 2,000 single-lead ECG recordings collected from patients with cardiovascular disease (CVD)
     2. Each of the recording last for 10 s
     3. Sampling rate = 500 Hz
-
-    NOTE
-    ----
-
-    ISSUES
-    ------
+    """,
+    usage=[
+        "ECG wave delineation",
+    ],
+    issues="""
     1. there're 13 records with unusual large values (> 20 mV):
         data_00098, data_00167, data_00173, data_00223, data_00224, data_00245, data_00813,
         data_00814, data_00815, data_00833, data_00841, data_00949, data_00950
-    >>> for rec in dr.all_records:
-    >>>     data = dr.load_data(rec)
-    >>>     if np.max(data) > 20:
-    >>>         print(f"{rec} has max value ({np.max(data)} mV) > 20 mV")
-    data_00173 has max value (32.72031811111111 mV) > 20 mV
-    data_00223 has max value (32.75516713333333 mV) > 20 mV
-    data_00224 has max value (32.7519272 mV) > 20 mV
-    data_00245 has max value (32.75305293939394 mV) > 20 mV
-    data_00813 has max value (32.75865595876289 mV) > 20 mV
-    data_00814 has max value (32.75865595876289 mV) > 20 mV
-    data_00815 has max value (32.75558282474227 mV) > 20 mV
-    data_00833 has max value (32.76330123809524 mV) > 20 mV
-    data_00841 has max value (32.727626558139534 mV) > 20 mV
-    data_00949 has max value (32.75699667692308 mV) > 20 mV
-    data_00950 has max value (32.769551661538465 mV) > 20 mV
-    2. rpeak references (annotations) loaded from files has dtype = uint16,
-    which would produce unexpected large positive values when subtracting values larger than it,
-    rather than the correct negative value.
-    This might cause confusion in computing metrics when using annotations subtracting
-    (instead of being subtracted by) predictions.
-    3. official scoring function has errors,
-    which would falsely omit the interval between the 0-th and the 1-st ref rpeaks,
-    thus potentially missing false positive
+        >>> for rec in dr.all_records:
+        >>>     data = dr.load_data(rec)
+        >>>     if np.max(data) > 20:
+        >>>         print(f"{rec} has max value ({np.max(data)} mV) > 20 mV")
+        data_00173 has max value (32.72031811111111 mV) > 20 mV
+        data_00223 has max value (32.75516713333333 mV) > 20 mV
+        data_00224 has max value (32.7519272 mV) > 20 mV
+        data_00245 has max value (32.75305293939394 mV) > 20 mV
+        data_00813 has max value (32.75865595876289 mV) > 20 mV
+        data_00814 has max value (32.75865595876289 mV) > 20 mV
+        data_00815 has max value (32.75558282474227 mV) > 20 mV
+        data_00833 has max value (32.76330123809524 mV) > 20 mV
+        data_00841 has max value (32.727626558139534 mV) > 20 mV
+        data_00949 has max value (32.75699667692308 mV) > 20 mV
+        data_00950 has max value (32.769551661538465 mV) > 20 mV
+    2. rpeak references (annotations) loaded from files has dtype = uint16, which would produce unexpected large positive values when subtracting values larger than it, rather than the correct negative value. This might cause confusion in computing metrics when using annotations subtracting (instead of being subtracted by) predictions.
+    3. official scoring function has errors, which would falsely omit the interval between the 0-th and the 1-st ref rpeaks, thus potentially missing false positive
+    """,
+    references=[
+        "http://2019.icbeb.org/Challenge.html",
+    ],
+    doi="10.1166/jmihi.2019.2800",
+)
 
-    Usage
-    -----
-    1. ECG wave delineation
 
-    References
-    ----------
-    1. <a name="ref1"></a> http://2019.icbeb.org/Challenge.html
+@add_docstring(_CPSC2019_INFO.format_database_docstring())
+class CPSC2019(CPSCDataBase):
+    """ """
 
-    """
+    __name__ = "CPSC2019"
 
     def __init__(
         self,
-        db_dir: Union[str, Path],
+        db_dir: Optional[Union[str, Path]] = None,
         working_dir: Optional[Union[str, Path]] = None,
-        verbose: int = 2,
+        verbose: int = 1,
         **kwargs: Any,
-    ) -> NoReturn:
+    ) -> None:
         """finished, to be improved,
 
         Parameters
         ----------
-        db_dir: str or Path,
+        db_dir: str or Path, optional,
             storage path of the database
         working_dir: str or Path, optional,
             working directory, to store intermediate files and log file
-        verbose: int, default 2,
+        verbose: int, default 1
             log verbosity
         kwargs: auxilliary key word arguments
 
@@ -112,42 +111,58 @@ class CPSC2019(CPSCDataBase):
         self.rec_dir = self.db_dir / "data"
         self.ann_dir = self.db_dir / "ref"
 
-        self.n_records = 2000
-        self._all_records = [f"data_{i:05d}" for i in range(1, 1 + self.n_records)]
-        self._all_annotations = [f"R_{i:05d}" for i in range(1, 1 + self.n_records)]
-        self._ls_rec()
-
         # aliases
         self.data_dir = self.rec_dir
         self.ref_dir = self.ann_dir
 
-    def _ls_rec(self) -> NoReturn:
+        self.n_records = 2000
+        self._all_records = None
+        self._all_annotations = None
+        self._ls_rec()
+
+    def _ls_rec(self) -> None:
         """ """
         records_fn = self.db_dir / "records.json"
-        if records_fn.is_file():
-            records_json = json.loads(records_fn.read_text())
-            self._all_records = records_json["rec"]
-            self._all_annotations = records_json["ann"]
-            return
-        print(
-            "Please allow some time for the reader to confirm the existence of corresponding data files and annotation files..."
+        self._all_records = [f"data_{i:05d}" for i in range(1, 1 + self.n_records)]
+        self._all_annotations = [f"R_{i:05d}" for i in range(1, 1 + self.n_records)]
+        self._df_records = pd.DataFrame()
+        self.logger.info(
+            "Please allow some time for the reader to confirm "
+            "the existence of corresponding data files and annotation files..."
         )
+        self._df_records["record"] = [
+            f"data_{i:05d}" for i in range(1, 1 + self.n_records)
+        ]
+        self._df_records["path"] = self._df_records["record"].apply(
+            lambda x: self.data_dir / x
+        )
+        self._df_records["annotation"] = self._df_records["record"].apply(
+            lambda x: x.replace("data", "R")
+        )
+        self._df_records.index = self._df_records["record"]
+        self._df_records = self._df_records.drop(columns="record")
+        self._all_annotations = [f"R_{i:05d}" for i in range(1, 1 + self.n_records)]
         self._all_records = [
             rec
             for rec in self._all_records
-            if (self.rec_dir / f"{rec}.{self.rec_ext}").is_file()
+            if self.get_absolute_path(rec, self.rec_ext).is_file()
         ]
         self._all_annotations = [
             ann
             for ann in self._all_annotations
-            if (self.ann_dir / f"{ann}.{self.ann_ext}").is_file()
+            if self.get_absolute_path(ann, self.ann_ext, ann=True).is_file()
         ]
         common = set([rec.split("_")[1] for rec in self._all_records]) & set(
             [ann.split("_")[1] for ann in self._all_annotations]
         )
-        common = sorted(list(common))
+        if self._subsample is not None:
+            # random subsample with ratio `self._subsample`
+            size = min(len(common), max(1, int(round(self._subsample * len(common)))))
+            common = DEFAULTS.RNG.choice(list(common), size, replace=False)
+        common = sorted(common)
         self._all_records = [f"data_{item}" for item in common]
         self._all_annotations = [f"R_{item}" for item in common]
+        self._df_records = self._df_records.loc[self._all_records]
         records_json = {"rec": self._all_records, "ann": self._all_annotations}
         records_fn.write_text(json.dumps(records_json, ensure_ascii=False))
 
@@ -167,7 +182,7 @@ class CPSC2019(CPSCDataBase):
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
 
         Returns
         -------
@@ -178,17 +193,60 @@ class CPSC2019(CPSCDataBase):
         pid = 0
         raise NotImplementedError
 
-    def load_data(
-        self, rec: Union[int, str], units: str = "mV", keep_dim: bool = True
-    ) -> np.ndarray:
+    def get_absolute_path(
+        self,
+        rec: Union[str, int],
+        extension: Optional[str] = None,
+        ann: bool = False,
+    ) -> Path:
         """
+        get the absolute path of the record `rec`
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
-        keep_dim: bool, default True,
-            whether or not to flatten the data of shape (n,1)
+            record name or index of the record in `self.all_records`
+        extension: str, optional,
+            extension of the file
+        ann: bool, default False,
+            whether to get the annotation file path or not
+
+        Returns
+        -------
+        Path,
+            absolute path of the file
+
+        """
+        if isinstance(rec, int):
+            rec = self[rec]
+        if extension is not None and not extension.startswith("."):
+            extension = f".{extension}"
+        if ann:
+            rec = rec.replace("data", "R")
+            return self.ann_dir / f"{rec}{extension or ''}"
+        return self.data_dir / f"{rec}{extension or ''}"
+
+    def load_data(
+        self,
+        rec: Union[int, str],
+        data_format: str = "channel_first",
+        units: str = "mV",
+        fs: Optional[Real] = None,
+    ) -> np.ndarray:
+        """
+        Parameters
+        ----------
+        rec: str or int,
+            record name or index of the record in `self.all_records`
+        data_format: str, default "channel_first",
+            format of the ecg data,
+            "channel_last" (alias "lead_last"), or
+            "channel_first" (alias "lead_first"), or
+            "flat" (alias "plain")
+        units: str or None, default "mV",
+            units of the output signal, can also be "μV", with aliases of "uV", "muV"
+        fs: real number, optional,
+            if not None, the loaded data will be resampled to this frequency
 
         Returns
         -------
@@ -196,28 +254,30 @@ class CPSC2019(CPSCDataBase):
             the ECG data
 
         """
-        if isinstance(rec, int):
-            rec = self[rec]
-        fp = self.data_dir / f"{self._get_rec_name(rec)}.{self.rec_ext}"
-        data = loadmat(str(fp))["ecg"]
-        if units.lower() in [
-            "uv",
-            "μv",
-        ]:
-            data = (1000 * data).astype(int)
-        if not keep_dim:
+        fp = self.get_absolute_path(rec, self.rec_ext)
+        data = loadmat(str(fp))["ecg"].astype(DEFAULTS.DTYPE.NP)
+        if fs is not None and fs != self.fs:
+            data = SS.resample_poly(data, fs, self.fs, axis=0).astype(data.dtype)
+        if data_format.lower() in ["channel_first", "lead_first"]:
+            data = data.T
+        elif data_format.lower() in ["flat", "plain"]:
             data = data.flatten()
+        elif data_format.lower() not in ["channel_last", "lead_last"]:
+            raise ValueError(f"Invalid `data_format`: {data_format}")
+        if units.lower() in ["uv", "μv", "muv"]:
+            data = (1000 * data).astype(int)
+        elif units.lower() != "mv":
+            raise ValueError(f"Invalid `units`: {units}")
         return data
 
-    def load_ann(self, rec: Union[int, str], keep_dim: bool = True) -> np.ndarray:
+    def load_ann(self, rec: Union[int, str]) -> np.ndarray:
         """
+        load the annotations (indices of R peaks) of the record `rec`
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
-        keep_dim: bool, default True,
-            whether or not to flatten the data of shape (n,1)
+            record name or index of the record in `self.all_records`
 
         Returns
         -------
@@ -225,68 +285,23 @@ class CPSC2019(CPSCDataBase):
             array of indices of R peaks
 
         """
-        if isinstance(rec, int):
-            rec = self[rec]
-        fp = self.ann_dir / f"{self._get_ann_name(rec)}.{self.ann_ext}"
-        ann = loadmat(str(fp))["R_peak"].astype(int)
-        if not keep_dim:
-            ann = ann.flatten()
+        fp = self.get_absolute_path(rec, self.ann_ext, ann=True)
+        ann = loadmat(str(fp))["R_peak"].astype(int).flatten()
         return ann
 
     @add_docstring(load_ann.__doc__)
-    def load_rpeaks(self, rec: Union[int, str], keep_dim: bool = True) -> np.ndarray:
+    def load_rpeaks(self, rec: Union[int, str]) -> np.ndarray:
         """
         alias of `self.load_ann`
         """
-        return self.load_ann(rec=rec, keep_dim=keep_dim)
+        return self.load_ann(rec=rec)
 
     @add_docstring(load_rpeaks.__doc__)
-    def load_rpeak_indices(
-        self, rec: Union[int, str], keep_dim: bool = True
-    ) -> np.ndarray:
+    def load_rpeak_indices(self, rec: Union[int, str]) -> np.ndarray:
         """
         alias of `self.load_rpeaks`
         """
-        return self.load_rpeaks(rec=rec, keep_dim=keep_dim)
-
-    def _get_rec_name(self, rec: Union[int, str]) -> str:
-        """
-
-        Parameters
-        ----------
-        rec: str or int,
-            name or index of the record
-
-        Returns
-        -------
-        rec_name: str,
-            filename of the record
-
-        """
-        if isinstance(rec, int):
-            rec = self[rec]
-        rec_name = rec
-        return rec_name
-
-    def _get_ann_name(self, rec: Union[int, str]) -> str:
-        """
-
-        Parameters
-        ----------
-        rec: str or int,
-            name or index of the record
-
-        Returns
-        -------
-        ann_name: str,
-            filename of annotations of the record `rec`
-
-        """
-        if isinstance(rec, int):
-            rec = self[rec]
-        rec_name = self._get_rec_name(rec)
-        ann_name = rec_name.replace("data", "R")
-        return ann_name
+        return self.load_rpeaks(rec=rec)
 
     def plot(
         self,
@@ -295,13 +310,12 @@ class CPSC2019(CPSCDataBase):
         ann: Optional[np.ndarray] = None,
         ticks_granularity: int = 0,
         **kwargs: Any,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         data: ndarray, optional,
             ECG signal to plot,
             if given, data of `rec` will not be used,
@@ -320,7 +334,7 @@ class CPSC2019(CPSCDataBase):
             import matplotlib.pyplot as plt
 
         if data is None:
-            _data = self.load_data(rec, units="μV", keep_dim=False)
+            _data = self.load_data(rec, units="μV", data_format="flat")
         else:
             units = self._auto_infer_units(data)
             if units == "mV":
@@ -331,7 +345,7 @@ class CPSC2019(CPSCDataBase):
         duration = len(_data) / self.fs
         secs = np.linspace(0, duration, len(_data))
         if ann is None or data is None:
-            rpeak_secs = self.load_rpeaks(rec, keep_dim=False) / self.fs
+            rpeak_secs = self.load_rpeaks(rec) / self.fs
         else:
             rpeak_secs = np.array(ann) / self.fs
 
@@ -368,11 +382,16 @@ class CPSC2019(CPSCDataBase):
 
     @property
     def url(self) -> str:
-        return "http://2019.icbeb.org/file/train.rar"
+        # return "http://2019.icbeb.org/file/train.rar"
+        return "https://www.dropbox.com/s/75nee0pqdy3f9r2/CPSC2019-train.zip?dl=1"
 
-    def download(self) -> NoReturn:
-        """download the database from self.url"""
-        http_get(self.url, self.db_dir, extract=False)
+    @property
+    def database_info(self) -> DataBaseInfo:
+        return _CPSC2019_INFO
+
+    @property
+    def webpage(self) -> str:
+        return "http://2019.icbeb.org/Challenge.html"
 
 
 def compute_metrics(
@@ -383,7 +402,6 @@ def compute_metrics(
     verbose: int = 0,
 ) -> float:
     """
-
     metric (scoring) function modified from the official one, with errors fixed
 
     Parameters
@@ -406,9 +424,10 @@ def compute_metrics(
         accuracy of predictions
 
     """
-    assert len(rpeaks_truths) == len(
-        rpeaks_preds
-    ), f"number of records does not match, truth indicates {len(rpeaks_truths)}, while pred indicates {len(rpeaks_preds)}"
+    assert len(rpeaks_truths) == len(rpeaks_preds), (
+        f"number of records does not match, truth indicates {len(rpeaks_truths)}, "
+        f"while pred indicates {len(rpeaks_preds)}"
+    )
     n_records = len(rpeaks_truths)
     record_flags = np.ones((len(rpeaks_truths),), dtype=float)
     thr_ = thr * fs
@@ -452,7 +471,8 @@ def compute_metrics(
 
         if verbose >= 2:
             print(
-                f"for the {idx}-th record,\ntrue positive = {true_positive}\nfalse positive = {false_positive}\nfalse negative = {false_negative}"
+                f"for the {idx}-th record,\ntrue positive = {true_positive}\n"
+                f"false positive = {false_positive}\nfalse negative = {false_negative}"
             )
 
     rec_acc = round(np.sum(record_flags) / n_records, 4)
