@@ -1,20 +1,24 @@
 # -*- coding: utf-8 -*-
 """
+The 3rd China Physiological Signal Challenge 2020:
+Searching for Premature Ventricular Contraction (PVC) and Supraventricular Premature Beat (SPB) from Long-term ECGs
 """
 
 import math
 from numbers import Real
 from pathlib import Path
-from typing import Any, Dict, List, NoReturn, Optional, Sequence, Tuple, Union
+from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
 
 import numpy as np
+import pandas as pd
+import scipy.signal as SS
 from scipy.io import loadmat
 
 from ...cfg import CFG, DEFAULTS
-from ...utils.download import http_get
-from ...utils.ecg_arrhythmia_knowledge import PVC, SPB  # noqa: F401
+from ...utils.misc import add_docstring
 from ...utils.utils_interval import get_optimal_covering
-from ..base import DEFAULT_FIG_SIZE_PER_SEC, CPSCDataBase
+from ..base import DEFAULT_FIG_SIZE_PER_SEC, CPSCDataBase, DataBaseInfo
+
 
 __all__ = [
     "CPSC2020",
@@ -22,20 +26,17 @@ __all__ = [
 ]
 
 
-class CPSC2020(CPSCDataBase):
-    """
-
+_CPSC2020_INFO = DataBaseInfo(
+    title="""
     The 3rd China Physiological Signal Challenge 2020:
     Searching for Premature Ventricular Contraction (PVC) and Supraventricular Premature Beat (SPB) from Long-term ECGs
-
-    ABOUT CPSC2020
-    --------------
+    """,
+    about="""
     1. training data consists of 10 single-lead ECG recordings collected from arrhythmia patients, each of the recording last for about 24 hours
     2. data and annotations are stored in v5 .mat files
     3. A02, A03, A08 are patient with atrial fibrillation
     4. sampling frequency = 400 Hz
     5. Detailed information:
-        -------------------------------------------------------------------------
         rec   ?AF   Length(h)   # N beats   # V beats   # S beats   # Total beats
         A01   No	25.89       109,062     0           24          109,086
         A02   Yes	22.83       98,936      4,554       0           103,490
@@ -49,76 +50,77 @@ class CPSC2020(CPSCDataBase):
         A10   No	23.64	    72,821	    169	        9,071	    82,061
     6. challenging factors for accurate detection of SPB and PVC:
         amplitude variation; morphological variation; noise
-
-    NOTE
-    ----
+    """,
+    note="""
     1. the records can roughly be classified into 4 groups:
         N:  A01, A03, A05, A06
         V:  A02, A08
         S:  A09, A10
         VS: A04, A07
     2. as premature beats and atrial fibrillation can co-exists
-    (via the following code, and data from CINC2020),
-    the situation becomes more complicated.
-    >>> from utils.scoring_aux_data import dx_cooccurrence_all
-    >>> dx_cooccurrence_all.loc["AF", ["PAC","PVC","SVPB","VPB"]]
-    PAC     20
-    PVC     19
-    SVPB     4
-    VPB     20
-    Name: AF, dtype: int64
-    this could also be seen from this dataset, via the following code as an example:
-    >>> from data_reader import CPSC2020Reader as CR
-    >>> db_dir = "/media/cfs/wenhao71/data/CPSC2020/TrainingSet/"
-    >>> dr = CR(db_dir)
-    >>> rec = dr.all_records[1]
-    >>> dr.plot(rec, sampfrom=0, sampto=4000, ticks_granularity=2)
+        (via the following code, and data from CINC2020),
+        the situation becomes more complicated.
+        >>> from utils.scoring_aux_data import dx_cooccurrence_all
+        >>> dx_cooccurrence_all.loc["AF", ["PAC","PVC","SVPB","VPB"]]
+        PAC     20
+        PVC     19
+        SVPB     4
+        VPB     20
+        Name: AF, dtype: int64
+        this could also be seen from this dataset, via the following code as an example:
+        >>> from data_reader import CPSC2020Reader as CR
+        >>> db_dir = "/media/cfs/wenhao71/data/CPSC2020/TrainingSet/"
+        >>> dr = CR(db_dir)
+        >>> rec = dr.all_records[1]
+        >>> dr.plot(rec, sampfrom=0, sampto=4000, ticks_granularity=2)
     3. PVC and SPB can also co-exist, as illustrated via the following code (from CINC2020):
-    >>> from utils.scoring_aux_data import dx_cooccurrence_all
-    >>> dx_cooccurrence_all.loc[["PVC","VPB"], ["PAC","SVPB",]]
-    PAC	SVPB
-    PVC	14	1
-    VPB	27	0
-    and also from the following code:
-    >>> for rec in dr.all_records:
-    >>>     ann = dr.load_ann(rec)
-    >>>     spb = ann["SPB_indices"]
-    >>>     pvc = ann["PVC_indices"]
-    >>>     if len(np.diff(spb)) > 0:
-    >>>         print(f"{rec}: min dist among SPB = {np.min(np.diff(spb))}")
-    >>>     if len(np.diff(pvc)) > 0:
-    >>>         print(f"{rec}: min dist among PVC = {np.min(np.diff(pvc))}")
-    >>>     diff = [s-p for s,p in product(spb, pvc)]
-    >>>     if len(diff) > 0:
-    >>>         print(f"{rec}: min dist between SPB and PVC = {np.min(np.abs(diff))}")
-    A01: min dist among SPB = 630
-    A02: min dist among SPB = 696
-    A02: min dist among PVC = 87
-    A02: min dist between SPB and PVC = 562
-    A03: min dist among SPB = 7044
-    A03: min dist among PVC = 151
-    A03: min dist between SPB and PVC = 3750
-    A04: min dist among SPB = 175
-    A04: min dist among PVC = 156
-    A04: min dist between SPB and PVC = 178
-    A05: min dist among SPB = 182
-    A05: min dist between SPB and PVC = 22320
-    A06: min dist among SPB = 455158
-    A07: min dist among SPB = 603
-    A07: min dist among PVC = 153
-    A07: min dist between SPB and PVC = 257
-    A08: min dist among SPB = 2903029
-    A08: min dist among PVC = 106
-    A08: min dist between SPB and PVC = 350
-    A09: min dist among SPB = 180
-    A09: min dist among PVC = 7719290
-    A09: min dist between SPB and PVC = 1271
-    A10: min dist among SPB = 148
-    A10: min dist among PVC = 708
-    A10: min dist between SPB and PVC = 177
-
-    ISSUES
-    ------
+        >>> from utils.scoring_aux_data import dx_cooccurrence_all
+        >>> dx_cooccurrence_all.loc[["PVC","VPB"], ["PAC","SVPB",]]
+        PAC	SVPB
+        PVC	14	1
+        VPB	27	0
+        and also from the following code:
+        >>> for rec in dr.all_records:
+        >>>     ann = dr.load_ann(rec)
+        >>>     spb = ann["SPB_indices"]
+        >>>     pvc = ann["PVC_indices"]
+        >>>     if len(np.diff(spb)) > 0:
+        >>>         print(f"{rec}: min dist among SPB = {np.min(np.diff(spb))}")
+        >>>     if len(np.diff(pvc)) > 0:
+        >>>         print(f"{rec}: min dist among PVC = {np.min(np.diff(pvc))}")
+        >>>     diff = [s-p for s,p in product(spb, pvc)]
+        >>>     if len(diff) > 0:
+        >>>         print(f"{rec}: min dist between SPB and PVC = {np.min(np.abs(diff))}")
+        A01: min dist among SPB = 630
+        A02: min dist among SPB = 696
+        A02: min dist among PVC = 87
+        A02: min dist between SPB and PVC = 562
+        A03: min dist among SPB = 7044
+        A03: min dist among PVC = 151
+        A03: min dist between SPB and PVC = 3750
+        A04: min dist among SPB = 175
+        A04: min dist among PVC = 156
+        A04: min dist between SPB and PVC = 178
+        A05: min dist among SPB = 182
+        A05: min dist between SPB and PVC = 22320
+        A06: min dist among SPB = 455158
+        A07: min dist among SPB = 603
+        A07: min dist among PVC = 153
+        A07: min dist between SPB and PVC = 257
+        A08: min dist among SPB = 2903029
+        A08: min dist among PVC = 106
+        A08: min dist between SPB and PVC = 350
+        A09: min dist among SPB = 180
+        A09: min dist among PVC = 7719290
+        A09: min dist between SPB and PVC = 1271
+        A10: min dist among SPB = 148
+        A10: min dist among PVC = 708
+        A10: min dist between SPB and PVC = 177
+    """,
+    usage=[
+        "ECG arrhythmia (PVC, SPB) detection",
+    ],
+    issues="""
     1. currently, using `xqrs` as qrs detector,
        a lot more (more than 1000) rpeaks would be detected for A02, A07, A08,
        which might be caused by motion artefacts (or AF?);
@@ -136,8 +138,7 @@ class CPSC2020(CPSCDataBase):
        A08   Yes    126908              118,311
        A09   No     89972               89,693
        A10   No     83509               82,061
-    2. (fixed by an official update)
-    A04 has duplicate "PVC_indices" (13534856,27147621,35141190 all appear twice):
+    2. (fixed by an official update) A04 has duplicate "PVC_indices" (13534856,27147621,35141190 all appear twice):
        before correction of `load_ann`:
        >>> from collections import Counter
        >>> db_dir = "/mnt/wenhao71/data/CPSC2020/TrainingSet/"
@@ -152,40 +153,37 @@ class CPSC2020(CPSCDataBase):
        for `y_values[n] = (y_values[n] - y_min) / (y_max - y_min)`.
        this is caused by the 13882273-th sample, which is contained in "PVC_indices",
        however, whether it is a PVC beat, or just motion artefact, is in doubt!
+    """,
+    references=[
+        "http://www.icbeb.org/CPSC2020.html",
+        "https://github.com/PIA-Group/BioSPPy",
+    ],
+    doi="10.1166/jmihi.2020.3289",
+)
 
-    TODO
-    ----
-    1. use SNR to filter out too noisy segments?
-    2. for ML, consider more features
 
-    Usage
-    -----
-    1. ECG arrhythmia (PVC, SPB) detection
-
-    References
-    ----------
-    [1] http://www.icbeb.org/CPSC2020.html
-    [2] https://github.com/PIA-Group/BioSPPy
-    """
+@add_docstring(_CPSC2020_INFO.format_database_docstring())
+class CPSC2020(CPSCDataBase):
+    """ """
 
     def __init__(
         self,
-        db_dir: Union[str, Path],
+        db_dir: Optional[Union[str, Path]] = None,
         working_dir: Optional[Union[str, Path]] = None,
-        verbose: int = 2,
+        verbose: int = 1,
         **kwargs: Any,
-    ) -> NoReturn:
-        """finished, to be improved,
-
+    ) -> None:
+        """
         Parameters
         ----------
-        db_dir: str or Path,
+        db_dir: str or Path, optional,
             storage path of the database
         working_dir: str or Path, optional,
             working directory, to store intermediate files and log file
-        verbose: int, default 2,
+        verbose: int, default 1
             log verbosity
         kwargs: auxilliary key word arguments
+
         """
         super().__init__(
             db_name="cpsc2020",
@@ -200,15 +198,10 @@ class CPSC2020(CPSCDataBase):
         self.rec_ext = "mat"
         self.ann_ext = "mat"
 
-        self.n_records = 10
+        self._df_records = None
         self._all_records = None
         self._all_annotations = None
         self._ls_rec()
-        self.rec_dir = self.db_dir / "data"
-        self.ann_dir = self.db_dir / "ref"
-        # aliases
-        self.data_dir = self.rec_dir
-        self.ref_dir = self.ann_dir
 
         self.subgroups = CFG(
             {
@@ -229,10 +222,45 @@ class CPSC2020(CPSCDataBase):
             "pvc": "red",
         }
 
-    def _ls_rec(self) -> NoReturn:
+    def _ls_rec(self) -> None:
         """ """
-        self._all_records = [f"A{i:02d}" for i in range(1, 1 + self.n_records)]
-        self._all_annotations = [f"R{i:02d}" for i in range(1, 1 + self.n_records)]
+        self._df_records = pd.DataFrame()
+        n_records = 10
+        all_records = [f"A{i:02d}" for i in range(1, 1 + n_records)]
+        self._df_records["path"] = [
+            path
+            for path in self.db_dir.rglob(f"*.{self.rec_ext}")
+            if path.stem in all_records
+        ]
+        self._df_records["record"] = self._df_records["path"].apply(lambda x: x.stem)
+        self._df_records.set_index("record", inplace=True)
+
+        all_annotations = [f"R{i:02d}" for i in range(1, 1 + n_records)]
+        df_ann = pd.DataFrame()
+        df_ann["ann_path"] = [
+            path
+            for path in self.db_dir.rglob(f"*.{self.ann_ext}")
+            if path.stem in all_annotations
+        ]
+        df_ann["record"] = df_ann["ann_path"].apply(lambda x: x.stem.replace("R", "A"))
+        df_ann.set_index("record", inplace=True)
+        # take the intersection by the index of `df_ann` and `self._df_records`
+        self._df_records = self._df_records.join(df_ann, how="inner")
+
+        if len(self._df_records) > 0:
+            if self._subsample is not None:
+                size = min(
+                    len(self._df_records),
+                    max(1, int(round(self._subsample * len(self._df_records)))),
+                )
+                self._df_records = self._df_records.sample(
+                    n=size, random_state=DEFAULTS.SEED, replace=False
+                )
+
+        self._all_records = self._df_records.index.tolist()
+        self._all_annotations = (
+            self._df_records["ann_path"].apply(lambda x: x.stem).tolist()
+        )
 
     @property
     def all_annotations(self):
@@ -250,38 +278,76 @@ class CPSC2020(CPSCDataBase):
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
 
         Returns
         -------
         pid: int,
             the `subject_id` corr. to `rec_no`
+
         """
         pid = 0
         raise NotImplementedError
 
-    def load_data(
+    def get_absolute_path(
         self,
-        rec: Union[int, str],
-        units: str = "mV",
-        sampfrom: Optional[int] = None,
-        sampto: Optional[int] = None,
-        keep_dim: bool = True,
-    ) -> np.ndarray:
+        rec: Union[str, int],
+        extension: Optional[str] = None,
+        ann: bool = False,
+    ) -> Path:
         """
+        get the absolute path of the record `rec`
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
-        units: str, default "mV",
-            units of the output signal, can also be "μV", with an alias of "uV"
+            record name or index of the record in `self.all_records`
+        extension: str, optional,
+            extension of the file
+        ann: bool, default False,
+            whether to get the annotation file path or not
+
+        Returns
+        -------
+        abs_path: Path,
+            absolute path of the file
+
+        """
+        if isinstance(rec, int):
+            rec = self[rec]
+        if extension is not None and not extension.startswith("."):
+            extension = f".{extension}"
+        col = "ann_path" if ann else "path"
+        abs_path = self._df_records.loc[rec, col].with_suffix(extension)
+        return abs_path
+
+    def load_data(
+        self,
+        rec: Union[int, str],
+        sampfrom: Optional[int] = None,
+        sampto: Optional[int] = None,
+        data_format: str = "channel_first",
+        units: str = "mV",
+        fs: Optional[Real] = None,
+    ) -> np.ndarray:
+        """
+        Parameters
+        ----------
+        rec: str or int,
+            record name or index of the record in `self.all_records`
         sampfrom: int, optional,
             start index of the data to be loaded
         sampto: int, optional,
             end index of the data to be loaded
-        keep_dim: bool, default True,
-            whether or not to flatten the data of shape (n,1)
+        data_format: str, default "channel_first",
+            format of the ecg data,
+            "channel_last" (alias "lead_last"), or
+            "channel_first" (alias "lead_first"), or
+            "flat" (alias "plain")
+        units: str or None, default "mV",
+            units of the output signal, can also be "μV", with aliases of "uV", "muV"
+        fs: real number, optional,
+            if not None, the loaded data will be resampled to this frequency
 
         Returns
         -------
@@ -289,15 +355,22 @@ class CPSC2020(CPSCDataBase):
             the ECG data
 
         """
-        rec_name = self._get_rec_name(rec)
-        rec_fp = self.data_dir / f"{rec_name}.{self.rec_ext}"
-        data = loadmat(str(rec_fp))["ecg"]
-        if units.lower() in ["uv", "μv"]:
-            data = (1000 * data).astype(int)
+        rec_fp = self.get_absolute_path(rec, self.rec_ext)
+        data = loadmat(str(rec_fp))["ecg"].astype(DEFAULTS.DTYPE.NP)
         sf, st = (sampfrom or 0), (sampto or len(data))
         data = data[sf:st]
-        if not keep_dim:
+        if fs is not None and fs != self.fs:
+            data = SS.resample_poly(data, fs, self.fs, axis=0).astype(data.dtype)
+        if data_format.lower() in ["channel_first", "lead_first"]:
+            data = data.T
+        elif data_format.lower() in ["flat", "plain"]:
             data = data.flatten()
+        elif data_format.lower() in ["channel_last", "lead_last"]:
+            pass
+        else:
+            raise ValueError(f"invalid `data_format`: {data_format}")
+        if units.lower() in ["uv", "muv", "μv"]:
+            data = (1000 * data).astype(int)
         return data
 
     def load_ann(
@@ -307,11 +380,10 @@ class CPSC2020(CPSCDataBase):
         sampto: Optional[int] = None,
     ) -> Dict[str, np.ndarray]:
         """
-
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         sampfrom: int, optional,
             start index of the data to be loaded
         sampto: int, optional,
@@ -322,11 +394,9 @@ class CPSC2020(CPSCDataBase):
         ann: dict,
             with items (ndarray) "SPB_indices" and "PVC_indices",
             which record the indices of SPBs and PVCs
+
         """
-        if isinstance(rec, int):
-            rec = self[rec]
-        ann_name = self._get_ann_name(rec)
-        ann_fp = self.ann_dir / f"{ann_name}.{self.ann_ext}"
+        ann_fp = self.get_absolute_path(rec, self.ann_ext, ann=True)
         ann = loadmat(str(ann_fp))["ref"]
         sf, st = (sampfrom or 0), (sampto or np.inf)
         spb_indices = ann["S_ref"][0, 0].flatten().astype(int)
@@ -343,46 +413,8 @@ class CPSC2020(CPSCDataBase):
         }
         return ann
 
-    def _get_ann_name(self, rec: Union[int, str]) -> str:
-        """
-
-        Parameters
-        ----------
-        rec: str or int,
-            name or index of the record
-
-        Returns
-        -------
-        ann_name: str,
-            filename of the annotation file
-
-        """
-        if isinstance(rec, int):
-            rec = self[rec]
-        ann_name = rec.replace("A", "R")
-        return ann_name
-
-    def _get_rec_name(self, rec: Union[int, str]) -> str:
-        """
-
-        Parameters
-        ----------
-        rec: str or int,
-            name or index of the record
-
-        Returns
-        -------
-        rec_name: str,
-            filename of the record
-        """
-        if isinstance(rec, int):
-            rec = self[rec]
-        rec_name = rec
-        return rec_name
-
     def train_test_split_rec(self, test_rec_num: int = 2) -> Dict[str, List[str]]:
         """
-
         split the records into train set and test set
 
         Parameters
@@ -394,6 +426,7 @@ class CPSC2020(CPSCDataBase):
         -------
         split_res: dict,
             with items `train`, `test`, both being list of record names
+
         """
         if test_rec_num == 1:
             test_records = DEFAULTS.RNG_sample(self.subgroups.VS, 1).tolist()
@@ -411,8 +444,10 @@ class CPSC2020(CPSCDataBase):
             test_records = []
             for k in self.subgroups.keys():
                 test_records += DEFAULTS.RNG_sample(self.subgroups[k], 1).tolist()
-        else:
+        elif 5 <= test_rec_num <= 10:
             raise ValueError("test data ratio too high")
+        else:
+            raise ValueError("Invalid `test_rec_num`")
         train_records = [r for r in self.all_records if r not in test_records]
 
         split_res = CFG(
@@ -428,12 +463,11 @@ class CPSC2020(CPSCDataBase):
         self,
         rec: Union[int, str],
         premature_type: Optional[str] = None,
-        window: int = 10000,
+        window: Real = 10,
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
     ) -> List[List[int]]:
         """
-
         locate the sample indices of premature beats in a record,
         in the form of a list of lists,
         each list contains the interval of sample indices of premature beats
@@ -441,11 +475,12 @@ class CPSC2020(CPSCDataBase):
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         premature_type: str, optional,
             premature beat type, can be one of "SPB", "PVC"
-        window: int, default 10000,
-            window length of each premature beat
+        window: real number, default 10,
+            window length of each premature beat,
+            with units in seconds
         sampfrom: int, optional,
             start index of the premature beats to locate
         sampto: int, optional,
@@ -455,9 +490,8 @@ class CPSC2020(CPSCDataBase):
         -------
         premature_intervals: list,
             list of intervals of premature beats
+
         """
-        if isinstance(rec, int):
-            rec = self[rec]
         ann = self.load_ann(rec)
         if premature_type:
             premature_inds = ann[f"{premature_type.upper()}_indices"]
@@ -471,11 +505,13 @@ class CPSC2020(CPSCDataBase):
             return premature_intervals
         premature_inds = premature_inds[(sf < premature_inds) & (premature_inds < st)]
         tot_interval = [sf, st]
-        premature_intervals, _ = get_optimal_covering(
+        windown_len = int(window * self.fs)
+        premature_intervals = get_optimal_covering(
             total_interval=tot_interval,
             to_cover=premature_inds,
-            min_len=window * self.fs // 1000,
-            split_threshold=window * self.fs // 1000,
+            min_len=windown_len,
+            isolated_point_dist_threshold=windown_len // 2,
+            split_threshold=windown_len,
             traceback=False,
         )
         return premature_intervals
@@ -489,13 +525,12 @@ class CPSC2020(CPSCDataBase):
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
         rpeak_inds: Optional[Union[Sequence[int], np.ndarray]] = None,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         data: ndarray, optional,
             ECG signal to plot,
             if given, data of `rec` will not be used,
@@ -514,6 +549,7 @@ class CPSC2020(CPSCDataBase):
         rpeak_inds: array_like, optional,
             indices of R peaks,
             if `data` is None, then indices should be the absolute indices in the record
+
         """
         if isinstance(rec, int):
             rec = self[rec]
@@ -525,7 +561,7 @@ class CPSC2020(CPSCDataBase):
 
         if data is None:
             _data = self.load_data(
-                rec, units="μV", sampfrom=sampfrom, sampto=sampto, keep_dim=False
+                rec, units="μV", sampfrom=sampfrom, sampto=sampto, data_format="flat"
             )
         else:
             units = self._auto_infer_units(data)
@@ -639,180 +675,19 @@ class CPSC2020(CPSCDataBase):
             ax.set_ylabel("Voltage [μV]")
             plt.show()
 
-
-def _ann_to_beat_ann_epoch_v1(
-    rpeaks: np.ndarray, ann: Dict[str, np.ndarray], bias_thr: Real
-) -> dict:
-    """
-
-    the naive method to label beat types using annotations provided by the dataset
-
-    Parameters
-    ----------
-    rpeaks: ndarray,
-        rpeaks for forming beats
-    ann: dict,
-        with items (ndarray) "SPB_indices" and "PVC_indices",
-        which record the indices of SPBs and PVCs
-    bias_thr: real number,
-        tolerance for using annotations (PVC, SPB indices provided by the dataset),
-        to label the type of beats given by `rpeaks`
-
-    Returns
-    -------
-    retval: dict, with the following items
-        - ann_matched: dict of ndarray,
-            indices of annotations ("SPB_indices" and "PVC_indices")
-            that match some beat from `rpeaks`.
-            for v1, this term is always the same as `ann`, hence useless
-        - beat_ann: ndarray,
-            label for each beat from `rpeaks`
-
-    """
-    beat_ann = np.array(["N" for _ in range(len(rpeaks))])
-    for idx, r in enumerate(rpeaks):
-        if any([abs(r - p) < bias_thr for p in ann["SPB_indices"]]):
-            beat_ann[idx] = "S"
-        elif any([abs(r - p) < bias_thr for p in ann["PVC_indices"]]):
-            beat_ann[idx] = "V"
-    ann_matched = ann.copy()
-    retval = dict(ann_matched=ann_matched, beat_ann=beat_ann)
-    return retval
-
-
-def _ann_to_beat_ann_epoch_v2(
-    rpeaks: np.ndarray, ann: Dict[str, np.ndarray], bias_thr: Real
-) -> dict:
-    """has flaws, deprecated,
-
-    similar to `_ann_to_beat_ann_epoch_v1`, but records those matched annotations,
-    for further post-process, adding those beats that are in annotation,
-    but not detected by the signal preprocessing algorithms (qrs detection)
-
-    however, the comparison process (the block inside the outer `for` loop)
-    is not quite correct
-
-    Parameters
-    ----------
-    rpeaks: ndarray,
-        rpeaks for forming beats
-    ann: dict,
-        with items (ndarray) "SPB_indices" and "PVC_indices",
-        which record the indices of SPBs and PVCs
-    bias_thr: real number,
-        tolerance for using annotations (PVC, SPB indices provided by the dataset),
-        to label the type of beats given by `rpeaks`
-
-    Returns
-    -------
-    retval: dict, with the following items
-        - ann_matched: dict of ndarray,
-            indices of annotations ("SPB_indices" and "PVC_indices")
-            that match some beat from `rpeaks`
-        - beat_ann: ndarray,
-            label for each beat from `rpeaks`
-
-    """
-    beat_ann = np.array(["N" for _ in range(len(rpeaks))], dtype="<U1")
-    # used to add back those beat that is not detected via proprocess algorithm
-    _ann = {k: v.astype(int).tolist() for k, v in ann.items()}
-    for idx_r, r in enumerate(rpeaks):
-        found = False
-        for idx_a, a in enumerate(_ann["SPB_indices"]):
-            if abs(r - a) < bias_thr:
-                found = True
-                beat_ann[idx_r] = "S"
-                del _ann["SPB_indices"][idx_a]
-                break
-        if found:
-            continue
-        for idx_a, a in enumerate(_ann["PVC_indices"]):
-            if abs(r - a) < bias_thr:
-                found = True
-                beat_ann[idx_r] = "V"
-                del _ann["PVC_indices"][idx_a]
-                break
-    ann_matched = {
-        k: np.array([a for a in v if a not in _ann[k]], dtype=int)
-        for k, v in ann.items()
-    }
-    retval = dict(ann_matched=ann_matched, beat_ann=beat_ann)
-    return retval
-    # _ann["SPB_indices"] = [a for a in _ann["SPB_indices"] if prev_r<a<next_r]
-    # _ann["PVC_indices"] = [a for a in _ann["PVC_indices"] if prev_r<a<next_r]
-    # augmented_rpeaks = np.concatenate((rpeaks, np.array(_ann["SPB_indices"]), np.array(_ann["PVC_indices"])))
-    # beat_ann = np.concatenate((beat_ann, np.array(["S" for _ in _ann["SPB_indices"]], dtype="<U1"), np.array(["V" for _ in _ann["PVC_indices"]], dtype="<U1")))
-    # sorted_indices = np.argsort(augmented_rpeaks)
-    # augmented_rpeaks = augmented_rpeaks[sorted_indices].astype(int)
-    # beat_ann = beat_ann[sorted_indices].astype("<U1")
-
-    # retval = dict(augmented_rpeaks=augmented_rpeaks, beat_ann=beat_ann)
-    # return retval
-
-
-def _ann_to_beat_ann_epoch_v3(
-    rpeaks: np.ndarray, ann: Dict[str, np.ndarray], bias_thr: Real
-) -> dict:
-    """
-
-    similar to `_ann_to_beat_ann_epoch_v2`, but more reasonable
-
-    Parameters
-    ----------
-    rpeaks: ndarray,
-        rpeaks for forming beats
-    ann: dict,
-        with items (ndarray) "SPB_indices" and "PVC_indices",
-        which record the indices of SPBs and PVCs
-    bias_thr: real number,
-        tolerance for using annotations (PVC, SPB indices provided by the dataset),
-        to label the type of beats given by `rpeaks`
-
-    Returns
-    -------
-    retval: dict, with the following items
-        - ann_matched: dict of ndarray,
-            indices of annotations ("SPB_indices" and "PVC_indices")
-            that match some beat from `rpeaks`
-        - beat_ann: ndarray,
-            label for each beat from `rpeaks`
-
-    """
-    beat_ann = np.array(["N" for _ in range(len(rpeaks))], dtype="<U1")
-    ann_matched = {k: [] for k, v in ann.items()}
-    for idx_r, r in enumerate(rpeaks):
-        dist_to_spb = np.abs(r - ann["SPB_indices"])
-        dist_to_pvc = np.abs(r - ann["PVC_indices"])
-        if len(dist_to_spb) == 0:
-            dist_to_spb = np.array([np.inf])
-        if len(dist_to_pvc) == 0:
-            dist_to_pvc = np.array([np.inf])
-        argmin = np.argmin([np.min(dist_to_spb), np.min(dist_to_pvc), bias_thr])
-        if argmin == 2:
-            pass
-        elif argmin == 1:
-            beat_ann[idx_r] = "V"
-            ann_matched["PVC_indices"].append(
-                ann["PVC_indices"][np.argmin(dist_to_pvc)]
-            )
-        elif argmin == 0:
-            beat_ann[idx_r] = "S"
-            ann_matched["SPB_indices"].append(
-                ann["SPB_indices"][np.argmin(dist_to_spb)]
-            )
-    ann_matched = {k: np.array(v) for k, v in ann_matched.items()}
-    retval = dict(ann_matched=ann_matched, beat_ann=beat_ann)
-    return retval
-
     @property
     def url(self) -> str:
         return (
             "https://opensz.oss-cn-beijing.aliyuncs.com/ICBEB2020/file/TrainingSet.zip"
         )
 
-    def download(self) -> NoReturn:
-        """download the database from self.url"""
-        http_get(self.url, self.db_dir, extract=True)
+    @property
+    def database_info(self) -> DataBaseInfo:
+        return _CPSC2020_INFO
+
+    @property
+    def webpage(self) -> str:
+        return "http://2020.icbeb.org/CSPC2020"
 
 
 def compute_metrics(
@@ -823,7 +698,6 @@ def compute_metrics(
     verbose: int = 0,
 ) -> Union[Tuple[int], dict]:
     """
-
     Score Function for all (test) records
 
     Parameters
@@ -914,3 +788,168 @@ def compute_metrics(
         retval = Score1, Score2
 
     return retval
+
+
+# out-dated functions
+
+# def _ann_to_beat_ann_epoch_v1(
+#     rpeaks: np.ndarray, ann: Dict[str, np.ndarray], bias_thr: Real
+# ) -> dict:
+#     """
+#     the naive method to label beat types using annotations provided by the dataset
+
+#     Parameters
+#     ----------
+#     rpeaks: ndarray,
+#         rpeaks for forming beats
+#     ann: dict,
+#         with items (ndarray) "SPB_indices" and "PVC_indices",
+#         which record the indices of SPBs and PVCs
+#     bias_thr: real number,
+#         tolerance for using annotations (PVC, SPB indices provided by the dataset),
+#         to label the type of beats given by `rpeaks`
+
+#     Returns
+#     -------
+#     retval: dict, with the following items
+#         - ann_matched: dict of ndarray,
+#             indices of annotations ("SPB_indices" and "PVC_indices")
+#             that match some beat from `rpeaks`.
+#             for v1, this term is always the same as `ann`, hence useless
+#         - beat_ann: ndarray,
+#             label for each beat from `rpeaks`
+
+#     """
+#     beat_ann = np.array(["N" for _ in range(len(rpeaks))])
+#     for idx, r in enumerate(rpeaks):
+#         if any([abs(r - p) < bias_thr for p in ann["SPB_indices"]]):
+#             beat_ann[idx] = "S"
+#         elif any([abs(r - p) < bias_thr for p in ann["PVC_indices"]]):
+#             beat_ann[idx] = "V"
+#     ann_matched = ann.copy()
+#     retval = dict(ann_matched=ann_matched, beat_ann=beat_ann)
+#     return retval
+
+
+# def _ann_to_beat_ann_epoch_v2(
+#     rpeaks: np.ndarray, ann: Dict[str, np.ndarray], bias_thr: Real
+# ) -> dict:
+#     """has flaws, deprecated,
+
+#     similar to `_ann_to_beat_ann_epoch_v1`, but records those matched annotations,
+#     for further post-process, adding those beats that are in annotation,
+#     but not detected by the signal preprocessing algorithms (qrs detection)
+
+#     however, the comparison process (the block inside the outer `for` loop)
+#     is not quite correct
+
+#     Parameters
+#     ----------
+#     rpeaks: ndarray,
+#         rpeaks for forming beats
+#     ann: dict,
+#         with items (ndarray) "SPB_indices" and "PVC_indices",
+#         which record the indices of SPBs and PVCs
+#     bias_thr: real number,
+#         tolerance for using annotations (PVC, SPB indices provided by the dataset),
+#         to label the type of beats given by `rpeaks`
+
+#     Returns
+#     -------
+#     retval: dict, with the following items
+#         - ann_matched: dict of ndarray,
+#             indices of annotations ("SPB_indices" and "PVC_indices")
+#             that match some beat from `rpeaks`
+#         - beat_ann: ndarray,
+#             label for each beat from `rpeaks`
+
+#     """
+#     beat_ann = np.array(["N" for _ in range(len(rpeaks))], dtype="<U1")
+#     # used to add back those beat that is not detected via proprocess algorithm
+#     _ann = {k: v.astype(int).tolist() for k, v in ann.items()}
+#     for idx_r, r in enumerate(rpeaks):
+#         found = False
+#         for idx_a, a in enumerate(_ann["SPB_indices"]):
+#             if abs(r - a) < bias_thr:
+#                 found = True
+#                 beat_ann[idx_r] = "S"
+#                 del _ann["SPB_indices"][idx_a]
+#                 break
+#         if found:
+#             continue
+#         for idx_a, a in enumerate(_ann["PVC_indices"]):
+#             if abs(r - a) < bias_thr:
+#                 found = True
+#                 beat_ann[idx_r] = "V"
+#                 del _ann["PVC_indices"][idx_a]
+#                 break
+#     ann_matched = {
+#         k: np.array([a for a in v if a not in _ann[k]], dtype=int)
+#         for k, v in ann.items()
+#     }
+#     retval = dict(ann_matched=ann_matched, beat_ann=beat_ann)
+#     return retval
+#     # _ann["SPB_indices"] = [a for a in _ann["SPB_indices"] if prev_r<a<next_r]
+#     # _ann["PVC_indices"] = [a for a in _ann["PVC_indices"] if prev_r<a<next_r]
+#     # augmented_rpeaks = np.concatenate((rpeaks, np.array(_ann["SPB_indices"]), np.array(_ann["PVC_indices"])))
+#     # beat_ann = np.concatenate((beat_ann, np.array(["S" for _ in _ann["SPB_indices"]], dtype="<U1"), np.array(["V" for _ in _ann["PVC_indices"]], dtype="<U1")))
+#     # sorted_indices = np.argsort(augmented_rpeaks)
+#     # augmented_rpeaks = augmented_rpeaks[sorted_indices].astype(int)
+#     # beat_ann = beat_ann[sorted_indices].astype("<U1")
+
+#     # retval = dict(augmented_rpeaks=augmented_rpeaks, beat_ann=beat_ann)
+#     # return retval
+
+
+# def _ann_to_beat_ann_epoch_v3(
+#     rpeaks: np.ndarray, ann: Dict[str, np.ndarray], bias_thr: Real
+# ) -> dict:
+#     """
+#     similar to `_ann_to_beat_ann_epoch_v2`, but more reasonable
+
+#     Parameters
+#     ----------
+#     rpeaks: ndarray,
+#         rpeaks for forming beats
+#     ann: dict,
+#         with items (ndarray) "SPB_indices" and "PVC_indices",
+#         which record the indices of SPBs and PVCs
+#     bias_thr: real number,
+#         tolerance for using annotations (PVC, SPB indices provided by the dataset),
+#         to label the type of beats given by `rpeaks`
+
+#     Returns
+#     -------
+#     retval: dict, with the following items
+#         - ann_matched: dict of ndarray,
+#             indices of annotations ("SPB_indices" and "PVC_indices")
+#             that match some beat from `rpeaks`
+#         - beat_ann: ndarray,
+#             label for each beat from `rpeaks`
+
+#     """
+#     beat_ann = np.array(["N" for _ in range(len(rpeaks))], dtype="<U1")
+#     ann_matched = {k: [] for k, v in ann.items()}
+#     for idx_r, r in enumerate(rpeaks):
+#         dist_to_spb = np.abs(r - ann["SPB_indices"])
+#         dist_to_pvc = np.abs(r - ann["PVC_indices"])
+#         if len(dist_to_spb) == 0:
+#             dist_to_spb = np.array([np.inf])
+#         if len(dist_to_pvc) == 0:
+#             dist_to_pvc = np.array([np.inf])
+#         argmin = np.argmin([np.min(dist_to_spb), np.min(dist_to_pvc), bias_thr])
+#         if argmin == 2:
+#             pass
+#         elif argmin == 1:
+#             beat_ann[idx_r] = "V"
+#             ann_matched["PVC_indices"].append(
+#                 ann["PVC_indices"][np.argmin(dist_to_pvc)]
+#             )
+#         elif argmin == 0:
+#             beat_ann[idx_r] = "S"
+#             ann_matched["SPB_indices"].append(
+#                 ann["SPB_indices"][np.argmin(dist_to_spb)]
+#             )
+#     ann_matched = {k: np.array(v) for k, v in ann_matched.items()}
+#     retval = dict(ann_matched=ann_matched, beat_ann=beat_ann)
+#     return retval
