@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 """
+The 4th China Physiological Signal Challenge 2021:
+Paroxysmal Atrial Fibrillation Events Detection from Dynamic ECG Recordings
 """
 
 import json
@@ -9,13 +11,12 @@ import time
 import warnings
 from numbers import Real
 from pathlib import Path
-from typing import Any, Dict, List, NoReturn, Optional, Sequence, Tuple, Union
+from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
 
 import numpy as np
 import pandas as pd
 import scipy.io as sio
 import wfdb
-from scipy.signal import resample, resample_poly  # noqa: F401
 
 from ...cfg import CFG
 from ...utils.misc import (
@@ -27,41 +28,28 @@ from ...utils.misc import (
 from ...utils.utils_interval import generalized_intervals_intersection
 from ..base import (  # noqa: F401
     DEFAULT_FIG_SIZE_PER_SEC,
+    DataBaseInfo,
     CPSCDataBase,
     PhysioNetDataBase,
     WFDB_Beat_Annotations,
     WFDB_Non_Beat_Annotations,
     WFDB_Rhythm_Annotations,
+    _PlotCfg,
 )
 
+
 __all__ = [
     "CPSC2021",
     "compute_metrics",
 ]
 
 
-# configurations for visualization
-PlotCfg = CFG()
-# default const for the plot function in dataset.py
-# used only when corr. values are absent
-# all values are time bias w.r.t. corr. peaks, with units in ms
-PlotCfg.p_onset = -40
-PlotCfg.p_offset = 40
-PlotCfg.q_onset = -20
-PlotCfg.s_offset = 40
-PlotCfg.qrs_radius = 60
-PlotCfg.t_onset = -100
-PlotCfg.t_offset = 60
-
-
-class CPSC2021(PhysioNetDataBase):
-    r"""
-
+_CPSC2021_INFO = DataBaseInfo(
+    title="""
     The 4th China Physiological Signal Challenge 2021:
     Paroxysmal Atrial Fibrillation Events Detection from Dynamic ECG Recordings
-
-    ABOUT CPSC2021
-    --------------
+    """,
+    about=r"""
     1. source ECG data are recorded from 12-lead Holter or 3-lead wearable ECG monitoring devices
     2. dataset provides variable-length ECG fragments extracted from lead I and lead II of the long-term source ECG data, each sampled at 200 Hz
     3. AF event is limited to be no less than 5 heart beats
@@ -84,52 +72,47 @@ class CPSC2021(PhysioNetDataBase):
         (3) final score (U):
         U = \dfrac{1}{N} \sum\limits_{i=1}^N \left( Ur_i + \dfrac{Ma_i}{\max\{Mr_i, Ma_i\}} \right)
         where N is the number of records, Ma is the number of annotated AF episodes, Mr the number of predicted AF episodes
-
-    NOTE
-    ----
+    """,
+    note="""
     1. if an ECG record is classified as AFf, the provided onset and offset locations should be the first and last record points. If an ECG record is classified as N, the answer should be an empty list
     2. it can be inferred from the classification scoring matrix that the punishment of false negatives of AFf is very heavy, while mixing-up of AFf and AFp is not punished
     3. flag of atrial fibrillation and atrial flutter ("AFIB" and "AFL") in annotated information are seemed as the same type when scoring the method
     4. the 3 classes can coexist in ONE subject (not one record). For example, subject 61 has 6 records with label "N", 1 with label "AFp", and 2 with label "AFf"
     5. rhythm change annotations ("(AFIB", "(AFL", "(N" in the `aux_note` field or "+" in the `symbol` field of the annotation files) are inserted 0.15s ahead of or behind (onsets or offset resp.) of corresponding R peaks.
-    6. some records are revised if there are heart beats of the AF episode or the pause between adjacent AF episodes less than 5. The id numbers of the revised records are summarized in the attached REVISED_RECORDS
+    6. some records are revised if there are heart beats of the AF episode or the pause between adjacent AF episodes less than 5. The id numbers of the revised records are summarized in the attached `REVISED_RECORDS`.
+    """,
+    usage=[
+        "AF (event, fine) detection",
+    ],
+    references=[
+        "http://www.icbeb.org/CPSC2021",
+        "https://www.physionet.org/content/cpsc2021/1.0.0/",
+        "https://archive.physionet.org/physiobank/annotations.shtml",
+    ],
+)
 
-    ISSUES
-    ------
-    1.
 
-    TODO
-    ----
-    1.
+@add_docstring(_CPSC2021_INFO.format_database_docstring())
+class CPSC2021(PhysioNetDataBase):
+    """ """
 
-    Usage
-    -----
-    1. AF (event, fine) detection
-
-    References
-    ----------
-    1. <a name="ref1"></a> http://www.icbeb.org/CPSC2021
-    2. <a name="ref2"></a> https://www.physionet.org/content/cpsc2021/1.0.0/
-    3. <a name="ref3"></a> https://archive.physionet.org/physiobank/annotations.shtml
-
-    """
+    __name__ = "CPSC2021"
 
     def __init__(
         self,
-        db_dir: str,
-        working_dir: Optional[str] = None,
-        verbose: int = 2,
+        db_dir: Optional[Union[str, Path]] = None,
+        working_dir: Optional[Union[str, Path]] = None,
+        verbose: int = 1,
         **kwargs: Any,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
-        db_dir: str,
+        db_dir: str or Path, optional,
             storage path of the database
-        working_dir: str, optional,
+        working_dir: str or Path, optional,
             working directory, to store intermediate files and log file
-        verbose: int, default 2,
+        verbose: int, default 1
             log verbosity
         kwargs: auxilliary key word arguments
 
@@ -155,7 +138,7 @@ class CPSC2021(PhysioNetDataBase):
         self.ann_ext = "atr"
         self.header_ext = "hea"
         self.all_leads = ["I", "II"]
-        self.rec_patterns_with_ext = f"^data_(?:\\d+)_(?:\\d+).{self.rec_ext}$"
+        self.rec_patterns_with_ext = f"^data_(?:\\d+)_(?:\\d+)\\.{self.rec_ext}$"
 
         self._labels_f2a = {  # fullname to abbreviation
             "non atrial fibrillation": "N",
@@ -204,12 +187,12 @@ class CPSC2021(PhysioNetDataBase):
             self._ls_rec()
         return self.__all_records
 
-    def _ls_rec(self) -> NoReturn:
+    def _ls_rec(self) -> None:
         """
-
         list all the records and load into `self._all_records`,
         facilitating further uses
 
+        TODO: implement subsampling of records
         """
         self._all_records = CFG({t: [] for t in self.db_tranches})
         self._all_subjects = CFG({t: [] for t in self.db_tranches})
@@ -264,12 +247,12 @@ class CPSC2021(PhysioNetDataBase):
             list_sum(self._all_subjects.values()), key=lambda s: int(s)
         )
 
-    def _ls_rec_split(self) -> NoReturn:
+    def _ls_rec_split(self) -> None:
         """
-
         list all the records assuming the records
         are split into two folders (training_I and training_II)
 
+        TODO: implement subsampling of records
         """
         fn = "RECORDS"
         rev_fn = "REVISED_RECORDS"
@@ -292,19 +275,21 @@ class CPSC2021(PhysioNetDataBase):
             else:
                 if not dir_tranche.is_dir():
                     continue
-                print("Please wait patiently to let the reader find all records...")
+                self.logger.info(
+                    "Please wait patiently to let the reader find all records..."
+                )
                 start = time.time()
                 self._all_records[t] = get_record_list_recursive3(
                     str(dir_tranche), self.rec_patterns_with_ext
                 )
-                print(f"Done in {time.time() - start:.5f} seconds!")
+                self.logger.info(f"Done in {time.time() - start:.5f} seconds!")
                 record_list_fp.write_text("\n".join(self._all_records[t]))
 
             record_list_fp = dir_tranche / rev_fn
             if record_list_fp.is_file():
                 self.__revised_records.extend(record_list_fp.read_text().splitlines())
 
-    def _aggregate_stats(self) -> NoReturn:
+    def _aggregate_stats(self) -> None:
         """aggregate stats on the whole dataset"""
         stats_file = "stats.csv"
         stats_file_fp = self.db_dir_base / stats_file
@@ -312,7 +297,7 @@ class CPSC2021(PhysioNetDataBase):
             self._stats = pd.read_csv(stats_file_fp)
 
         if self._stats.empty or set(self._stats_columns) != set(self._stats.columns):
-            print(
+            self.logger.info(
                 "Please wait patiently to let the reader aggregate statistics on the whole dataset..."
             )
             start = time.time()
@@ -333,7 +318,7 @@ class CPSC2021(PhysioNetDataBase):
             )
             self._stats["fs"] = self.fs
             self._stats["sig_len"] = self._stats["record"].apply(
-                lambda s: wfdb.rdheader(str(self._get_path(s))).sig_len
+                lambda s: wfdb.rdheader(str(self.get_absolute_path(s))).sig_len
             )
             self._stats["sig_len_sec"] = self._stats["sig_len"] / self._stats["fs"]
             self._stats["revised"] = self._stats["record"].apply(
@@ -344,11 +329,18 @@ class CPSC2021(PhysioNetDataBase):
             )
             self._stats = self._stats[self._stats_columns]
             self._stats.to_csv(stats_file_fp, index=False)
-            print(f"Done in {time.time() - start:.5f} seconds!")
+            self.logger.info(f"Done in {time.time() - start:.5f} seconds!")
         else:
             pass  # currently no need to parse the loaded csv file
         self._stats["subject_id"] = self._stats["subject_id"].apply(lambda s: str(s))
         self.__all_records = self._stats["record"].tolist()
+        self._df_records = self._stats[["record", "tranche", "subject_id"]].copy(
+            deep=True
+        )
+        self._df_records["path"] = self._df_records["record"].apply(
+            lambda s: self.db_dirs[self._all_records_inv[s]] / s
+        )
+        self._df_records.set_index("record", inplace=True)
 
     @property
     def all_subjects(self) -> List[str]:
@@ -365,7 +357,7 @@ class CPSC2021(PhysioNetDataBase):
         """ """
         return self._stats
 
-    def _ls_diagnoses_records(self) -> NoReturn:
+    def _ls_diagnoses_records(self) -> None:
         """list all the records for all diagnoses"""
         fn = "diagnoses_records_list.json"
         dr_fp = self.db_dir_base / fn
@@ -374,7 +366,7 @@ class CPSC2021(PhysioNetDataBase):
         else:
             start = time.time()
             if self.df_stats.empty:
-                print(
+                self.logger.info(
                     "Please wait several minutes patiently to let the reader list records for each diagnosis..."
                 )
                 self._diagnoses_records_list = {
@@ -383,7 +375,7 @@ class CPSC2021(PhysioNetDataBase):
                 for rec in self.all_records:
                     lb = self.load_label(rec)
                     self._diagnoses_records_list[lb].append(rec)
-                print(f"Done in {time.time() - start:.5f} seconds!")
+                self.logger.info(f"Done in {time.time() - start:.5f} seconds!")
             else:
                 self._diagnoses_records_list = {
                     d: self.df_stats[self.df_stats["label"] == d]["record"].tolist()
@@ -403,11 +395,10 @@ class CPSC2021(PhysioNetDataBase):
 
     def get_subject_id(self, rec: Union[str, int]) -> str:
         """
-
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
 
         Returns
         -------
@@ -420,27 +411,30 @@ class CPSC2021(PhysioNetDataBase):
         sid = rec.split("_")[1]
         return sid
 
-    def _get_path(self, rec: Union[str, int], ext: Optional[str] = None) -> Path:
+    def get_absolute_path(
+        self, rec: Union[str, int], extension: Optional[str] = None
+    ) -> Path:
         """
+        get the absolute path of the record `rec`
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
-        ext: str, optional,
-            file extension of the path
+            record name or index of the record in `self.all_records`
+        extension: str, optional,
+            extension of the file
 
         Returns
         -------
-        p: Path,
-            path (with or without file extension) of the record
+        Path,
+            absolute path of the file
+
         """
         if isinstance(rec, int):
             rec = self[rec]
-        if ext:
-            rec += f".{ext}"
-        p = self.db_dirs[self._all_records_inv[rec]] / rec
-        return p
+        if extension is not None and not extension.startswith("."):
+            extension = f".{extension}"
+        return self.db_dirs[self._all_records_inv[rec]] / f"{rec}{extension or ''}"
 
     def _validate_samp_interval(
         self,
@@ -454,7 +448,7 @@ class CPSC2021(PhysioNetDataBase):
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         sampfrom: int, optional,
             start index of the data to be loaded
         sampto: int, optional,
@@ -471,89 +465,21 @@ class CPSC2021(PhysioNetDataBase):
         """
         if isinstance(rec, int):
             rec = self[rec]
+        sig_len = self.df_stats[self.df_stats.record == rec].iloc[0].sig_len
         sf, st = (
-            sampfrom or 0,
-            sampto or self.df_stats[self.df_stats.record == rec].iloc[0].sig_len,
+            sampfrom if sampfrom is not None else 0,
+            min(sampto, sig_len) if sampto is not None else sig_len,
         )
+        if sampto is not None and sampto > sig_len:
+            warnings.warn(
+                f"the end index {sampto} is larger than the signal length {sig_len}, "
+                f"so it is set to {sig_len}",
+                RuntimeWarning,
+            )
         if sf >= st:
             raise ValueError("Invalid `sampfrom` and `sampto`")
         return sf, st
 
-    def load_data(
-        self,
-        rec: Union[str, int],
-        leads: Optional[Union[str, List[str]]] = None,
-        data_format: str = "channel_first",
-        units: str = "mV",
-        sampfrom: Optional[int] = None,
-        sampto: Optional[int] = None,
-        fs: Optional[Real] = None,
-    ) -> np.ndarray:
-        """
-
-        load physical (converted from digital) ECG data,
-        which is more understandable for humans
-
-        Parameters
-        ----------
-        rec: str or int,
-            name or index of the record
-        leads: str or list of str, optional,
-            the leads to load
-        data_format: str, default "channel_first",
-            format of the ECG data,
-            "channel_last" (alias "lead_last"), or
-            "channel_first" (alias "lead_first")
-        units: str, default "mV",
-            units of the output signal, can also be "μV", with an alias of "uV"
-        sampfrom: int, optional,
-            start index of the data to be loaded
-        sampto: int, optional,
-            end index of the data to be loaded
-        fs: real number, optional,
-            if not None, the loaded data will be resampled to this sampling frequency
-
-        Returns
-        -------
-        data: ndarray,
-            the ECG data
-
-        """
-        if isinstance(rec, int):
-            rec = self[rec]
-        assert data_format.lower() in [
-            "channel_first",
-            "lead_first",
-            "channel_last",
-            "lead_last",
-        ]
-        if not leads:
-            _leads = self.all_leads
-        elif isinstance(leads, str):
-            _leads = [leads]
-        else:
-            _leads = leads
-        assert all([ld in self.all_leads for ld in _leads])
-
-        rec_fp = self._get_path(rec)
-        sf, st = self._validate_samp_interval(rec, sampfrom, sampto)
-        wfdb_rec = wfdb.rdrecord(
-            str(rec_fp), sampfrom=sf, sampto=st, physical=True, channel_names=_leads
-        )
-        data = np.asarray(wfdb_rec.p_signal.T)
-        # lead_units = np.vectorize(lambda s: s.lower())(wfdb_rec.units)
-
-        if units.lower() in ["uv", "μv"]:
-            data = data * 1000
-
-        if fs is not None and fs != self.fs:
-            data = resample_poly(data, fs, self.fs, axis=1)
-
-        if data_format.lower() in ["channel_last", "lead_last"]:
-            data = data.T
-
-        return data
-
     def load_ann(
         self,
         rec: Union[str, int],
@@ -563,15 +489,14 @@ class CPSC2021(PhysioNetDataBase):
         **kwargs: Any,
     ) -> Union[dict, np.ndarray, List[List[int]], str]:
         """
-
         load annotations of the record
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         field: str, optional
-            field of the annotation, can be one of "rpeaks", "af_episodes", "label", "raw", "wfdb",
+            field of the annotation, can be one of "rpeaks", "af_episodes", "label", "raw", "wfdb";
             if not specified, all fields of the annotation will be returned in the form of a dict,
             if is "raw" or "wfdb", then the corresponding wfdb "Annotation" will be returned
         sampfrom: int, optional,
@@ -592,12 +517,14 @@ class CPSC2021(PhysioNetDataBase):
         -------
         ann: dict, or list, or ndarray, or str,
             annotaton of the record
+
         """
-        if isinstance(rec, int):
-            rec = self[rec]
         sf, st = self._validate_samp_interval(rec, sampfrom, sampto)
         ann = wfdb.rdann(
-            str(self._get_path(rec)), extension=self.ann_ext, sampfrom=sf, sampto=st
+            str(self.get_absolute_path(rec)),
+            extension=self.ann_ext,
+            sampfrom=sf,
+            sampto=st,
         )
         # `load_af_episodes` should not use sampfrom, sampto
         func = {
@@ -609,7 +536,8 @@ class CPSC2021(PhysioNetDataBase):
             ann = {k: f(rec, ann, sf, st) for k, f in func.items()}
             if kwargs:
                 warnings.warn(
-                    f"key word arguments {list(kwargs.keys())} ignored when field is not specified!"
+                    f"key word arguments `{list(kwargs.keys())}` ignored when `field` is not specified!",
+                    RuntimeWarning,
                 )
             return ann
         elif field.lower() in [
@@ -621,7 +549,7 @@ class CPSC2021(PhysioNetDataBase):
         try:
             f = func[field.lower()]
         except Exception:
-            raise ValueError("invalid field")
+            raise ValueError(f"Invalid `field`: {field}")
         ann = f(rec, ann, sf, st, **kwargs)
         return ann
 
@@ -632,16 +560,16 @@ class CPSC2021(PhysioNetDataBase):
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
         keep_original: bool = False,
+        valid_only: bool = True,
         fs: Optional[Real] = None,
     ) -> np.ndarray:
         """
-
         load position (in terms of samples) of rpeaks
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         ann: Annotation, optional,
             the wfdb Annotation of the record,
             if None, corresponding annotation file will be read
@@ -652,6 +580,11 @@ class CPSC2021(PhysioNetDataBase):
         keep_original: bool, default False,
             if True, indices will keep the same with the annotation file
             otherwise subtract `sampfrom` if specified
+        valid_only: bool, default True,
+            if True, only valid rpeaks will be returned,
+            otherwise, all indices in the `sample` field of the annotation will be returned.
+            Valid rpeaks are those with symbol in `WFDB_Beat_Annotations`;
+            symbols in `WFDB_Non_Beat_Annotations` are considered as invalid rpeaks
         fs: real number, optional,
             if not None, positions of the loaded rpeaks will be ajusted according to this sampling frequency
 
@@ -661,23 +594,27 @@ class CPSC2021(PhysioNetDataBase):
             position (in terms of samples) of rpeaks of the record
 
         """
-        if isinstance(rec, int):
-            rec = self[rec]
         if ann is None:
             sf, st = self._validate_samp_interval(rec, sampfrom, sampto)
             ann = wfdb.rdann(
-                str(self._get_path(rec)), extension=self.ann_ext, sampfrom=sf, sampto=st
+                str(self.get_absolute_path(rec)),
+                extension=self.ann_ext,
+                sampfrom=sf,
+                sampto=st,
             )
         critical_points = ann.sample
         symbols = ann.symbol
-        rpeaks_valid = np.isin(symbols, list(WFDB_Beat_Annotations.keys()))
         if sampfrom and not keep_original:
             critical_points = critical_points - sampfrom
         if fs is not None and fs != self.fs:
             critical_points = np.round(
                 critical_points * fs / self.fs + self._epsilon
             ).astype(int)
-        rpeaks = critical_points[rpeaks_valid]
+        if valid_only:
+            rpeaks_valid = np.isin(symbols, list(WFDB_Beat_Annotations.keys()))
+            rpeaks = critical_points[rpeaks_valid]
+        else:
+            rpeaks = critical_points
         return rpeaks
 
     @add_docstring(load_rpeaks.__doc__)
@@ -688,12 +625,19 @@ class CPSC2021(PhysioNetDataBase):
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
         keep_original: bool = False,
+        valid_only: bool = True,
         fs: Optional[Real] = None,
     ) -> np.ndarray:
-        """
-        alias of `self.load_rpeaks`
-        """
-        return self.load_rpeaks(rec, ann, sampfrom, sampto, keep_original, fs)
+        """alias of `self.load_rpeaks`"""
+        return self.load_rpeaks(
+            rec=rec,
+            ann=ann,
+            sampfrom=sampfrom,
+            sampto=sampto,
+            keep_original=keep_original,
+            valid_only=valid_only,
+            fs=fs,
+        )
 
     def load_af_episodes(
         self,
@@ -706,13 +650,12 @@ class CPSC2021(PhysioNetDataBase):
         fmt: str = "intervals",
     ) -> Union[List[List[int]], np.ndarray]:
         """
-
         load the episodes of atrial fibrillation, in terms of intervals or mask
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         ann: Annotation, optional,
             the wfdb Annotation of the record,
             if None, corresponding annotation file will be read
@@ -737,12 +680,10 @@ class CPSC2021(PhysioNetDataBase):
             episodes of atrial fibrillation, in terms of intervals or mask
 
         """
-        if isinstance(rec, int):
-            rec = self[rec]
-        header = wfdb.rdheader(str(self._get_path(rec)))
+        header = wfdb.rdheader(str(self.get_absolute_path(rec)))
         label = self._labels_f2a[header.comments[0]]
         siglen = header.sig_len
-        _ann = wfdb.rdann(str(self._get_path(rec)), extension=self.ann_ext)
+        _ann = wfdb.rdann(str(self.get_absolute_path(rec)), extension=self.ann_ext)
         sf, st = self._validate_samp_interval(rec, sampfrom, sampto)
         aux_note = np.array(_ann.aux_note)
         critical_points = _ann.sample
@@ -813,7 +754,6 @@ class CPSC2021(PhysioNetDataBase):
         fmt: str = "a",
     ) -> str:
         """
-
         load (classifying) label of the record,
         among the following three classes:
         "non atrial fibrillation",
@@ -823,7 +763,7 @@ class CPSC2021(PhysioNetDataBase):
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         ann: Annotation, optional,
             not used, to keep in accordance with other methods
         sampfrom: int, optional,
@@ -842,15 +782,13 @@ class CPSC2021(PhysioNetDataBase):
             classifying label of the record
 
         """
-        if isinstance(rec, int):
-            rec = self[rec]
-        header = wfdb.rdheader(str(self._get_path(rec)))
+        header = wfdb.rdheader(str(self.get_absolute_path(rec)))
         label = header.comments[0]
         if fmt.lower() in ["a", "abbr", "abbreviation"]:
             label = self._labels_f2a[label]
         elif fmt.lower() in ["n", "num", "number"]:
             label = self._labels_f2n[label]
-        elif not fmt.lower() in ["f", "fullname"]:
+        elif fmt.lower() not in ["f", "fullname"]:
             raise ValueError(f"format `{fmt}` of labels is not supported!")
         return label
 
@@ -858,13 +796,12 @@ class CPSC2021(PhysioNetDataBase):
         self, rec: Union[str, int], bias: dict = {1: 1, 2: 0.5}
     ) -> Tuple[np.ndarray, np.ndarray]:
         """
-
         generate the scoring mask for the onsets and offsets of af episodes,
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         bias: dict, default {1:1, 2:0.5},
             keys are bias (with ±) in terms of number of rpeaks
             values are corresponding scores
@@ -885,7 +822,7 @@ class CPSC2021(PhysioNetDataBase):
         masks = gen_endpoint_score_mask(
             siglen=self.df_stats[self.df_stats.record == rec].iloc[0].sig_len,
             critical_points=wfdb.rdann(
-                str(self._get_path(rec)), extension=self.ann_ext
+                str(self.get_absolute_path(rec)), extension=self.ann_ext
             ).sample,
             af_intervals=self.load_af_episodes(rec, fmt="c_intervals"),
             bias=bias,
@@ -901,12 +838,11 @@ class CPSC2021(PhysioNetDataBase):
         ticks_granularity: int = 0,
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
-        leads: Optional[Union[str, List[str]]] = None,
+        leads: Optional[Union[str, int, List[Union[str, int]]]] = None,
         waves: Optional[Dict[str, Sequence[int]]] = None,
         **kwargs,
-    ) -> NoReturn:
-        """to improve,
-
+    ) -> None:
+        """
         plot the signals of a record or external signals (units in μV),
         with metadata (labels, episodes of atrial fibrillation, etc.),
         possibly also along with wave delineations
@@ -914,7 +850,7 @@ class CPSC2021(PhysioNetDataBase):
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         data: ndarray, optional,
             (2-lead) ECG signal to plot,
             should be of the format "channel_first", and compatible with `leads`
@@ -960,13 +896,8 @@ class CPSC2021(PhysioNetDataBase):
             import matplotlib.pyplot as plt
 
             plt.MultipleLocator.MAXTICKS = 3000
-        if leads is None or leads == "all":
-            _leads = self.all_leads
-        elif isinstance(leads, str):
-            _leads = [leads]
-        else:
-            _leads = leads
-        assert all([ld in self.all_leads for ld in _leads])
+
+        _leads = self._normalize_leads(leads)
 
         if data is None:
             _data = self.load_data(
@@ -979,7 +910,7 @@ class CPSC2021(PhysioNetDataBase):
             )
         else:
             units = self._auto_infer_units(data)
-            print(f"input data is auto detected to have units in {units}")
+            self.logger.info(f"input data is auto detected to have units in {units}")
             if units.lower() == "mv":
                 _data = 1000 * data
             else:
@@ -999,9 +930,10 @@ class CPSC2021(PhysioNetDataBase):
             elif waves.get("p_peaks", None):
                 p_waves = [
                     [
-                        max(0, p + ms2samples(PlotCfg.p_onset, fs=self.fs)),
+                        max(0, p + ms2samples(_PlotCfg.p_onset, fs=self.fs)),
                         min(
-                            _data.shape[1], p + ms2samples(PlotCfg.p_offset, fs=self.fs)
+                            _data.shape[1],
+                            p + ms2samples(_PlotCfg.p_offset, fs=self.fs),
                         ),
                     ]
                     for p in waves["p_peaks"]
@@ -1016,9 +948,10 @@ class CPSC2021(PhysioNetDataBase):
             elif waves.get("q_peaks", None) and waves.get("s_peaks", None):
                 qrs = [
                     [
-                        max(0, q + ms2samples(PlotCfg.q_onset, fs=self.fs)),
+                        max(0, q + ms2samples(_PlotCfg.q_onset, fs=self.fs)),
                         min(
-                            _data.shape[1], s + ms2samples(PlotCfg.s_offset, fs=self.fs)
+                            _data.shape[1],
+                            s + ms2samples(_PlotCfg.s_offset, fs=self.fs),
                         ),
                     ]
                     for q, s in zip(waves["q_peaks"], waves["s_peaks"])
@@ -1026,10 +959,10 @@ class CPSC2021(PhysioNetDataBase):
             elif waves.get("r_peaks", None):
                 qrs = [
                     [
-                        max(0, r + ms2samples(PlotCfg.qrs_radius, fs=self.fs)),
+                        max(0, r + ms2samples(_PlotCfg.qrs_radius, fs=self.fs)),
                         min(
                             _data.shape[1],
-                            r + ms2samples(PlotCfg.qrs_radius, fs=self.fs),
+                            r + ms2samples(_PlotCfg.qrs_radius, fs=self.fs),
                         ),
                     ]
                     for r in waves["r_peaks"]
@@ -1044,9 +977,10 @@ class CPSC2021(PhysioNetDataBase):
             elif waves.get("t_peaks", None):
                 t_waves = [
                     [
-                        max(0, t + ms2samples(PlotCfg.t_onset, fs=self.fs)),
+                        max(0, t + ms2samples(_PlotCfg.t_onset, fs=self.fs)),
                         min(
-                            _data.shape[1], t + ms2samples(PlotCfg.t_offset, fs=self.fs)
+                            _data.shape[1],
+                            t + ms2samples(_PlotCfg.t_offset, fs=self.fs),
                         ),
                     ]
                     for t in waves["t_peaks"]
@@ -1165,7 +1099,8 @@ class CPSC2021(PhysioNetDataBase):
 
     def _round(self, n: Real) -> int:
         """
-        dealing with round(0.5) = 0, hence keeping accordance with output length of `resample_poly`
+        dealing with round(0.5) = 0,
+        hence keeping accordance with output length of `resample_poly`
         """
         return int(round(n + self._epsilon))
 
@@ -1176,9 +1111,17 @@ class CPSC2021(PhysioNetDataBase):
             return self._url_compressed
         # currently, cpsc2021 is not included in the list obtained
         # using `wfdb.get_dbs()`
-        self._url_compressed = f"https://www.physionet.org/static/published-projects/cpsc2021/paroxysmal-atrial-fibrillation-events-detection-from-dynamic-ECG-recordings-the-4th-china-physiological-signal-challenge-2021-{self.version}.zip"
+        self._url_compressed = (
+            "https://www.physionet.org/static/published-projects/cpsc2021/"
+            "paroxysmal-atrial-fibrillation-events-detection-from-dynamic-ECG-recordings"
+            f"-the-4th-china-physiological-signal-challenge-2021-{self.version}.zip"
+        )
         return self._url_compressed
 
+    @property
+    def database_info(self) -> DataBaseInfo:
+        return _CPSC2021_INFO
+
 
 ###################################################################
 # copied and modified from the official scoring code
@@ -1240,8 +1183,8 @@ class RefInfo:
 
     def _gen_endpoint_score_range(self, verbose=0):
         """ """
-        onset_range = np.zeros((self.len_sig,), dtype=np.float)
-        offset_range = np.zeros((self.len_sig,), dtype=np.float)
+        onset_range = np.zeros((self.len_sig,), dtype=float)
+        offset_range = np.zeros((self.len_sig,), dtype=float)
         for i, af_start in enumerate(self.af_starts):
             if self.class_true == 2:
                 if max(af_start - 1, 0) == 0:
@@ -1501,7 +1444,6 @@ def compute_challenge_metric(
     offset_score_range: Sequence[float],
 ) -> float:
     """
-
     compute challenge metric for a single record
 
     Parameters
@@ -1523,6 +1465,7 @@ def compute_challenge_metric(
     -------
     u: float,
         the final score for the prediction
+
     """
     ur_score = ur_calculate(class_true, class_pred)
     ue_score = ue_calculate(
@@ -1540,7 +1483,6 @@ def gen_endpoint_score_mask(
     verbose: int = 0,
 ) -> Tuple[np.ndarray, np.ndarray]:
     """
-
     generate the scoring mask for the onsets and offsets of af episodes,
 
     Parameters
@@ -1572,6 +1514,7 @@ def gen_endpoint_score_mask(
     2. for records [data_39_4,data_48_4,data_68_23,data_98_5,data_101_5,data_101_7,data_101_8,data_104_25,data_104_27],
     the official `RefInfo._gen_endpoint_score_range` slightly expands the scoring intervals at heads or tails of the records,
     which strictly is incorrect as defined in the `Scoring` section of the official webpage (http://www.icbeb.org/CPSC2021)
+
     """
     _critical_points = list(critical_points)
     if 0 not in _critical_points:
