@@ -4,27 +4,21 @@ data generator for feeding data into pytorch models
 
 import json
 import time
+import warnings
 from copy import deepcopy
-from functools import reduce
-from pathlib import Path
 from random import sample, shuffle
-from typing import List, NoReturn, Optional, Sequence, Set, Tuple
+from typing import List, Optional, Sequence, Set, Tuple, Any
 
 import numpy as np
-
-try:
-    from tqdm.auto import tqdm
-except ModuleNotFoundError:
-    from tqdm import tqdm
-
 import torch
 from torch.utils.data.dataset import Dataset
+from tqdm.auto import tqdm
 
 from ...._preprocessors import PreprocManager
 from ....cfg import CFG
 from ....databases import CINC2020 as CR
 from ....utils.misc import ReprMixin, list_sum
-from torch_ecg.utils.utils_data import ensure_siglen
+from ....utils.utils_data import ensure_siglen
 from ....utils.utils_signal import remove_spikes_naive
 from .cinc2020_cfg import CINC2020TrainCfg  # noqa: F401
 
@@ -41,10 +35,13 @@ class CINC2020Dataset(ReprMixin, Dataset):
     __name__ = "CINC2020Dataset"
 
     def __init__(
-        self, config: CFG, training: bool = True, lazy: bool = True
-    ) -> NoReturn:
+        self,
+        config: CFG,
+        training: bool = True,
+        lazy: bool = True,
+        **reader_kwargs: Any,
+    ) -> None:
         """
-
         Parameters
         ----------
         config: dict,
@@ -57,17 +54,23 @@ class CINC2020Dataset(ReprMixin, Dataset):
         training: bool, default True,
             if True, the training set will be loaded, otherwise the test set
         lazy: bool, default True,
-            if True, the data will not be loaded immediately,
+            if True, the data will not be loaded immediately
+        reader_kwargs: dict,
+            keyword arguments for the data reader class
 
         """
         super().__init__()
         self.config = deepcopy(config)
-        assert self.config.db_dir is not None, "db_dir must be specified"
-        self.config.db_dir = Path(self.config.db_dir)
+        # assert self.config.db_dir is not None, "db_dir must be specified"
+        if reader_kwargs.pop("db_dir", None) is not None:
+            warnings.warn(
+                "db_dir is specified in both config and reader_kwargs", RuntimeWarning
+            )
+        self.reader = CR(db_dir=self.config.db_dir, **reader_kwargs)
+        self.config.db_dir = self.reader.db_dir
         self._TRANCHES = (
             self.config.tranche_classes.keys()
         )  # ["A", "B", "AB", "E", "F"]
-        self.reader = CR(db_dir=self.config.db_dir)
         self.tranches = self.config.tranches_for_training
         self.training = training
         if self.config.torch_dtype == torch.float64:
@@ -108,7 +111,7 @@ class CINC2020Dataset(ReprMixin, Dataset):
         ppm_config = CFG(random=False)
         ppm_config.update(self.config)
         self.ppm = PreprocManager.from_config(ppm_config)
-        self.ppm.rearrange(["bandpass", "normalize"])
+        # self.ppm.rearrange(["bandpass", "normalize"])
 
         self._signals = np.array([], dtype=self.dtype).reshape(
             0, len(self.config.leads), self.siglen
@@ -117,11 +120,17 @@ class CINC2020Dataset(ReprMixin, Dataset):
         if not self.lazy:
             self._load_all_data()
 
-    def _load_all_data(self) -> NoReturn:
+    def _load_all_data(self) -> None:
         """ """
         fdr = FastDataReader(self.reader, self.records, self.config, self.ppm)
         self._signals, self._labels = [], []
-        with tqdm(range(len(fdr)), desc="Loading data", unit="records") as pbar:
+        with tqdm(
+            range(len(fdr)),
+            desc="Loading data",
+            unit="record",
+            dynamic_ncols=True,
+            mininterval=1.0,
+        ) as pbar:
             for idx in pbar:
                 sig, lb = fdr[idx]
                 self._signals.append(sig)
@@ -131,7 +140,6 @@ class CINC2020Dataset(ReprMixin, Dataset):
 
     def _load_one_record(self, rec: str) -> Tuple[np.ndarray, np.ndarray]:
         """
-
         load a record from the database using data reader
 
         NOTE
@@ -195,10 +203,12 @@ class CINC2020Dataset(ReprMixin, Dataset):
         return len(self._signals)
 
     def _train_test_split(
-        self, train_ratio: float = 0.8, force_recompute: bool = False
+        self,
+        train_ratio: float = 0.8,
+        force_recompute: bool = False,
+        force_valid: bool = False,
     ) -> List[str]:
         """
-
         do train test split,
         it is ensured that both the train and the test set contain all classes
 
@@ -209,6 +219,9 @@ class CINC2020Dataset(ReprMixin, Dataset):
         force_recompute: bool, default False,
             if True, force redo the train-test split,
             regardless of the existing ones stored in json files
+        force_valid: bool, default False,
+            if True, force redo the train-test split
+            if validity check fails
 
         Returns
         -------
@@ -228,9 +241,13 @@ class CINC2020Dataset(ReprMixin, Dataset):
         ns = "_ns" if len(self.config.special_classes) == 0 else ""
         file_suffix = f"_siglen_{self.siglen}{ns}.json"
         train_file = (
-            self.reader.db_dir_base / f"train_ratio_{_train_ratio}{file_suffix}"
+            self.reader.db_dir_base
+            / f"{self.reader.db_name}_train_ratio_{_train_ratio}{file_suffix}"
+        )
+        test_file = (
+            self.reader.db_dir_base
+            / f"{self.reader.db_name}_test_ratio_{_test_ratio}{file_suffix}"
         )
-        test_file = self.reader.db_dir_base / f"test_ratio_{_test_ratio}{file_suffix}"
 
         if force_recompute or not all([train_file.is_file(), test_file.is_file()]):
             tranche_records = {t: [] for t in _TRANCHES}
@@ -238,7 +255,10 @@ class CINC2020Dataset(ReprMixin, Dataset):
             test_set = {t: [] for t in _TRANCHES}
             for t in _TRANCHES:
                 with tqdm(
-                    self.reader.all_records[t], total=len(self.reader.all_records[t])
+                    self.reader.all_records[t],
+                    total=len(self.reader.all_records[t]),
+                    dynamic_ncols=True,
+                    mininterval=1.0,
                 ) as bar:
                     for rec in bar:
                         if rec in self.reader.exceptional_records:
@@ -267,9 +287,14 @@ class CINC2020Dataset(ReprMixin, Dataset):
                     split_idx = int(len(tranche_records[t]) * train_ratio)
                     train_set[t] = tranche_records[t][:split_idx]
                     test_set[t] = tranche_records[t][split_idx:]
-                    is_valid = self._check_train_test_split_validity(
-                        train_set[t], test_set[t], set(self.config.tranche_classes[t])
-                    )
+                    if force_valid:
+                        is_valid = self._check_train_test_split_validity(
+                            train_set[t],
+                            test_set[t],
+                            set(self.config.tranche_classes[t]),
+                        )
+                    else:
+                        is_valid = True
             train_file.write_text(json.dumps(train_set, ensure_ascii=False))
             test_file.write_text(json.dumps(test_set, ensure_ascii=False))
         else:
@@ -287,7 +312,6 @@ class CINC2020Dataset(ReprMixin, Dataset):
         self, train_set: List[str], test_set: List[str], all_classes: Set[str]
     ) -> bool:
         """
-
         the train-test split is valid iff
         records in both `train_set` and `test` contain all classes in `all_classes`
 
@@ -306,30 +330,26 @@ class CINC2020Dataset(ReprMixin, Dataset):
             the split is valid or not
 
         """
-
-        def add(a, b):
-            return a + b
-
         train_classes = set(
-            reduce(add, [self.reader.get_labels(rec, fmt="a") for rec in train_set])
+            list_sum([self.reader.get_labels(rec, fmt="a") for rec in train_set])
         )
         train_classes.intersection_update(all_classes)
         test_classes = set(
-            reduce(add, [self.reader.get_labels(rec, fmt="a") for rec in test_set])
+            list_sum([self.reader.get_labels(rec, fmt="a") for rec in test_set])
         )
         test_classes.intersection_update(all_classes)
         is_valid = len(all_classes) == len(train_classes) == len(test_classes)
         print(
-            f"all_classes = {all_classes}\ntrain_classes = {train_classes}\ntest_classes = {test_classes}\nis_valid = {is_valid}"
+            f"all_classes = {all_classes}\n"
+            f"train_classes = {train_classes}\ntest_classes = {test_classes}\n"
+            f"is_valid = {is_valid}"
         )
         return is_valid
 
-    def persistence(self) -> NoReturn:
+    def persistence(self) -> None:
         """
         make the dataset persistent w.r.t. the tranches and the ratios in `self.config`
-
         """
-        prev_state = self.__data_aug
         _TRANCHES = "ABEF"
         if self.training:
             ratio = int(self.config.train_ratio * 100)
@@ -343,7 +363,12 @@ class CINC2020Dataset(ReprMixin, Dataset):
         fn_suffix = fn_suffix + f"_siglen_{self.siglen}"
 
         X, y = [], []
-        with tqdm(range(self.__len__()), total=self.__len__()) as bar:
+        with tqdm(
+            range(self.__len__()),
+            total=self.__len__(),
+            dynamic_ncols=True,
+            mininterval=1.0,
+        ) as bar:
             for idx in bar:
                 values, labels = self.__getitem__(idx)
                 X.append(values)
@@ -357,15 +382,12 @@ class CINC2020Dataset(ReprMixin, Dataset):
         np.save(self.reader.db_dir_base / filename, y)
         print(f"y saved to {filename}")
 
-        self.__data_aug = prev_state
-
-    def _check_nan(self) -> NoReturn:
+    def _check_nan(self) -> None:
         """
         during training, sometimes nan values are encountered,
         which ruins the whole training process
-
         """
-        for idx, (values, labels) in self:
+        for idx, (values, labels) in enumerate(self):
             if np.isnan(values).any():
                 print(f"values of {self.records[idx]} have nan values")
             if np.isnan(labels).any():
@@ -388,7 +410,7 @@ class FastDataReader(ReprMixin, Dataset):
         records: Sequence[str],
         config: CFG,
         ppm: Optional[PreprocManager] = None,
-    ) -> NoReturn:
+    ) -> None:
         """ """
         self.reader = reader
         self.records = records
