@@ -6,7 +6,7 @@ along with some constants
 """
 
 from copy import deepcopy
-from typing import List, NoReturn
+from typing import List
 
 from ....cfg import CFG, DEFAULTS
 from ...aux_data.cinc2021_aux_data import get_class_weight
@@ -43,7 +43,7 @@ _SPECIAL_CLASSES = []
 _NAME = "cinc2021"
 
 
-def _assign_classes(cfg: CFG, special_classes: List[str]) -> NoReturn:
+def _assign_classes(cfg: CFG, special_classes: List[str]) -> None:
     """ """
     cfg.special_classes = deepcopy(special_classes)
     cfg.tranche_class_weights = CFG(
@@ -82,7 +82,7 @@ def _assign_classes(cfg: CFG, special_classes: List[str]) -> NoReturn:
 # training configurations for machine learning and deep learning
 
 CINC2021TrainCfg = CFG()
-CINC2021TrainCfg.torch_dtype = DEFAULTS.torch_dtype
+CINC2021TrainCfg.torch_dtype = DEFAULTS.DTYPE.TORCH
 
 # configs of files
 CINC2021TrainCfg.db_dir = None
