@@ -5,27 +5,22 @@ data generator for feeding data into pytorch models
 import json
 import textwrap
 import time
+import warnings
 from copy import deepcopy
-from pathlib import Path
 from random import sample, shuffle
-from typing import List, NoReturn, Optional, Sequence, Set, Tuple
+from typing import List, Optional, Sequence, Set, Tuple, Any
 
 import numpy as np
-
-try:
-    from tqdm.auto import tqdm
-except ModuleNotFoundError:
-    from tqdm import tqdm
-
 import torch
 from torch.utils.data.dataset import Dataset
+from tqdm.auto import tqdm
 
 from ...._preprocessors import PreprocManager
 from ....cfg import CFG
 from ....databases import CINC2021 as CR
 from ....utils.ecg_arrhythmia_knowledge import Standard12Leads
 from ....utils.misc import ReprMixin, list_sum
-from torch_ecg.utils.utils_data import ensure_siglen
+from ....utils.utils_data import ensure_siglen
 from ....utils.utils_signal import remove_spikes_naive
 from .cinc2021_cfg import CINC2021TrainCfg  # noqa: F401
 
@@ -42,10 +37,13 @@ class CINC2021Dataset(ReprMixin, Dataset):
     __name__ = "CINC2021Dataset"
 
     def __init__(
-        self, config: Optional[CFG] = None, training: bool = True, lazy: bool = True
-    ) -> NoReturn:
+        self,
+        config: Optional[CFG] = None,
+        training: bool = True,
+        lazy: bool = True,
+        **reader_kwargs: Any,
+    ) -> None:
         """
-
         Parameters
         ----------
         config: dict,
@@ -58,17 +56,23 @@ class CINC2021Dataset(ReprMixin, Dataset):
         training: bool, default True,
             if True, the training set will be loaded, otherwise the test set
         lazy: bool, default True,
-            if True, the data will not be loaded immediately,
+            if True, the data will not be loaded immediately
+        reader_kwargs: dict,
+            keyword arguments for the data reader class
 
         """
         super().__init__()
         self.config = deepcopy(config)
-        assert self.config.db_dir is not None, "db_dir must be specified"
-        self.config.db_dir = Path(self.config.db_dir)
+        if reader_kwargs.pop("db_dir", None) is not None:
+            warnings.warn(
+                "db_dir is specified in both config and reader_kwargs", RuntimeWarning
+            )
+        self.reader = CR(db_dir=self.config.db_dir, **reader_kwargs)
+        # assert self.config.db_dir is not None, "db_dir must be specified"
+        self.config.db_dir = self.reader.db_dir
         self._TRANCHES = (
             self.config.tranche_classes.keys()
         )  # ["A", "B", "AB", "E", "F", "G",]
-        self.reader = CR(db_dir=self.config.db_dir)
         self.tranches = self.config.tranches_for_training
         self.training = training
         if self.config.torch_dtype == torch.float64:
@@ -114,7 +118,7 @@ class CINC2021Dataset(ReprMixin, Dataset):
         ppm_config = CFG(random=False)
         ppm_config.update(self.config)
         self.ppm = PreprocManager.from_config(ppm_config)
-        self.ppm.rearrange(["bandpass", "normalize"])
+        # self.ppm.rearrange(["bandpass", "normalize"])
 
         self._signals = np.array([], dtype=self.dtype).reshape(
             0, len(self.config.leads), self.siglen
@@ -123,23 +127,21 @@ class CINC2021Dataset(ReprMixin, Dataset):
         if not self.lazy:
             self._load_all_data()
 
-    def _load_all_data(self) -> NoReturn:
+    def _load_all_data(self) -> None:
         """ """
-        # self.reader can not be pickled
-        # with mp.Pool(processes=max(1, mp.cpu_count()-2)) as pool:
-        #     self._signals, self._labels = \
-        #         zip(*pool.starmap(_load_record, [(self.reader, r, self.config) for r in self.records]))
-
-        # self._signals = np.array([]).reshape(0, len(self.config.leads), self.siglen)
-        # self._labels = np.array([]).reshape(0, self.n_classes)
-
         fdr = FastDataReader(self.reader, self.records, self.config, self.ppm)
 
-        # with tqdm(self.records, desc="Loading data", unit="records") as pbar:
+        # with tqdm(self.records, desc="Loading data", unit="record") as pbar:
         #     for rec in pbar:
         # s, l = self._load_one_record(rec)  # self._load_one_record is much slower than FastDataReader
         self._signals, self._labels = [], []
-        with tqdm(desc="Loading data", total=len(fdr), unit="records") as pbar:
+        with tqdm(
+            desc="Loading data",
+            total=len(fdr),
+            unit="record",
+            dynamic_ncols=True,
+            mininterval=1.0,
+        ) as pbar:
             for idx in range(len(fdr)):
                 sig, lb = fdr[idx]
                 # np.concatenate slows down the process severely
@@ -153,7 +155,6 @@ class CINC2021Dataset(ReprMixin, Dataset):
 
     def _load_one_record(self, rec: str) -> Tuple[np.ndarray, np.ndarray]:
         """
-
         load a record from the database using data reader
 
         NOTE
@@ -202,14 +203,17 @@ class CINC2021Dataset(ReprMixin, Dataset):
 
         return values, labels
 
-    def to(self, leads: Sequence[str]) -> NoReturn:
+    def to(self, leads: Sequence[str]) -> None:
         """ """
+        assert set(leads) <= set(
+            self.config.leads
+        ), "One is not able to change to a set of leads which is not a subset of the current leads"
         prev_leads = self.config.leads
         self.config.leads = leads
         self._indices = [prev_leads.index(ld) for ld in leads]
         self._signals = self._signals[:, self._indices, :]
 
-    def emtpy(self, leads: Optional[Sequence[str]] = None) -> NoReturn:
+    def empty(self, leads: Optional[Sequence[str]] = None) -> None:
         """ """
         if leads is None:
             leads = self.config.leads
@@ -218,6 +222,7 @@ class CINC2021Dataset(ReprMixin, Dataset):
         self._signals = np.array([], dtype=self.dtype).reshape(
             0, len(leads), self.siglen
         )
+        self._labels = np.array([], dtype=self.dtype).reshape(0, self.n_classes)
 
     @classmethod
     def from_extern(cls, ext_ds: "CINC2021Dataset", config: CFG) -> "CINC2021Dataset":
@@ -228,8 +233,11 @@ class CINC2021Dataset(ReprMixin, Dataset):
         new_ds._labels = ext_ds._labels.copy()
         return new_ds
 
-    def reload_from_extern(self, ext_ds: "CINC2021Dataset") -> NoReturn:
+    def reload_from_extern(self, ext_ds: "CINC2021Dataset") -> None:
         """ """
+        assert set(self.config.leads) <= set(
+            ext_ds.config.leads
+        ), "One is not able to reload from a dataset whose `leads` is not a superset of the current one"
         indices = [ext_ds.config.leads.index(ld) for ld in self.config.leads]
         self._signals = ext_ds._signals[:, indices, :]
         self._labels = ext_ds._labels.copy()
@@ -253,10 +261,12 @@ class CINC2021Dataset(ReprMixin, Dataset):
         return len(self._signals)
 
     def _train_test_split(
-        self, train_ratio: float = 0.8, force_recompute: bool = False
+        self,
+        train_ratio: float = 0.8,
+        force_recompute: bool = False,
+        force_valid: bool = False,
     ) -> List[str]:
         """
-
         do train test split,
         it is ensured that both the train and the test set contain all classes
 
@@ -267,6 +277,9 @@ class CINC2021Dataset(ReprMixin, Dataset):
         force_recompute: bool, default False,
             if True, force redo the train-test split,
             regardless of the existing ones stored in json files
+        force_valid: bool, default False,
+            if True, force redo the train-test split
+            if validity check fails
 
         Returns
         -------
@@ -285,9 +298,13 @@ class CINC2021Dataset(ReprMixin, Dataset):
         ns = "_ns" if len(self.config.special_classes) == 0 else ""
         file_suffix = f"_siglen_{self.siglen}{ns}.json"
         train_file = (
-            self.reader.db_dir_base / f"train_ratio_{_train_ratio}{file_suffix}"
+            self.reader.db_dir_base
+            / f"{self.reader.db_name}_train_ratio_{_train_ratio}{file_suffix}"
+        )
+        test_file = (
+            self.reader.db_dir_base
+            / f"{self.reader.db_name}_test_ratio_{_test_ratio}{file_suffix}"
         )
-        test_file = self.reader.db_dir_base / f"test_ratio_{_test_ratio}{file_suffix}"
 
         # TODO: use self.reader.df_stats (precomputed and stored in utils/stats.csv)
         # to accelerate the validity examinations
@@ -297,7 +314,10 @@ class CINC2021Dataset(ReprMixin, Dataset):
             test_set = {t: [] for t in _TRANCHES}
             for t in _TRANCHES:
                 with tqdm(
-                    self.reader.all_records[t], total=len(self.reader.all_records[t])
+                    self.reader.all_records[t],
+                    total=len(self.reader.all_records[t]),
+                    dynamic_ncols=True,
+                    mininterval=1.0,
                 ) as bar:
                     for rec in bar:
                         if rec in self.reader.exceptional_records:
@@ -330,9 +350,14 @@ class CINC2021Dataset(ReprMixin, Dataset):
                     split_idx = int(len(tranche_records[t]) * train_ratio)
                     train_set[t] = tranche_records[t][:split_idx]
                     test_set[t] = tranche_records[t][split_idx:]
-                    is_valid = self._check_train_test_split_validity(
-                        train_set[t], test_set[t], set(self.config.tranche_classes[t])
-                    )
+                    if force_valid:
+                        is_valid = self._check_train_test_split_validity(
+                            train_set[t],
+                            test_set[t],
+                            set(self.config.tranche_classes[t]),
+                        )
+                    else:
+                        is_valid = True
             train_file.write_text(json.dumps(train_set, ensure_ascii=False))
             test_file.write_text(json.dumps(test_set, ensure_ascii=False))
             print(
@@ -364,7 +389,6 @@ class CINC2021Dataset(ReprMixin, Dataset):
         self, train_set: List[str], test_set: List[str], all_classes: Set[str]
     ) -> bool:
         """
-
         the train-test split is valid iff
         records in both `train_set` and `test` contain all classes in `all_classes`
 
@@ -404,10 +428,9 @@ class CINC2021Dataset(ReprMixin, Dataset):
         )
         return is_valid
 
-    def persistence(self) -> NoReturn:
+    def persistence(self) -> None:
         """
         make the dataset persistent w.r.t. the tranches and the ratios in `self.config`
-
         """
         _TRANCHES = "ABEFG"
         if self.training:
@@ -422,7 +445,12 @@ class CINC2021Dataset(ReprMixin, Dataset):
         fn_suffix = fn_suffix + f"_siglen_{self.siglen}"
 
         X, y = [], []
-        with tqdm(range(self.__len__()), total=self.__len__()) as bar:
+        with tqdm(
+            range(self.__len__()),
+            total=self.__len__(),
+            dynamic_ncols=True,
+            mininterval=1.0,
+        ) as bar:
             for idx in bar:
                 values, labels = self.__getitem__(idx)
                 X.append(values)
@@ -436,7 +464,7 @@ class CINC2021Dataset(ReprMixin, Dataset):
         np.save(self.reader.db_dir_base / filename, y)
         print(f"y saved to {filename}")
 
-    def _check_nan(self) -> NoReturn:
+    def _check_nan(self) -> None:
         """
         during training, sometimes nan values are encountered,
         which ruins the whole training process
@@ -465,7 +493,7 @@ class FastDataReader(ReprMixin, Dataset):
         records: Sequence[str],
         config: CFG,
         ppm: Optional[PreprocManager] = None,
-    ) -> NoReturn:
+    ) -> None:
         """ """
         self.reader = reader
         self.records = records
@@ -517,39 +545,3 @@ class FastDataReader(ReprMixin, Dataset):
             "reader",
             "ppm",
         ]
-
-
-def _load_record(reader: CR, rec: str, config: CFG) -> Tuple[np.ndarray, np.ndarray]:
-    """finished, NOT checked,
-
-    load a record from the database using data reader
-
-    Parameters
-    ----------
-    reader: CR,
-        the data reader
-    rec: str,
-        the record to load
-    config: dict,
-        the configuration for loading record
-
-    Returns
-    -------
-    values: np.ndarray,
-        the values of the record
-    labels: np.ndarray,
-        the labels of the record
-
-    """
-    values = reader.load_resampled_data(
-        rec, leads=config.leads, data_format="channel_first", siglen=None
-    )
-    values = ensure_siglen(values, siglen=config.input_len, fmt="channel_first")
-
-    labels = reader.get_labels(rec, scored_only=True, fmt="a", normalize=True)
-    labels = np.isin(config.all_classes, labels).astype(int)
-
-    if config.data_format.lower() in ["channel_last", "lead_last"]:
-        values = values.T
-
-    return values, labels
