@@ -4,18 +4,15 @@ data generator for feeding data into pytorch models
 
 import json
 import math
+import warnings
 from copy import deepcopy
 from random import shuffle
-from typing import List, NoReturn, Optional, Sequence, Tuple
+from typing import List, Optional, Sequence, Tuple, Any
 
 import numpy as np
 import torch
 from torch.utils.data.dataset import Dataset
-
-try:
-    from tqdm.auto import tqdm
-except ModuleNotFoundError:
-    from tqdm import tqdm
+from tqdm.auto import tqdm
 
 from ...._preprocessors import PreprocManager
 from ....cfg import CFG
@@ -40,9 +37,9 @@ class CPSC2019Dataset(ReprMixin, Dataset):
         config: CFG,
         training: bool = True,
         lazy: bool = False,
-    ) -> NoReturn:
+        **reader_kwargs: Any,
+    ) -> None:
         """
-
         Parameters
         ----------
         config: dict,
@@ -55,12 +52,19 @@ class CPSC2019Dataset(ReprMixin, Dataset):
         training: bool, default True,
             if True, the training set will be loaded, otherwise the test set
         lazy: bool, default False,
-            if True, the data will not be loaded immediately,
+            if True, the data will not be loaded immediately
+        reader_kwargs: dict,
+            keyword arguments for the data reader class
 
         """
         super().__init__()
         self.config = deepcopy(config)
-        self.reader = CR(db_dir=config.db_dir)
+        if reader_kwargs.pop("db_dir", None) is not None:
+            warnings.warn(
+                "db_dir is specified in both config and reader_kwargs", RuntimeWarning
+            )
+        self.reader = CR(db_dir=config.db_dir, **reader_kwargs)
+        self.config.db_dir = self.reader.db_dir
         self.training = training
         self.n_classes = 1
         self.lazy = lazy
@@ -97,10 +101,16 @@ class CPSC2019Dataset(ReprMixin, Dataset):
         """ """
         return len(self.fdr)
 
-    def _load_all_data(self) -> NoReturn:
+    def _load_all_data(self) -> None:
         """ """
         self._signals, self._labels = [], []
-        with tqdm(self.fdr, desc="loading data", unit="records") as pbar:
+        with tqdm(
+            self.fdr,
+            desc="loading data",
+            unit="record",
+            dynamic_ncols=True,
+            mininterval=1.0,
+        ) as pbar:
             for sig, lab in pbar:
                 self._signals.append(sig)
                 self._labels.append(lab)
@@ -179,7 +189,7 @@ class FastDataReader(ReprMixin, Dataset):
         records: Sequence[str],
         config: CFG,
         ppm: Optional[PreprocManager] = None,
-    ) -> NoReturn:
+    ) -> None:
         """ """
         self.reader = reader
         self.records = records
@@ -195,8 +205,8 @@ class FastDataReader(ReprMixin, Dataset):
     def __getitem__(self, index: int) -> Tuple[np.ndarray, np.ndarray]:
         """ """
         rec_name = self.records[index]
-        values = self.reader.load_data(rec_name, units="mV", keep_dim=False)
-        rpeaks = self.reader.load_ann(rec_name, keep_dim=False)
+        values = self.reader.load_data(rec_name, units="mV", data_format="flat")
+        rpeaks = self.reader.load_ann(rec_name)
         if self.config.get("recover_length", False):
             reduction = 1
         else:
