@@ -1,17 +1,8 @@
 """
 """
 
-from pathlib import Path
-
-try:
-    import torch_ecg  # noqa: F401
-except ModuleNotFoundError:
-    import sys
-
-    sys.path.insert(0, str(Path(__file__).absolute().parents[2]))
-
-from torch_ecg.cfg import CFG, DEFAULTS
-from torch_ecg.model_configs import (  # noqa: F401
+from ....cfg import CFG, DEFAULTS
+from ....model_configs import (  # noqa: F401
     ECG_SEQ_LAB_NET_CONFIG,
     ECG_SUBTRACT_UNET_CONFIG,
     ECG_UNET_VANILLA_CONFIG,
@@ -79,7 +70,7 @@ CPSC2021TrainCfg.checkpoints.mkdir(parents=True, exist_ok=True)
 
 CPSC2021TrainCfg.fs = 200
 CPSC2021TrainCfg.n_leads = 2
-CPSC2021TrainCfg.torch_dtype = DEFAULTS.torch_dtype
+CPSC2021TrainCfg.torch_dtype = DEFAULTS.DTYPE.TORCH
 
 CPSC2021TrainCfg.class_fn2abbr = {  # fullname to abbreviation
     "non atrial fibrillation": "N",
