@@ -40,21 +40,17 @@ import json
 import os
 import re
 import time
+import warnings
 from copy import deepcopy
 from pathlib import Path
-from typing import Dict, List, NoReturn, Optional, Sequence, Tuple, Union
+from typing import Dict, List, Optional, Sequence, Tuple, Union, Any
 
 import numpy as np
 from scipy import signal as SS
-
-try:
-    from tqdm.auto import tqdm
-except ModuleNotFoundError:
-    from tqdm import tqdm
-
-import torch
 from scipy.io import loadmat, savemat
+import torch
 from torch.utils.data.dataset import Dataset
+from tqdm.auto import tqdm
 
 from ...._preprocessors import PreprocManager
 from ....databases import CPSC2021 as CR
@@ -78,16 +74,37 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         the data files stores segments of fixed length of preprocessed ECGs,
         the annotation files contain "qrs_mask", and "af_mask"
 
+    The returned values (tuple) of `__getitem__` depends on the task:
+    1. "qrs_detection": (`data`, `qrs_mask`, None)
+    2. "rr_lstm": (`rr_seq`, `rr_af_mask`, `rr_weight_mask`)
+    3. "main": (`data`, `af_mask`, `weight_mask`)
+
+    - `data` shape: (`n_lead`, `n_sample`)
+    - `qrs_mask` shape: (`n_sample`, 1)
+    - `af_mask` shape: (`n_sample`, 1)
+    - `weight_mask` shape: (`n_sample`, 1)
+    - `rr_seq` shape: (`n_rr`, 1)
+    - `rr_af_mask` shape: (`n_rr`, 1)
+    - `rr_weight_mask` shape: (`n_rr`, 1)
+
+    typical values of `n_sample` and `n_rr` are 6000 and 30, respectively
+
+    `n_lead` is typically 2, which is the number of leads in the ECG signal of the CPSC2021 dataset
+
     """
 
     __DEBUG__ = False
     __name__ = "CPSC2021Dataset"
 
     def __init__(
-        self, config: CFG, task: str, training: bool = True, lazy: bool = True
-    ) -> NoReturn:
+        self,
+        config: CFG,
+        task: str,
+        training: bool = True,
+        lazy: bool = True,
+        **reader_kwargs: Any,
+    ) -> None:
         """
-
         Parameters
         ----------
         config: dict,
@@ -96,18 +113,24 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             a simple example is:
             >>> config = deepcopy(CPSC2021TrainCfg)
             >>> config.db_dir = "some/path/to/db"
-            >>> dataset = CPSC2021Dataset(config, training=True, lazy=False)
+            >>> dataset = CPSC2021Dataset(config, task="main", training=True, lazy=False)
         training: bool, default True,
             if True, the training set will be loaded, otherwise the test set
         lazy: bool, default False,
-            if True, the data will not be loaded immediately,
+            if True, the data will not be loaded immediately
+        reader_kwargs: dict,
+            keyword arguments for the data reader class
 
         """
         super().__init__()
         self.config = deepcopy(config)
-        assert self.config.db_dir is not None, "db_dir must be specified"
-        self.config.db_dir = Path(self.config.db_dir)
-        self.reader = CR(db_dir=self.config.db_dir)
+        if reader_kwargs.pop("db_dir", None) is not None:
+            warnings.warn(
+                "db_dir is specified in both config and reader_kwargs", RuntimeWarning
+            )
+        self.reader = CR(db_dir=self.config.db_dir, **reader_kwargs)
+        # assert self.config.db_dir is not None, "db_dir must be specified"
+        self.config.db_dir = self.reader.db_dir
         if self.config.torch_dtype == torch.float64:
             self.dtype = np.float64
         else:
@@ -154,13 +177,12 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         self._all_masks = None
         self.__set_task(task, lazy=self.lazy)
 
-    def _load_all_data(self) -> NoReturn:
+    def _load_all_data(self) -> None:
         """ """
         self.__set_task(self.task, lazy=False)
 
-    def __set_task(self, task: str, lazy: bool = True) -> NoReturn:
+    def __set_task(self, task: str, lazy: bool = True) -> None:
         """
-
         Parameters
         ----------
         task: str,
@@ -223,7 +245,11 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                 return
             self._all_data, self._all_labels, self._all_masks = [], [], []
             with tqdm(
-                range(len(self.fdr)), desc="Loading data", unit="records"
+                range(len(self.fdr)),
+                desc="Loading data",
+                unit="record",
+                dynamic_ncols=True,
+                mininterval=1.0,
             ) as pbar:
                 for idx in pbar:
                     d, l, m = self.fdr[idx]
@@ -265,7 +291,11 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                 return
             self._all_data, self._all_labels, self._all_masks = [], [], []
             with tqdm(
-                range(len(self.fdr)), desc="Loading data", unit="records"
+                range(len(self.fdr)),
+                desc="Loading data",
+                unit="record",
+                dynamic_ncols=True,
+                mininterval=1.0,
             ) as pbar:
                 for idx in pbar:
                     d, l, m = self.fdr[idx]
@@ -280,11 +310,11 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                 f"data generator for task \042{self.task}\042 not implemented"
             )
 
-    def reset_task(self, task: str, lazy: bool = True) -> NoReturn:
+    def reset_task(self, task: str, lazy: bool = True) -> None:
         """ """
         self.__set_task(task, lazy)
 
-    def _ls_segments(self) -> NoReturn:
+    def _ls_segments(self) -> None:
         """list all the segments"""
         for item in ["data", "ann"]:
             self.segments_dirs[item] = CFG()
@@ -297,7 +327,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         print(
             f"please allow the reader a few minutes to collect the segments from {self.segments_base_dir}..."
         )
-        seg_filename_pattern = f"{self.segment_name_pattern}.{self.segment_ext}"
+        seg_filename_pattern = f"{self.segment_name_pattern}\\.{self.segment_ext}"
         self.__all_segments = CFG(
             {
                 s: get_record_list_recursive3(
@@ -311,7 +341,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                 json.dumps(self.__all_segments, ensure_ascii=False)
             )
 
-    def _ls_rr_seq(self) -> NoReturn:
+    def _ls_rr_seq(self) -> None:
         """list all the rr sequences"""
         for s in self.reader.all_subjects:
             self.rr_seq_dirs[s] = self.rr_seq_base_dir / s
@@ -322,7 +352,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         print(
             f"please allow the reader a few minutes to collect the rr sequences from {self.rr_seq_base_dir}..."
         )
-        rr_seq_filename_pattern = f"{self.rr_seq_name_pattern}.{self.rr_seq_ext}"
+        rr_seq_filename_pattern = f"{self.rr_seq_name_pattern}\\.{self.rr_seq_ext}"
         self.__all_rr_seq = CFG(
             {
                 s: get_record_list_recursive3(
@@ -376,7 +406,6 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
     def _get_seg_data_path(self, seg: str) -> Path:
         """
-
         Parameters
         ----------
         seg: str,
@@ -394,7 +423,6 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
     def _get_seg_ann_path(self, seg: str) -> Path:
         """
-
         Parameters
         ----------
         seg: str,
@@ -412,7 +440,6 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
     def _load_seg_data(self, seg: str) -> np.ndarray:
         """
-
         Parameters
         ----------
         seg: str,
@@ -430,7 +457,6 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
     def _load_seg_ann(self, seg: str) -> dict:
         """
-
         Parameters
         ----------
         seg: str,
@@ -458,7 +484,6 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         self, seg: str, task: Optional[str] = None
     ) -> Union[np.ndarray, Dict[str, np.ndarray]]:
         """
-
         Parameters
         ----------
         seg: str,
@@ -498,7 +523,6 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
     def _load_seg_seq_lab(self, seg: str, reduction: int) -> np.ndarray:
         """
-
         Parameters
         ----------
         seg: str,
@@ -531,7 +555,6 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
     def _get_rr_seq_path(self, seq_name: str) -> Path:
         """
-
         Parameters
         ----------
         seq_name: str,
@@ -549,7 +572,6 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
     def _load_rr_seq(self, seq_name: str) -> Dict[str, np.ndarray]:
         """
-
         Parameters
         ----------
         seq_name: str,
@@ -573,9 +595,8 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         rr_seq["interval"] = rr_seq["interval"].flatten()
         return rr_seq
 
-    def persistence(self, force_recompute: bool = False, verbose: int = 0) -> NoReturn:
+    def persistence(self, force_recompute: bool = False, verbose: int = 0) -> None:
         """
-
         make the dataset persistent w.r.t. the ratios in `self.config`
 
         Parameters
@@ -587,29 +608,34 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
         """
         if verbose >= 1:
-            print(" preprocessing data ".center("#", 110))
+            print(" preprocessing data ".center(110, "#"))
         self._preprocess_data(
             force_recompute=force_recompute,
             verbose=verbose,
         )
+
+        original_task = self.task
+        original_lazy = self.lazy
+        self.__set_task("main", lazy=True)
         if verbose >= 1:
-            print("\n" + " slicing data into segments ".center("#", 110))
+            print("\n" + " slicing data into segments ".center(110, "#"))
         self._slice_data(
             force_recompute=force_recompute,
             verbose=verbose,
         )
+
+        self.__set_task("rr_lstm", lazy=True)
         if verbose >= 1:
-            print("\n" + " generating rr sequences ".center("#", 110))
+            print("\n" + " generating rr sequences ".center(110, "#"))
         self._slice_rr_seq(
             force_recompute=force_recompute,
             verbose=verbose,
         )
 
-    def _preprocess_data(
-        self, force_recompute: bool = False, verbose: int = 0
-    ) -> NoReturn:
-        """
+        self.__set_task(original_task, lazy=original_lazy)
 
+    def _preprocess_data(self, force_recompute: bool = False, verbose: int = 0) -> None:
+        """
         preprocesses the ecg data in advance for further use,
         offline for `self.persistence`
 
@@ -632,9 +658,8 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
     def _preprocess_one_record(
         self, rec: str, force_recompute: bool = False, verbose: int = 0
-    ) -> NoReturn:
+    ) -> None:
         """
-
         preprocesses the ecg data in advance for further use,
         offline for `self.persistence`
 
@@ -658,7 +683,6 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
     def load_preprocessed_data(self, rec: str) -> np.ndarray:
         """
-
         Parameters
         ----------
         rec: str,
@@ -684,7 +708,6 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
     def _get_rec_suffix(self, operations: List[str]) -> str:
         """
-
         Parameters
         ----------
         operations: list of str,
@@ -700,9 +723,8 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         suffix = "-".join(sorted([item.lower() for item in operations]))
         return suffix
 
-    def _slice_data(self, force_recompute: bool = False, verbose: int = 0) -> NoReturn:
+    def _slice_data(self, force_recompute: bool = False, verbose: int = 0) -> None:
         """
-
         slice all records into segments of length `self.seglen`,
         and perform data augmentations specified in `self.config`
 
@@ -742,9 +764,8 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         force_recompute: bool = False,
         update_segments_json: bool = False,
         verbose: int = 0,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         slice one record into segments of length `self.seglen`,
         and perform data augmentations specified in `self.config`
 
@@ -851,7 +872,6 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         end_idx: Optional[int] = None,
     ) -> CFG:
         """
-
         generate segment, with possible data augmentation
 
         Parameter
@@ -976,9 +996,8 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
     def __save_segments(
         self, rec: str, segments: List[CFG], update_segments_json: bool = False
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         rec: str,
@@ -1015,9 +1034,8 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                 json.dumps(self.__all_segments, ensure_ascii=False)
             )
 
-    def _clear_cached_segments(self, recs: Optional[Sequence[str]] = None) -> NoReturn:
+    def _clear_cached_segments(self, recs: Optional[Sequence[str]] = None) -> None:
         """
-
         Parameters
         ----------
         recs: sequence of str, optional
@@ -1065,11 +1083,8 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             [self.__all_segments[subject] for subject in self.subjects]
         )
 
-    def _slice_rr_seq(
-        self, force_recompute: bool = False, verbose: int = 0
-    ) -> NoReturn:
+    def _slice_rr_seq(self, force_recompute: bool = False, verbose: int = 0) -> None:
         """
-
         slice sequences of rr intervals into fixed length (sub)sequences
 
         Parameters
@@ -1103,7 +1118,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         force_recompute: bool = False,
         update_rr_seq_json: bool = False,
         verbose: int = 0,
-    ) -> NoReturn:
+    ) -> None:
         """ """
         self.__assert_task(["rr_lstm"])
         subject = self.reader.get_subject_id(rec)
@@ -1185,9 +1200,8 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
     def __save_rr_seq(
         self, rec: str, rr_seq: List[CFG], update_rr_seq_json: bool = False
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         rec: str,
@@ -1212,9 +1226,8 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                 json.dumps(self.__all_rr_seq, ensure_ascii=False)
             )
 
-    def _clear_cached_rr_seq(self, recs: Optional[Sequence[str]] = None) -> NoReturn:
+    def _clear_cached_rr_seq(self, recs: Optional[Sequence[str]] = None) -> None:
         """
-
         Parameters
         ----------
         recs: sequence of str, optional
@@ -1245,7 +1258,6 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
     def _get_rec_name(self, seg_or_rr: str) -> str:
         """
-
         Parameters
         ----------
         seg_or_rr: str,
@@ -1264,7 +1276,6 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         self, train_ratio: float = 0.8, force_recompute: bool = False
     ) -> Dict[str, List[str]]:
         """
-
         do train test split,
         it is ensured that both the train and the test set contain all classes
 
@@ -1311,15 +1322,15 @@ class CPSC2021Dataset(ReprMixin, Dataset):
 
             test_set = (
                 DEFAULTS.RNG_sample(
-                    afp_subjects,
+                    list(afp_subjects),
                     max(1, int(round(len(afp_subjects) * _test_ratio / 100))),
                 ).tolist()
                 + DEFAULTS.RNG_sample(
-                    aff_subjects,
+                    list(aff_subjects),
                     max(1, int(round(len(aff_subjects) * _test_ratio / 100))),
                 ).tolist()
                 + DEFAULTS.RNG_sample(
-                    normal_subjects,
+                    list(normal_subjects),
                     max(1, int(round(len(normal_subjects) * _test_ratio / 100))),
                 ).tolist()
             )
@@ -1352,15 +1363,14 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         )
         return split_res
 
-    def __assert_task(self, tasks: List[str]) -> NoReturn:
+    def __assert_task(self, tasks: List[str]) -> None:
         """ """
         assert (
             self.task in tasks
         ), f"DO NOT call this method when the current task is {self.task}. Switch task using `reset_task`"
 
-    def plot_seg(self, seg: str, ticks_granularity: int = 0) -> NoReturn:
+    def plot_seg(self, seg: str, ticks_granularity: int = 0) -> None:
         """
-
         Parameters
         ----------
         seg: str,
@@ -1402,7 +1412,7 @@ class FastDataReader(ReprMixin, Dataset):
         file_dirs: dict,
         files: List[str],
         file_ext: str,
-    ) -> NoReturn:
+    ) -> None:
         """ """
         self.config = config
         self.task = task
@@ -1496,282 +1506,4 @@ class FastDataReader(ReprMixin, Dataset):
         ]
 
 
-class StandaloneSegmentSlicer(ReprMixin, Dataset):
-    """ """
-
-    def __init__(
-        self,
-        reader: CR,
-        config: CFG,
-        task: str,
-        seg_ppm: PreprocManager,
-        allowed_preproc: List[str],
-        segment_ext: str,
-        preprocess_dir: str,
-    ) -> NoReturn:
-        """ """
-        self.reader = reader
-        self.config = config
-
-        self.seg_ppm = seg_ppm
-        if self.config.torch_dtype == torch.float64:
-            self.dtype = np.float64
-        else:
-            self.dtype = np.float32
-
-        self.task = task
-        self.seglen = self.config[self.task].input_len
-        self.allowed_preproc = allowed_preproc
-        self.segment_ext = segment_ext
-        self.preprocess_dir = preprocess_dir
-
-    def __len__(self) -> int:
-        """ """
-        return len(self.reader.all_records)
-
-    def __getitem__(self, index: int) -> np.ndarray:
-        """ """
-        rec = self.reader.all_records[index]
-        data = self.load_preprocessed_data(rec)
-        siglen = data.shape[1]
-        rpeaks = self.reader.load_rpeaks(rec)
-        af_mask = self.reader.load_af_episodes(rec, fmt="mask")
-        forward_len = self.seglen - self.config[self.task].overlap_len
-        critical_forward_len = self.seglen - self.config[self.task].critical_overlap_len
-        critical_forward_len = [critical_forward_len // 4, critical_forward_len]
-
-        # skip those records that are too short
-        if siglen < self.seglen:
-            return
-
-        # find critical points
-        critical_points = np.where(np.diff(af_mask) != 0)[0]
-        critical_points = [
-            p
-            for p in critical_points
-            if critical_forward_len[1] <= p < siglen - critical_forward_len[1]
-        ]
-
-        segments = []
-
-        # ordinary segments with constant forward_len
-        for idx in range((siglen - self.seglen) // forward_len + 1):
-            start_idx = idx * forward_len
-            new_seg = self.__generate_segment(
-                rec=rec,
-                data=data,
-                start_idx=start_idx,
-            )
-            segments.append(new_seg)
-        # the tail segment
-        new_seg = self.__generate_segment(
-            rec=rec,
-            data=data,
-            end_idx=siglen,
-        )
-        segments.append(new_seg)
-
-        # special segments around critical_points with random forward_len in critical_forward_len
-        for cp in critical_points:
-            start_idx = max(
-                0,
-                cp
-                - self.seglen
-                + DEFAULTS.RNG_randint(
-                    critical_forward_len[0], critical_forward_len[1]
-                ),
-            )
-            while start_idx <= min(cp - critical_forward_len[1], siglen - self.seglen):
-                new_seg = self.__generate_segment(
-                    rec=rec,
-                    data=data,
-                    start_idx=start_idx,
-                )
-                segments.append(new_seg)
-                start_idx += DEFAULTS.RNG_randint(
-                    critical_forward_len[0], critical_forward_len[1]
-                )
-
-        return segments
-
-    def __generate_segment(
-        self,
-        rec: str,
-        data: np.ndarray,
-        start_idx: Optional[int] = None,
-        end_idx: Optional[int] = None,
-    ) -> CFG:
-        """
-
-        generate segment, with possible data augmentation
-
-        Parameter
-        ---------
-        rec: str,
-            filename of the record
-        data: ndarray,
-            the whole of (preprocessed) ECG record
-        start_idx: int, optional,
-            start index of the signal of `rec` for generating the segment
-        end_idx: int, optional,
-            end index of the signal of `rec` for generating the segment,
-            if `start_idx` is set, `end_idx` is ignored,
-            at least one of `start_idx` and `end_idx` should be set
-
-        Returns
-        -------
-        new_seg: dict,
-            segments (meta-)data, containing:
-            - data: values of the segment, with units in mV
-            - rpeaks: indices of rpeaks of the segment
-            - qrs_mask: mask of qrs complexes of the segment
-            - af_mask: mask of af episodes of the segment
-            - interval: interval ([start_idx, end_idx]) in the original ECG record of the segment
-
-        """
-        assert not all(
-            [start_idx is None, end_idx is None]
-        ), "at least one of `start_idx` and `end_idx` should be set"
-        siglen = data.shape[1]
-        # offline augmentations are done, including strech-or-compress, ...
-        if self.config.stretch_compress != 0:
-            sign = DEFAULTS.RNG_sample(self.config.stretch_compress_choices, 1)[0]
-            if sign != 0:
-                sc_ratio = self.config.stretch_compress
-                sc_ratio = (
-                    1 + (DEFAULTS.RNG.uniform(sc_ratio / 4, sc_ratio) * sign) / 100
-                )
-                sc_len = int(round(sc_ratio * self.seglen))
-                if start_idx is not None:
-                    end_idx = start_idx + sc_len
-                else:
-                    start_idx = end_idx - sc_len
-                if end_idx > siglen:
-                    end_idx = siglen
-                    start_idx = max(0, end_idx - sc_len)
-                    sc_ratio = (end_idx - start_idx) / self.seglen
-                aug_seg = data[..., start_idx:end_idx]
-                aug_seg = SS.resample(x=aug_seg, num=self.seglen, axis=1)
-            else:
-                if start_idx is not None:
-                    end_idx = start_idx + self.seglen
-                    if end_idx > siglen:
-                        end_idx = siglen
-                        start_idx = end_idx - self.seglen
-                else:
-                    start_idx = end_idx - self.seglen
-                    if start_idx < 0:
-                        start_idx = 0
-                        end_idx = self.seglen
-                # the segment of original signal, with no augmentation
-                aug_seg = data[..., start_idx:end_idx]
-                sc_ratio = 1
-        else:
-            if start_idx is not None:
-                end_idx = start_idx + self.seglen
-                if end_idx > siglen:
-                    end_idx = siglen
-                    start_idx = end_idx - self.seglen
-            else:
-                start_idx = end_idx - self.seglen
-                if start_idx < 0:
-                    start_idx = 0
-                    end_idx = self.seglen
-            aug_seg = data[..., start_idx:end_idx]
-            sc_ratio = 1
-        # adjust rpeaks
-        seg_rpeaks = self.reader.load_rpeaks(
-            rec=rec,
-            sampfrom=start_idx,
-            sampto=end_idx,
-            keep_original=False,
-        )
-        seg_rpeaks = [
-            int(round(r / sc_ratio))
-            for r in seg_rpeaks
-            if self.config.rpeaks_dist2border
-            <= r
-            < self.seglen - self.config.rpeaks_dist2border
-        ]
-        # generate qrs_mask from rpeaks
-        seg_qrs_mask = np.zeros((self.seglen,), dtype=int)
-        for r in seg_rpeaks:
-            seg_qrs_mask[
-                r - self.config.qrs_mask_bias : r + self.config.qrs_mask_bias
-            ] = 1
-        # adjust af_intervals
-        seg_af_intervals = self.reader.load_af_episodes(
-            rec=rec,
-            sampfrom=start_idx,
-            sampto=end_idx,
-            keep_original=False,
-            fmt="intervals",
-        )
-        seg_af_intervals = [
-            [int(round(itv[0] / sc_ratio)), int(round(itv[1] / sc_ratio))]
-            for itv in seg_af_intervals
-        ]
-        # generate af_mask from af_intervals
-        seg_af_mask = np.zeros((self.seglen,), dtype=int)
-        for itv in seg_af_intervals:
-            seg_af_mask[itv[0] : itv[1]] = 1
-
-        new_seg = CFG(
-            data=aug_seg,
-            rpeaks=seg_rpeaks,
-            qrs_mask=seg_qrs_mask,
-            af_mask=seg_af_mask,
-            interval=[start_idx, end_idx],
-        )
-        return new_seg
-
-    def load_preprocessed_data(self, rec: str) -> np.ndarray:
-        """
-
-        Parameters
-        ----------
-        rec: str,
-            filename of the record
-
-        Returns
-        -------
-        p_sig: ndarray,
-            the pre-computed processed ECG
-
-        """
-        preproc = self.allowed_preproc
-        suffix = _get_rec_suffix(preproc)
-        fp = self.preprocess_dir / f"{rec}-{suffix}.{self.segment_ext}"
-        if not fp.exists():
-            raise FileNotFoundError(
-                f"preprocess(es) \042{preproc}\042 not done for {rec} yet"
-            )
-        p_sig = loadmat(fp)["ecg"]
-        if p_sig.shape[0] != 2:
-            p_sig = p_sig.T
-        return p_sig
-
-    def extra_repr_keys(self) -> List[str]:
-        return [
-            "task",
-            "reader",
-            "seg_ppm",
-        ]
-
-
-def _get_rec_suffix(operations: List[str]) -> str:
-    """
-
-    Parameters
-    ----------
-    operations: list of str,
-        names of operations to perform (or has performed),
-
-    Returns
-    -------
-    suffix: str,
-        suffix of the filename of the preprocessed ecg signal
-
-    """
-    suffix = "-".join(sorted([item.lower() for item in operations]))
-    return suffix
+# `StandaloneSegmentSlicer` can be found in `benchmarks/train_hybrid_cpsc2021/dataset.py`
