@@ -47,7 +47,7 @@ LUDBTrainCfg.keep_checkpoint_max = 20
 LUDBTrainCfg.bias_thr = int(0.075 * LUDBTrainCfg.fs)  # TODO: renew this const
 # detected waves that are within `skip_dist` from two ends of the signal will be ignored,
 LUDBTrainCfg.skip_dist = int(0.5 * LUDBTrainCfg.fs)
-LUDBTrainCfg.torch_dtype = DEFAULTS.torch_dtype
+LUDBTrainCfg.torch_dtype = DEFAULTS.DTYPE.TORCH
 
 LUDBTrainCfg.fs = 500
 LUDBTrainCfg.train_ratio = 0.8
