@@ -2,19 +2,15 @@
 """
 
 import json
+import warnings
 from copy import deepcopy
 from random import randint, shuffle
-from typing import List, NoReturn, Optional, Sequence, Tuple
+from typing import List, Optional, Sequence, Tuple, Any
 
 import numpy as np
-
-try:
-    from tqdm.auto import tqdm
-except ModuleNotFoundError:
-    from tqdm import tqdm
-
 import torch
 from torch.utils.data.dataset import Dataset
+from tqdm.auto import tqdm
 
 from ...._preprocessors import PreprocManager
 from ....cfg import CFG
@@ -38,9 +34,9 @@ class LUDBDataset(ReprMixin, Dataset):
         config: CFG,
         training: bool = True,
         lazy: bool = False,
-    ) -> NoReturn:
+        **reader_kwargs: Any,
+    ) -> None:
         """
-
         Parameters
         ----------
         config: dict,
@@ -50,11 +46,18 @@ class LUDBDataset(ReprMixin, Dataset):
             if True, the training set will be loaded, otherwise the test set
         lazy: bool, default False,
             if True, the data will not be loaded immediately
+        reader_kwargs: dict,
+            keyword arguments for the data reader class
 
         """
         super().__init__()
         self.config = deepcopy(config)
-        self.reader = LR(db_dir=self.config.db_dir)
+        if reader_kwargs.pop("db_dir", None) is not None:
+            warnings.warn(
+                "db_dir is specified in both config and reader_kwargs", RuntimeWarning
+            )
+        self.reader = LR(db_dir=self.config.db_dir, **reader_kwargs)
+        self.config.db_dir = self.reader.db_dir
         self.training = training
         if self.config.torch_dtype == torch.float64:
             self.dtype = np.float64
@@ -120,11 +123,13 @@ class LUDBDataset(ReprMixin, Dataset):
 
         return signals, labels
 
-    def _load_all_data(self) -> NoReturn:
+    def _load_all_data(self) -> None:
         """ """
         self._signals, self._labels = [], []
 
-        with tqdm(self.fdr, total=len(self.fdr)) as bar:
+        with tqdm(
+            self.fdr, total=len(self.fdr), dynamic_ncols=True, mininterval=1.0
+        ) as bar:
             for signals, labels in bar:
                 self._signals.append(signals)
                 self._labels.append(labels)
@@ -146,7 +151,6 @@ class LUDBDataset(ReprMixin, Dataset):
         self, train_ratio: float = 0.8, force_recompute: bool = False
     ) -> List[str]:
         """
-
         Parameters
         ----------
         train_ratio: float, default 0.8,
@@ -167,14 +171,18 @@ class LUDBDataset(ReprMixin, Dataset):
         train_file = self.reader.db_dir / f"train_ratio_{_train_ratio}.json"
         test_file = self.reader.db_dir / f"test_ratio_{_test_ratio}.json"
 
+        if self.reader._subsample is not None:
+            force_recompute = True
+
         if force_recompute or not all([train_file.is_file(), test_file.is_file()]):
             all_records = deepcopy(self.reader.all_records)
             shuffle(all_records)
             split_idx = int(_train_ratio * len(all_records) / 100)
             train_set = all_records[:split_idx]
             test_set = all_records[split_idx:]
-            train_file.write_text(json.dumps(train_set, ensure_ascii=False))
-            test_file.write_text(json.dumps(test_set, ensure_ascii=False))
+            if self.reader._subsample is None:
+                train_file.write_text(json.dumps(train_set, ensure_ascii=False))
+                test_file.write_text(json.dumps(test_set, ensure_ascii=False))
         else:
             train_set = json.loads(train_file.read_text())
             test_set = json.loads(test_file.read_text())
@@ -203,7 +211,7 @@ class FastDataReader(ReprMixin, Dataset):
         records: Sequence[str],
         config: CFG,
         ppm: Optional[PreprocManager] = None,
-    ) -> NoReturn:
+    ) -> None:
         """ """
         self.reader = reader
         self.records = records
