@@ -6,19 +6,14 @@ import os
 import warnings
 from copy import deepcopy
 from pathlib import Path
-from typing import Dict, List, NoReturn, Optional, Sequence, Tuple, Union
+from typing import Dict, List, Optional, Sequence, Tuple, Union, Any
 
 import numpy as np
 from scipy import signal as SS
-
-try:
-    from tqdm.auto import tqdm
-except ModuleNotFoundError:
-    from tqdm import tqdm
-
-import torch
 from scipy.io import loadmat, savemat
+import torch
 from torch.utils.data.dataset import Dataset
+from tqdm.auto import tqdm
 
 from ...._preprocessors import PreprocManager
 from ....databases import MITDB as DR
@@ -46,10 +41,14 @@ class MITDBDataset(ReprMixin, Dataset):
     __name__ = "MITDBDataset"
 
     def __init__(
-        self, config: CFG, task: str, training: bool = True, lazy: bool = True
-    ) -> NoReturn:
+        self,
+        config: CFG,
+        task: str,
+        training: bool = True,
+        lazy: bool = True,
+        **reader_kwargs: Any,
+    ) -> None:
         """
-
         Parameters
         ----------
         config: dict,
@@ -58,18 +57,24 @@ class MITDBDataset(ReprMixin, Dataset):
             a simple example is:
             >>> config = deepcopy(MITDBDataset)
             >>> config.db_dir = "some/path/to/db"
-            >>> dataset = MITDBDataset(config, training=True, lazy=False)
+            >>> dataset = MITDBDataset(config, task="qrs_detection", training=True, lazy=False)
         training: bool, default True,
             if True, the training set will be loaded, otherwise the test set
         lazy: bool, default False,
-            if True, the data will not be loaded immediately,
+            if True, the data will not be loaded immediately
+        reader_kwargs: dict,
+            keyword arguments for the data reader class
 
         """
         super().__init__()
         self.config = deepcopy(config)
-        assert self.config.db_dir is not None, "db_dir must be specified"
-        self.config.db_dir = Path(self.config.db_dir)
-        self.reader = DR(db_dir=self.config.db_dir)
+        if reader_kwargs.pop("db_dir", None) is not None:
+            warnings.warn(
+                "db_dir is specified in both config and reader_kwargs", RuntimeWarning
+            )
+        self.reader = DR(db_dir=self.config.db_dir, **reader_kwargs)
+        # assert self.config.db_dir is not None, "db_dir must be specified"
+        self.config.db_dir = self.reader.db_dir
         if self.config.torch_dtype == torch.float64:
             self.dtype = np.float64
         else:
@@ -93,9 +98,9 @@ class MITDBDataset(ReprMixin, Dataset):
         # ppm_config.pop("normalize")
         seg_ppm_config = CFG(random=False)
         seg_ppm_config.update(deepcopy(self.config))
-        seg_ppm_config.pop("bandpass")
+        seg_ppm_config.pop("bandpass", None)
         self.ppm = PreprocManager.from_config(ppm_config)
-        self.ppm.rearrange(["bandpass", "baseline_remove", "normalize"])
+        # self.ppm.rearrange(["bandpass", "baseline_remove", "normalize"])
         self.seg_ppm = PreprocManager.from_config(seg_ppm_config)
 
         # create directories if needed
@@ -115,13 +120,12 @@ class MITDBDataset(ReprMixin, Dataset):
         self._all_masks = None
         self.__set_task(task, lazy=self.lazy)
 
-    def _load_all_data(self) -> NoReturn:
+    def _load_all_data(self) -> None:
         """ """
         self.__set_task(self.task, lazy=False)
 
-    def __set_task(self, task: str, lazy: bool = True) -> NoReturn:
+    def __set_task(self, task: str, lazy: bool = True) -> None:
         """
-
         Parameters
         ----------
         task: str,
@@ -153,8 +157,14 @@ class MITDBDataset(ReprMixin, Dataset):
         if self.task in ["beat_classification"]:
             # finished, tested
             self._all_data, self._all_labels = [], []
+            if self.lazy:
+                return
             with tqdm(
-                range(len(self.records)), desc="Loading data", unit="record"
+                range(len(self.records)),
+                desc="Loading data",
+                unit="record",
+                dynamic_ncols=True,
+                mininterval=1.0,
             ) as pbar:
                 for idx in pbar:
                     data = self.reader.load_data(self.records[idx])
@@ -207,15 +217,23 @@ class MITDBDataset(ReprMixin, Dataset):
                 self.segments_dirs,
                 self.segments,
                 self.segment_ext,
+                self.reader.rhythm_types_map,
             )
             if self.lazy:
                 return
             self._all_data, self._all_labels, self._all_masks = [], [], []
             if len(self.fdr) == 0:
                 warnings.warn(
-                    f"No data found for task {self.task}, slice the data first."
+                    f"No data found for task {self.task}, slice the data first.",
+                    RuntimeWarning,
                 )
-            with tqdm(range(len(self.fdr)), desc="Loading data", unit="record") as pbar:
+            with tqdm(
+                range(len(self.fdr)),
+                desc="Loading data",
+                unit="record",
+                dynamic_ncols=True,
+                mininterval=1.0,
+            ) as pbar:
                 for idx in pbar:
                     d, l, m = self.fdr[idx]
                     self._all_data.append(d)
@@ -249,11 +267,18 @@ class MITDBDataset(ReprMixin, Dataset):
                 self.rr_seq_dirs,
                 self.rr_seq,
                 self.rr_seq_ext,
+                self.reader.rhythm_types_map,
             )
             if self.lazy:
                 return
             self._all_data, self._all_labels, self._all_masks = [], [], []
-            with tqdm(range(len(self.fdr)), desc="Loading data", unit="record") as pbar:
+            with tqdm(
+                range(len(self.fdr)),
+                desc="Loading data",
+                unit="record",
+                dynamic_ncols=True,
+                mininterval=1.0,
+            ) as pbar:
                 for idx in pbar:
                     d, l, m = self.fdr[idx]
                     self._all_data.append(d)
@@ -267,11 +292,11 @@ class MITDBDataset(ReprMixin, Dataset):
                 f"data generator for task \042{self.task}\042 not implemented"
             )
 
-    def reset_task(self, task: str, lazy: bool = True) -> NoReturn:
+    def reset_task(self, task: str, lazy: bool = True) -> None:
         """ """
         self.__set_task(task, lazy)
 
-    def _ls_segments(self) -> NoReturn:
+    def _ls_segments(self) -> None:
         """list all the segments"""
         for item in ["data", "ann"]:
             self.segments_dirs[item] = CFG()
@@ -282,9 +307,10 @@ class MITDBDataset(ReprMixin, Dataset):
             self.__all_segments = json.loads(self.segments_json.read_text())
             # return
         print(
-            f"please allow the reader a few minutes to collect the segments from {self.segments_base_dir}..."
+            "please allow the reader a few minutes to collect "
+            f"the segments from {self.segments_base_dir}..."
         )
-        seg_filename_pattern = f"{self.segment_name_pattern}.{self.segment_ext}"
+        seg_filename_pattern = f"{self.segment_name_pattern}\\.{self.segment_ext}"
         self.__all_segments = CFG(
             {
                 rec: get_record_list_recursive3(
@@ -298,7 +324,7 @@ class MITDBDataset(ReprMixin, Dataset):
                 json.dumps(self.__all_segments, ensure_ascii=False)
             )
 
-    def _ls_rr_seq(self) -> NoReturn:
+    def _ls_rr_seq(self) -> None:
         """list all the rr sequences"""
         for rec in self.reader:
             self.rr_seq_dirs[rec] = self.rr_seq_base_dir / rec
@@ -307,9 +333,10 @@ class MITDBDataset(ReprMixin, Dataset):
             self.__all_rr_seq = json.loads(self.rr_seq_json.read_text())
             # return
         print(
-            f"please allow the reader a few minutes to collect the rr sequences from {self.rr_seq_base_dir}..."
+            "please allow the reader a few minutes to collect "
+            f"the rr sequences from {self.rr_seq_base_dir}..."
         )
-        rr_seq_filename_pattern = f"{self.rr_seq_name_pattern}.{self.rr_seq_ext}"
+        rr_seq_filename_pattern = f"{self.rr_seq_name_pattern}\\.{self.rr_seq_ext}"
         self.__all_rr_seq = CFG(
             {
                 rec: get_record_list_recursive3(
@@ -344,6 +371,8 @@ class MITDBDataset(ReprMixin, Dataset):
             return CFG()
 
     def __len__(self) -> int:
+        if self.task in ["beat_classification"]:
+            return len(self._all_data)
         return len(self.fdr)
 
     def __getitem__(self, index: int) -> Tuple[np.ndarray, ...]:
@@ -366,7 +395,6 @@ class MITDBDataset(ReprMixin, Dataset):
 
     def _get_seg_data_path(self, seg: str) -> Path:
         """
-
         Parameters
         ----------
         seg: str,
@@ -384,7 +412,6 @@ class MITDBDataset(ReprMixin, Dataset):
 
     def _get_seg_ann_path(self, seg: str) -> Path:
         """
-
         Parameters
         ----------
         seg: str,
@@ -402,7 +429,6 @@ class MITDBDataset(ReprMixin, Dataset):
 
     def _load_seg_data(self, seg: str) -> np.ndarray:
         """
-
         Parameters
         ----------
         seg: str,
@@ -420,7 +446,6 @@ class MITDBDataset(ReprMixin, Dataset):
 
     def _load_seg_ann(self, seg: str) -> dict:
         """
-
         Parameters
         ----------
         seg: str,
@@ -448,7 +473,6 @@ class MITDBDataset(ReprMixin, Dataset):
         self, seg: str, task: Optional[str] = None
     ) -> Union[np.ndarray, Dict[str, np.ndarray]]:
         """
-
         Parameters
         ----------
         seg: str,
@@ -486,7 +510,6 @@ class MITDBDataset(ReprMixin, Dataset):
 
     def _load_seg_seq_lab(self, seg: str, reduction: int) -> np.ndarray:
         """
-
         Parameters
         ----------
         seg: str,
@@ -519,7 +542,6 @@ class MITDBDataset(ReprMixin, Dataset):
 
     def _get_rr_seq_path(self, seq_name: str) -> Path:
         """
-
         Parameters
         ----------
         seq_name: str,
@@ -537,7 +559,6 @@ class MITDBDataset(ReprMixin, Dataset):
 
     def _load_rr_seq(self, seq_name: str) -> Dict[str, np.ndarray]:
         """
-
         Parameters
         ----------
         seq_name: str,
@@ -561,9 +582,8 @@ class MITDBDataset(ReprMixin, Dataset):
         rr_seq["interval"] = rr_seq["interval"].flatten()
         return rr_seq
 
-    def persistence(self, force_recompute: bool = False, verbose: int = 0) -> NoReturn:
+    def persistence(self, force_recompute: bool = False, verbose: int = 0) -> None:
         """
-
         make the dataset persistent w.r.t. the ratios in `self.config`
 
         Parameters
@@ -574,28 +594,36 @@ class MITDBDataset(ReprMixin, Dataset):
             print verbosity
 
         """
+        # TODO: consider whether preprocessing should be added
+        # if verbose >= 1:
+        #     print(" preprocessing data ".center(110, "#"))
+        # self._preprocess_data(
+        #     force_recompute=force_recompute,
+        #     verbose=verbose,
+        # )
+
+        original_task = self.task
+        original_lazy = self.lazy
+        self.__set_task("qrs_detection", lazy=True)
         if verbose >= 1:
-            print(" preprocessing data ".center("#", 110))
-        self._preprocess_data(
-            force_recompute=force_recompute,
-            verbose=verbose,
-        )
-        if verbose >= 1:
-            print("\n" + " slicing data into segments ".center("#", 110))
+            print("\n" + " slicing data into segments ".center(110, "#"))
         self._slice_data(
             force_recompute=force_recompute,
             verbose=verbose,
         )
+
+        self.__set_task("rr_lstm", lazy=True)
         if verbose >= 1:
-            print("\n" + " generating rr sequences ".center("#", 110))
+            print("\n" + " generating rr sequences ".center(110, "#"))
         self._slice_rr_seq(
             force_recompute=force_recompute,
             verbose=verbose,
         )
 
+        self.__set_task(original_task, lazy=original_lazy)
+
     def _get_rec_suffix(self, operations: List[str]) -> str:
         """
-
         Parameters
         ----------
         operations: list of str,
@@ -611,9 +639,8 @@ class MITDBDataset(ReprMixin, Dataset):
         suffix = "-".join(sorted([item.lower() for item in operations]))
         return suffix
 
-    def _slice_data(self, force_recompute: bool = False, verbose: int = 0) -> NoReturn:
+    def _slice_data(self, force_recompute: bool = False, verbose: int = 0) -> None:
         """
-
         slice all records into segments of length `self.seglen`,
         and perform data augmentations specified in `self.config`
 
@@ -639,6 +666,8 @@ class MITDBDataset(ReprMixin, Dataset):
             total=len(self.reader),
             desc="Slicing data",
             unit="record",
+            dynamic_ncols=True,
+            mininterval=1.0,
         ) as pbar:
             for idx, rec in pbar:
                 self._slice_one_record(
@@ -660,9 +689,8 @@ class MITDBDataset(ReprMixin, Dataset):
         force_recompute: bool = False,
         update_segments_json: bool = False,
         verbose: int = 0,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         slice one record into segments of length `self.seglen`,
         and perform data augmentations specified in `self.config`
 
@@ -716,11 +744,13 @@ class MITDBDataset(ReprMixin, Dataset):
         segments = []
 
         # ordinary segments with constant forward_len
-        print("Slicing ordinary segments with constant forward_len")
+        print("Slicing ordinary segments with constant `forward_len`")
         with tqdm(
             range((siglen - self.seglen) // forward_len + 1),
             desc=f"Slicing segments for record {rec}",
             unit="segment",
+            dynamic_ncols=True,
+            mininterval=1.0,
         ) as pbar:
             for idx in pbar:
                 start_idx = idx * forward_len
@@ -745,10 +775,15 @@ class MITDBDataset(ReprMixin, Dataset):
 
         # special segments around critical_points with random forward_len in critical_forward_len
         print(
-            "Slicing special segments around critical_points with random forward_len in critical_forward_len"
+            "Slicing special segments around `critical_points` "
+            "with random `forward_len` in `critical_forward_len`"
         )
         with tqdm(
-            critical_points, desc=f"Slicing segments for record {rec}", unit="segment"
+            critical_points,
+            desc=f"Slicing segments for record {rec}",
+            unit="segment",
+            dynamic_ncols=True,
+            mininterval=1.0,
         ) as pbar:
             for cp in pbar:
                 start_idx = max(
@@ -783,7 +818,6 @@ class MITDBDataset(ReprMixin, Dataset):
         end_idx: Optional[int] = None,
     ) -> CFG:
         """
-
         generate segment, with possible data augmentation
 
         Parameter
@@ -915,9 +949,8 @@ class MITDBDataset(ReprMixin, Dataset):
 
     def __save_segments(
         self, rec: str, segments: List[CFG], update_segments_json: bool = False
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         rec: str,
@@ -953,9 +986,8 @@ class MITDBDataset(ReprMixin, Dataset):
                 json.dumps(self.__all_segments, ensure_ascii=False)
             )
 
-    def _clear_cached_segments(self, recs: Optional[Sequence[str]] = None) -> NoReturn:
+    def _clear_cached_segments(self, recs: Optional[Sequence[str]] = None) -> None:
         """
-
         Parameters
         ----------
         recs: sequence of str, optional
@@ -984,11 +1016,8 @@ class MITDBDataset(ReprMixin, Dataset):
                         self.__all_segments[rec].remove(os.path.splitext(f)[0])
         self.segments = list_sum([self.__all_segments[rec] for rec in self.records])
 
-    def _slice_rr_seq(
-        self, force_recompute: bool = False, verbose: int = 0
-    ) -> NoReturn:
+    def _slice_rr_seq(self, force_recompute: bool = False, verbose: int = 0) -> None:
         """
-
         slice sequences of rr intervals into fixed length (sub)sequences
 
         Parameters
@@ -1007,6 +1036,8 @@ class MITDBDataset(ReprMixin, Dataset):
             total=len(self.reader),
             desc="Slicing rr_seq",
             unit="record",
+            dynamic_ncols=True,
+            mininterval=1.0,
         ) as pbar:
             for idx, rec in pbar:
                 self._slice_rr_seq_one_record(
@@ -1028,7 +1059,7 @@ class MITDBDataset(ReprMixin, Dataset):
         force_recompute: bool = False,
         update_rr_seq_json: bool = False,
         verbose: int = 0,
-    ) -> NoReturn:
+    ) -> None:
         """ """
         self.__assert_task(["rr_lstm"])
         rec_rr_seq = self.__all_rr_seq[rec]
@@ -1063,11 +1094,13 @@ class MITDBDataset(ReprMixin, Dataset):
         rr_seq = []
 
         # ordinary rr_seq with constant forward_len
-        print("Slicing ordinary rr_seq with constant forward_len")
+        print("Slicing ordinary rr_seq with constant `forward_len`")
         with tqdm(
             range((len(rr) - self.seglen) // forward_len + 1),
-            desc=f"Slicing rr_seq for record {rec}",
+            desc=f"Slicing rr_seq for record `{rec}`",
             unit="segment",
+            dynamic_ncols=True,
+            mininterval=1.0,
         ) as pbar:
             for idx in pbar:
                 start_idx = idx * forward_len
@@ -1096,10 +1129,15 @@ class MITDBDataset(ReprMixin, Dataset):
 
         # special rr_seq around critical_points with random forward_len in critical_forward_len
         print(
-            "Slicing special rr_seq around critical_points with random forward_len in critical_forward_len"
+            "Slicing special rr_seq around `critical_points` "
+            "with random `forward_len` in `critical_forward_len`"
         )
         with tqdm(
-            critical_points, desc=f"Slicing rr_seq for record {rec}", unit="segment"
+            critical_points,
+            desc=f"Slicing rr_seq for record {rec}",
+            unit="segment",
+            dynamic_ncols=True,
+            mininterval=1.0,
         ) as pbar:
             for cp in pbar:
                 start_idx = max(
@@ -1128,9 +1166,8 @@ class MITDBDataset(ReprMixin, Dataset):
 
     def __save_rr_seq(
         self, rec: str, rr_seq: List[CFG], update_rr_seq_json: bool = False
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         rec: str,
@@ -1154,9 +1191,8 @@ class MITDBDataset(ReprMixin, Dataset):
                 json.dumps(self.__all_rr_seq, ensure_ascii=False)
             )
 
-    def _clear_cached_rr_seq(self, recs: Optional[Sequence[str]] = None) -> NoReturn:
+    def _clear_cached_rr_seq(self, recs: Optional[Sequence[str]] = None) -> None:
         """
-
         Parameters
         ----------
         recs: sequence of str, optional
@@ -1177,7 +1213,6 @@ class MITDBDataset(ReprMixin, Dataset):
 
     def _get_rec_name(self, seg_or_rr: str) -> str:
         """
-
         Parameters
         ----------
         seg_or_rr: str,
@@ -1194,7 +1229,6 @@ class MITDBDataset(ReprMixin, Dataset):
 
     def _train_test_split(self, task: str) -> Dict[str, List[str]]:
         """
-
         do train test split,
         it is ensured that both the train and the test set contain all classes
 
@@ -1214,11 +1248,13 @@ class MITDBDataset(ReprMixin, Dataset):
             "beat_classification",
             "qrs_detection",
         ]:
-            test_set = ["101", "102", "108", "114", "207", "223"]
-            train_set = [rec for rec in self.reader if rec not in test_set]
+            full_test_set = ["101", "102", "108", "114", "207", "223"]
+            train_set = [rec for rec in self.reader if rec not in full_test_set]
+            test_set = [rec for rec in self.reader if rec in full_test_set]
         else:  # rhythm segmentation, af event, rr_lstm
-            test_set = ["106", "114", "124", "202", "217", "232"]
-            train_set = [rec for rec in self.reader if rec not in test_set]
+            full_test_set = ["106", "114", "124", "202", "217", "232"]
+            train_set = [rec for rec in self.reader if rec not in full_test_set]
+            test_set = [rec for rec in self.reader if rec in full_test_set]
 
         split_res = CFG(
             {
@@ -1228,15 +1264,14 @@ class MITDBDataset(ReprMixin, Dataset):
         )
         return split_res
 
-    def __assert_task(self, tasks: List[str]) -> NoReturn:
+    def __assert_task(self, tasks: List[str]) -> None:
         """ """
         assert (
             self.task in tasks
         ), f"DO NOT call this method when the current task is {self.task}. Switch task using `reset_task`"
 
-    def plot_seg(self, seg: str, ticks_granularity: int = 0) -> NoReturn:
+    def plot_seg(self, seg: str, ticks_granularity: int = 0) -> None:
         """
-
         Parameters
         ----------
         seg: str,
@@ -1246,12 +1281,9 @@ class MITDBDataset(ReprMixin, Dataset):
             0 (no ticks) --> 1 (major ticks) --> 2 (major + minor ticks)
 
         """
-        raise NotImplementedError
         seg_data = self._load_seg_data(seg)
-        print(f"seg_data.shape = {seg_data.shape}")
         seg_ann = self._load_seg_ann(seg)
         seg_ann["rhythm_intervals"] = mask_to_intervals(seg_ann["rhythm_mask"], vals=1)
-        print(f"seg_ann = {seg_ann}")
         rec_name = self._get_rec_name(seg)
         self.reader.plot(
             rec=rec_name,  # unnecessary indeed
@@ -1279,7 +1311,8 @@ class FastDataReader(ReprMixin, Dataset):
         file_dirs: dict,
         files: List[str],
         file_ext: str,
-    ) -> NoReturn:
+        rhythm_types_map: dict,
+    ) -> None:
         """ """
         self.config = config
         self.task = task
@@ -1287,6 +1320,7 @@ class FastDataReader(ReprMixin, Dataset):
         self.file_dirs = file_dirs
         self.files = files
         self.file_ext = file_ext
+        self.rhythm_types_map = rhythm_types_map
 
         self.seglen = self.config[self.task].input_len
         self.n_classes = len(self.config[task].classes)
@@ -1352,6 +1386,13 @@ class FastDataReader(ReprMixin, Dataset):
             rr_seq = loadmat(str(rr_seq_path))
             rr_seq["rr"] = rr_seq["rr"].reshape((self.seglen, 1))
             rr_seq["label"] = rr_seq["label"].reshape((self.seglen, self.n_classes))
+            # map values of `rr_seq["label"]` to 0, 1 according to `self.reader.rhythm_types_map`
+            rr_seq["label"][
+                np.where(rr_seq["label"] == self.rhythm_types_map["AFIB"])
+            ] = 1
+            rr_seq["label"][
+                np.where(rr_seq["label"] != self.rhythm_types_map["AFIB"])
+            ] = 0
             weight_mask = generate_weight_mask(
                 target_mask=rr_seq["label"].squeeze(-1),
                 fg_weight=2,
@@ -1380,7 +1421,6 @@ class FastDataReader(ReprMixin, Dataset):
 
 def _get_rec_suffix(operations: List[str]) -> str:
     """
-
     Parameters
     ----------
     operations: list of str,
