@@ -1,34 +1,32 @@
 # -*- coding: utf-8 -*-
 """
+MIT-BIH Atrial Fibrillation Database
 """
 
 import math
 from copy import deepcopy
-from numbers import Real
 from pathlib import Path
-from typing import Any, Dict, List, NoReturn, Optional, Sequence, Union
+from typing import Any, Dict, List, Optional, Sequence, Union
 
 import numpy as np
 import wfdb
-from scipy.signal import resample_poly
 
 from ...cfg import CFG
 from ...utils.misc import add_docstring, get_record_list_recursive
 from ...utils.utils_interval import generalized_intervals_intersection
-from ..base import DEFAULT_FIG_SIZE_PER_SEC, PhysioNetDataBase
+from ..base import DEFAULT_FIG_SIZE_PER_SEC, PhysioNetDataBase, DataBaseInfo
+
 
 __all__ = [
     "AFDB",
 ]
 
 
-class AFDB(PhysioNetDataBase):
-    """
-
+_AFDB_INFO = DataBaseInfo(
+    title="""
     MIT-BIH Atrial Fibrillation Database
-
-    ABOUT afdb
-    ----------
+    """,
+    about="""
     1. contains 25 long-term (each 10 hours) ECG recordings of human subjects with atrial fibrillation (mostly paroxysmal)
     2. 23 records out of 25 include the two ECG signals, the left 2 records 00735 and 03665 are represented only by the rhythm (.atr) and unaudited beat (.qrs) annotation files
     3. signals are sampled at 250 samples per second with 12-bit resolution over a range of ±10 millivolts, with a typical recording bandwidth of approximately 0.1 Hz to 40 Hz
@@ -38,36 +36,39 @@ class AFDB(PhysioNetDataBase):
         - J:     AV junctional rhythm
         - N:     all other rhythms
     5. rhythm annotations almost all start with "(N", except for 4 which start with '(AFIB', which are all within 1 second (250 samples)
-
-    NOTE
-    ----
+    """,
+    note="""
     1. beat annotation files (.qrs files) were prepared using an automated detector and have NOT been corrected manually
     2. for some records, manually corrected beat annotation files (.qrsc files) are available
     3. one should never use wfdb.rdann with arguments `sampfrom`, since one has to know the `aux_note` (with values in ["(N", "(J", "(AFL", "(AFIB"]) before the index at `sampfrom`
+    """,
+    usage=[
+        "Atrial fibrillation (AF) detection",
+    ],
+    references=[
+        "https://physionet.org/content/afdb/1.0.0/",
+        "Moody GB, Mark RG. A new method for detecting atrial fibrillation using R-R intervals. Computers in Cardiology. 10:227-230 (1983).",
+    ],
+    doi=[
+        "10.13026/C2MW2D",
+    ],
+)
 
-    ISSUES
-    ------
 
-    Usage
-    -----
-    1. AF detection
+@add_docstring(_AFDB_INFO.format_database_docstring())
+class AFDB(PhysioNetDataBase):
+    """ """
 
-    References
-    ----------
-    1. <a name="ref1"></a> https://physionet.org/content/afdb/1.0.0/
-    2. <a name="ref2"></a> Moody GB, Mark RG. A new method for detecting atrial fibrillation using R-R intervals. Computers in Cardiology. 10:227-230 (1983).
-
-    """
+    __name__ = "AFDB"
 
     def __init__(
         self,
         db_dir: Optional[Union[str, Path]] = None,
         working_dir: Optional[str] = None,
-        verbose: int = 2,
+        verbose: int = 1,
         **kwargs: Any,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         db_dir: str or Path, optional,
@@ -75,7 +76,7 @@ class AFDB(PhysioNetDataBase):
             if not specified, data will be fetched from Physionet
         working_dir: str, optional,
             working directory, to store intermediate files and log file
-        verbose: int, default 2,
+        verbose: int, default 1
             log verbosity
         kwargs: auxilliary key word arguments
 
@@ -98,11 +99,9 @@ class AFDB(PhysioNetDataBase):
             "ECG2",
         ]
 
-        self._ls_rec()
         self.special_records = ["00735", "03665"]
-        self.qrsc_records = get_record_list_recursive(
-            self.db_dir, self.manual_beat_ann_ext
-        )
+        self.qrsc_records = None
+        self._ls_rec()
 
         self.class_map = CFG(AFIB=1, AFL=2, J=3, N=0)  # an extra isoelectric
         self.palette = kwargs.get("palette", None)
@@ -115,70 +114,25 @@ class AFDB(PhysioNetDataBase):
                 qrs="green",
             )
 
-    def load_data(
-        self,
-        rec: Union[str, int],
-        leads: Optional[Union[str, List[str]]] = None,
-        sampfrom: Optional[int] = None,
-        sampto: Optional[int] = None,
-        data_format: str = "channel_first",
-        units: str = "mV",
-        fs: Optional[Real] = None,
-    ) -> np.ndarray:
+    def _ls_rec(self, local: bool = True) -> None:
         """
-        load physical (converted from digital) ecg data,
-        which is more understandable for humans
+        find all records (relative path without file extension),
+        and save into `self._all_records` for further use
 
         Parameters
         ----------
-        rec: str or int,
-            name or index of the record
-        leads: str or list of str, optional,
-            the leads to load
-        sampfrom: int, optional,
-            start index of the data to be loaded
-        sampto: int, optional,
-            end index of the data to be loaded
-        data_format: str, default "channel_first",
-            format of the ecg data,
-            "channel_last" (alias "lead_last"), or
-            "channel_first" (alias "lead_first")
-        units: str, default "mV",
-            units of the output signal, can also be "μV", with an alias of "uV"
-        fs: real number, optional,
-            if not None, the loaded data will be resampled to this frequency
-
-        Returns
-        -------
-        data: ndarray,
-            the ecg data
+        local: bool, default True,
+            if True, read from local storage, prior to using `wfdb.get_record_list`
 
         """
-        if isinstance(rec, int):
-            rec = self[rec]
-        fp = str(self.db_dir / rec)
-        if not leads:
-            _leads = self.all_leads
-        elif isinstance(leads, str):
-            _leads = [leads]
-        else:
-            _leads = leads
-        assert set(_leads).issubset(self.all_leads)
-        # p_signal in the format of "lead_last", and in units "mV"
-        data = wfdb.rdrecord(
-            fp,
-            sampfrom=sampfrom or 0,
-            sampto=sampto,
-            physical=True,
-            channel_names=_leads,
-        ).p_signal
-        if units.lower() in ["μv", "uv"]:
-            data = 1000 * data
-        if fs is not None and fs != self.fs:
-            data = resample_poly(data, fs, self.fs, axis=0)
-        if data_format.lower() in ["channel_first", "lead_first"]:
-            data = data.T
-        return data
+        super()._ls_rec(local=local)
+        self._all_records = [
+            rec for rec in self._all_records if rec not in self.special_records
+        ]
+        self.qrsc_records = get_record_list_recursive(
+            self.db_dir, self.manual_beat_ann_ext, relative=False
+        )
+        self.qrsc_records = [Path(rec).stem for rec in self.qrsc_records]
 
     def load_ann(
         self,
@@ -193,8 +147,8 @@ class AFDB(PhysioNetDataBase):
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            record name or index of the record in `self.all_records`
         sampfrom: int, optional,
             start index of the annotations to be loaded
         sampto: int, optional,
@@ -212,7 +166,7 @@ class AFDB(PhysioNetDataBase):
             the annotations in the format of intervals, or in the format of mask
 
         """
-        fp = str(self.db_dir / rec)
+        fp = str(self.get_absolute_path(rec))
         wfdb_ann = wfdb.rdann(fp, extension=self.ann_ext)
         header = wfdb.rdheader(fp)
         sig_len = header.sig_len
@@ -267,7 +221,7 @@ class AFDB(PhysioNetDataBase):
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         sampfrom: int, optional,
             start index of the annotations to be loaded
         sampto: int, optional,
@@ -287,7 +241,7 @@ class AFDB(PhysioNetDataBase):
         """
         if isinstance(rec, int):
             rec = self[rec]
-        fp = str(self.db_dir / rec)
+        fp = str(self.get_absolute_path(rec))
         if use_manual and rec in self.qrsc_records:
             ext = self.manual_beat_ann_ext
         else:
@@ -324,14 +278,13 @@ class AFDB(PhysioNetDataBase):
         ann: Optional[Dict[str, np.ndarray]] = None,
         rpeak_inds: Optional[Union[Sequence[int], np.ndarray]] = None,
         ticks_granularity: int = 0,
-        leads: Optional[Union[str, List[str]]] = None,
+        leads: Optional[Union[str, int, List[str], List[int]]] = None,
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
         same_range: bool = False,
         **kwargs: Any,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         plot the signals of a record or external signals (units in μV),
         with metadata (fs, labels, tranche, etc.),
         possibly also along with wave delineations
@@ -339,7 +292,7 @@ class AFDB(PhysioNetDataBase):
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         data: ndarray, optional,
             (2-lead) ecg signal to plot,
             should be of the format "channel_first", and compatible with `leads`
@@ -356,7 +309,7 @@ class AFDB(PhysioNetDataBase):
         ticks_granularity: int, default 0,
             the granularity to plot axis ticks, the higher the more,
             0 (no ticks) --> 1 (major ticks) --> 2 (major + minor ticks)
-        leads: str or list of str, optional,
+        leads: str or int or list of str or list of int, optional,
             the leads to plot
         sampfrom: int, optional,
             start index of the data to plot
@@ -378,11 +331,16 @@ class AFDB(PhysioNetDataBase):
             _leads = self.all_leads
         elif isinstance(leads, str):
             _leads = [leads]
+        elif isinstance(leads, int):
+            _leads = [self.all_leads[leads]]
         else:
             _leads = leads
-        assert all([ld in self.all_leads for ld in _leads])
+        assert all([ld in self.all_leads for ld in _leads]) or set(_leads) <= {0, 1}
 
-        lead_indices = [self.all_leads.index(ld) for ld in _leads]
+        try:
+            lead_indices = [self.all_leads.index(ld) for ld in _leads]
+        except ValueError:
+            lead_indices = _leads
         if data is None:
             _data = self.load_data(
                 rec,
@@ -394,7 +352,7 @@ class AFDB(PhysioNetDataBase):
             )
         else:
             units = self._auto_infer_units(data)
-            print(f"input data is auto detected to have units in {units}")
+            self.logger.info(f"input data is auto detected to have units in `{units}`")
             if units.lower() == "mv":
                 _data = 1000 * data
             else:
@@ -507,3 +465,7 @@ class AFDB(PhysioNetDataBase):
                 axes[idx].set_ylabel("Voltage [μV]")
             plt.subplots_adjust(hspace=0.2)
             plt.show()
+
+    @property
+    def database_info(self) -> DataBaseInfo:
+        return _AFDB_INFO
