@@ -1,76 +1,79 @@
 # -*- coding: utf-8 -*-
 """
+AF Classification from a Short Single Lead ECG Recording
+-- The PhysioNet Computing in Cardiology Challenge 2017
 """
 
 import math
-import zipfile
+import time
 import warnings
 from pathlib import Path
-from typing import Any, NoReturn, Optional, Sequence, Union, List
+from typing import Any, Optional, Sequence, Union, List
 
 import numpy as np
 import pandas as pd
-import wfdb
+import wfdb  # noqa: F401
+
+from ...cfg import DEFAULTS
+from ...utils.misc import get_record_list_recursive3, add_docstring
+from ..base import DEFAULT_FIG_SIZE_PER_SEC, PhysioNetDataBase, DataBaseInfo
 
-from ...utils.misc import get_record_list_recursive3
-from ..base import DEFAULT_FIG_SIZE_PER_SEC, PhysioNetDataBase
 
 __all__ = [
     "CINC2017",
 ]
 
 
-class CINC2017(PhysioNetDataBase):
-    """
-
+_CINC2017_INFO = DataBaseInfo(
+    title="""
     AF Classification from a Short Single Lead ECG Recording
-    - The PhysioNet Computing in Cardiology Challenge 2017
-
-    ABOUT CINC2017
-    --------------
+    -- The PhysioNet Computing in Cardiology Challenge 2017
+    """,
+    about="""
     1. training set contains 8,528 single lead ECG recordings lasting from 9 s to just over 60 s, and the test set contains 3,658 ECG recordings of similar lengths
     2. records are of frequency 300 Hz and have been band pass filtered
     3. data distribution:
-        Type	        	                    Time length (s)
-                        # recording     Mean	SD	    Max	    Median	Min
-        Normal	        5154	        31.9	10.0	61.0	30	    9.0
-        AF              771             31.6	12.5	60	    30	    10.0
-        Other rhythm	2557	        34.1	11.8	60.9	30	    9.1
-        Noisy	        46	            27.1	9.0	    60	    30	    10.2
-        Total	        8528	        32.5	10.9	61.0	30	    9.0
+        Type            # recording             Time length (s)
+                                        Mean    SD      Max     Median  Min
+        Normal          5154            31.9    10.0    61.0    30      9.0
+        AF              771             31.6    12.5    60      30      10.0
+        Other rhythm    2557            34.1    11.8    60.9    30      9.1
+        Noisy           46              27.1    9.0     60      30      10.2
+        Total           8528            32.5    10.9    61.0    30      9.0
+    """,
+    usage=[
+        "Atrial fibrillation (AF) detection",
+    ],
+    references=[
+        "https://physionet.org/content/challenge-2017/1.0.0/",
+    ],
+    doi=[
+        "10.22489/CinC.2017.065-469",
+    ],
+)
 
-    NOTE
-    ----
 
-    ISSUES
-    ------
+@add_docstring(_CINC2017_INFO.format_database_docstring())
+class CINC2017(PhysioNetDataBase):
+    """ """
 
-    Usage
-    -----
-    1. atrial fibrillation (AF) detection
-
-    References
-    ----------
-    1. <a name="ref1"></a> https://physionet.org/content/challenge-2017/1.0.0/
-
-    """
+    __name__ = "CINC2017"
 
     def __init__(
         self,
-        db_dir: Union[str, Path],
+        db_dir: Optional[Union[str, Path]] = None,
         working_dir: Optional[Union[str, Path]] = None,
-        verbose: int = 2,
+        verbose: int = 1,
         **kwargs: Any,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
-        db_dir: str or Path,
+        db_dir: str or Path, optional,
             storage path of the database
         working_dir: str, optional,
             working directory, to store intermediate files and log file
-        verbose: int, default 2,
+        verbose: int, default 1
             log verbosity
         kwargs: auxilliary key word arguments
 
@@ -82,6 +85,7 @@ class CINC2017(PhysioNetDataBase):
             verbose=verbose,
             **kwargs,
         )
+
         self.fs = 300
 
         self.rec_ext = "mat"
@@ -91,7 +95,6 @@ class CINC2017(PhysioNetDataBase):
         self._df_ann = pd.DataFrame()
         self._df_ann_ori = pd.DataFrame()
         self._all_ann = []
-        self.data_dir = None
         self._ls_rec()
 
         self.d_ann_names = {
@@ -107,45 +110,89 @@ class CINC2017(PhysioNetDataBase):
             "~": "blue",
         }
 
-        self._url_compressed = (
-            "https://physionet.org/static/published-projects/challenge-2017/"
-            "af-classification-from-a-short-single-lead-ecg-recording-"
-            "the-physionetcomputing-in-cardiology-challenge-2017-1.0.0.zip"
-        )
+        # self._url_compressed = (
+        #     "https://physionet.org/static/published-projects/challenge-2017/"
+        #     "af-classification-from-a-short-single-lead-ecg-recording-"
+        #     "the-physionetcomputing-in-cardiology-challenge-2017-1.0.0.zip"
+        # )
+        self._url_compressed = self.get_file_download_url("training2017.zip")
 
-    def _ls_rec(self) -> NoReturn:
+    def _ls_rec(self) -> None:
         """ """
-        fp = self.db_dir / "RECORDS"
-        if fp.exists():
-            self._all_records = fp.read_text().splitlines()
-            # return
-        self._all_records = get_record_list_recursive3(
-            db_dir=str(self.db_dir), rec_patterns=f"A[\\d]{{5}}.{self.rec_ext}"
-        )
-        parent_dir = set([str(Path(item).parent) for item in self.all_records])
-        if len(parent_dir) > 1:
-            raise ValueError("all records should be in the same directory")
-        self.data_dir = self.db_dir / parent_dir.pop()
-        self._all_records = [Path(item).name for item in self.all_records]
-        # fp.write_text("\n".join(self._all_records) + "\n")
-        if len(self._all_records) == 0:
+        record_list_fp = self.db_dir / "RECORDS"
+        self._df_records = pd.DataFrame()
+        if record_list_fp.is_file():
+            self._df_records["record"] = [
+                item
+                for item in record_list_fp.read_text().splitlines()
+                if len(item) > 0
+            ]
+            if len(self._df_records) > 0:
+                if self._subsample is not None:
+                    size = min(
+                        len(self._df_records),
+                        max(1, int(round(self._subsample * len(self._df_records)))),
+                    )
+                    self._df_records = self._df_records.sample(
+                        n=size, random_state=DEFAULTS.SEED, replace=False
+                    )
+                self._df_records["path"] = self._df_records["record"].apply(
+                    lambda x: (self.db_dir / x).resolve()
+                )
+                self._df_records = self._df_records[
+                    self._df_records["path"].apply(lambda x: x.is_file())
+                ]
+                self._df_records["record"] = self._df_records["path"].apply(
+                    lambda x: x.name
+                )
+
+        if len(self._df_records) == 0:
+            self.logger.info(
+                "Please wait patiently to let the reader find "
+                "all records of the database from local storage..."
+            )
+            start = time.time()
+            self._df_records["path"] = get_record_list_recursive3(
+                db_dir=str(self.db_dir),
+                rec_patterns=f"A[\\d]{{5}}\\.{self.rec_ext}",
+                relative=False,
+            )
+            if self._subsample is not None:
+                size = min(
+                    len(self._df_records),
+                    max(1, int(round(self._subsample * len(self._df_records)))),
+                )
+                self._df_records = self._df_records.sample(
+                    n=size, random_state=DEFAULTS.SEED, replace=False
+                )
+            self._df_records["path"] = self._df_records["path"].apply(lambda x: Path(x))
+            self.logger.info(f"Done in {time.time() - start:.3f} seconds!")
+            self._df_records["record"] = self._df_records["path"].apply(
+                lambda x: x.name
+            )
+        self._df_records.set_index("record", inplace=True)
+        self._all_records = self._df_records.index.values.tolist()
+
+        ann_file = list(self.db_dir.rglob("REFERENCE.csv"))
+        if len(ann_file) > 0:
+            self._df_ann = pd.read_csv(ann_file[0], header=None)
+            self._df_ann.columns = ["rec", "ann"]
+        else:
+            self._df_ann = pd.DataFrame(columns=["rec", "ann"])
             warnings.warn(
-                f"No record found in {self.db_dir}. "
-                "Perhaps the user should call the `download` method to download the database first."
+                "Cannot find the annotation file `REFERENCE.csv`!",
+                RuntimeWarning,
+            )
+        ann_file = list(self.db_dir.rglob("REFERENCE-original.csv"))
+        if len(ann_file) > 0:
+            self._df_ann_ori = pd.read_csv(ann_file[0], header=None)
+            self._df_ann_ori.columns = ["rec", "ann"]
+        else:
+            self._df_ann_ori = pd.DataFrame(columns=["rec", "ann"])
+            warnings.warn(
+                "Cannot find the annotation file `REFERENCE-original.csv`!",
+                RuntimeWarning,
             )
-            return
-        self._df_ann = pd.read_csv(self.data_dir / "REFERENCE.csv", header=None)
-        self._df_ann.columns = [
-            "rec",
-            "ann",
-        ]
-        self._df_ann_ori = pd.read_csv(
-            self.data_dir / "REFERENCE-original.csv", header=None
-        )
-        self._df_ann_ori.columns = [
-            "rec",
-            "ann",
-        ]
         # ["N", "A", "O", "~"]
         self._all_ann = list(
             set(
@@ -154,71 +201,16 @@ class CINC2017(PhysioNetDataBase):
             )
         )
 
-    def load_data(
-        self,
-        rec: Union[str, int],
-        data_format: str = "channel_first",
-        units: str = "mV",
-    ) -> np.ndarray:
-        """
-
-        Parameters
-        ----------
-        rec: str or int,
-            name or index of the record
-        data_format: str, default "channel_first",
-            format of the ecg data, case insensitive, can be
-            "channel_last" (alias "lead_last"), or
-            "channel_first" (alias "lead_first"), or
-            "flat" (of dimension 1, without channel dimension)
-        units: str, default "mV",
-            units of the output signal, can also be "μV", with an alias of "uV"
-
-        Returns
-        -------
-        data: ndarray,
-            data loaded from `rec`, with given units and format
-        """
-        if isinstance(rec, int):
-            rec = self[rec]
-        assert data_format.lower() in [
-            "channel_first",
-            "lead_first",
-            "channel_last",
-            "lead_last",
-            "flat",
-        ]
-        assert units.lower() in [
-            "mv",
-            "uv",
-            "μv",
-        ]
-        wr = wfdb.rdrecord(str(self.data_dir / rec))
-        data = wr.p_signal
-
-        if wr.units[0].lower() == units.lower():
-            pass
-        elif wr.units[0].lower() in ["uv", "μv"] and units.lower() == "mv":
-            data = data / 1000
-        elif units.lower() in ["uv", "μv"] and wr.units[0].lower() == "mv":
-            data = data * 1000
-
-        data = data.squeeze()
-        if data_format.lower() in ["channel_first", "lead_first"]:
-            data = data[np.newaxis, ...]
-        elif data_format.lower() in ["channel_last", "lead_last"]:
-            data = data[..., np.newaxis]
-        return data
-
     def load_ann(
         self, rec: Union[str, int], original: bool = False, ann_format: str = "a"
     ) -> str:
         """
+        load the annotation of the record `rec`
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         original: bool, default False,
             if True, load annotations from the file `REFERENCE-original.csv`,
             otherwise from `REFERENCE.csv`
@@ -253,13 +245,12 @@ class CINC2017(PhysioNetDataBase):
         ann: Optional[str] = None,
         ticks_granularity: int = 0,
         rpeak_inds: Optional[Union[Sequence[int], np.ndarray]] = None,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         data: ndarray, optional,
             ecg signal to plot,
             if given, data of `rec` will not be used,
@@ -334,13 +325,7 @@ class CINC2017(PhysioNetDataBase):
             ax.set_ylabel("Voltage [μV]")
             plt.show()
 
-    def download(self, compressed: bool = False) -> NoReturn:
-        """ """
-        super().download(compressed=compressed)
-        if compressed:
-            with zipfile.ZipFile(str(self.db_dir / "training2017.zip")) as zip_ref:
-                zip_ref.extractall()
-
+    @property
     def _validation_set(self) -> List[str]:
         """
         the validation set specified at https://physionet.org/content/challenge-2017/1.0.0/
@@ -377,3 +362,7 @@ class CINC2017(PhysioNetDataBase):
             "A01772,A01833,A02168,A02372,A02772,A02785,A02833,A03549,A03738,A04086,"
             "A04137,A04170,A04186,A04216,A04282,A04452,A04522,A04701,A04735,A04805"
         ).split(",")
+
+    @property
+    def database_info(self) -> DataBaseInfo:
+        return _CINC2017_INFO
