@@ -1,19 +1,23 @@
 # -*- coding: utf-8 -*-
 """
+Lobachevsky University Electrocardiography Database
 """
 
+from copy import deepcopy
 from numbers import Real
 from pathlib import Path
-from typing import Any, Dict, List, NoReturn, Optional, Sequence, Union
+from typing import Any, Dict, List, Optional, Sequence, Union
 
 import numpy as np
 import pandas as pd
 import wfdb
-from scipy.signal import resample_poly
 
 from ...cfg import CFG, DEFAULTS
+from ...utils import EAK
+from ...utils.misc import add_docstring
 from ...utils.utils_data import ECGWaveForm, masks_to_waveforms
-from ..base import PhysioNetDataBase
+from ..base import PhysioNetDataBase, DataBaseInfo
+
 
 __all__ = [
     "LUDB",
@@ -21,151 +25,160 @@ __all__ = [
 ]
 
 
-class LUDB(PhysioNetDataBase):
-    """finished, to be improved,
-
+_LUDB_INFO = DataBaseInfo(
+    title="""
     Lobachevsky University Electrocardiography Database
-
-    ABOUT ludb
-    ----------
+    """,
+    about="""
     1. consist of 200 10-second conventional 12-lead (i, ii, iii, avr, avl, avf, v1, v2, v3, v4, v5, v6) ECG signal records, with sampling frequency 500 Hz
     2. boundaries of P, T waves and QRS complexes were manually annotated by cardiologists, and with the corresponding diagnosis
     3. annotated are 16797 P waves, 21966 QRS complexes, 19666 T waves (in total, 58429 annotated waves)
     4. distributions of data:
         4.1. rhythm distribution:
-            Rhythms	                        Number of ECGs
-            Sinus rhythm	                143
-            Sinus tachycardia	            4
-            Sinus bradycardia	            25
-            Sinus arrhythmia	            8
-            Irregular sinus rhythm	        2
-            Abnormal rhythm	                19
+            | Rhythms                 | Number of ECGs |
+            | ----------------------- | -------------- |
+            | Sinus rhythm            | 143            |
+            | Sinus tachycardia       | 4              |
+            | Sinus bradycardia       | 25             |
+            | Sinus arrhythmia        | 8              |
+            | Irregular sinus rhythm  | 2              |
+            | Abnormal rhythm         | 19             |
         4.2. electrical axis distribution:
-            Heart electric axis	            Number of ECGs
-            Normal	                        75
-            Left axis deviation (LAD)	    66
-            Vertical	                    26
-            Horizontal	                    20
-            Right axis deviation (RAD)	    3
-            Undetermined	                10
+            | Heart electric axis        | Number of ECGs |
+            | -------------------------- | -------------- |
+            | Normal                     | 75             |
+            | Left axis deviation (LAD)  | 66             |
+            | Vertical                   | 26             |
+            | Horizontal                 | 20             |
+            | Right axis deviation (RAD) | 3              |
+            | Undetermined               | 10             |
         4.3. distribution of records with conduction abnomalities (totally 79):
-            Conduction abnormalities	                        Number of ECGs
-            Sinoatrial blockade, undetermined	                1
-            I degree AV block	                                10
-            III degree AV-block	                                5
-            Incomplete right bundle branch block	            29
-            Incomplete left bundle branch block	                6
-            Left anterior hemiblock	                            16
-            Complete right bundle branch block	                4
-            Complete left bundle branch block	                4
-            Non-specific intravintricular conduction delay	    4
+            | Conduction abnormalities                       | Number of ECGs |
+            | ---------------------------------------------- | -------------- |
+            | Sinoatrial blockade, undetermined              | 1              |
+            | I degree AV block                              | 10             |
+            | III degree AV-block                            | 5              |
+            | Incomplete right bundle branch block           | 29             |
+            | Incomplete left bundle branch block            | 6              |
+            | Left anterior hemiblock                        | 16             |
+            | Complete right bundle branch block             | 4              |
+            | Complete left bundle branch block              | 4              |
+            | Non-specific intravintricular conduction delay | 4              |
         4.4. distribution of records with extrasystoles (totally 35):
-            Extrasystoles	                                                    Number of ECGs
-            Atrial extrasystole, undetermined	                                2
-            Atrial extrasystole, low atrial	                                    1
-            Atrial extrasystole, left atrial	                                2
-            Atrial extrasystole, SA-nodal extrasystole	                        3
-            Atrial extrasystole, type: single PAC	                            4
-            Atrial extrasystole, type: bigemini	                                1
-            Atrial extrasystole, type: quadrigemini	                            1
-            Atrial extrasystole, type: allorhythmic pattern	                    1
-            Ventricular extrasystole, morphology: polymorphic	                2
-            Ventricular extrasystole, localisation: RVOT, anterior wall	        3
-            Ventricular extrasystole, localisation: RVOT, antero-septal part	1
-            Ventricular extrasystole, localisation: IVS, middle part	        1
-            Ventricular extrasystole, localisation: LVOT, LVS	                2
-            Ventricular extrasystole, localisation: LV, undefined	            1
-            Ventricular extrasystole, type: single PVC	                        6
-            Ventricular extrasystole, type: intercalary PVC	                    2
-            Ventricular extrasystole, type: couplet	                            2
+            | Extrasystoles                                                    | Number of ECGs |
+            | ---------------------------------------------------------------- | -------------- |
+            | Atrial extrasystole, undetermined                                | 2              |
+            | Atrial extrasystole, low atrial                                  | 1              |
+            | Atrial extrasystole, left atrial                                 | 2              |
+            | Atrial extrasystole, SA-nodal extrasystole                       | 3              |
+            | Atrial extrasystole, type: single PAC                            | 4              |
+            | Atrial extrasystole, type: bigemini                              | 1              |
+            | Atrial extrasystole, type: quadrigemini                          | 1              |
+            | Atrial extrasystole, type: allorhythmic pattern                  | 1              |
+            | Ventricular extrasystole, morphology: polymorphic                | 2              |
+            | Ventricular extrasystole, localisation: RVOT, anterior wall      | 3              |
+            | Ventricular extrasystole, localisation: RVOT, antero-septal part | 1              |
+            | Ventricular extrasystole, localisation: IVS, middle part         | 1              |
+            | Ventricular extrasystole, localisation: LVOT, LVS                | 2              |
+            | Ventricular extrasystole, localisation: LV, undefined            | 1              |
+            | Ventricular extrasystole, type: single PVC                       | 6              |
+            | Ventricular extrasystole, type: intercalary PVC                  | 2              |
+            | Ventricular extrasystole, type: couplet                          | 2              |
         4.5. distribution of records with hypertrophies (totally 253):
-            Hypertrophies	                    Number of ECGs
-            Right atrial hypertrophy	        1
-            Left atrial hypertrophy	            102
-            Right atrial overload	            17
-            Left atrial overload	            11
-            Left ventricular hypertrophy	    108
-            Right ventricular hypertrophy	    3
-            Left ventricular overload	        11
+            | Hypertrophies                 | Number of ECGs |
+            | ----------------------------- | -------------- |
+            | Right atrial hypertrophy      | 1              |
+            | Left atrial hypertrophy       | 102            |
+            | Right atrial overload         | 17             |
+            | Left atrial overload          | 11             |
+            | Left ventricular hypertrophy  | 108            |
+            | Right ventricular hypertrophy | 3              |
+            | Left ventricular overload     | 11             |
         4.6. distribution of records of pacing rhythms (totally 12):
-            Cardiac pacing	                Number of ECGs
-            UNIpolar atrial pacing	        1
-            UNIpolar ventricular pacing	    6
-            BIpolar ventricular pacing	    2
-            Biventricular pacing	        1
-            P-synchrony	                    2
+            | Cardiac pacing               | Number of ECGs |
+            | ---------------------------- | -------------- |
+            | UNIpolar atrial pacing       | 1              |
+            | UNIpolar ventricular pacing  | 6              |
+            | BIpolar ventricular pacing   | 2              |
+            | Biventricular pacing         | 1              |
+            | P-synchrony                  | 2              |
         4.7. distribution of records with ischemia (totally 141):
-            Ischemia	                                            Number of ECGs
-            STEMI: anterior wall	                                8
-            STEMI: lateral wall	                                    7
-            STEMI: septal	                                        8
-            STEMI: inferior wall	                                1
-            STEMI: apical	                                        5
-            Ischemia: anterior wall	                                5
-            Ischemia: lateral wall	                                8
-            Ischemia: septal	                                    4
-            Ischemia: inferior wall	                                10
-            Ischemia: posterior wall	                            2
-            Ischemia: apical	                                    6
-            Scar formation: lateral wall	                        3
-            Scar formation: septal	                                9
-            Scar formation: inferior wall	                        3
-            Scar formation: posterior wall	                        6
-            Scar formation: apical	                                5
-            Undefined ischemia/scar/supp.NSTEMI: anterior wall	    12
-            Undefined ischemia/scar/supp.NSTEMI: lateral wall	    16
-            Undefined ischemia/scar/supp.NSTEMI: septal	            5
-            Undefined ischemia/scar/supp.NSTEMI: inferior wall	    3
-            Undefined ischemia/scar/supp.NSTEMI: posterior wall	    4
-            Undefined ischemia/scar/supp.NSTEMI: apical	            11
+            | Ischemia                                              | Number of ECGs |
+            | ----------------------------------------------------- | -------------- |
+            | STEMI: anterior wall                                  | 8              |
+            | STEMI: lateral wall                                   | 7              |
+            | STEMI: septal                                         | 8              |
+            | STEMI: inferior wall                                  | 1              |
+            | STEMI: apical                                         | 5              |
+            | Ischemia: anterior wall                               | 5              |
+            | Ischemia: lateral wall                                | 8              |
+            | Ischemia: septal                                      | 4              |
+            | Ischemia: inferior wall                               | 10             |
+            | Ischemia: posterior wall                              | 2              |
+            | Ischemia: apical                                      | 6              |
+            | Scar formation: lateral wall                          | 3              |
+            | Scar formation: septal                                | 9              |
+            | Scar formation: inferior wall                         | 3              |
+            | Scar formation: posterior wall                        | 6              |
+            | Scar formation: apical                                | 5              |
+            | Undefined ischemia/scar/supp.NSTEMI: anterior wall    | 12             |
+            | Undefined ischemia/scar/supp.NSTEMI: lateral wall     | 16             |
+            | Undefined ischemia/scar/supp.NSTEMI: septal           | 5              |
+            | Undefined ischemia/scar/supp.NSTEMI: inferior wall    | 3              |
+            | Undefined ischemia/scar/supp.NSTEMI: posterior wall   | 4              |
+            | Undefined ischemia/scar/supp.NSTEMI: apical           | 11             |
         4.8. distribution of records with non-specific repolarization abnormalities (totally 85):
-            Non-specific repolarization abnormalities	    Number of ECGs
-            Anterior wall	                                18
-            Lateral wall	                                13
-            Septal	                                        15
-            Inferior wall	                                19
-            Posterior wall	                                9
-            Apical	                                        11
+            | Non-specific repolarization abnormalities | Number of ECGs |
+            | ----------------------------------------- | -------------- |
+            | Anterior wall                             |      18        |
+            | Lateral wall                              |      13        |
+            | Septal                                    |      15        |
+            | Inferior wall                             |      19        |
+            | Posterior wall                            |      9         |
+            | Apical                                    |      11        |
         4.9. there are also 9 records with early repolarization syndrome
         there might well be records with multiple conditions.
     5. ludb.csv stores information about the subjects (gender, age, rhythm type, direction of the electrical axis of the heart, the presence of a cardiac pacemaker, etc.)
-
-
-    NOTE
-    ----
-
-    ISSUES
-    ------
+    """,
+    usage=[
+        "ECG wave delineation",
+        "ECG arrhythmia classification",
+    ],
+    issues="""
     1. (version 1.0.0, fixed in version 1.0.1) ADC gain might be wrong, either `units` should be μV, or `adc_gain` should be 1000 times larger
+    """,
+    references=[
+        "https://physionet.org/content/ludb/1.0.1/",
+        "Kalyakulina, A., Yusipov, I., Moskalenko, V., Nikolskiy, A., Kozlov, A., Kosonogov, K., Zolotykh, N., & Ivanchenko, M. (2020). Lobachevsky University Electrocardiography Database (version 1.0.1).",
+    ],
+    doi=[
+        "10.1109/ACCESS.2020.3029211",
+        "10.13026/eegm-h675",
+    ],
+)
 
-    Usage
-    -----
-    1. ECG wave delineation
-    2. ECG arrhythmia classification
 
-    References
-    ----------
-    1. <a name="ref1"></a> https://physionet.org/content/ludb/1.0.1/
-    2. <a name="ref2"></a> Kalyakulina, A., Yusipov, I., Moskalenko, V., Nikolskiy, A., Kozlov, A., Kosonogov, K., Zolotykh, N., & Ivanchenko, M. (2020). Lobachevsky University Electrocardiography Database (version 1.0.1).
+@add_docstring(_LUDB_INFO.format_database_docstring())
+class LUDB(PhysioNetDataBase):
+    """ """
 
-    """
+    __name__ = "LUDB"
 
     def __init__(
         self,
-        db_dir: Union[str, Path],
+        db_dir: Optional[Union[str, Path]] = None,
         working_dir: Optional[Union[str, Path]] = None,
-        verbose: int = 2,
+        verbose: int = 1,
         **kwargs: Any,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
-        db_dir: str or Path,
+        db_dir: str or Path, optional,
             storage path of the database
         working_dir: str or Path, optional,
             working directory, to store intermediate files and log file
-        verbose: int, default 2,
+        verbose: int, default 1
             log verbosity
         kwargs: auxilliary key word arguments
 
@@ -177,26 +190,14 @@ class LUDB(PhysioNetDataBase):
             verbose=verbose,
             **kwargs,
         )
-        print(
-            "Version of LUDB 1.0.0 has bugs, make sure that version 1.0.1 or higher is used"
-        )
+        if self.version == "1.0.0":
+            self.logger.info(
+                "Version of LUDB 1.0.0 has bugs, make sure that version 1.0.1 or higher is used"
+            )
         self.fs = 500
         self.spacing = 1000 / self.fs
         self.data_ext = "dat"
-        self.all_leads = [
-            "I",
-            "II",
-            "III",
-            "aVR",
-            "aVL",
-            "aVF",
-            "V1",
-            "V2",
-            "V3",
-            "V4",
-            "V5",
-            "V6",
-        ]
+        self.all_leads = deepcopy(EAK.Standard12Leads)
         self.all_leads_lower = [ld.lower() for ld in self.all_leads]
         # Version 1.0.0 has different beat_ann_ext: [f"atr_{item}" for item in self.all_leads_lower]
         self.beat_ann_ext = [f"{item}" for item in self.all_leads_lower]
@@ -204,11 +205,11 @@ class LUDB(PhysioNetDataBase):
         self._all_symbols = ["(", ")", "p", "N", "t"]
         """
         this can be obtained using the following code:
-        >>> data_gen = LUDB(db_dir="/home/wenhao71/data/PhysioNet/ludb/1.0.0/")
+        >>> data_gen = LUDB(db_dir="/home/wenhao71/data/PhysioNet/ludb/1.0.1/")
         >>> all_symbols = set()
         >>> for rec in data_gen.all_records:
         ...     for ext in data_gen.beat_ann_ext:
-        ...         ann = wfdb.rdann(data_gen._get_path(rec), extension=ext)
+        ...         ann = wfdb.rdann(data_gen.get_absolute_path(rec), extension=ext)
         ...         all_symbols.update(ann.symbol)
         """
         self._symbol_to_wavename = CFG(N="qrs", p="pwave", t="twave")
@@ -247,108 +248,49 @@ class LUDB(PhysioNetDataBase):
 
         self._ls_rec()
 
-    def get_subject_id(self, rec: str) -> int:
+    def get_subject_id(self, rec: Union[str, int]) -> int:
         """ """
         raise NotImplementedError
 
-    def _get_path(self, rec: Union[str, int]) -> str:
+    def get_absolute_path(
+        self, rec: Union[str, int], extension: Optional[str] = None
+    ) -> Path:
         """
-
-        get the path of a record, without file extension
+        get the absolute path of the record `rec`
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
+        extension: str, optional,
+            extension of the file
 
         Returns
         -------
-        rec_fp: str,
-            path of the record, without file extension
+        Path,
+            absolute path of the file
 
         """
         if isinstance(rec, int):
             rec = self[rec]
-        rec_fp = str(self.db_dir / "data" / rec)
-        return rec_fp
-
-    def load_data(
-        self,
-        rec: Union[str, int],
-        leads: Optional[Union[str, List[str]]] = None,
-        data_format="channel_first",
-        units: str = "mV",
-        fs: Optional[Real] = None,
-    ) -> np.ndarray:
-        """
-
-        load physical (converted from digital) ECG data,
-        which is more understandable for humans
-
-        Parameters
-        ----------
-        rec: str or int,
-            name or index of the record
-        leads: str or list of str, optional,
-            the leads to load
-        data_format: str, default "channel_first",
-            format of the ECG data,
-            "channel_last" (alias "lead_last"), or
-            "channel_first" (alias "lead_first", original)
-        units: str, default "mV",
-            units of the output signal, can also be "μV", with an alias of "uV"
-        fs: real number, optional,
-            if not None, the loaded data will be resampled to this frequency
-
-        Returns
-        -------
-        data: ndarray,
-            the ECG data
-
-        """
-        if isinstance(rec, int):
-            rec = self[rec]
-        assert data_format.lower() in [
-            "channel_first",
-            "lead_first",
-            "channel_last",
-            "lead_last",
-        ]
-        _leads = self._normalize_leads(leads, standard_ordering=True, lower_cases=True)
-
-        rec_fp = self._get_path(rec)
-        wfdb_rec = wfdb.rdrecord(rec_fp, physical=True, channel_names=_leads)
-        # p_signal of "lead_last" format
-        # ref. ISSUES 1. (fixed in version 1.0.1)
-        # data = np.asarray(wfdb_rec.p_signal.T / 1000, dtype=np.float64)
-        data = np.asarray(wfdb_rec.p_signal.T, dtype=np.float64)
-
-        if units.lower() in ["uv", "μv"]:
-            data = data * 1000
-
-        if fs is not None and fs != self.fs:
-            data = resample_poly(data, fs, self.fs, axis=1)
-
-        if data_format.lower() in ["channel_last", "lead_last"]:
-            data = data.T
-
-        return data
+        if extension is not None and not extension.startswith("."):
+            extension = f".{extension}"
+        return self.db_dir / "data" / f"{rec}{extension or ''}"
 
     def load_ann(
         self,
         rec: Union[str, int],
-        leads: Optional[Sequence[str]] = None,
+        leads: Optional[Union[str, int, Sequence[Union[str, int]]]] = None,
         metadata: bool = False,
     ) -> dict:
         """
-
         load the wave delineation, along with metadata if specified
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
-        leads: str or list of str, optional,
+            record name or index of the record in `self.all_records`
+        leads: str or int or list of str or int, optional,
             the leads to load
         metadata: bool, default False,
             if True, load metadata from corresponding head file
@@ -356,14 +298,16 @@ class LUDB(PhysioNetDataBase):
         Returns
         -------
         ann_dict: dict,
+            the wave delineation
+
         """
         if isinstance(rec, int):
             rec = self[rec]
         ann_dict = CFG()
-        rec_fp = self._get_path(rec)
+        rec_fp = str(self.get_absolute_path(rec))
 
         # wave delineation annotations
-        _leads = self._normalize_leads(leads, standard_ordering=True, lower_cases=False)
+        _leads = self._normalize_leads(leads)
         _ann_ext = [
             f"{ld.lower()}" for ld in _leads
         ]  # for Version 1.0.0, it is f"{l.lower()}"
@@ -429,13 +373,12 @@ class LUDB(PhysioNetDataBase):
 
     def load_diagnoses(self, rec: Union[str, int]) -> List[str]:
         """
-
         load diagnoses of the `rec`
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
 
         Returns
         -------
@@ -449,19 +392,18 @@ class LUDB(PhysioNetDataBase):
     def load_masks(
         self,
         rec: Union[str, int],
-        leads: Optional[Sequence[str]] = None,
+        leads: Optional[Union[str, int, Sequence[Union[str, int]]]] = None,
         mask_format: str = "channel_first",
         class_map: Optional[Dict[str, int]] = None,
     ) -> np.ndarray:
         """
-
         load the wave delineation in the form of masks
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
-        leads: str or list of str, optional,
+            record name or index of the record in `self.all_records`
+        leads: str or int or list of str or int, optional,
             the leads to load
         mask_format: str, default "channel_first",
             format of the mask,
@@ -480,10 +422,9 @@ class LUDB(PhysioNetDataBase):
         if isinstance(rec, int):
             rec = self[rec]
         _class_map = CFG(class_map) if class_map is not None else self.class_map
-        _leads = self._normalize_leads(leads, standard_ordering=True, lower_cases=True)
-        data = self.load_data(rec, leads=_leads, data_format="channel_first")
+        data = self.load_data(rec, leads=leads, data_format="channel_first")
         masks = np.full_like(data, fill_value=_class_map.i, dtype=int)
-        waves = self.load_ann(rec, leads=_leads, metadata=False)["waves"]
+        waves = self.load_ann(rec, leads=leads, metadata=False)["waves"]
         for idx, (l, l_w) in enumerate(waves.items()):
             for w in l_w:
                 masks[idx, w.onset : w.offset] = _class_map[
@@ -500,12 +441,11 @@ class LUDB(PhysioNetDataBase):
         self,
         masks: np.ndarray,
         mask_format: str = "channel_first",
-        leads: Optional[Sequence[str]] = None,
+        leads: Optional[Union[str, int, Sequence[Union[str, int]]]] = None,
         class_map: Optional[Dict[str, int]] = None,
         fs: Optional[Real] = None,
     ) -> Dict[str, List[ECGWaveForm]]:
         """
-
         convert masks into lists of waveforms
 
         Parameters
@@ -517,7 +457,7 @@ class LUDB(PhysioNetDataBase):
             format of the mask, used only when `masks.ndim = 2`
             "channel_last" (alias "lead_last"), or
             "channel_first" (alias "lead_first")
-        leads: str or list of str, optional,
+        leads: str or int or list of str or int, optional,
             the leads to load
         class_map: dict, optional,
             custom class map,
@@ -548,9 +488,7 @@ class LUDB(PhysioNetDataBase):
             )
 
         if leads is not None:
-            _leads = self._normalize_leads(
-                leads, standard_ordering=False, lower_cases=False
-            )
+            _leads = self._normalize_leads(leads)
         else:
             _leads = [f"lead_{idx+1}" for idx in range(_masks.shape[0])]
         assert len(_leads) == _masks.shape[0]
@@ -593,13 +531,12 @@ class LUDB(PhysioNetDataBase):
 
     def _load_header(self, rec: Union[str, int]) -> dict:
         """
-
         load header data into a dict
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
 
         Returns
         -------
@@ -607,10 +544,8 @@ class LUDB(PhysioNetDataBase):
             the header data
 
         """
-        if isinstance(rec, int):
-            rec = self[rec]
         header_dict = CFG({})
-        rec_fp = self._get_path(rec)
+        rec_fp = str(self.get_absolute_path(rec))
         header_reader = wfdb.rdheader(rec_fp)
         header_dict["units"] = header_reader.units
         header_dict["baseline"] = header_reader.baseline
@@ -638,55 +573,14 @@ class LUDB(PhysioNetDataBase):
         header_dict["diagnoses"] = header_reader.comments[d_start:]
         return header_dict
 
-    def _normalize_leads(
-        self,
-        leads: Optional[Sequence[str]] = None,
-        standard_ordering: bool = True,
-        lower_cases: bool = False,
-    ) -> List[str]:
-        """
-
-        Parameters
-        ----------
-        leads: str or list of str, optional,
-            the (names of) leads to normalize
-        starndard_ordering: bool, default True,
-            if True, the ordering will be re-aranged to be accordance with `self.all_leads`
-        lower_cases: bool, default False,
-            if True, all names of the leads will be in lower cases
-
-        Returns
-        -------
-        leads: list of str,
-            the leads in the standard names and order
-        """
-        if leads is None:
-            _leads = self.all_leads_lower
-        elif isinstance(leads, str):
-            _leads = [leads.lower()]
-        else:
-            _leads = [ld.lower() for ld in leads]
-
-        if standard_ordering:
-            _leads = [ld for ld in self.all_leads_lower if ld in _leads]
-
-        if not lower_cases:
-            _lead_indices = [
-                idx for idx, ld in enumerate(self.all_leads_lower) if ld in _leads
-            ]
-            _leads = [self.all_leads[idx] for idx in _lead_indices]
-
-        return _leads
-
     def load_subject_info(
         self, rec: Union[str, int], fields: Optional[Union[str, Sequence[str]]] = None
     ) -> Union[dict, str]:
         """
-
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         fields: str or sequence of str, optional,
             field(s) of the subject info of record `rec`,
             if not specified, all fields of the subject info will be returned
@@ -719,11 +613,11 @@ class LUDB(PhysioNetDataBase):
         rec: Union[str, int],
         data: Optional[np.ndarray] = None,
         ticks_granularity: int = 0,
-        leads: Optional[Union[str, List[str]]] = None,
+        leads: Optional[Union[str, int, Sequence[Union[str, int]]]] = None,
         same_range: bool = False,
         waves: Optional[ECGWaveForm] = None,
         **kwargs: Any,
-    ) -> NoReturn:
+    ) -> None:
         """to improve,
 
         plot the signals of a record or external signals (units in μV),
@@ -733,7 +627,7 @@ class LUDB(PhysioNetDataBase):
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         data: ndarray, optional,
             12-lead ECG signal to plot,
             if given, data of `rec` will not be used,
@@ -741,7 +635,7 @@ class LUDB(PhysioNetDataBase):
         ticks_granularity: int, default 0,
             the granularity to plot axis ticks, the higher the more,
             0 (no ticks) --> 1 (major ticks) --> 2 (major + minor ticks)
-        leads: str or list of str, optional,
+        leads: str or int or list of str or int, optional,
             the leads to plot
         same_range: bool, default False,
             if True, forces all leads to have the same y range
@@ -768,7 +662,7 @@ class LUDB(PhysioNetDataBase):
             import matplotlib.pyplot as plt
 
             plt.MultipleLocator.MAXTICKS = 3000
-        _leads = self._normalize_leads(leads, standard_ordering=True, lower_cases=False)
+        _leads = self._normalize_leads(leads)
 
         # lead_list = self.load_ann(rec)["df_leads"]["lead_name"].tolist()
         # _lead_indices = [lead_list.index(ld) for ld in leads]
@@ -779,7 +673,7 @@ class LUDB(PhysioNetDataBase):
             ]
         else:
             units = self._auto_infer_units(data)
-            print(f"input data is auto detected to have units in {units}")
+            self.logger.info(f"input data is auto detected to have units in {units}")
             if units.lower() == "mv":
                 _data = 1000 * data
             else:
@@ -876,6 +770,10 @@ class LUDB(PhysioNetDataBase):
         else:
             plt.show()
 
+    @property
+    def database_info(self) -> DataBaseInfo:
+        return _LUDB_INFO
+
 
 __TOLERANCE = 150  # ms
 __WaveNames = ["pwave", "qrs", "twave"]
