@@ -1,78 +1,75 @@
 # -*- coding: utf-8 -*-
 """
+MIT-BIH Arrhythmia Database
 """
 
 from collections import defaultdict, Counter
-from numbers import Real
 from pathlib import Path
-from typing import Any, Dict, List, NoReturn, Optional, Sequence, Union
+from typing import Any, Dict, List, Optional, Sequence, Union
 
 import numpy as np
 import pandas as pd
 import wfdb
-from scipy.signal import resample_poly
+from tqdm.auto import tqdm
 
-try:
-    from tqdm.auto import tqdm
-except ModuleNotFoundError:
-    from tqdm import tqdm
-
-from ...cfg import CFG
-from ...utils.misc import get_record_list_recursive3
+from ...cfg import CFG, DEFAULTS
+from ...utils.misc import get_record_list_recursive3, add_docstring
 from ...utils.utils_interval import generalized_intervals_intersection
 from ..base import (
     BeatAnn,
+    DataBaseInfo,
     PhysioNetDataBase,
     WFDB_Beat_Annotations,
     WFDB_Non_Beat_Annotations,
     WFDB_Rhythm_Annotations,
 )
 
+
 __all__ = [
     "MITDB",
 ]
 
 
-class MITDB(PhysioNetDataBase):
-    """NOT finished,
-
+_MITDB_INFO = DataBaseInfo(
+    title="""
     MIT-BIH Arrhythmia Database
-
-    ABOUT mitdb
-    -----------
+    """,
+    about="""
     1. contains 48 half-hour excerpts of two-channel ambulatory ECG recordings, obtained from 47 subjects.
     2. recordings were digitized at 360 samples per second per channel with 11-bit resolution over a 10 mV range.
     3. annotations contains:
         - beat-wise or finer (e.g. annotations of flutter wave) annotations, accessed via the `symbol` attribute of an `Annotation`.
         - rhythm annotations, accessed via the `aux_note` attribute of an `Annotation`.
+    """,
+    usage=[
+        "Beat classification",
+        "Rhythm classification (segmentation)",
+        "R peaks detection",
+    ],
+    references=[
+        "https://physionet.org/content/mitdb/1.0.0/",
+    ],
+    doi=[
+        "10.1109/51.932724",
+        "10.13026/C2F305",
+    ],
+)
 
-    NOTE
-    ----
 
-    ISSUES
-    ------
+@add_docstring(_MITDB_INFO.format_database_docstring())
+class MITDB(PhysioNetDataBase):
+    """ """
 
-    Usage
-    -----
-    1. Beat classification
-    2. rhythm classification (segmentation)
-    3. R peaks detection
-
-    References
-    ----------
-    1. <a name="ref1"></a> https://physionet.org/content/mitdb/1.0.0/
-
-    """
+    __name__ = "MITDB"
 
     def __init__(
         self,
         db_dir: Optional[Union[str, Path]] = None,
         working_dir: Optional[Union[str, Path]] = None,
-        verbose: int = 2,
+        verbose: int = 1,
         **kwargs: Any,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         db_dir: str or Path, optional,
@@ -80,7 +77,7 @@ class MITDB(PhysioNetDataBase):
             if not specified, data will be fetched from Physionet
         working_dir: str or Path, optional,
             working directory, to store intermediate files and log file
-        verbose: int, default 2,
+        verbose: int, default 1
             log verbosity
         kwargs: auxilliary key word arguments
 
@@ -94,6 +91,8 @@ class MITDB(PhysioNetDataBase):
         )
         self.fs = 360
         self.data_ext = "dat"
+        self.data_pattern = "^[\\d]{3}$"
+        self.data_pattern_with_ext = f"^[\\d]{{3}}\\.{self.data_ext}$"
         self.ann_ext = "atr"
 
         self.beat_types_extended = list("""!"+/AEFJLNQRSV[]aefjx|~""")
@@ -135,7 +134,10 @@ class MITDB(PhysioNetDataBase):
         self.rhythm_types_map = {rt: idx for idx, rt in enumerate(self.rhythm_types)}
         self._rhythm_ignore_index = -100
 
-        self.all_leads = ["MLII", "V1", "V2", "V4", "V5"]
+        # records have different lead names
+        # therefore, self.all_leads should not be set
+        # otherwise, it will cause problems when loading data using `self.load_data`
+        self._all_leads = ["MLII", "V1", "V2", "V4", "V5"]
 
         self._ls_rec()
 
@@ -143,18 +145,52 @@ class MITDB(PhysioNetDataBase):
         self._stats_columns = ["record", "beat_num", "beat_type_num", "rhythm_len"]
         self._aggregate_stats()
 
-    def _ls_rec(self) -> NoReturn:
+    def _ls_rec(self) -> None:
         """ """
+        subsample = self._subsample
+        self._subsample = None  # so that no subsampling in super()._ls_rec()
         super()._ls_rec()
+        # filters out records with names not matching `self.data_pattern`
+        if len(self._df_records) > 0:
+            self._df_records = self._df_records[
+                self._df_records.index.str.match(self.data_pattern)
+            ]
         if len(self._all_records) == 0:
-            self._all_records = get_record_list_recursive3(
-                self.db_dir, f"^[\\d]{{3}}.{self.data_ext}$"
+            self._df_records = pd.DataFrame()
+            self._df_records["path"] = get_record_list_recursive3(
+                self.db_dir, self.data_pattern_with_ext, relative=False
             )
+            self._df_records["record"] = self._df_records["path"].apply(
+                lambda x: x.stem
+            )
+            self._df_records.set_index("record", inplace=True)
+        if subsample is not None:
+            size = min(
+                len(self._df_records),
+                max(1, int(round(subsample * len(self._df_records)))),
+            )
+            self.logger.debug(
+                f"subsample `{size}` records from `{len(self._df_records)}`"
+            )
+            self._df_records = self._df_records.sample(
+                n=size, random_state=DEFAULTS.SEED, replace=False
+            )
+        self._all_records = self._df_records.index.tolist()
+        self._subsample = subsample
 
-    def _aggregate_stats(self) -> NoReturn:
+    def _aggregate_stats(self) -> None:
         """ """
         self._stats = pd.DataFrame(columns=self._stats_columns)
-        with tqdm(range(len(self)), desc="Aggregating stats", unit="records") as pbar:
+        if len(self) == 0:
+            return
+        with tqdm(
+            range(len(self)),
+            desc="Aggregating stats",
+            unit="record",
+            dynamic_ncols=True,
+            mininterval=1.0,
+            disable=(self.verbose < 1),
+        ) as pbar:
             for idx in pbar:
                 rec_ann = self.load_ann(idx)
                 beat_type_num = {
@@ -168,82 +204,24 @@ class MITDB(PhysioNetDataBase):
                     k: sum([itv[1] - itv[0] for itv in v])
                     for k, v in rec_ann["rhythm"].items()
                 }
-                self._stats = self._stats.append(
-                    dict(
-                        record=self[idx],
-                        beat_num=beat_num,
-                        beat_type_num=beat_type_num,
-                        rhythm_len=rhythm_len,
-                    ),
+                self._stats = pd.concat(
+                    [
+                        self._stats,
+                        pd.DataFrame(
+                            [
+                                [
+                                    self._all_records[idx],
+                                    beat_num,
+                                    beat_type_num,
+                                    rhythm_len,
+                                ]
+                            ],
+                            columns=self._stats_columns,
+                        ),
+                    ],
                     ignore_index=True,
                 )
 
-    def load_data(
-        self,
-        rec: Union[str, int],
-        leads: Optional[Union[str, List[str]]] = None,
-        sampfrom: Optional[int] = None,
-        sampto: Optional[int] = None,
-        data_format: str = "channel_first",
-        units: str = "mV",
-        fs: Optional[Real] = None,
-    ) -> np.ndarray:
-        """
-
-        load physical (converted from digital) ECG data,
-        which is more understandable for humans
-
-        Parameters
-        ----------
-        rec: str or int,
-            name or index of the record
-        leads: str or list of str, optional,
-            the leads to load
-        sampfrom: int, optional,
-            start index of the data to be loaded
-        sampto: int, optional,
-            end index of the data to be loaded
-        data_format: str, default "channel_first",
-            format of the ECG data,
-            "channel_last" (alias "lead_last"), or
-            "channel_first" (alias "lead_first")
-        units: str, default "mV",
-            units of the output signal, can also be "μV", with an alias of "uV"
-        fs: real number, optional,
-            if not None, the loaded data will be resampled to this frequency
-
-        Returns
-        -------
-        data: ndarray,
-            the ECG data
-
-        """
-        if isinstance(rec, int):
-            rec = self[rec]
-        fp = str(self.db_dir / rec)
-        if not leads:
-            _leads = self._get_lead_names(rec)
-        elif isinstance(leads, str):
-            _leads = [leads]
-        else:
-            _leads = leads
-        assert set(_leads).issubset(self.all_leads)
-        # p_signal in the format of "lead_last", and in units "mV"
-        data = wfdb.rdrecord(
-            fp,
-            sampfrom=sampfrom or 0,
-            sampto=sampto,
-            physical=True,
-            channel_names=_leads,
-        ).p_signal
-        if units.lower() in ["μv", "uv"]:
-            data = 1000 * data
-        if fs is not None and fs != self.fs:
-            data = resample_poly(data, fs, self.fs, axis=0)
-        if data_format.lower() in ["channel_first", "lead_first"]:
-            data = data.T
-        return data
-
     def load_ann(
         self,
         rec: Union[str, int],
@@ -253,11 +231,9 @@ class MITDB(PhysioNetDataBase):
         rhythm_types: Optional[Sequence[str]] = None,
         beat_format: str = "beat",
         beat_types: Optional[Sequence[str]] = None,
-        extended_beats: bool = False,
         keep_original: bool = False,
     ) -> dict:
         """
-
         load rhythm and beat annotations,
         which are stored in the `aux_note`, `symbol` attributes of corresponding annotation files.
         NOTE that qrs annotations (.qrs files) do NOT contain any rhythm annotations
@@ -265,7 +241,7 @@ class MITDB(PhysioNetDataBase):
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         sampfrom: int, optional,
             start index of the annotations to be loaded
         sampto: int, optional,
@@ -292,8 +268,6 @@ class MITDB(PhysioNetDataBase):
             `beat` annotations in the format of `dict` or `BeatAnn`
 
         """
-        if isinstance(rec, int):
-            rec = self[rec]
         assert rhythm_format.lower() in [
             "intervals",
             "mask",
@@ -302,7 +276,7 @@ class MITDB(PhysioNetDataBase):
             "beat",
             "dict",
         ], f"`beat_format` must be one of ['beat', 'dict'], got {beat_format}"
-        fp = str(self.db_dir / rec)
+        fp = str(self.get_absolute_path(rec))
         wfdb_ann = wfdb.rdann(fp, extension=self.ann_ext)
         header = wfdb.rdheader(fp)
         sig_len = header.sig_len
@@ -355,8 +329,8 @@ class MITDB(PhysioNetDataBase):
             for b in beat_ann:
                 b.index -= sf
 
-        if not extended_beats:
-            beat_ann = [b for b in beat_ann if b.symbol in self.beat_types]
+        # if not extended_beats:
+        #     beat_ann = [b for b in beat_ann if b.symbol in self.beat_types]
 
         if beat_format.lower() == "dict":
             beat_ann = {
@@ -383,22 +357,19 @@ class MITDB(PhysioNetDataBase):
         keep_original: bool = False,
     ) -> Union[Dict[str, list], np.ndarray]:
         """
-
         load rhythm annotations,
         which are stored in the `aux_note` attribute of corresponding annotation files.
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         sampfrom: int, optional,
             start index of the annotations to be loaded
         sampto: int, optional,
             end index of the annotations to be loaded
         rhythm_format: str, default "intervals", case insensitive,
             format of returned annotation, can also be "mask"
-        beat_format: str, default "beat", case insensitive,
-            format of returned annotation, can also be "dict"
         rhythm_types: list of str, optional,
             defaults to `self.rhythm_types`
             if not None, only the rhythm annotations with the specified types will be returned
@@ -412,8 +383,6 @@ class MITDB(PhysioNetDataBase):
             the annotations in the format of intervals, or in the format of mask
 
         """
-        if isinstance(rec, int):
-            rec = self[rec]
         return self.load_ann(
             rec,
             sampfrom,
@@ -433,14 +402,13 @@ class MITDB(PhysioNetDataBase):
         keep_original: bool = False,
     ) -> Union[Dict[str, np.ndarray], List[BeatAnn]]:
         """
-
         load beat annotations,
         which are stored in the `symbol` attribute of corresponding annotation files
 
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         sampfrom: int, optional,
             start index of the annotations to be loaded
         sampto: int, optional,
@@ -460,8 +428,6 @@ class MITDB(PhysioNetDataBase):
             locations (indices) of the all the beat types ("A", "N", "Q", "V",)
 
         """
-        if isinstance(rec, int):
-            rec = self[rec]
         return self.load_ann(
             rec,
             sampfrom,
@@ -479,7 +445,6 @@ class MITDB(PhysioNetDataBase):
         keep_original: bool = False,
     ) -> np.ndarray:
         """
-
         load rpeak indices, or equivalently qrs complex locations,
         which are stored in the `symbol` attribute of corresponding annotation files,
         regardless of their beat types,
@@ -487,7 +452,7 @@ class MITDB(PhysioNetDataBase):
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
         sampfrom: int, optional,
             start index of the annotations to be loaded
         sampto: int, optional,
@@ -498,13 +463,11 @@ class MITDB(PhysioNetDataBase):
 
         Returns
         -------
-        rpeak_inds, ndarray,
+        rpeak_inds: ndarray,
             locations (indices) of the all the rpeaks (qrs complexes)
 
         """
-        if isinstance(rec, int):
-            rec = self[rec]
-        fp = str(self.db_dir / rec)
+        fp = str(self.get_absolute_path(rec))
         wfdb_ann = wfdb.rdann(fp, extension=self.ann_ext)
         header = wfdb.rdheader(fp)
         sig_len = header.sig_len
@@ -525,11 +488,10 @@ class MITDB(PhysioNetDataBase):
 
     def _get_lead_names(self, rec: Union[str, int]) -> List[str]:
         """
-
         Parameters
         ----------
         rec: str or int,
-            name or index of the record
+            record name or index of the record in `self.all_records`
 
         Returns
         -------
@@ -537,10 +499,7 @@ class MITDB(PhysioNetDataBase):
             a list of names of the leads contained in the record
 
         """
-        if isinstance(rec, int):
-            rec = self[rec]
-        fp = str(self.db_dir / rec)
-        return wfdb.rdheader(fp).sig_name
+        return wfdb.rdheader(str(self.get_absolute_path(rec))).sig_name
 
     @property
     def df_stats(self) -> pd.DataFrame:
@@ -586,7 +545,6 @@ class MITDB(PhysioNetDataBase):
 
     def _categorize_records(self, by: str) -> Dict[str, List[str]]:
         """
-
         categorize records by specific attributes
 
         Parameters
@@ -640,6 +598,10 @@ class MITDB(PhysioNetDataBase):
         sampto: Optional[int] = None,
         same_range: bool = False,
         **kwargs: Any,
-    ) -> NoReturn:
+    ) -> None:
         """ """
         raise NotImplementedError
+
+    @property
+    def database_info(self) -> DataBaseInfo:
+        return _MITDB_INFO
