@@ -6,9 +6,8 @@ from copy import deepcopy
 
 from ..cfg import CFG
 from ..utils.utils_nn import adjust_cnn_filter_lengths
-from .attn import global_context, non_local, squeeze_excitation  # noqa: F401
-from .cnn import multi_scopic, multi_scopic_block, multi_scopic_leadwise  # noqa: F401
-from .ecg_crnn import ECG_CRNN_CONFIG  # noqa: F401
+from .attn import global_context, non_local
+from .cnn import multi_scopic, multi_scopic_leadwise
 
 
 __all__ = [
@@ -21,7 +20,6 @@ ECG_SEQ_LAB_NET_CONFIG = CFG()
 ECG_SEQ_LAB_NET_CONFIG.fs = 500
 
 
-# ECG_SEQ_LAB_NET_CONFIG.cnn = deepcopy(ECG_CRNN_CONFIG.cnn)
 ECG_SEQ_LAB_NET_CONFIG.cnn = CFG()
 ECG_SEQ_LAB_NET_CONFIG.cnn.name = "multi_scopic"
 ECG_SEQ_LAB_NET_CONFIG.cnn.multi_scopic = deepcopy(multi_scopic)
@@ -43,14 +41,15 @@ ECG_SEQ_LAB_NET_CONFIG.cnn.multi_scopic.num_filters = [
         _base_num_filters * 16,
     ],
 ]
-ECG_SEQ_LAB_NET_CONFIG.cnn.multi_scopic.block = deepcopy(multi_scopic_block)
-# ECG_SEQ_LAB_NET_CONFIG.cnn.multi_scopic_leadwise = deepcopy(multi_scopic_leadwise)
-# ECG_SEQ_LAB_NET_CONFIG.cnn.multi_scopic_leadwise.block = deepcopy(multi_scopic_block)
-
 ECG_SEQ_LAB_NET_CONFIG.cnn.multi_scopic = adjust_cnn_filter_lengths(
     ECG_SEQ_LAB_NET_CONFIG.cnn.multi_scopic, ECG_SEQ_LAB_NET_CONFIG.fs
 )
 
+ECG_SEQ_LAB_NET_CONFIG.cnn.multi_scopic_leadwise = deepcopy(multi_scopic_leadwise)
+ECG_SEQ_LAB_NET_CONFIG.cnn.multi_scopic_leadwise = adjust_cnn_filter_lengths(
+    ECG_SEQ_LAB_NET_CONFIG.cnn.multi_scopic_leadwise, ECG_SEQ_LAB_NET_CONFIG.fs
+)
+
 
 ECG_SEQ_LAB_NET_CONFIG.rnn = CFG()
 ECG_SEQ_LAB_NET_CONFIG.rnn.name = "lstm"  # "none"
@@ -63,7 +62,7 @@ ECG_SEQ_LAB_NET_CONFIG.rnn.lstm.bidirectional = True
 
 
 ECG_SEQ_LAB_NET_CONFIG.attn = CFG()
-ECG_SEQ_LAB_NET_CONFIG.attn.name = "se"  # "gc"
+ECG_SEQ_LAB_NET_CONFIG.attn.name = "se"  # "gc", "nl", "none"
 ECG_SEQ_LAB_NET_CONFIG.attn.se = CFG()
 ECG_SEQ_LAB_NET_CONFIG.attn.se.reduction = 8  # not including the last linear layer
 ECG_SEQ_LAB_NET_CONFIG.attn.se.activation = "relu"
@@ -71,6 +70,8 @@ ECG_SEQ_LAB_NET_CONFIG.attn.se.kw_activation = CFG(inplace=True)
 ECG_SEQ_LAB_NET_CONFIG.attn.se.bias = True
 ECG_SEQ_LAB_NET_CONFIG.attn.se.kernel_initializer = "he_normal"
 # ECG_SEQ_LAB_NET_CONFIG.attn.se.dropouts = [0.2, 0.0]
+ECG_SEQ_LAB_NET_CONFIG.attn.gc = deepcopy(global_context)
+ECG_SEQ_LAB_NET_CONFIG.attn.nl = deepcopy(non_local)
 
 
 ECG_SEQ_LAB_NET_CONFIG.clf = CFG()
