@@ -2,12 +2,8 @@
 configs of the yolo model for qrs complex (or more?) detection
 """
 
-from itertools import repeat
-from copy import deepcopy
-
 from ..cfg import CFG
 
-
 __all__ = [
     "ECG_YOLO_CONFIG",
 ]
