@@ -10,12 +10,10 @@ References
 [5] Wang, C. Y., Bochkovskiy, A., & Liao, H. Y. M. (2020). Scaled-YOLOv4: Scaling Cross Stage Partial Network. arXiv preprint arXiv:2011.08036.
 """
 
-from typing import NoReturn
+from typing import List
 
-import torch
 from torch import nn
 
-from ...cfg import DEFAULTS
 from ...models._nets import (  # noqa: F401
     Conv_Bn_Activation,
     DownSample,
@@ -23,10 +21,7 @@ from ...models._nets import (  # noqa: F401
     NonLocalBlock,
     SEBlock,
 )
-from ...utils.utils_nn import SizeMixin
-
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
+from ...utils import SizeMixin, CitationMixin
 
 
 __all__ = [
@@ -34,13 +29,16 @@ __all__ = [
 ]
 
 
-class DarkNet(nn.Sequential, SizeMixin):
+class DarkNet(nn.Sequential, SizeMixin, CitationMixin):
     """ """
 
-    __DEBUG__ = True
     __name__ = "DarkNet"
 
-    def __init__(self, in_channels: int, **config) -> NoReturn:
+    def __init__(self, in_channels: int, **config) -> None:
         """ """
         super().__init__()
         raise NotImplementedError
+
+    @property
+    def doi(self) -> List[str]:
+        return list(set(self.config.get("doi", []) + ["10.1109/CVPR.2016.91"]))
