@@ -14,12 +14,12 @@ Its key points:
 import math
 from copy import deepcopy
 from itertools import repeat
-from typing import NoReturn, Optional, Sequence, Union
+from typing import Optional, Sequence, Union, List
 
 import torch
 from torch import Tensor, nn
 
-from ...cfg import CFG, DEFAULTS
+from ...cfg import CFG
 from ...models._nets import (  # noqa: F401
     Conv_Bn_Activation,
     DownSample,
@@ -27,16 +27,13 @@ from ...models._nets import (  # noqa: F401
     NonLocalBlock,
     SEBlock,
 )
-from ...utils.misc import dict_to_str, list_sum, add_docstring
+from ...utils.misc import list_sum, add_docstring, CitationMixin
 from ...utils.utils_nn import (
     SizeMixin,
     compute_sequential_output_shape,
     compute_sequential_output_shape_docstring,
 )
 
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
-
 
 __all__ = [
     "DenseNet",
@@ -49,14 +46,11 @@ __all__ = [
 
 class DenseBasicBlock(nn.Module, SizeMixin):
     """
-
     the basic building block for DenseNet,
     consisting of normalization -> activation -> convolution (-> dropout (optional)),
     the output Tensor is the concatenation of old features (input) with new features
-
     """
 
-    __DEBUG__ = True
     __name__ = "DenseBasicBlock"
     __DEFAULT_CONFIG__ = CFG(
         activation="relu",
@@ -73,9 +67,8 @@ class DenseBasicBlock(nn.Module, SizeMixin):
         bias: bool = False,
         dropout: float = 0.0,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -127,7 +120,6 @@ class DenseBasicBlock(nn.Module, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -169,7 +161,6 @@ class DenseBasicBlock(nn.Module, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -190,13 +181,11 @@ class DenseBasicBlock(nn.Module, SizeMixin):
 
 class DenseBottleNeck(nn.Module, SizeMixin):
     """
-
     bottleneck modification of `DenseBasicBlock`,
     with an additional prefixed sequence of
     (normalization -> activation -> convolution of kernel size 1)
     """
 
-    __DEBUG__ = True
     __name__ = "DenseBottleNeck"
     __DEFAULT_CONFIG__ = CFG(
         activation="relu",
@@ -214,9 +203,8 @@ class DenseBottleNeck(nn.Module, SizeMixin):
         bias: bool = False,
         dropout: float = 0.0,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -284,7 +272,6 @@ class DenseBottleNeck(nn.Module, SizeMixin):
 
     def bn_function(self, input: Tensor) -> Tensor:
         """
-
         the `not memory_efficient` way
 
         Parameters
@@ -303,7 +290,6 @@ class DenseBottleNeck(nn.Module, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -349,7 +335,6 @@ class DenseBottleNeck(nn.Module, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -370,13 +355,10 @@ class DenseBottleNeck(nn.Module, SizeMixin):
 
 class DenseMacroBlock(nn.Sequential, SizeMixin):
     """
-
     macro blocks for `DenseNet`,
     stacked sequence of builing blocks of similar pattern
-
     """
 
-    __DEBUG__ = True
     __name__ = "DenseMacroBlock"
     building_block = DenseBottleNeck
 
@@ -391,9 +373,8 @@ class DenseMacroBlock(nn.Sequential, SizeMixin):
         bias: bool = False,
         dropout: float = 0.0,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -468,14 +449,11 @@ class DenseMacroBlock(nn.Sequential, SizeMixin):
 
 class DenseTransition(nn.Sequential, SizeMixin):
     """
-
     transition blocks between `DenseMacroBlock`s,
     used to perform sub-sampling,
     and compression of channels if specified
-
     """
 
-    __DEBUG__ = True
     __name__ = "DenseTransition"
     __DEFAULT_CONFIG__ = CFG(
         activation="relu",
@@ -491,9 +469,8 @@ class DenseTransition(nn.Sequential, SizeMixin):
         groups: int = 1,
         bias: bool = False,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -561,9 +538,8 @@ class DenseTransition(nn.Sequential, SizeMixin):
         return compute_sequential_output_shape(self, seq_len, batch_size)
 
 
-class DenseNet(nn.Sequential, SizeMixin):
+class DenseNet(nn.Sequential, SizeMixin, CitationMixin):
     """
-
     The core part of the SOTA model (framework) of CPSC2020
 
     References
@@ -586,7 +562,6 @@ class DenseNet(nn.Sequential, SizeMixin):
 
     """
 
-    __DEBUG__ = True
     __name__ = "DenseNet"
     __DEFAULT_CONFIG__ = CFG(
         bias=False,
@@ -597,9 +572,8 @@ class DenseNet(nn.Sequential, SizeMixin):
         init_subsample_mode="avg",
     )
 
-    def __init__(self, in_channels: int, **config) -> NoReturn:
+    def __init__(self, in_channels: int, **config) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -647,10 +621,6 @@ class DenseNet(nn.Sequential, SizeMixin):
         self.config = CFG(deepcopy(self.__DEFAULT_CONFIG__))
         self.config.update(deepcopy(config))
         self.__num_blocks = len(self.config.num_layers)
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
 
         self.add_module(
             "init_cba",
@@ -743,3 +713,7 @@ class DenseNet(nn.Sequential, SizeMixin):
     @property
     def in_channels(self) -> int:
         return self.__in_channels
+
+    @property
+    def doi(self) -> List[str]:
+        return list(set(self.config.get("doi", []) + ["10.1109/cvpr.2017.243"]))
