@@ -9,12 +9,10 @@ References
 
 """
 
-from typing import NoReturn
+from typing import List
 
-import torch
 from torch import nn
 
-from ...cfg import DEFAULTS
 from ...models._nets import (  # noqa: F401
     Conv_Bn_Activation,
     DownSample,
@@ -22,10 +20,7 @@ from ...models._nets import (  # noqa: F401
     NonLocalBlock,
     SEBlock,
 )
-from ...utils.utils_nn import SizeMixin
-
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
+from ...utils import SizeMixin, CitationMixin
 
 
 __all__ = [
@@ -33,21 +28,20 @@ __all__ = [
 ]
 
 
-class EfficientNet(nn.Module, SizeMixin):
+class EfficientNet(nn.Module, SizeMixin, CitationMixin):
     """
-
     Reference
     ---------
     1. Tan, M., & Le, Q. V. (2019). Efficientnet: Rethinking model scaling for convolutional neural networks. arXiv preprint arXiv:1905.11946.
     2. https://github.com/lukemelas/EfficientNet-PyTorch/blob/master/efficientnet_pytorch/model.py
     3. https://github.com/rwightman/pytorch-image-models/blob/master/timm/models/efficientnet.py
     3. https://github.com/google/automl
+
     """
 
-    __DEBUG__ = True
     __name__ = "EfficientNet"
 
-    def __init__(self, in_channels: int, **config) -> NoReturn:
+    def __init__(self, in_channels: int, **config) -> None:
         """ """
         super().__init__()
         raise NotImplementedError
@@ -62,10 +56,13 @@ class EfficientNet(nn.Module, SizeMixin):
         """ """
         raise NotImplementedError
 
+    @property
+    def doi(self) -> List[str]:
+        return list(set(self.config.get("doi", []) + ["10.48550/ARXIV.1905.11946"]))
+
 
 class EfficientNetV2(nn.Module, SizeMixin):
     """
-
     Reference
     ---------
     1. Tan, M., & Le, Q. V. (2021). Efficientnetv2: Smaller models and faster training. arXiv preprint arXiv:2104.00298.
@@ -73,12 +70,12 @@ class EfficientNetV2(nn.Module, SizeMixin):
     3. https://github.com/lukemelas/EfficientNet-PyTorch/blob/master/efficientnet_pytorch/model.py
     4. https://github.com/rwightman/pytorch-image-models/blob/master/timm/models/efficientnet.py
     5. https://github.com/google/automl
+
     """
 
-    __DEBUG__ = True
     __name__ = "EfficientNetV2"
 
-    def __init__(self, in_channels: int, **config) -> NoReturn:
+    def __init__(self, in_channels: int, **config) -> None:
         """ """
         super().__init__()
         raise NotImplementedError
