@@ -7,12 +7,9 @@ References
 
 """
 
-from typing import NoReturn
-
-import torch
 from torch import nn
 
-from ...cfg import DEFAULTS
+from ...cfg import CFG  # noqa: F401
 from ...models._nets import (  # noqa: F401
     Activations,
     Conv_Bn_Activation,
@@ -22,10 +19,7 @@ from ...models._nets import (  # noqa: F401
     SEBlock,
     ZeroPadding,
 )
-from ...utils.utils_nn import SizeMixin
-
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
+from ...utils import SizeMixin, CitationMixin
 
 
 __all__ = [
@@ -35,25 +29,25 @@ __all__ = [
 ]
 
 
-class MidPointResNet(nn.Module, SizeMixin):
+class MidPointResNet(nn.Module, SizeMixin, CitationMixin):
     """ """
 
-    def __init__(self, in_channels: int, **config) -> NoReturn:
+    def __init__(self, in_channels: int, **config) -> None:
         """ """
         raise NotImplementedError
 
 
-class RK4ResNet(nn.Module, SizeMixin):
+class RK4ResNet(nn.Module, SizeMixin, CitationMixin):
     """ """
 
-    def __init__(self, in_channels: int, **config) -> NoReturn:
+    def __init__(self, in_channels: int, **config) -> None:
         """ """
         raise NotImplementedError
 
 
-class RK8ResNet(nn.Module, SizeMixin):
+class RK8ResNet(nn.Module, SizeMixin, CitationMixin):
     """ """
 
-    def __init__(self, in_channels: int, **config) -> NoReturn:
+    def __init__(self, in_channels: int, **config) -> None:
         """ """
         raise NotImplementedError
