@@ -11,13 +11,13 @@ References
 
 from copy import deepcopy
 from itertools import repeat
-from typing import Any, NoReturn, Optional, Sequence, Union
 from numbers import Real
+from typing import Any, Optional, Sequence, Union, List
 
-import torch
 from torch import Tensor, nn
+from deprecate_kwargs import deprecate_kwargs
 
-from ...cfg import CFG, DEFAULTS
+from ...cfg import CFG
 from ...models._nets import (  # noqa: F401
     Conv_Bn_Activation,
     DownSample,
@@ -25,16 +25,13 @@ from ...models._nets import (  # noqa: F401
     MultiConv,
     make_attention_layer,
 )
-from ...utils.misc import dict_to_str, deprecate_kwargs, add_docstring
+from ...utils.misc import add_docstring, CitationMixin
 from ...utils.utils_nn import (
     SizeMixin,
     compute_sequential_output_shape,
     compute_sequential_output_shape_docstring,
 )
 
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
-
 
 __all__ = [
     "MobileNetV1",
@@ -56,13 +53,11 @@ _DEFAULT_CONV_CONFIGS_MobileNetV1 = CFG(
 
 class MobileNetSeparableConv(nn.Sequential, SizeMixin):
     """
-
     similar to `_nets.SeparableConv`,
     the difference is that there are normalization and activation between depthwise conv and pointwise conv
 
     """
 
-    __DEBUG__ = True
     __name__ = "MobileNetSeparableConv"
 
     @deprecate_kwargs([["norm", "batch_norm"]])
@@ -82,9 +77,8 @@ class MobileNetSeparableConv(nn.Sequential, SizeMixin):
         depth_multiplier: int = 1,
         width_multiplier: float = 1.0,
         **kwargs: Any,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -204,9 +198,8 @@ class MobileNetSeparableConv(nn.Sequential, SizeMixin):
         return compute_sequential_output_shape(self, seq_len, batch_size)
 
 
-class MobileNetV1(nn.Sequential, SizeMixin):
+class MobileNetV1(nn.Sequential, SizeMixin, CitationMixin):
     """
-
     Similar to Xception, but without skip connections,
     separable convolutions are slightly different too
 
@@ -222,12 +215,10 @@ class MobileNetV1(nn.Sequential, SizeMixin):
 
     """
 
-    __DEBUG__ = True
     __name__ = "MobileNetV1"
 
-    def __init__(self, in_channels: int, **config) -> NoReturn:
+    def __init__(self, in_channels: int, **config) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -328,7 +319,6 @@ class MobileNetV1(nn.Sequential, SizeMixin):
         ordering: str = "cba",
     ) -> nn.Sequential:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -410,7 +400,6 @@ class MobileNetV1(nn.Sequential, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -441,10 +430,13 @@ class MobileNetV1(nn.Sequential, SizeMixin):
     def in_channels(self) -> int:
         return self.__in_channels
 
+    @property
+    def doi(self) -> List[str]:
+        return list(set(self.config.get("doi", []) + ["10.48550/ARXIV.1704.04861"]))
+
 
 class InvertedResidual(nn.Module, SizeMixin):
     """
-
     inverted residual block
 
     expansion (via pointwise conv) --> depthwise conv --> pointwise conv (without activation) ---> output
@@ -453,7 +445,6 @@ class InvertedResidual(nn.Module, SizeMixin):
 
     """
 
-    __DEBUG__ = True
     __name__ = "InvertedResidual"
 
     @deprecate_kwargs([["norm", "batch_norm"]])
@@ -470,9 +461,8 @@ class InvertedResidual(nn.Module, SizeMixin):
         activation: Optional[Union[str, nn.Module]] = "relu6",
         width_multiplier: float = 1.0,
         attn: Optional[CFG] = None,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -587,9 +577,8 @@ class InvertedResidual(nn.Module, SizeMixin):
         current_pos = self._add_attn_layer_if_needed(conv_in_channels, current_pos)
         _, self.__out_channels, _ = pw_linear.compute_output_shape()
 
-    def _add_attn_layer_if_needed(self, in_channels: int, current_pos: int) -> NoReturn:
+    def _add_attn_layer_if_needed(self, in_channels: int, current_pos: int) -> None:
         """
-
         add attention layer at the position specified by `self.__attn.pos`
 
         Parameters
@@ -610,7 +599,6 @@ class InvertedResidual(nn.Module, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -635,7 +623,6 @@ class InvertedResidual(nn.Module, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -658,9 +645,8 @@ class InvertedResidual(nn.Module, SizeMixin):
         return output_shape
 
 
-class MobileNetV2(nn.Sequential, SizeMixin):
+class MobileNetV2(nn.Sequential, SizeMixin, CitationMixin):
     """
-
     References
     ----------
     1. Sandler, M., Howard, A., Zhu, M., Zhmoginov, A., & Chen, L. C. (2018). Mobilenetv2: Inverted residuals and linear bottlenecks. In Proceedings of the IEEE conference on computer vision and pattern recognition (pp. 4510-4520).
@@ -669,12 +655,10 @@ class MobileNetV2(nn.Sequential, SizeMixin):
 
     """
 
-    __DEBUG__ = True
     __name__ = "MobileNetV2"
 
-    def __init__(self, in_channels: int, **config: CFG) -> NoReturn:
+    def __init__(self, in_channels: int, **config: CFG) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -725,10 +709,6 @@ class MobileNetV2(nn.Sequential, SizeMixin):
         super().__init__()
         self.__in_channels = in_channels
         self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
 
         # stem
         if isinstance(self.config.stem.num_filters, int):
@@ -837,11 +817,14 @@ class MobileNetV2(nn.Sequential, SizeMixin):
     def in_channels(self) -> int:
         return self.__in_channels
 
+    @property
+    def doi(self) -> List[str]:
+        return list(set(self.config.get("doi", []) + ["10.1109/cvpr.2018.00474"]))
+
 
 class InvertedResidualBlock(nn.Sequential, SizeMixin):
     """ """
 
-    __DEBUG__ = True
     __name__ = "InvertedResidualBlock"
 
     @deprecate_kwargs([["norm", "batch_norm"]])
@@ -862,9 +845,8 @@ class InvertedResidualBlock(nn.Sequential, SizeMixin):
         out_channels: Union[int, Sequence[int]] = None,
         attn: Optional[Union[CFG, Sequence[CFG]]] = None,
         **kwargs: Any,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -997,7 +979,6 @@ class InvertedResidualBlock(nn.Sequential, SizeMixin):
 class MobileNetV3_STEM(nn.Sequential, SizeMixin):
     """ """
 
-    __DEBUG__ = True
     __name__ = "MobileNetV3_STEM"
 
     @deprecate_kwargs([["norm", "batch_norm"]])
@@ -1012,9 +993,8 @@ class MobileNetV3_STEM(nn.Sequential, SizeMixin):
         ] = "relu",
         width_multiplier: Union[float, Sequence[float]] = 1.0,
         **config: CFG,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -1085,9 +1065,8 @@ class MobileNetV3_STEM(nn.Sequential, SizeMixin):
         return compute_sequential_output_shape(self, seq_len, batch_size)
 
 
-class MobileNetV3(nn.Sequential, SizeMixin):
+class MobileNetV3(nn.Sequential, SizeMixin, CitationMixin):
     """
-
     References
     ----------
     1. Howard, A., Sandler, M., Chu, G., Chen, L. C., Chen, B., Tan, M., ... & Adam, H. (2019). Searching for mobilenetv3. In Proceedings of the IEEE International Conference on Computer Vision (pp. 1314-1324).
@@ -1095,12 +1074,10 @@ class MobileNetV3(nn.Sequential, SizeMixin):
 
     """
 
-    __DEBUG__ = True
     __name__ = "MobileNetV3"
 
-    def __init__(self, in_channels: int, **config: CFG) -> NoReturn:
+    def __init__(self, in_channels: int, **config: CFG) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -1165,10 +1142,6 @@ class MobileNetV3(nn.Sequential, SizeMixin):
         super().__init__()
         self.__in_channels = in_channels
         self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
 
         # stem
         self.add_module(
@@ -1251,3 +1224,11 @@ class MobileNetV3(nn.Sequential, SizeMixin):
     ) -> Sequence[Union[int, None]]:
         """ """
         return compute_sequential_output_shape(self, seq_len, batch_size)
+
+    @property
+    def in_channels(self) -> int:
+        return self.__in_channels
+
+    @property
+    def doi(self) -> List[str]:
+        return list(set(self.config.get("doi", []) + ["10.1109/iccv.2019.00140"]))
