@@ -6,27 +6,21 @@ branched, and has different scope (in terms of dilation) in each branch
 from collections import OrderedDict
 from copy import deepcopy
 from itertools import repeat
-from typing import NoReturn, Optional, Sequence, Union
+from typing import Optional, Sequence, Union, List
 
 import numpy as np
 import torch
 from torch import Tensor, nn
 
-from ...cfg import CFG, DEFAULTS
-from ...models._nets import (  # noqa: F401
-    Conv_Bn_Activation,
-    DownSample,
-)
-from ...utils.misc import dict_to_str, list_sum, add_docstring
+from ...cfg import CFG
+from ...models._nets import Conv_Bn_Activation, DownSample
+from ...utils.misc import list_sum, add_docstring, CitationMixin
 from ...utils.utils_nn import (
     SizeMixin,
     compute_sequential_output_shape,
     compute_sequential_output_shape_docstring,
 )
 
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
-
 
 __all__ = [
     "MultiScopicCNN",
@@ -37,7 +31,6 @@ __all__ = [
 
 class MultiScopicBasicBlock(nn.Sequential, SizeMixin):
     """
-
     basic building block of the CNN part of the SOTA model
     from CPSC2019 challenge (entry 0416)
 
@@ -45,7 +38,6 @@ class MultiScopicBasicBlock(nn.Sequential, SizeMixin):
 
     """
 
-    __DEBUG__ = False
     __name__ = "MultiScopicBasicBlock"
 
     def __init__(
@@ -57,9 +49,8 @@ class MultiScopicBasicBlock(nn.Sequential, SizeMixin):
         subsample_length: int,
         groups: int = 1,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -136,7 +127,6 @@ class MultiScopicBasicBlock(nn.Sequential, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -155,7 +145,6 @@ class MultiScopicBasicBlock(nn.Sequential, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -181,7 +170,7 @@ class MultiScopicBasicBlock(nn.Sequential, SizeMixin):
 
     def _assign_weights_lead_wise(
         self, other: "MultiScopicBasicBlock", indices: Sequence[int]
-    ) -> NoReturn:
+    ) -> None:
         """ """
         assert not any([isinstance(m, nn.LayerNorm) for m in self]) and not any(
             [isinstance(m, nn.LayerNorm) for m in other]
@@ -213,13 +202,11 @@ class MultiScopicBasicBlock(nn.Sequential, SizeMixin):
 
 class MultiScopicBranch(nn.Sequential, SizeMixin):
     """
-
     branch path of the CNN part of the SOTA model
     from CPSC2019 challenge (entry 0416)
 
     """
 
-    __DEBUG__ = False
     __name__ = "MultiScopicBranch"
 
     def __init__(
@@ -231,9 +218,8 @@ class MultiScopicBranch(nn.Sequential, SizeMixin):
         subsample_lengths: Union[int, Sequence[int]],
         groups: int = 1,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -313,7 +299,7 @@ class MultiScopicBranch(nn.Sequential, SizeMixin):
 
     def _assign_weights_lead_wise(
         self, other: "MultiScopicBranch", indices: Sequence[int]
-    ) -> NoReturn:
+    ) -> None:
         """ """
         for blk, o_blk in zip(self, other):
             blk._assign_weights_lead_wise(o_blk, indices)
@@ -323,15 +309,13 @@ class MultiScopicBranch(nn.Sequential, SizeMixin):
         return self.__in_channels
 
 
-class MultiScopicCNN(nn.Module, SizeMixin):
+class MultiScopicCNN(nn.Module, SizeMixin, CitationMixin):
     """CNN part of the SOTA model from CPSC2019 challenge (entry 0416)"""
 
-    __DEBUG__ = False
     __name__ = "MultiScopicCNN"
 
-    def __init__(self, in_channels: int, **config) -> NoReturn:
+    def __init__(self, in_channels: int, **config) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -370,11 +354,6 @@ class MultiScopicCNN(nn.Module, SizeMixin):
         self.__scopes = self.config.scopes
         self.__num_branches = len(self.__scopes)
 
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
-
         self.branches = nn.ModuleDict()
         for idx in range(self.__num_branches):
             self.branches[f"branch_{idx}"] = MultiScopicBranch(
@@ -390,7 +369,6 @@ class MultiScopicCNN(nn.Module, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -420,7 +398,6 @@ class MultiScopicCNN(nn.Module, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -446,9 +423,8 @@ class MultiScopicCNN(nn.Module, SizeMixin):
 
     def assign_weights_lead_wise(
         self, other: "MultiScopicCNN", indices: Sequence[int]
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Assign weights to the `other` MultiScopicCNN module in the lead-wise manner
 
         Parameters
@@ -485,7 +461,7 @@ class MultiScopicCNN(nn.Module, SizeMixin):
         >>> model6.eval()
         >>> # we create tensor12, tensor6 to check the correctness of the assignment of the weights
         >>> tensor12 = torch.zeros(1, 12, 200)  # batch, leads, seq_len
-        >>> tensor6 = torch.rand(1, 6, 200)
+        >>> tensor6 = torch.randn(1, 6, 200)
         >>> # we make tensor12 has identical values as tensor6 at the given leads, and let the other leads have zero values
         >>> tensor12[:, indices, :] = tensor6
         >>> b = "branch_0"  # similarly for other branches
@@ -527,3 +503,7 @@ class MultiScopicCNN(nn.Module, SizeMixin):
     @property
     def num_branches(self) -> int:
         return self.__num_branches
+
+    @property
+    def doi(self) -> List[str]:
+        return list(set(self.config.get("doi", []) + ["10.1109/access.2020.2997473"]))
