@@ -6,13 +6,12 @@ whose performance however seems exceeded by newer networks
 from copy import deepcopy
 from itertools import repeat
 from numbers import Real
-from typing import NoReturn, Optional, Sequence, Union
+from typing import Optional, Sequence, Union, List
 
-import torch
 import torch.nn.functional as F
 from torch import Tensor, nn
 
-from ...cfg import CFG, DEFAULTS
+from ...cfg import CFG
 from ...models._nets import (  # noqa: F401
     Activations,
     Conv_Bn_Activation,
@@ -22,16 +21,13 @@ from ...models._nets import (  # noqa: F401
     ZeroPadding,
     make_attention_layer,
 )
-from ...utils.misc import dict_to_str, add_docstring
+from ...utils.misc import add_docstring, CitationMixin
 from ...utils.utils_nn import (
     SizeMixin,
     compute_sequential_output_shape,
     compute_sequential_output_shape_docstring,
 )
 
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
-
 
 __all__ = [
     "ResNet",
@@ -43,10 +39,8 @@ __all__ = [
 class ResNetBasicBlock(nn.Module, SizeMixin):
     """
     building blocks for `ResNet`, as implemented in ref. [2] of `ResNet`
-
     """
 
-    __DEBUG__ = False
     __name__ = "ResNetBasicBlock"
     expansion = 1  # not used
 
@@ -60,9 +54,8 @@ class ResNetBasicBlock(nn.Module, SizeMixin):
         dilation: int = 1,
         attn: Optional[dict] = None,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -103,10 +96,6 @@ class ResNetBasicBlock(nn.Module, SizeMixin):
         self.__stride = subsample_length
         self.__groups = groups
         self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
 
         if (
             self.config.increase_channels_method.lower() == "zero_padding"
@@ -176,10 +165,6 @@ class ResNetBasicBlock(nn.Module, SizeMixin):
 
     def _make_shortcut_layer(self) -> Union[nn.Module, None]:
         """ """
-        if self.__DEBUG__:
-            print(
-                f"down_scale = {self.__down_scale}, increase_channels = {self.__increase_channels}"
-            )
         if self.__down_scale > 1 or self.__increase_channels:
             if self.config.increase_channels_method.lower() == "conv":
                 shortcut = DownSample(
@@ -212,7 +197,6 @@ class ResNetBasicBlock(nn.Module, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -251,7 +235,6 @@ class ResNetBasicBlock(nn.Module, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -276,13 +259,11 @@ class ResNetBasicBlock(nn.Module, SizeMixin):
 
 class ResNetBottleNeck(nn.Module, SizeMixin):
     """
-
     bottle neck blocks for `ResNet`, as implemented in ref. [2] of `ResNet`,
     as for 1D ECG, should be of the "baby-giant-baby" pattern?
 
     """
 
-    __DEBUG__ = False
     __name__ = "ResNetBottleNeck"
     expansion = 4
     __DEFAULT_BASE_WIDTH__ = 12 * 4
@@ -300,9 +281,8 @@ class ResNetBottleNeck(nn.Module, SizeMixin):
         base_filter_length: int = 1,
         attn: Optional[dict] = None,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -344,10 +324,6 @@ class ResNetBottleNeck(nn.Module, SizeMixin):
         super().__init__()
         self.__num_convs = 3
         self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
         self.expansion = self.config.get("expansion", self.expansion)
 
         self.__in_channels = in_channels
@@ -363,12 +339,6 @@ class ResNetBottleNeck(nn.Module, SizeMixin):
             neck_num_filters,
             num_filters * self.expansion,
         ]
-        if self.__DEBUG__:
-            print(
-                f"__DEFAULT_BASE_WIDTH__ = {self.__DEFAULT_BASE_WIDTH__}, "
-                f"in_channels = {in_channels}, num_filters = {num_filters}, "
-                f"base_width = {base_width}, neck_num_filters = {neck_num_filters}."
-            )
         self.__base_filter_length = base_filter_length
         self.__kernel_size = [
             base_filter_length,
@@ -462,10 +432,6 @@ class ResNetBottleNeck(nn.Module, SizeMixin):
 
     def _make_shortcut_layer(self) -> Union[nn.Module, None]:
         """ """
-        if self.__DEBUG__:
-            print(
-                f"down_scale = {self.__down_scale}, increase_channels = {self.__increase_channels}"
-            )
         if self.__down_scale > 1 or self.__increase_channels:
             if self.config.increase_channels_method.lower() == "conv":
                 shortcut = DownSample(
@@ -498,7 +464,6 @@ class ResNetBottleNeck(nn.Module, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -537,7 +502,6 @@ class ResNetBottleNeck(nn.Module, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -560,26 +524,6 @@ class ResNetBottleNeck(nn.Module, SizeMixin):
         return output_shape
 
 
-class ResNetMacroBlock(nn.Sequential, SizeMixin):
-    """NOT finished, NOT checked,"""
-
-    __DEBUG__ = True
-    __name__ = "ResNetMacroBlock"
-
-    def __init__(self) -> NoReturn:
-        """ """
-        super().__init__()
-        raise NotImplementedError
-
-    def forward(self):
-        """ """
-        raise NotImplementedError
-
-    def compute_output_shape(self):
-        """ """
-        raise NotImplementedError
-
-
 class ResNetStem(nn.Sequential, SizeMixin):
     """
     the input stem of ResNet
@@ -598,9 +542,8 @@ class ResNetStem(nn.Sequential, SizeMixin):
         subsample_mode: str = "max",
         groups: int = 1,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         f"""
-
         Parameters
         ----------
         in_channels: int,
@@ -680,9 +623,8 @@ class ResNetStem(nn.Sequential, SizeMixin):
         return compute_sequential_output_shape(self, seq_len, batch_size)
 
 
-class ResNet(nn.Sequential, SizeMixin):
+class ResNet(nn.Sequential, SizeMixin, CitationMixin):
     """
-
     References
     ----------
     [1] https://github.com/awni/ecg
@@ -695,7 +637,6 @@ class ResNet(nn.Sequential, SizeMixin):
 
     """
 
-    __DEBUG__ = False
     __name__ = "ResNet"
     building_block = ResNetBasicBlock
     __DEFAULT_CONFIG__ = CFG(
@@ -706,9 +647,8 @@ class ResNet(nn.Sequential, SizeMixin):
         dropouts=0,
     )
 
-    def __init__(self, in_channels: int, **config) -> NoReturn:
+    def __init__(self, in_channels: int, **config) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -742,10 +682,6 @@ class ResNet(nn.Sequential, SizeMixin):
         self.__in_channels = in_channels
         self.config = CFG(deepcopy(self.__DEFAULT_CONFIG__))
         self.config.update(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
         if not isinstance(self.config.get("building_block", None), str):
             # for those architectures that have multiple type of building blocks
             self.building_block = []
@@ -794,8 +730,6 @@ class ResNet(nn.Sequential, SizeMixin):
             )
         else:
             self.additional_kw = CFG()
-        if self.__DEBUG__:
-            print(f"additional_kw = {self.additional_kw}")
 
         self.add_module(
             "input_stem",
@@ -887,6 +821,7 @@ class ResNet(nn.Sequential, SizeMixin):
                 bb_config = self.config.block
                 bb_kw = self.additional_kw
             for block_idx in range(nb):
+                dropout = bb_config.pop("dropout", self.__dropouts[macro_idx])
                 self.add_module(
                     f"{bb.__name__}_{macro_idx}_{block_idx}",
                     bb(
@@ -896,7 +831,7 @@ class ResNet(nn.Sequential, SizeMixin):
                         subsample_length=block_subsample_lengths[block_idx],
                         groups=self.config.groups,
                         dilation=1,
-                        dropout=self.__dropouts[macro_idx],
+                        dropout=dropout,
                         **(bb_kw),
                         **(bb_config),
                     ),
@@ -914,3 +849,17 @@ class ResNet(nn.Sequential, SizeMixin):
     @property
     def in_channels(self) -> int:
         return self.__in_channels
+
+    @property
+    def doi(self) -> List[str]:
+        return list(
+            set(
+                self.config.get("doi", [])
+                + [
+                    "10.1109/cvpr.2016.90",
+                    "10.1038/s41591-018-0268-3",
+                    "10.1038/s41467-020-15432-4",
+                    "10.1088/1361-6579/ac6aa3",
+                ]
+            )
+        )
