@@ -3,19 +3,13 @@ the most basic CNN
 """
 
 from copy import deepcopy
-from typing import NoReturn, Optional, Sequence, Union
+from typing import Optional, Sequence, Union, List
 
-import torch
 from torch import nn
 
-from ...cfg import CFG, DEFAULTS
-from ...models._nets import (  # noqa: F401
-    Conv_Bn_Activation,
-    GlobalContextBlock,
-    NonLocalBlock,
-    SEBlock,
-)
-from ...utils.misc import dict_to_str, add_docstring
+from ...cfg import CFG
+from ...models._nets import Conv_Bn_Activation
+from ...utils.misc import add_docstring, CitationMixin
 from ...utils.utils_nn import (
     SizeMixin,
     compute_maxpool_output_shape,
@@ -23,12 +17,8 @@ from ...utils.utils_nn import (
     compute_sequential_output_shape_docstring,
 )
 
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
-
 
 __all__ = [
-    "VGGBlock",
     "VGG16",
 ]
 
@@ -36,7 +26,6 @@ __all__ = [
 class VGGBlock(nn.Sequential, SizeMixin):
     """building blocks of the CNN feature extractor `VGG16`"""
 
-    __DEBUG__ = False
     __name__ = "VGGBlock"
 
     def __init__(
@@ -46,9 +35,8 @@ class VGGBlock(nn.Sequential, SizeMixin):
         out_channels: int,
         groups: int = 1,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         num_convs: int,
@@ -72,10 +60,6 @@ class VGGBlock(nn.Sequential, SizeMixin):
         self.__out_channels = out_channels
         self.__groups = groups
         self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
 
         self.add_module(
             "cba_1",
@@ -116,7 +100,6 @@ class VGGBlock(nn.Sequential, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -128,6 +111,7 @@ class VGGBlock(nn.Sequential, SizeMixin):
         -------
         output_shape: sequence,
             the output shape, given `seq_len` and `batch_size`
+
         """
         num_layers = 0
         for module in self:
@@ -138,25 +122,22 @@ class VGGBlock(nn.Sequential, SizeMixin):
                 output_shape = compute_maxpool_output_shape(
                     input_shape=[batch_size, self.__out_channels, seq_len],
                     kernel_size=self.config.pool_size,
-                    stride=self.config.pool_size,
+                    stride=self.config.pool_stride,
                     channel_last=False,
                 )
             num_layers += 1
         return output_shape
 
 
-class VGG16(nn.Sequential, SizeMixin):
+class VGG16(nn.Sequential, SizeMixin, CitationMixin):
     """
-
     CNN feature extractor of the CRNN models proposed in refs of `ECG_CRNN`
     """
 
-    __DEBUG__ = False
     __name__ = "VGG16"
 
-    def __init__(self, in_channels: int, **config) -> NoReturn:
+    def __init__(self, in_channels: int, **config) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -181,10 +162,6 @@ class VGG16(nn.Sequential, SizeMixin):
         self.__in_channels = in_channels
         # self.config = deepcopy(ECG_CRNN_CONFIG.cnn.vgg16)
         self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
 
         module_in_channels = in_channels
         for idx, (nc, nf) in enumerate(
@@ -213,3 +190,12 @@ class VGG16(nn.Sequential, SizeMixin):
     @property
     def in_channels(self) -> int:
         return self.__in_channels
+
+    @property
+    def doi(self) -> List[str]:
+        return list(
+            set(
+                self.config.get("doi", [])
+                + ["10.48550/ARXIV.1409.1556", "10.1016/j.inffus.2019.06.024"]
+            )
+        )
