@@ -7,12 +7,11 @@ however seems not have been used in physiological signal processing tasks
 from copy import deepcopy
 from itertools import repeat
 from numbers import Real
-from typing import NoReturn, Optional, Sequence, Union
+from typing import Optional, Sequence, Union, List
 
-import torch
 from torch import Tensor, nn
 
-from ...cfg import CFG, DEFAULTS
+from ...cfg import CFG
 from ...models._nets import (  # noqa: F401
     Conv_Bn_Activation,
     DownSample,
@@ -22,16 +21,13 @@ from ...models._nets import (  # noqa: F401
     SEBlock,
     SeparableConv,
 )
-from ...utils.misc import dict_to_str, add_docstring
+from ...utils.misc import add_docstring, CitationMixin
 from ...utils.utils_nn import (
     SizeMixin,
     compute_sequential_output_shape,
     compute_sequential_output_shape_docstring,
 )
 
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
-
 
 __all__ = [
     "Xception",
@@ -54,14 +50,12 @@ _DEFAULT_CONV_CONFIGS = CFG(
 )
 
 
-class XceptionMultiConv(nn.Module, SizeMixin):
+class XceptionMultiConv(nn.Module, SizeMixin, CitationMixin):
     """
-
     -> n(2 or 3) x (activation -> norm -> sep_conv) (-> optional sub-sample) ->
     |-------------------------------- shortcut ------------------------------|
     """
 
-    __DEBUG__ = False
     __name__ = "XceptionMultiConv"
 
     def __init__(
@@ -75,9 +69,8 @@ class XceptionMultiConv(nn.Module, SizeMixin):
         groups: int = 1,
         dropouts: Union[Sequence[float], float] = 0.0,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -103,6 +96,7 @@ class XceptionMultiConv(nn.Module, SizeMixin):
             activation choices, weight initializer, batch normalization choices, etc.,
             for the convolutional layers,
             and subsampling modes for subsampling layers, etc.
+
         """
         super().__init__()
         self.__in_channels = in_channels
@@ -169,7 +163,6 @@ class XceptionMultiConv(nn.Module, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -179,6 +172,7 @@ class XceptionMultiConv(nn.Module, SizeMixin):
         -------
         output: Tensor,
             of shape (batch_size, n_channels, seq_len)
+
         """
         main_out = self.main_stream_conv(input)
         if self.subsample:
@@ -193,7 +187,6 @@ class XceptionMultiConv(nn.Module, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -205,6 +198,7 @@ class XceptionMultiConv(nn.Module, SizeMixin):
         -------
         output_shape: sequence,
             the output shape of this `MultiConv` layer, given `seq_len` and `batch_size`
+
         """
         output_shape = self.main_stream_conv.compute_output_shape(seq_len, batch_size)
         if self.subsample is not None:
@@ -216,13 +210,12 @@ class XceptionMultiConv(nn.Module, SizeMixin):
 
 class XceptionEntryFlow(nn.Sequential, SizeMixin):
     """
-
     Entry flow of the Xception model,
     consisting of 2 initial convolutions which subsamples at the first one,
     followed by several Xception blocks of 2 convolutions and of sub-sampling size 2
+
     """
 
-    __DEBUG__ = False
     __name__ = "XceptionEntryFlow"
 
     def __init__(
@@ -240,9 +233,8 @@ class XceptionEntryFlow(nn.Sequential, SizeMixin):
         dropouts: Union[float, Sequence[float], Sequence[Sequence[float]]] = 0.0,
         block_dropouts: Union[float, Sequence[float]] = 0.0,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -274,6 +266,7 @@ class XceptionEntryFlow(nn.Sequential, SizeMixin):
             activation choices, weight initializer, batch normalization choices, etc.
             for the convolutional layers,
             and subsampling modes for subsampling layers, etc.
+
         """
         super().__init__()
         self.__in_channels = in_channels
@@ -379,7 +372,6 @@ class XceptionEntryFlow(nn.Sequential, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -389,6 +381,7 @@ class XceptionEntryFlow(nn.Sequential, SizeMixin):
         -------
         output: Tensor,
             of shape (batch_size, n_channels, seq_len)
+
         """
         output = super().forward(input)
         return output
@@ -397,7 +390,6 @@ class XceptionEntryFlow(nn.Sequential, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -409,6 +401,7 @@ class XceptionEntryFlow(nn.Sequential, SizeMixin):
         -------
         output_shape: sequence,
             the output shape of this `MultiConv` layer, given `seq_len` and `batch_size`
+
         """
         _seq_len = seq_len
         for module in self:
@@ -421,12 +414,11 @@ class XceptionEntryFlow(nn.Sequential, SizeMixin):
 
 class XceptionMiddleFlow(nn.Sequential, SizeMixin):
     """
-
     Middle flow of the Xception model,
     consisting of several Xception blocks of 3 convolutions and without sub-sampling
+
     """
 
-    __DEBUG__ = False
     __name__ = "XceptionMiddleFlow"
 
     def __init__(
@@ -439,9 +431,8 @@ class XceptionMiddleFlow(nn.Sequential, SizeMixin):
         dropouts: Union[float, Sequence[float], Sequence[Sequence[float]]] = 0.0,
         block_dropouts: Union[float, Sequence[float]] = 0.0,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -463,6 +454,7 @@ class XceptionMiddleFlow(nn.Sequential, SizeMixin):
             activation choices, weight initializer, batch normalization choices, etc.
             for the convolutional layers,
             and subsampling modes for subsampling layers, etc.
+
         """
         super().__init__()
         self.__in_channels = in_channels
@@ -527,7 +519,6 @@ class XceptionMiddleFlow(nn.Sequential, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -537,6 +528,7 @@ class XceptionMiddleFlow(nn.Sequential, SizeMixin):
         -------
         output: Tensor,
             of shape (batch_size, n_channels, seq_len)
+
         """
         output = super().forward(input)
         return output
@@ -545,7 +537,6 @@ class XceptionMiddleFlow(nn.Sequential, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -557,6 +548,7 @@ class XceptionMiddleFlow(nn.Sequential, SizeMixin):
         -------
         output_shape: sequence,
             the output shape of this `MultiConv` layer, given `seq_len` and `batch_size`
+
         """
         _seq_len = seq_len
         for module in self:
@@ -569,13 +561,12 @@ class XceptionMiddleFlow(nn.Sequential, SizeMixin):
 
 class XceptionExitFlow(nn.Sequential, SizeMixin):
     """
-
     Exit flow of the Xception model,
     consisting of several Xception blocks of 2 convolutions,
     followed by several separable convolutions
+
     """
 
-    __DEBUG__ = False
     __name__ = "XceptionExitFlow"
 
     def __init__(
@@ -592,9 +583,8 @@ class XceptionExitFlow(nn.Sequential, SizeMixin):
         dropouts: Union[float, Sequence[float], Sequence[Sequence[float]]] = 0.0,
         block_dropouts: Union[float, Sequence[float]] = 0.0,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -626,6 +616,7 @@ class XceptionExitFlow(nn.Sequential, SizeMixin):
             activation choices, weight initializer, batch normalization choices, etc.
             for the convolutional layers,
             and subsampling modes for subsampling layers, etc.
+
         """
         super().__init__()
         self.__in_channels = in_channels
@@ -726,7 +717,6 @@ class XceptionExitFlow(nn.Sequential, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -736,6 +726,7 @@ class XceptionExitFlow(nn.Sequential, SizeMixin):
         -------
         output: Tensor,
             of shape (batch_size, n_channels, seq_len)
+
         """
         output = super().forward(input)
         return output
@@ -744,7 +735,6 @@ class XceptionExitFlow(nn.Sequential, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -756,6 +746,7 @@ class XceptionExitFlow(nn.Sequential, SizeMixin):
         -------
         output_shape: sequence,
             the output shape of this `MultiConv` layer, given `seq_len` and `batch_size`
+
         """
         _seq_len = seq_len
         for module in self:
@@ -766,22 +757,20 @@ class XceptionExitFlow(nn.Sequential, SizeMixin):
         return output_shape
 
 
-class Xception(nn.Sequential, SizeMixin):
+class Xception(nn.Sequential, SizeMixin, CitationMixin):
     """
-
     References
     ----------
     [1] Chollet, François. "Xception: Deep learning with depthwise separable convolutions." Proceedings of the IEEE conference on computer vision and pattern recognition. 2017.
     [2] https://github.com/keras-team/keras-applications/blob/master/keras_applications/xception.py
     [3] https://github.com/Cadene/pretrained-models.pytorch/blob/master/pretrainedmodels/models/xception.py
+
     """
 
-    __DEBUG__ = False
     __name__ = "Xception"
 
-    def __init__(self, in_channels: int, **config) -> NoReturn:
+    def __init__(self, in_channels: int, **config) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -791,14 +780,11 @@ class Xception(nn.Sequential, SizeMixin):
             key word arguments that have to be set in 3 sub-dict,
             namely in "entry_flow", "middle_flow", and "exit_flow",
             ref. corresponding docstring of each class
+
         """
         super().__init__()
         self.__in_channels = in_channels
         self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
 
         entry_flow_in_channels = self.__in_channels
         entry_flow = XceptionEntryFlow(
@@ -823,7 +809,6 @@ class Xception(nn.Sequential, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -833,6 +818,7 @@ class Xception(nn.Sequential, SizeMixin):
         -------
         output: Tensor,
             of shape (batch_size, n_channels, seq_len)
+
         """
         output = super().forward(input)
         return output
@@ -847,3 +833,7 @@ class Xception(nn.Sequential, SizeMixin):
     @property
     def in_channels(self) -> int:
         return self.__in_channels
+
+    @property
+    def doi(self) -> List[str]:
+        return list(set(self.config.get("doi", []) + ["10.1109/cvpr.2017.195"]))
