@@ -7,17 +7,12 @@ References
 https://github.com/jacobgil/pytorch-grad-cam
 """
 
-from typing import List, NoReturn, Optional, Sequence, Tuple
+from typing import List, Optional, Sequence, Tuple
 
 import numpy as np
 import torch
 from torch import Tensor, nn
 
-from ..cfg import DEFAULTS
-
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
-
 
 __all__ = [
     "GradCam",
@@ -30,19 +25,19 @@ class FeatureExtractor(object):
     registering gradients from targetted intermediate layers
     """
 
-    def __init__(self, model: nn.Module, target_layers: Sequence[str]) -> NoReturn:
+    def __init__(self, model: nn.Module, target_layers: Sequence[str]) -> None:
         """
-
         Parameters
         ----------
         model: Module,
         target_layers: sequence of str,
+
         """
         self.model = model
         self.target_layers = target_layers
         self.gradients = []
 
-    def save_gradient(self, grad: Tensor) -> NoReturn:
+    def save_gradient(self, grad: Tensor) -> None:
         """ """
         self.gradients.append(grad)
 
@@ -65,11 +60,12 @@ class ModelOutputs(object):
     1. The network output.
     2. Activations from intermeddiate targetted layers.
     3. Gradients from intermeddiate targetted layers.
+
     """
 
     def __init__(
         self, model: nn.Module, feature_module: nn.Module, target_layers: Sequence[str]
-    ) -> NoReturn:
+    ) -> None:
         """ """
         self.model = model
         self.feature_module = feature_module
@@ -97,7 +93,6 @@ class ModelOutputs(object):
 class GradCam(object):
     """NOT finished,"""
 
-    __DEBUG__ = True
     __name__ = "GradCam"
 
     def __init__(
@@ -107,9 +102,8 @@ class GradCam(object):
         target_layer_names: Sequence[str],
         target_channel_last: bool = False,
         device: str = "cpu",
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         to write
