@@ -1,16 +1,15 @@
 """
 """
 
-from typing import Any, NoReturn, Optional, Sequence, Union
+import warnings
+from typing import Any, Optional, Sequence, Union
 
 import torch
 import torch.nn as nn
+from einops.layers.torch import Rearrange
 
-from ...cfg import DEFAULTS
 from ...utils.utils_nn import SizeMixin
-
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
+from ...utils.misc import get_kwargs
 
 
 __all__ = [
@@ -21,7 +20,6 @@ __all__ = [
 class Transformer(nn.Module, SizeMixin):
     """ """
 
-    __DEBUG__ = True
     __name__ = "Transformer"
 
     def __init__(
@@ -31,10 +29,10 @@ class Transformer(nn.Module, SizeMixin):
         num_heads: int = 8,
         num_layers: int = 1,
         dropout: float = 0.1,
+        batch_first: bool = True,
         **kwargs: Any,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         input_size: int,
@@ -47,6 +45,8 @@ class Transformer(nn.Module, SizeMixin):
             number of encoding layers
         dropout: float, default 0.1,
             dropout probability
+        batch_first: bool, default True,
+            whether the input is of shape (batch_size, seq_len, input_size)
         kwargs: keyword arguments,
 
         """
@@ -55,8 +55,32 @@ class Transformer(nn.Module, SizeMixin):
         self.__hidden_size = hidden_size
         self.__num_layers = num_layers
         self.__num_heads = num_heads
-        self.__batch_first = kwargs.get("batch_first", False)
-        try:
+        self.__batch_first = batch_first
+
+        if self.__input_size % self.__num_heads != 0:
+            input_size = self.__input_size
+            self.__input_size = self.__input_size // self.__num_heads * self.__num_heads
+            warnings.warn(
+                f"`input_size` {input_size} is not divisible by `num_heads` {self.__num_heads}, "
+                f"adjusted to {self.__input_size}",
+                RuntimeWarning,
+            )
+            if self.__batch_first:
+                self.project = nn.Linear(input_size, self.__input_size)
+            else:
+                self.project = nn.Sequential(
+                    Rearrange(
+                        "seq_len batch_size input_size -> batch_size seq_len input_size"
+                    ),
+                    nn.Linear(input_size, self.__input_size),
+                    Rearrange(
+                        "batch_size seq_len input_size -> seq_len batch_size input_size"
+                    ),
+                )
+        else:
+            self.project = nn.Identity()
+
+        if "batch_first" in get_kwargs(nn.TransformerEncoderLayer):
             encoder_layer = nn.TransformerEncoderLayer(
                 d_model=self.__input_size,
                 nhead=self.__num_heads,
@@ -65,8 +89,7 @@ class Transformer(nn.Module, SizeMixin):
                 batch_first=self.__batch_first,
                 activation=kwargs.get("activation", "relu"),
             )
-        except Exception:
-            self.__batch_first = False
+        else:
             encoder_layer = nn.TransformerEncoderLayer(
                 d_model=self.__input_size,
                 nhead=self.__num_heads,
@@ -74,14 +97,18 @@ class Transformer(nn.Module, SizeMixin):
                 dropout=dropout,
                 activation=kwargs.get("activation", "relu"),
             )
-            print("batch_first only supports torch >= 1.9, defaults to False")
+            if self.__batch_first:
+                warnings.warn(
+                    "`batch_first` only supports torch >= 1.9, defaults to False",
+                    RuntimeWarning,
+                )
+            self.__batch_first = False
         self.encoder = nn.TransformerEncoder(
             encoder_layer, num_layers=self.__num_layers
         )
 
     def forward(self, x: torch.Tensor) -> torch.Tensor:
         """
-
         Parameters
         ----------
         x: torch.Tensor,
@@ -94,13 +121,13 @@ class Transformer(nn.Module, SizeMixin):
             of shape (seq_len, batch_size, input_size) or (batch_size, seq_len, input_size)
 
         """
+        x = self.project(x)
         return self.encoder(x)
 
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -118,3 +145,8 @@ class Transformer(nn.Module, SizeMixin):
             return (batch_size, seq_len, self.__input_size)
         else:
             return (seq_len, batch_size, self.__input_size)
+
+    @property
+    def batch_first(self) -> bool:
+        """ """
+        return self.__batch_first
