@@ -8,22 +8,23 @@ the main differences to a normal Unet are that
 
 """
 
+import warnings
 from copy import deepcopy
 from itertools import repeat
-from typing import NoReturn, Optional, Sequence, Union
+from typing import Optional, Union, Sequence, List
 
 import numpy as np
 import torch
 from torch import Tensor, nn
 
-from ...cfg import CFG, DEFAULTS
+from ...cfg import CFG
 from ...models._nets import (  # noqa: F401
     BranchedConv,
     Conv_Bn_Activation,
     DownSample,
     MultiConv,
 )
-from ...utils.misc import dict_to_str, add_docstring
+from ...utils.misc import add_docstring
 from ...utils.utils_nn import (
     CkptMixin,
     SizeMixin,
@@ -31,9 +32,7 @@ from ...utils.utils_nn import (
     compute_sequential_output_shape,
     compute_sequential_output_shape_docstring,
 )
-
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
+from ...model_configs import ECG_SUBTRACT_UNET_CONFIG
 
 
 __all__ = [
@@ -43,11 +42,9 @@ __all__ = [
 
 class TripleConv(MultiConv):
     """
-
     CBA --> (Dropout) --> CBA --> (Dropout) --> CBA --> (Dropout)
     """
 
-    __DEBUG__ = False
     __name__ = "TripleConv"
 
     def __init__(
@@ -60,9 +57,8 @@ class TripleConv(MultiConv):
         dropouts: Union[Sequence[float], float] = 0.0,
         out_activation: bool = True,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -84,6 +80,7 @@ class TripleConv(MultiConv):
             other parameters, including
             activation choices, weight initializer, batch normalization choices, etc.
             for the convolutional layers
+
         """
         _num_convs = 3
         if isinstance(out_channels, int):
@@ -91,10 +88,6 @@ class TripleConv(MultiConv):
         else:
             _out_channels = list(out_channels)
             assert _num_convs == len(_out_channels)
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(config)}"
-            )
 
         super().__init__(
             in_channels=in_channels,
@@ -111,7 +104,6 @@ class TripleConv(MultiConv):
 class DownTripleConv(nn.Sequential, SizeMixin):
     """ """
 
-    __DEBUG__ = False
     __name__ = "DownTripleConv"
     __MODES__ = deepcopy(DownSample.__MODES__)
 
@@ -125,9 +117,8 @@ class DownTripleConv(nn.Sequential, SizeMixin):
         dropouts: Union[Sequence[float], float] = 0.0,
         mode: str = "max",
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         down_scale: int,
@@ -148,6 +139,7 @@ class DownTripleConv(nn.Sequential, SizeMixin):
             other parameters, including
             activation choices, weight initializer, batch normalization choices, etc.
             for the convolutional layers
+
         """
         super().__init__()
         self.__mode = mode.lower()
@@ -156,10 +148,6 @@ class DownTripleConv(nn.Sequential, SizeMixin):
         self.__in_channels = in_channels
         self.__out_channels = out_channels
         self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
 
         self.add_module(
             "down_sample",
@@ -185,7 +173,6 @@ class DownTripleConv(nn.Sequential, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -195,6 +182,7 @@ class DownTripleConv(nn.Sequential, SizeMixin):
         -------
         out: Tensor,
             of shape (batch_size, channels, seq_len)
+
         """
         out = super().forward(input)
         return out
@@ -212,7 +200,6 @@ class DownBranchedDoubleConv(nn.Module, SizeMixin):
     the bottom block of the `subtract_unet`
     """
 
-    __DEBUG__ = False
     __name__ = "DownBranchedDoubleConv"
     __MODES__ = deepcopy(DownSample.__MODES__)
 
@@ -227,9 +214,8 @@ class DownBranchedDoubleConv(nn.Module, SizeMixin):
         dropouts: Union[Sequence[float], float] = 0.0,
         mode: str = "max",
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         down_scale: int,
@@ -250,6 +236,7 @@ class DownBranchedDoubleConv(nn.Module, SizeMixin):
             other parameters, including
             activation choices, weight initializer, batch normalization choices, etc.
             for the convolutional layers
+
         """
         super().__init__()
         self.__mode = mode.lower()
@@ -258,10 +245,6 @@ class DownBranchedDoubleConv(nn.Module, SizeMixin):
         self.__in_channels = in_channels
         self.__out_channels = out_channels
         self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
 
         self.down_sample = DownSample(
             down_scale=self.__down_scale,
@@ -282,7 +265,6 @@ class DownBranchedDoubleConv(nn.Module, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -292,6 +274,7 @@ class DownBranchedDoubleConv(nn.Module, SizeMixin):
         -------
         out: Tensor,
             of shape (batch_size, channels, seq_len)
+
         """
         out = self.down_sample(input)
         out = self.branched_conv(out)
@@ -306,7 +289,6 @@ class DownBranchedDoubleConv(nn.Module, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -318,6 +300,7 @@ class DownBranchedDoubleConv(nn.Module, SizeMixin):
         -------
         output_shape: sequence,
             the output shape of this `DownDoubleConv` layer, given `seq_len` and `batch_size`
+
         """
         _seq_len = seq_len
         output_shape = self.down_sample.compute_output_shape(seq_len=_seq_len)
@@ -342,9 +325,9 @@ class UpTripleConv(nn.Module, SizeMixin):
     extra input
 
     channels are shrinked after up sampling
+
     """
 
-    __DEBUG__ = False
     __name__ = "UpTripleConv"
     __MODES__ = [
         "nearest",
@@ -364,7 +347,7 @@ class UpTripleConv(nn.Module, SizeMixin):
         dropouts: Union[Sequence[float], float] = 0.0,
         mode: str = "deconv",
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """finished, NOT checked,
 
         Parameters
@@ -390,6 +373,7 @@ class UpTripleConv(nn.Module, SizeMixin):
             other parameters, including
             activation choices, weight initializer, batch normalization choices, etc.
             for the deconvolutional layers
+
         """
         super().__init__()
         self.__up_scale = up_scale
@@ -399,10 +383,6 @@ class UpTripleConv(nn.Module, SizeMixin):
         self.__mode = mode.lower()
         assert self.__mode in self.__MODES__
         self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
 
         # the following has to be checked
         # if bilinear, use the normal convolutions to reduce the number of channels
@@ -435,13 +415,13 @@ class UpTripleConv(nn.Module, SizeMixin):
 
     def forward(self, input: Tensor, down_output: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
             input tensor from the previous layer
         down_output:Tensor: Tensor,
             input tensor of the last layer of corr. down block
+
         """
         output = self.up(input)
         output = torch.cat(
@@ -455,7 +435,6 @@ class UpTripleConv(nn.Module, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -467,6 +446,7 @@ class UpTripleConv(nn.Module, SizeMixin):
         -------
         output_shape: sequence,
             the output shape of this `DownDoubleConv` layer, given `seq_len` and `batch_size`
+
         """
         _sep_len = seq_len
         if self.__mode == "deconv":
@@ -486,37 +466,40 @@ class UpTripleConv(nn.Module, SizeMixin):
 
 class ECG_SUBTRACT_UNET(nn.Module, CkptMixin, SizeMixin):
     """
-
     entry 0433 of CPSC2019
     """
 
-    __DEBUG__ = False
     __name__ = "ECG_SUBTRACT_UNET"
 
-    def __init__(self, classes: Sequence[str], n_leads: int, config: dict) -> NoReturn:
+    def __init__(
+        self,
+        classes: Sequence[str],
+        n_leads: int,
+        config: Optional[CFG] = None,
+    ) -> None:
         """
-
         Parameters
         ----------
         classes: sequence of int,
             name of the classes
         n_leads: int,
-            number of input leads
-        config: dict,
+            number of input leads (number of input channels)
+        config: CFG, optional,
             other hyper-parameters, including kernel sizes, etc.
             ref. the corresponding config file
+
         """
         super().__init__()
         self.classes = list(classes)
         self.n_classes = len(classes)
         self.__out_channels = len(classes)
         self.__in_channels = n_leads
-        self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
+        self.config = deepcopy(ECG_SUBTRACT_UNET_CONFIG)
+        if not config:
+            warnings.warn(
+                "No config is provided, using default config.", RuntimeWarning
             )
-            __debug_seq_len = 5000
+        self.config.update(deepcopy(config) or {})
 
         # TODO: an init batch normalization?
         if self.config.init_batch_norm:
@@ -539,12 +522,6 @@ class ECG_SUBTRACT_UNET(nn.Module, CkptMixin, SizeMixin):
             kernel_initializer=self.config.kernel_initializer,
             kw_initializer=self.config.kw_initializer,
         )
-        if self.__DEBUG__:
-            __debug_output_shape = self.init_conv.compute_output_shape(__debug_seq_len)
-            print(
-                f"given seq_len = {__debug_seq_len}, init_conv output shape = {__debug_output_shape}"
-            )
-            _, _, __debug_seq_len = __debug_output_shape
 
         self.down_blocks = nn.ModuleDict()
         in_channels = self.config.init_num_filters
@@ -560,14 +537,6 @@ class ECG_SUBTRACT_UNET(nn.Module, CkptMixin, SizeMixin):
                 **(self.config.down_block),
             )
             in_channels = self.config.down_num_filters[idx][-1]
-            if self.__DEBUG__:
-                __debug_output_shape = self.down_blocks[
-                    f"down_{idx}"
-                ].compute_output_shape(__debug_seq_len)
-                print(
-                    f"given seq_len = {__debug_seq_len}, down_{idx} output shape = {__debug_output_shape}"
-                )
-                _, _, __debug_seq_len = __debug_output_shape
 
         self.bottom_block = DownBranchedDoubleConv(
             down_scale=self.config.down_scales[-1],
@@ -580,14 +549,6 @@ class ECG_SUBTRACT_UNET(nn.Module, CkptMixin, SizeMixin):
             mode=self.config.down_mode,
             **(self.config.down_block),
         )
-        if self.__DEBUG__:
-            __debug_output_shape = self.bottom_block.compute_output_shape(
-                __debug_seq_len
-            )
-            print(
-                f"given seq_len = {__debug_seq_len}, bottom_block output shape = {__debug_output_shape}"
-            )
-            _, _, __debug_seq_len = __debug_output_shape
 
         self.up_blocks = nn.ModuleDict()
         # in_channels = sum([branch[-1] for branch in self.config.bottom_num_filters])
@@ -605,14 +566,6 @@ class ECG_SUBTRACT_UNET(nn.Module, CkptMixin, SizeMixin):
                 **(self.config.up_block),
             )
             in_channels = self.config.up_num_filters[idx][-1]
-            if self.__DEBUG__:
-                __debug_output_shape = self.up_blocks[f"up_{idx}"].compute_output_shape(
-                    __debug_seq_len
-                )
-                print(
-                    f"given seq_len = {__debug_seq_len}, up_{idx} output shape = {__debug_output_shape}"
-                )
-                _, _, __debug_seq_len = __debug_output_shape
 
         self.out_conv = Conv_Bn_Activation(
             in_channels=self.config.up_num_filters[-1][-1],
@@ -625,11 +578,6 @@ class ECG_SUBTRACT_UNET(nn.Module, CkptMixin, SizeMixin):
             kernel_initializer=self.config.kernel_initializer,
             kw_initializer=self.config.kw_initializer,
         )
-        if self.__DEBUG__:
-            __debug_output_shape = self.out_conv.compute_output_shape(__debug_seq_len)
-            print(
-                f"given seq_len = {__debug_seq_len}, out_conv output shape = {__debug_output_shape}"
-            )
 
         # for inference
         # if background counted in `classes`, use softmax
@@ -639,7 +587,6 @@ class ECG_SUBTRACT_UNET(nn.Module, CkptMixin, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -649,6 +596,7 @@ class ECG_SUBTRACT_UNET(nn.Module, CkptMixin, SizeMixin):
         -------
         output: Tensor,
             of shape (batch_size, n_channels, seq_len)
+
         """
         if self.config.init_batch_norm:
             x = self.init_bn(input)
@@ -657,15 +605,9 @@ class ECG_SUBTRACT_UNET(nn.Module, CkptMixin, SizeMixin):
 
         # down
         to_concat = [self.init_conv(x)]
-        # if self.__DEBUG__:
-        #     print(f"shape of the init conv block output = {to_concat[-1].shape}")
         for idx in range(self.config.down_up_block_num - 1):
             to_concat.append(self.down_blocks[f"down_{idx}"](to_concat[-1]))
-            # if self.__DEBUG__:
-            #     print(f"shape of the {idx}-th down block output = {to_concat[-1].shape}")
         to_concat.append(self.bottom_block(to_concat[-1]))
-        # if self.__DEBUG__:
-        #     print(f"shape of the bottom block output = {to_concat[-1].shape}")
 
         # up
         up_input = to_concat[-1]
@@ -673,13 +615,9 @@ class ECG_SUBTRACT_UNET(nn.Module, CkptMixin, SizeMixin):
         for idx in range(self.config.down_up_block_num):
             up_output = self.up_blocks[f"up_{idx}"](up_input, to_concat[idx])
             up_input = up_output
-            # if self.__DEBUG__:
-            #     print(f"shape of the {idx}-th up block output = {up_output.shape}")
 
         # output
         output = self.out_conv(up_output)
-        # if self.__DEBUG__:
-        #     print(f"shape of out_conv layer output = {output.shape}")
 
         # to keep in accordance with other models
         # (batch_size, channels, seq_len) --> (batch_size, seq_len, channels)
@@ -694,13 +632,12 @@ class ECG_SUBTRACT_UNET(nn.Module, CkptMixin, SizeMixin):
         self, input: Union[np.ndarray, Tensor], bin_pred_thr: float = 0.5
     ) -> Tensor:
         """ """
-        NotImplementedError("implement a task specific inference method")
+        raise NotImplementedError("implement a task specific inference method")
 
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -712,6 +649,12 @@ class ECG_SUBTRACT_UNET(nn.Module, CkptMixin, SizeMixin):
         -------
         output_shape: sequence,
             the output shape of this model, given `seq_len` and `batch_size`
+
         """
         output_shape = (batch_size, seq_len, self.n_classes)
         return output_shape
+
+    @property
+    def doi(self) -> List[str]:
+        # TODO: add doi
+        return list(set(self.config.get("doi", []) + []))
