@@ -9,20 +9,22 @@ References
 
 """
 
+import warnings
 from copy import deepcopy
-from typing import NoReturn, Optional, Sequence, Union
+from typing import Optional, Sequence, Union, List
 
 import torch
 import torch.nn.functional as F
 from torch import Tensor, nn
 
-from ...cfg import DEFAULTS, CFG
+from ...cfg import CFG
 from ...models._nets import (
     Conv_Bn_Activation,
     DownSample,
     MultiConv,
 )
-from ...utils.misc import dict_to_str, add_docstring
+from ...model_configs import ECG_UNET_VANILLA_CONFIG
+from ...utils.misc import add_docstring, CitationMixin
 from ...utils.utils_nn import (
     CkptMixin,
     SizeMixin,
@@ -31,9 +33,6 @@ from ...utils.utils_nn import (
     compute_sequential_output_shape_docstring,
 )
 
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
-
 
 __all__ = [
     "ECG_UNET",
@@ -42,7 +41,6 @@ __all__ = [
 
 class DoubleConv(MultiConv):
     """
-
     building blocks of UNet
 
     References
@@ -51,7 +49,6 @@ class DoubleConv(MultiConv):
 
     """
 
-    __DEBUG__ = False
     __name__ = "DoubleConv"
 
     def __init__(
@@ -65,9 +62,8 @@ class DoubleConv(MultiConv):
         out_activation: bool = True,
         mid_channels: Optional[int] = None,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         in_channels: int,
@@ -122,7 +118,6 @@ class DownDoubleConv(nn.Sequential, SizeMixin):
 
     """
 
-    __DEBUG__ = False
     __name__ = "DownDoubleConv"
     __MODES__ = deepcopy(DownSample.__MODES__)
 
@@ -137,9 +132,8 @@ class DownDoubleConv(nn.Sequential, SizeMixin):
         mid_channels: Optional[int] = None,
         mode: str = "max",
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         down_scale: int,
@@ -174,10 +168,6 @@ class DownDoubleConv(nn.Sequential, SizeMixin):
         self.__mid_channels = mid_channels if mid_channels is not None else out_channels
         self.__out_channels = out_channels
         self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
 
         self.add_module(
             "down_sample",
@@ -204,7 +194,6 @@ class DownDoubleConv(nn.Sequential, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -239,7 +228,6 @@ class UpDoubleConv(nn.Module, SizeMixin):
 
     """
 
-    __DEBUG__ = False
     __name__ = "UpDoubleConv"
     __MODES__ = [
         "nearest",
@@ -261,9 +249,8 @@ class UpDoubleConv(nn.Module, SizeMixin):
         mode: str = "deconv",
         mid_channels: Optional[int] = None,
         **config,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         up_scale: int,
@@ -306,10 +293,6 @@ class UpDoubleConv(nn.Module, SizeMixin):
         self.__mode = mode.lower()
         assert self.__mode in self.__MODES__
         self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
-            )
 
         # the following has to be checked
         # if bilinear, use the normal convolutions to reduce the number of channels
@@ -342,7 +325,6 @@ class UpDoubleConv(nn.Module, SizeMixin):
 
     def forward(self, input: Tensor, down_output: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -375,7 +357,6 @@ class UpDoubleConv(nn.Module, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -405,9 +386,8 @@ class UpDoubleConv(nn.Module, SizeMixin):
         return output_shape
 
 
-class ECG_UNET(nn.Module, CkptMixin, SizeMixin):
+class ECG_UNET(nn.Module, CkptMixin, SizeMixin, CitationMixin):
     """
-
     UNet for (multi-lead) ECG wave delineation
 
     References
@@ -417,19 +397,22 @@ class ECG_UNET(nn.Module, CkptMixin, SizeMixin):
 
     """
 
-    __DEBUG__ = False
     __name__ = "ECG_UNET"
 
-    def __init__(self, classes: Sequence[str], n_leads: int, config: dict) -> NoReturn:
+    def __init__(
+        self,
+        classes: Sequence[str],
+        n_leads: int,
+        config: Optional[CFG] = None,
+    ) -> None:
         """
-
         Parameters
         ----------
         classes: sequence of int,
             name of the classes
         n_leads: int,
             number of input leads (number of input channels)
-        config: dict,
+        config: CFG, optional,
             other hyper-parameters, including kernel sizes, etc.
             ref. the corresponding config file
 
@@ -439,12 +422,12 @@ class ECG_UNET(nn.Module, CkptMixin, SizeMixin):
         self.n_classes = len(classes)  # final out_channels
         self.__out_channels = self.n_classes
         self.__in_channels = n_leads
-        self.config = CFG(deepcopy(config))
-        if self.__DEBUG__:
-            print(
-                f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
+        self.config = deepcopy(ECG_UNET_VANILLA_CONFIG)
+        if not config:
+            warnings.warn(
+                "No config is provided, using default config.", RuntimeWarning
             )
-            __debug_seq_len = 4000
+        self.config.update(deepcopy(config) or {})
 
         self.init_conv = DoubleConv(
             in_channels=self.__in_channels,
@@ -458,12 +441,6 @@ class ECG_UNET(nn.Module, CkptMixin, SizeMixin):
             kernel_initializer=self.config.kernel_initializer,
             kw_initializer=self.config.kw_initializer,
         )
-        if self.__DEBUG__:
-            __debug_output_shape = self.init_conv.compute_output_shape(__debug_seq_len)
-            print(
-                f"given seq_len = {__debug_seq_len}, init_conv output shape = {__debug_output_shape}"
-            )
-            _, _, __debug_seq_len = __debug_output_shape
 
         self.down_blocks = nn.ModuleDict()
         in_channels = self.config.init_num_filters
@@ -478,14 +455,6 @@ class ECG_UNET(nn.Module, CkptMixin, SizeMixin):
                 **(self.config.down_block),
             )
             in_channels = self.config.down_num_filters[idx]
-            if self.__DEBUG__:
-                __debug_output_shape = self.down_blocks[
-                    f"down_{idx}"
-                ].compute_output_shape(__debug_seq_len)
-                print(
-                    f"given seq_len = {__debug_seq_len}, down_{idx} output shape = {__debug_output_shape}"
-                )
-                _, _, __debug_seq_len = __debug_output_shape
 
         self.up_blocks = nn.ModuleDict()
         in_channels = self.config.down_num_filters[-1]
@@ -501,14 +470,6 @@ class ECG_UNET(nn.Module, CkptMixin, SizeMixin):
                 **(self.config.up_block),
             )
             in_channels = self.config.up_num_filters[idx]
-            if self.__DEBUG__:
-                __debug_output_shape = self.up_blocks[f"up_{idx}"].compute_output_shape(
-                    __debug_seq_len
-                )
-                print(
-                    f"given seq_len = {__debug_seq_len}, up_{idx} output shape = {__debug_output_shape}"
-                )
-                _, _, __debug_seq_len = __debug_output_shape
 
         self.out_conv = Conv_Bn_Activation(
             in_channels=self.config.up_num_filters[-1],
@@ -521,11 +482,6 @@ class ECG_UNET(nn.Module, CkptMixin, SizeMixin):
             kernel_initializer=self.config.kernel_initializer,
             kw_initializer=self.config.kw_initializer,
         )
-        if self.__DEBUG__:
-            __debug_output_shape = self.out_conv.compute_output_shape(__debug_seq_len)
-            print(
-                f"given seq_len = {__debug_seq_len}, out_conv output shape = {__debug_output_shape}"
-            )
 
         # for inference
         # if background counted in `classes`, use softmax
@@ -535,7 +491,6 @@ class ECG_UNET(nn.Module, CkptMixin, SizeMixin):
 
     def forward(self, input: Tensor) -> Tensor:
         """
-
         Parameters
         ----------
         input: Tensor,
@@ -548,25 +503,14 @@ class ECG_UNET(nn.Module, CkptMixin, SizeMixin):
 
         """
         to_concat = [self.init_conv(input)]
-        # if self.__DEBUG__:
-        #     print(f"shape of init conv block output = {to_concat[-1].shape}")
         for idx in range(self.config.down_up_block_num):
             to_concat.append(self.down_blocks[f"down_{idx}"](to_concat[-1]))
-            # if self.__DEBUG__:
-            #     print(f"shape of {idx}-th down block output = {to_concat[-1].shape}")
         up_input = to_concat[-1]
         to_concat = to_concat[-2::-1]
         for idx in range(self.config.down_up_block_num):
-            # if self.__DEBUG__:
-            #     print(f"shape of {idx}-th up block 1st input = {up_input.shape}")
-            #     print(f"shape of {idx}-th up block 2nd input (from down) = {to_concat[idx].shape}")
             up_output = self.up_blocks[f"up_{idx}"](up_input, to_concat[idx])
             up_input = up_output
-            # if self.__DEBUG__:
-            #     print(f"shape of {idx}-th up block output = {up_output.shape}")
         output = self.out_conv(up_output)
-        # if self.__DEBUG__:
-        #     print(f"shape of out_conv layer output = {output.shape}")
 
         # to keep in accordance with other models
         # (batch_size, channels, seq_len) --> (batch_size, seq_len, channels)
@@ -585,7 +529,6 @@ class ECG_UNET(nn.Module, CkptMixin, SizeMixin):
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
         """
-
         Parameters
         ----------
         seq_len: int,
@@ -601,3 +544,7 @@ class ECG_UNET(nn.Module, CkptMixin, SizeMixin):
         """
         output_shape = (batch_size, seq_len, self.n_classes)
         return output_shape
+
+    @property
+    def doi(self) -> List[str]:
+        return list(set(self.config.get("doi", []) + ["10.1007/978-3-030-30425-6_29"]))
