@@ -3,7 +3,7 @@
 
 import warnings
 from numbers import Real
-from typing import Any, NoReturn
+from typing import Any
 
 import torch
 
@@ -26,9 +26,8 @@ class BaselineRemove(torch.nn.Module):
         window2: float = 0.6,
         inplace: bool = True,
         **kwargs: Any
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         fs: real number,
@@ -48,12 +47,13 @@ class BaselineRemove(torch.nn.Module):
         self.window2 = window2
         if self.window2 < self.window1:
             self.window1, self.window2 = self.window2, self.window1
-            warnings.warn("values of window1 and window2 are switched")
+            warnings.warn(
+                "values of `window1` and `window2` are switched", RuntimeWarning
+            )
         self.inplace = inplace
 
     def forward(self, sig: torch.Tensor) -> torch.Tensor:
         """
-
         apply the preprocessor to `sig`
 
         Parameters
