@@ -2,7 +2,7 @@
 """
 
 from numbers import Real
-from typing import Any, Iterable, NoReturn, Union
+from typing import Any, Iterable, Union
 
 import torch
 
@@ -43,9 +43,8 @@ class Normalize(torch.nn.Module):
         per_channel: bool = False,
         inplace: bool = True,
         **kwargs: Any
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         method: str, default "z-score",
@@ -81,7 +80,6 @@ class Normalize(torch.nn.Module):
 
     def forward(self, sig: torch.Tensor) -> torch.Tensor:
         """
-
         Parameters
         ----------
         sig: Tensor,
@@ -117,18 +115,20 @@ class MinMaxNormalize(Normalize):
     __name__ = "MinMaxNormalize"
 
     def __init__(
-        self,
-        per_channel: bool = False,
-    ) -> NoReturn:
+        self, per_channel: bool = False, inplace: bool = True, **kwargs: Any
+    ) -> None:
         """
-
         Parameters
         ----------
         per_channel: bool, default False,
             if True, normalization will be done per channel
+        inplace: bool, default True,
+            if True, normalization will be done inplace (on the signal)
 
         """
-        super().__init__(method="min-max", per_channel=per_channel)
+        super().__init__(
+            method="min-max", per_channel=per_channel, inplace=inplace, **kwargs
+        )
 
 
 class NaiveNormalize(Normalize):
@@ -147,10 +147,10 @@ class NaiveNormalize(Normalize):
         mean: Union[Real, Iterable[Real]] = 0.0,
         std: Union[Real, Iterable[Real]] = 1.0,
         per_channel: bool = False,
+        inplace: bool = True,
         **kwargs: Any
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         mean: real number or array_like, default 0.0,
@@ -159,6 +159,8 @@ class NaiveNormalize(Normalize):
             value(s) to be divided
         per_channel: bool, default False,
             if True, normalization will be done per channel
+        inplace: bool, default True,
+            if True, normalization will be done inplace (on the signal)
 
         """
         super().__init__(
@@ -166,6 +168,7 @@ class NaiveNormalize(Normalize):
             mean=mean,
             std=std,
             per_channel=per_channel,
+            inplace=inplace,
         )
 
 
@@ -185,10 +188,10 @@ class ZScoreNormalize(Normalize):
         mean: Union[Real, Iterable[Real]] = 0.0,
         std: Union[Real, Iterable[Real]] = 1.0,
         per_channel: bool = False,
+        inplace: bool = True,
         **kwargs: Any
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         mean: real number or array_like, default 0.0,
@@ -199,6 +202,8 @@ class ZScoreNormalize(Normalize):
             or standard deviations for each lead of the normalized signal,
         per_channel: bool, default False,
             if True, normalization will be done per channel
+        inplace: bool, default True,
+            if True, normalization will be done inplace (on the signal)
 
         """
         super().__init__(
@@ -206,4 +211,6 @@ class ZScoreNormalize(Normalize):
             mean=mean,
             std=std,
             per_channel=per_channel,
+            inplace=inplace,
+            **kwargs
         )
