@@ -1,8 +1,9 @@
 """
 """
 
+import warnings
 from random import sample
-from typing import List, NoReturn, Optional, Tuple
+from typing import List, Optional, Tuple
 
 import torch
 import torch.nn as nn
@@ -20,6 +21,7 @@ __all__ = [
 
 class PreprocManager(ReprMixin, nn.Module):
     """
+    Manager class for preprocessors
 
     Examples
     --------
@@ -34,7 +36,7 @@ class PreprocManager(ReprMixin, nn.Module):
         normalize={"method": "min-max"},
     )
     ppm = PreprocManager.from_config(config)
-    sig = torch.rand(2,12,8000)
+    sig = torch.randn(2, 12, 8000)
     sig = ppm(sig)
     ```
 
@@ -47,9 +49,8 @@ class PreprocManager(ReprMixin, nn.Module):
         *pps: Optional[Tuple[nn.Module, ...]],
         random: bool = False,
         inplace: bool = True,
-    ) -> NoReturn:
+    ) -> None:
         """
-
         Parameters
         ----------
         pps: tuple of `nn.Module`, optional,
@@ -64,25 +65,24 @@ class PreprocManager(ReprMixin, nn.Module):
         self.random = random
         self._preprocessors = list(pps)
 
-    def _add_bandpass(self, **config: dict) -> NoReturn:
+    def _add_bandpass(self, **config: dict) -> None:
         """ """
         self._preprocessors.append(BandPass(**config))
 
-    def _add_baseline_remove(self, **config: dict) -> NoReturn:
+    def _add_baseline_remove(self, **config: dict) -> None:
         """ """
         self._preprocessors.append(BaselineRemove(**config))
 
-    def _add_normalize(self, **config: dict) -> NoReturn:
+    def _add_normalize(self, **config: dict) -> None:
         """ """
         self._preprocessors.append(Normalize(**config))
 
-    def _add_resample(self, **config: dict) -> NoReturn:
+    def _add_resample(self, **config: dict) -> None:
         """ """
         self._preprocessors.append(Resample(**config))
 
     def forward(self, sig: torch.Tensor) -> torch.Tensor:
         """
-
         Parameters
         ----------
         sig: Tensor,
@@ -95,7 +95,9 @@ class PreprocManager(ReprMixin, nn.Module):
 
         """
         if len(self.preprocessors) == 0:
-            raise ValueError("No preprocessors added to the manager.")
+            # raise ValueError("No preprocessors added to the manager.")
+            # allow dummy (empty) preprocessors
+            return sig
         ordering = list(range(len(self.preprocessors)))
         if self.random:
             ordering = sample(ordering, len(ordering))
@@ -106,7 +108,6 @@ class PreprocManager(ReprMixin, nn.Module):
     @classmethod
     def from_config(cls, config: dict) -> "PreprocManager":
         """
-
         Parameters
         ----------
         config: dict,
@@ -140,33 +141,53 @@ class PreprocManager(ReprMixin, nn.Module):
                 # just ignore the other items
                 pass
                 # raise ValueError(f"Unknown preprocessor: {pp_name}")
+        if ppm.empty:
+            warnings.warn(
+                "No preprocessors added to the manager. You are using a dummy preprocessor.",
+                RuntimeWarning,
+            )
         return ppm
 
-    def rearrange(self, new_ordering: List[str]) -> NoReturn:
+    def rearrange(self, new_ordering: List[str]) -> None:
         """
-
         Parameters
         ----------
         new_ordering: list of str,
             the new ordering of the preprocessors
 
         """
-        _mapping = {
+        if self.random:
+            warnings.warn(
+                "The preprocessors are applied in random order, "
+                "rearranging the preprocessors will not take effect.",
+                RuntimeWarning,
+            )
+        _mapping = {  # built-in preprocessors
             "Resample": "resample",
             "BandPass": "bandpass",
             "BaselineRemove": "baseline_remove",
             "Normalize": "normalize",
         }
+        _mapping.update({v: k for k, v in _mapping.items()})
         for k in new_ordering:
             if k not in _mapping:
+                # allow custom preprocessors
+                assert k in [
+                    item.__class__.__name__ for item in self._preprocessors
+                ], f"Unknown preprocessor name: `{k}`"
                 _mapping.update({k: k})
+        assert len(new_ordering) == len(
+            set(new_ordering)
+        ), "Duplicate preprocessor names."
+        assert len(new_ordering) == len(
+            self._preprocessors
+        ), "Number of preprocessors mismatch."
         self._preprocessors.sort(
-            key=lambda aug: new_ordering.index(_mapping[aug.__name__])
+            key=lambda item: new_ordering.index(_mapping[item.__class__.__name__])
         )
 
-    def add_(self, pp: nn.Module, pos: int = -1) -> NoReturn:
+    def add_(self, pp: nn.Module, pos: int = -1) -> None:
         """
-
         add a (custom) preprocessor to the manager,
         this method is preferred against directly manipulating
         the internal list of preprocessors via `PreprocManager.preprocessors.append(pp)`
@@ -196,10 +217,13 @@ class PreprocManager(ReprMixin, nn.Module):
     def preprocessors(self) -> List[nn.Module]:
         return self._preprocessors
 
+    @property
+    def empty(self) -> bool:
+        return len(self.preprocessors) == 0
+
     def extra_repr_keys(self) -> List[str]:
         """
         return the extra keys for `__repr__`
-
         """
         return super().extra_repr_keys() + [
             "random",
