@@ -2,6 +2,7 @@
 """
 
 import datetime
+import inspect
 import logging
 import os
 import re
@@ -9,7 +10,6 @@ import sys
 import signal
 import time
 import warnings
-import inspect
 from contextlib import contextmanager
 from copy import deepcopy
 from functools import reduce, wraps
@@ -22,7 +22,6 @@ from typing import (
     Dict,
     Iterable,
     List,
-    NoReturn,
     Optional,
     Sequence,
     Union,
@@ -31,8 +30,19 @@ from typing import (
 
 import numpy as np
 import pandas as pd
+from deprecated import deprecated
+
+try:
+    from bib_lookup import CitationMixin as _CitationMixin
+except ImportError:
+    __bib_lookup_lowest_version = "0.0.19"
+    raise ImportError(
+        f"`bib-lookup` requires version {__bib_lookup_lowest_version} or higher. "
+        "Please upgrade it via `pip install -U bib-lookup`."
+    )
+
+from ..cfg import DEFAULTS, _DATA_CACHE
 
-from ..cfg import DEFAULTS
 
 __all__ = [
     "get_record_list_recursive",
@@ -52,19 +62,23 @@ __all__ = [
     "dicts_equal",
     "default_class_repr",
     "ReprMixin",
+    "CitationMixin",
     "MovingAverage",
     "nildent",
-    "isclass",
     "add_docstring",
-    "deprecate_kwargs",
+    "remove_parameters_returns_from_docstring",
     "timeout",
     "Timer",
+    "get_kwargs",
+    "get_required_args",
+    "add_kwargs",
 ]
 
 
-def get_record_list_recursive(db_dir: Union[str, Path], rec_ext: str) -> List[str]:
+def get_record_list_recursive(
+    db_dir: Union[str, Path], rec_ext: str, relative: bool = True
+) -> List[str]:
     """
-
     get the list of records in `db_dir` recursively,
     for example, there are two folders "patient1", "patient2" in `db_dir`,
     and there are records "A0001", "A0002", ... in "patient1"; "B0001", "B0002", ... in "patient2",
@@ -77,6 +91,8 @@ def get_record_list_recursive(db_dir: Union[str, Path], rec_ext: str) -> List[st
         the parent (root) path of the whole database
     rec_ext: str,
         extension of the record files
+    relative: bool,
+        if True, the output is relative to `db_dir`, otherwise absolute paths
 
     Returns
     -------
@@ -84,28 +100,23 @@ def get_record_list_recursive(db_dir: Union[str, Path], rec_ext: str) -> List[st
         list of records, in lexicographical order
 
     """
-    res = []
-    roots = [str(db_dir)]
-    while len(roots) > 0:
-        new_roots = []
-        for r in roots:
-            tmp = [os.path.join(r, item) for item in os.listdir(r)]
-            res += [item for item in tmp if os.path.isfile(item)]
-            new_roots += [item for item in tmp if os.path.isdir(item)]
-        roots = deepcopy(new_roots)
+    if not rec_ext.startswith("."):
+        res = Path(db_dir).rglob(f"*.{rec_ext}")
+    else:
+        res = Path(db_dir).rglob(f"*{rec_ext}")
     res = [
-        os.path.splitext(item)[0].replace(str(db_dir), "").strip(os.sep)
+        str((item.relative_to(db_dir) if relative else item).with_suffix(""))
         for item in res
-        if item.endswith(rec_ext)
+        if str(item).endswith(rec_ext)
     ]
     res = sorted(res)
 
     return res
 
 
+@deprecated(reason="use `get_record_list_recursive3` instead")
 def get_record_list_recursive2(db_dir: Union[str, Path], rec_pattern: str) -> List[str]:
     """
-
     get the list of records in `db_dir` recursively,
     for example, there are two folders "patient1", "patient2" in `db_dir`,
     and there are records "A0001", "A0002", ... in "patient1"; "B0001", "B0002", ... in "patient2",
@@ -144,10 +155,11 @@ def get_record_list_recursive2(db_dir: Union[str, Path], rec_pattern: str) -> Li
 
 
 def get_record_list_recursive3(
-    db_dir: Union[str, Path], rec_patterns: Union[str, Dict[str, str]]
+    db_dir: Union[str, Path],
+    rec_patterns: Union[str, Dict[str, str]],
+    relative: bool = True,
 ) -> Union[List[str], Dict[str, List[str]]]:
     r"""
-
     get the list of records in `db_dir` recursively,
     for example, there are two folders "patient1", "patient2" in `db_dir`,
     and there are records "A0001", "A0002", ... in "patient1"; "B0001", "B0002", ... in "patient2",
@@ -161,6 +173,8 @@ def get_record_list_recursive3(
     rec_patterns: str or dict,
         pattern of the record filenames, e.g. "A(?:\d+).mat",
         or patterns of several subsets, e.g. `{"A": "A(?:\d+).mat"}`
+    relative: bool,
+        if True, the output is relative to `db_dir`, otherwise absolute paths
 
     Returns
     -------
@@ -172,39 +186,34 @@ def get_record_list_recursive3(
         res = []
     elif isinstance(rec_patterns, dict):
         res = {k: [] for k in rec_patterns.keys()}
-    roots = [str(db_dir)]
+    _db_dir = Path(db_dir).resolve()  # make absolute
+    roots = [_db_dir]
     while len(roots) > 0:
         new_roots = []
         for r in roots:
-            # tmp = [os.path.join(r, item) for item in os.listdir(r)]
             tmp = os.listdir(r)
             if isinstance(rec_patterns, str):
                 res += [
-                    os.path.join(r, item)
-                    for item in filter(re.compile(rec_patterns).search, tmp)
+                    r / item for item in filter(re.compile(rec_patterns).search, tmp)
                 ]
             elif isinstance(rec_patterns, dict):
                 for k in rec_patterns.keys():
                     res[k] += [
-                        os.path.join(r, item)
+                        r / item
                         for item in filter(re.compile(rec_patterns[k]).search, tmp)
                     ]
-            new_roots += [
-                os.path.join(r, item)
-                for item in tmp
-                if os.path.isdir(os.path.join(r, item))
-            ]
+            new_roots += [r / item for item in tmp if (r / item).is_dir()]
         roots = deepcopy(new_roots)
     if isinstance(rec_patterns, str):
         res = [
-            os.path.splitext(item)[0].replace(str(db_dir), "").strip(os.sep)
+            str((item.relative_to(_db_dir) if relative else item).with_suffix(""))
             for item in res
         ]
         res = sorted(res)
     elif isinstance(rec_patterns, dict):
         for k in rec_patterns.keys():
             res[k] = [
-                os.path.splitext(item)[0].replace(str(db_dir), "").strip(os.sep)
+                str((item.relative_to(_db_dir) if relative else item).with_suffix(""))
                 for item in res[k]
             ]
             res[k] = sorted(res[k])
@@ -215,7 +224,6 @@ def dict_to_str(
     d: Union[dict, list, tuple], current_depth: int = 1, indent_spaces: int = 4
 ) -> str:
     """
-
     convert a (possibly) nested dict into a `str` of json-like formatted form,
     this nested dict might also contain lists or tuples of dict (and of str, int, etc.)
 
@@ -296,7 +304,6 @@ def dict_to_str(
 
 def str2bool(v: Union[str, bool]) -> bool:
     """
-
     converts a "boolean" value possibly in the format of str to bool
 
     Parameters
@@ -327,7 +334,6 @@ def str2bool(v: Union[str, bool]) -> bool:
 
 def diff_with_step(a: np.ndarray, step: int = 1, **kwargs) -> np.ndarray:
     """
-
     compute a[n+step] - a[n] for all valid n
 
     Parameters
@@ -354,7 +360,6 @@ def diff_with_step(a: np.ndarray, step: int = 1, **kwargs) -> np.ndarray:
 
 def ms2samples(t: Real, fs: Real) -> int:
     """
-
     convert time `t` with units in ms to number of samples
 
     Parameters
@@ -376,7 +381,6 @@ def ms2samples(t: Real, fs: Real) -> int:
 
 def samples2ms(n_samples: int, fs: Real) -> Real:
     """
-
     inverse function of `ms2samples`
 
     Parameters
@@ -402,7 +406,7 @@ def plot_single_lead(
     ax: Optional[Any] = None,
     ticks_granularity: int = 0,
     **kwargs,
-) -> NoReturn:
+) -> None:
     """finished, NOT checked,
 
     Parameters
@@ -460,22 +464,27 @@ def plot_single_lead(
 
 
 def init_logger(
-    log_dir: Union[str, Path],
+    log_dir: Optional[Union[str, Path, bool]] = None,
     log_file: Optional[str] = None,
     log_name: Optional[str] = None,
+    suffix: Optional[str] = None,
     mode: str = "a",
     verbose: int = 0,
 ) -> logging.Logger:
     """
-
     Parameters
     ----------
-    log_dir: str or Path,
-        directory of the log file
+    log_dir: str or Path, optional,
+        directory of the log file,
+        default to `DEFAULTS.log_dir`;
+        if is `False`, then no log file will be created
     log_file: str, optional,
         name of the log file
     log_name: str, optional,
         name of the logger
+    suffix: str, optional,
+        suffix of the log filename,
+        ignored if `log_file` is not `None`
     mode: str, default "a",
         mode of writing the log file, can be one of "a", "w"
     verbose: int, default 0,
@@ -486,50 +495,68 @@ def init_logger(
     logger: Logger
 
     """
-    if log_file is None:
-        log_file = f"log_{get_date_str()}.txt"
-    log_dir = Path(log_dir)
-    log_dir.mkdir(parents=True, exist_ok=True)
-    log_file = log_dir / log_file
-    print(f"log file path: {str(log_file)}")
+    if log_dir is False:
+        log_file = None
+    else:
+        if log_file is None:
+            log_file = f"{DEFAULTS.prefix}-log-{get_date_str()}.txt"
+        log_dir = (
+            Path(log_dir).expanduser().resolve()
+            if log_dir is not None
+            else DEFAULTS.log_dir
+        )
+        log_dir.mkdir(parents=True, exist_ok=True)
+        log_file = log_dir / log_file
+        print(f"log file path: {str(log_file)}")
 
-    logger = logging.getLogger(
-        log_name or DEFAULTS.prefix
-    )  # "ECG" to prevent from using the root logger
+    log_name = (log_name or DEFAULTS.prefix) + (f"-{suffix}" if suffix else "")
+    # if a logger with the same name already exists, remove it
+    if log_name in logging.root.manager.loggerDict:
+        logging.getLogger(log_name).handlers = []
+    logger = logging.getLogger(log_name)  # to prevent from using the root logger
 
     c_handler = logging.StreamHandler(sys.stdout)
-    f_handler = logging.FileHandler(str(log_file))
+    if log_file is not None:
+        f_handler = logging.FileHandler(str(log_file))
 
     if verbose >= 2:
-        print("levels of c_handler and f_handler are set DEBUG")
+        # print("level of `c_handler` is set DEBUG")
         c_handler.setLevel(logging.DEBUG)
-        f_handler.setLevel(logging.DEBUG)
+        if log_file is not None:
+            # print("level of `f_handler` is set DEBUG")
+            f_handler.setLevel(logging.DEBUG)
         logger.setLevel(logging.DEBUG)
     elif verbose >= 1:
-        print("level of c_handler is set INFO, level of f_handler is set DEBUG")
+        # print("level of `c_handler` is set INFO")
         c_handler.setLevel(logging.INFO)
-        f_handler.setLevel(logging.DEBUG)
+        if log_file is not None:
+            # print("level of `f_handler` is set DEBUG")
+            f_handler.setLevel(logging.DEBUG)
         logger.setLevel(logging.DEBUG)
     else:
-        print("level of c_handler is set WARNING, level of f_handler is set INFO")
+        # print("level of `c_handler` is set WARNING")
         c_handler.setLevel(logging.WARNING)
-        f_handler.setLevel(logging.INFO)
+        if log_file is not None:
+            # print("level of `f_handler` is set INFO")
+            f_handler.setLevel(logging.INFO)
         logger.setLevel(logging.INFO)
 
     c_format = logging.Formatter("%(name)s - %(levelname)s - %(message)s")
-    f_format = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
     c_handler.setFormatter(c_format)
-    f_handler.setFormatter(f_format)
-
     logger.addHandler(c_handler)
-    logger.addHandler(f_handler)
+
+    if log_file is not None:
+        f_format = logging.Formatter(
+            "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
+        )
+        f_handler.setFormatter(f_format)
+        logger.addHandler(f_handler)
 
     return logger
 
 
 def get_date_str(fmt: Optional[str] = None):
     """
-
     Parameters
     ----------
     fmt: str, optional,
@@ -548,7 +575,6 @@ def get_date_str(fmt: Optional[str] = None):
 
 def list_sum(lst: Sequence[list]) -> list:
     """
-
     Parameters
     ----------
     lst: sequence of list,
@@ -571,7 +597,6 @@ def read_log_txt(
     scalar_startswith: Union[str, Iterable[str]] = "train/|test/",
 ) -> pd.DataFrame:
     """
-
     read from log txt file, in case tensorboard not working
 
     Parameters
@@ -618,7 +643,6 @@ def read_event_scalars(
     fp: str, keys: Optional[Union[str, Iterable[str]]] = None
 ) -> Union[pd.DataFrame, Dict[str, pd.DataFrame]]:
     """
-
     read scalars from event file, in case tensorboard not working
 
     Parameters
@@ -665,13 +689,18 @@ def read_event_scalars(
     return summary
 
 
-def dicts_equal(d1: dict, d2: dict) -> bool:
+def dicts_equal(d1: dict, d2: dict, allow_array_diff_types: bool = True) -> bool:
     """
+    Determine if two dicts are equal
 
     Parameters
     ----------
     d1, d2: dict,
         the two dicts to compare equality
+    allow_array_diff_types: bool, default True,
+        whether to allow the equality of two arrays with different types,
+        including list, tuple, np.ndarray, torch.Tensor,
+        NOT including pd.DataFrame, pd.Series
 
     Returns
     -------
@@ -688,10 +717,12 @@ def dicts_equal(d1: dict, d2: dict) -> bool:
 
     Example
     -------
+    ```python
     >>> d1 = {"a": pd.DataFrame([{"hehe":1,"haha":2}])[["haha","hehe"]]}
     >>> d2 = {"a": pd.DataFrame([{"hehe":1,"haha":2}])[["hehe","haha"]]}
     >>> dicts_equal(d1, d2)
     True
+    ```
 
     """
     import torch
@@ -699,17 +730,27 @@ def dicts_equal(d1: dict, d2: dict) -> bool:
     if len(d1) != len(d2):
         return False
     for k, v in d1.items():
-        if k not in d2 or not isinstance(d2[k], type(v)):
+        if k not in d2:
             return False
+        if not allow_array_diff_types and not isinstance(d2[k], type(v)):
+            return False
+        if allow_array_diff_types and isinstance(
+            v, (list, tuple, np.ndarray, torch.Tensor)
+        ):
+            if not isinstance(d2[k], (list, tuple, np.ndarray, torch.Tensor)):
+                return False
+            if not np.array_equal(v, d2[k]):
+                return False
+        elif allow_array_diff_types and not isinstance(
+            v, (list, tuple, np.ndarray, torch.Tensor)
+        ):
+            if not isinstance(d2[k], type(v)):
+                return False
         if isinstance(v, dict):
             if not dicts_equal(v, d2[k]):
                 return False
-        elif isinstance(v, np.ndarray):
-            if v.shape != d2[k].shape or not (v == d2[k]).all():
-                return False
-        elif isinstance(v, torch.Tensor):
-            if v.shape != d2[k].shape or not (v == d2[k]).all().item():
-                return False
+        elif isinstance(v, (list, tuple, np.ndarray, torch.Tensor)):
+            return np.array_equal(v, d2[k])
         elif isinstance(v, pd.DataFrame):
             if v.shape != d2[k].shape or set(v.columns) != set(d2[k].columns):
                 # consider: should one check index be equal?
@@ -731,9 +772,50 @@ def dicts_equal(d1: dict, d2: dict) -> bool:
     return True
 
 
-def default_class_repr(c: object, align: str = "center", depth: int = 1) -> str:
+def add_docstring(doc: str, mode: str = "replace") -> Callable:
+    """
+    decorator to add docstring to a function or a class
+
+    Parameters
+    ----------
+    doc: str,
+        the docstring to be added
+    mode: str, default "replace",
+        the mode of the docstring,
+        can be "replace", "append" or "prepend",
+        case insensitive
+
     """
 
+    def decorator(func_or_cls: Callable) -> Callable:
+        """ """
+
+        pattern = "(\\s^\n){1,}"
+        if mode.lower() == "replace":
+            func_or_cls.__doc__ = doc
+        elif mode.lower() == "append":
+            tmp = re.sub(pattern, "", func_or_cls.__doc__)
+            new_lines = 1 - (len(tmp) - len(tmp.rstrip("\n")))
+            tmp = re.sub(pattern, "", doc)
+            new_lines -= len(tmp) - len(tmp.lstrip("\n"))
+            new_lines = max(0, new_lines) * "\n"
+            func_or_cls.__doc__ += new_lines + doc
+        elif mode.lower() == "prepend":
+            tmp = re.sub(pattern, "", doc)
+            new_lines = 1 - (len(tmp) - len(tmp.rstrip("\n")))
+            tmp = re.sub(pattern, "", func_or_cls.__doc__)
+            new_lines -= len(tmp) - len(tmp.lstrip("\n"))
+            new_lines = max(0, new_lines) * "\n"
+            func_or_cls.__doc__ = doc + new_lines + func_or_cls.__doc__
+        else:
+            raise ValueError(f"mode {mode} is not supported")
+        return func_or_cls
+
+    return decorator
+
+
+def default_class_repr(c: object, align: str = "center", depth: int = 1) -> str:
+    """
     Parameters
     ----------
     c: object,
@@ -773,8 +855,7 @@ def default_class_repr(c: object, align: str = "center", depth: int = 1) -> str:
 
 class ReprMixin(object):
     """
-    Mixin for enhanced __repr__ and __str__ methods.
-
+    Mixin class for enhanced __repr__ and __str__ methods.
     """
 
     def __repr__(self) -> str:
@@ -787,6 +868,61 @@ class ReprMixin(object):
         return []
 
 
+class CitationMixin(_CitationMixin):
+    """
+    Mixin class for getting citations from DOIs.
+    """
+
+    # backwar compatibility
+    if (_DATA_CACHE / "database_citation.csv").exists():
+        try:
+            df_old = pd.read_csv(_DATA_CACHE / "database_citation.csv")
+        except pd.errors.EmptyDataError:
+            df_old = pd.DataFrame(columns=["doi", "citation"])
+        if set(df_old.columns) != set(["doi", "citation"]):
+            df_old = pd.DataFrame(columns=["doi", "citation"])
+        df_old = df_old[["doi", "citation"]]
+        if _CitationMixin.citation_cache.exists():
+            df = pd.read_csv(_CitationMixin.citation_cache)
+        else:
+            df = pd.DataFrame(columns=["doi", "citation"])
+        # merge the old and new tables and drop duplicates
+        df = pd.concat([df, df_old], axis=0, ignore_index=True)
+        df = df.drop_duplicates(subset="doi", keep="first")
+        df = df.reset_index(drop=True)
+        df.to_csv(_CitationMixin.citation_cache, index=False)
+        del df_old, df
+        # delete the old cache
+        (_DATA_CACHE / "database_citation.csv").unlink()
+
+    @add_docstring(
+        _CitationMixin.get_citation.__doc__.replace(
+            "print_result: bool, default False", "print_result: bool, default True"
+        ),
+        mode="prepend",
+    )
+    def get_citation(
+        self,
+        lookup: bool = True,
+        format: Optional[str] = None,
+        style: Optional[str] = None,
+        timeout: Optional[float] = None,
+        print_result: bool = True,
+    ) -> Union[str, type(None)]:
+        """
+        override the default method to make the `print_result` argument
+        have default value `True`.
+
+        """
+        return super().get_citation(
+            lookup=lookup,
+            format=format,
+            style=style,
+            timeout=timeout,
+            print_result=print_result,
+        )
+
+
 class MovingAverage(object):
     """to be improved,
 
@@ -798,7 +934,7 @@ class MovingAverage(object):
 
     """
 
-    def __init__(self, data: Optional[Sequence] = None, **kwargs: Any) -> NoReturn:
+    def __init__(self, data: Optional[Sequence] = None, **kwargs: Any) -> None:
         """
         Parameters
         ----------
@@ -844,7 +980,7 @@ class MovingAverage(object):
         elif m in ["wma", "weighted", "weighted moving average"]:
             func = self._wma
         else:
-            raise NotImplementedError
+            raise NotImplementedError(f"method `{method}` is not implemented yet")
         if data is not None:
             self.data = np.array(data)
         return func(**kwargs)
@@ -862,6 +998,11 @@ class MovingAverage(object):
             otherwise the previous `window` points of the current point will be used
 
         """
+        if len(kwargs) > 0:
+            warnings.warn(
+                f"the following arguments are not used: `{kwargs}` for simple moving average",
+                RuntimeWarning,
+            )
         smoothed = []
         if center:
             hw = window // 2
@@ -893,6 +1034,11 @@ class MovingAverage(object):
             weight of the previous data point
 
         """
+        if len(kwargs) > 0:
+            warnings.warn(
+                f"the following arguments are not used: `{kwargs}` for exponential moving average",
+                RuntimeWarning,
+            )
         smoothed = []
         prev = self.data[0]
         for d in self.data:
@@ -905,8 +1051,12 @@ class MovingAverage(object):
     def _cma(self, **kwargs) -> np.ndarray:
         """
         cumulative moving average
-
         """
+        if len(kwargs) > 0:
+            warnings.warn(
+                f"the following arguments are not used: `{kwargs}` for cumulative moving average",
+                RuntimeWarning,
+            )
         smoothed = []
         prev = 0
         for n, d in enumerate(self.data):
@@ -926,6 +1076,11 @@ class MovingAverage(object):
             window length of the moving average
 
         """
+        if len(kwargs) > 0:
+            warnings.warn(
+                f"the following arguments are not used: `{kwargs}` for weighted moving average",
+                RuntimeWarning,
+            )
         conv = np.arange(1, window + 1)[::-1]
         deno = np.sum(conv)
         smoothed = np.convolve(conv, self.data, mode="same") / deno
@@ -934,7 +1089,6 @@ class MovingAverage(object):
 
 def nildent(text: str) -> str:
     """
-
     kill all leading white spaces in each line of `text`,
     while keeping all lines (including empty)
 
@@ -955,114 +1109,88 @@ def nildent(text: str) -> str:
     return new_text
 
 
-def isclass(obj: Any) -> bool:
+def remove_parameters_returns_from_docstring(
+    doc: str,
+    parameters: Optional[Union[str, List[str]]] = None,
+    returns: Optional[Union[str, List[str]]] = None,
+    parameters_indicator: str = "Parameters",
+    returns_indicator: str = "Returns",
+) -> str:
     """
-
-    Parameters
-    ----------
-    obj: any object,
-        any object, including class, instance of class, etc
-
-    Returns
-    -------
-    bool:
-        True if `obj` is a class, False otherwise
-
-    """
-    try:
-        return issubclass(obj, object)
-    except TypeError:
-        return False
-
-
-def add_docstring(doc: str, mode: str = "replace") -> Callable:
-    """
-    decorator to add docstring to a function
+    remove parameters and/or returns from docstring,
+    which is of the format of numpydoc
 
     Parameters
     ----------
     doc: str,
-        the docstring to be added
-    mode: str, default "replace",
-        the mode of the docstring,
-        can be "replace", "append" or "prepend",
-        case insensitive
+        docstring to be processed
+    parameters: str or list of str, default None,
+        parameters to be removed
+    returns: str or list of str, default None,
+        returned values to be removed
+    parameters_indicator: str, default "Parameters",
+        the indicator of the parameters section
+    returns_indicator: str, default "Returns",
+        the indicator of the returns section
+
+    Returns
+    -------
+    new_doc: str,
+        the processed docstring
 
     """
+    if parameters is None:
+        parameters = []
+    elif isinstance(parameters, str):
+        parameters = [parameters]
+    if returns is None:
+        returns = []
+    elif isinstance(returns, str):
+        returns = [returns]
 
-    def decorator(func: Callable) -> Callable:
-        """ """
-
-        @wraps(func)
-        def wrapper(*args, **kwargs) -> Callable:
-            """ """
-            return func(*args, **kwargs)
-
-        pattern = "(\\s^\n){1,}"
-        if mode.lower() == "replace":
-            wrapper.__doc__ = doc
-        elif mode.lower() == "append":
-            tmp = re.sub(pattern, "", wrapper.__doc__)
-            new_lines = 1 - (len(tmp) - len(tmp.rstrip("\n")))
-            tmp = re.sub(pattern, "", doc)
-            new_lines -= len(tmp) - len(tmp.lstrip("\n"))
-            new_lines = max(0, new_lines) * "\n"
-            wrapper.__doc__ += new_lines + doc
-        elif mode.lower() == "prepend":
-            tmp = re.sub(pattern, "", doc)
-            new_lines = 1 - (len(tmp) - len(tmp.rstrip("\n")))
-            tmp = re.sub(pattern, "", wrapper.__doc__)
-            new_lines -= len(tmp) - len(tmp.lstrip("\n"))
-            new_lines = max(0, new_lines) * "\n"
-            wrapper.__doc__ = doc + new_lines + wrapper.__doc__
-        else:
-            raise ValueError(f"mode {mode} is not supported")
-        return wrapper
-
-    return decorator
-
-
-def deprecate_kwargs(l_kwargs: Sequence[Sequence[str]]):
-    """
-
-    decorator to deprecate old kwargs in a function
-
-    Parameters
-    ----------
-    l_kwargs: Sequence[Sequence[str]],
-        a list of kwargs to be deprecated,
-        each element is a sequence of length 2,
-        of the form (new_kwarg, old_kwarg)
-
-    """
-
-    def decorator(func: Callable) -> Callable:
-        """ """
-
-        @wraps(func)
-        def wrapper(*args, **kwargs) -> Callable:
-            """ """
-            old_kwargs = deepcopy(kwargs)
-            for new_kw, old_kw in l_kwargs:
-                if new_kw in kwargs:
-                    old_kwargs.pop(new_kw, None)
-                    old_kwargs[old_kw] = kwargs[new_kw]
-                elif old_kw in kwargs:
-                    warnings.warn(
-                        f"key word argument \042{old_kw}\042 is deprecated, use \042{new_kw}\042 instead"
-                    )
-            return func(*args, **old_kwargs)
-
-        func_params = list(inspect.signature(func).parameters.values())
-        func_param_names = list(inspect.signature(func).parameters.keys())
-        for new_kw, old_kw in l_kwargs:
-            idx = func_param_names.index(old_kw)
-            func_params[idx] = func_params[idx].replace(name=new_kw)
-            wrapper.__doc__ = func.__doc__.replace(old_kw, new_kw)
-        wrapper.__signature__ = inspect.Signature(parameters=func_params)
-        return wrapper
-
-    return decorator
+    new_doc = doc.splitlines()
+    parameters_indent = None
+    returns_indent = None
+    start_idx = None
+    indices2remove = []
+    for idx, line in enumerate(new_doc):
+        if line.strip().startswith(parameters_indicator):
+            parameters_indent = " " * line.index(parameters_indicator)
+        if line.strip().startswith(returns_indicator):
+            returns_indent = " " * line.index(returns_indicator)
+        if parameters_indent is not None and len(line.lstrip()) == len(line) - len(
+            parameters_indent
+        ):
+            if any([line.lstrip().startswith(p) for p in parameters]):
+                if start_idx is not None:
+                    indices2remove.extend(list(range(start_idx, idx)))
+                start_idx = idx
+            elif start_idx is not None:
+                if (
+                    line.lstrip().startswith(returns_indicator)
+                    and len(new_doc[idx - 1].strip()) == 0
+                ):
+                    indices2remove.extend(list(range(start_idx, idx - 1)))
+                else:
+                    indices2remove.extend(list(range(start_idx, idx)))
+                start_idx = None
+        if returns_indent is not None and len(line.lstrip()) == len(line) - len(
+            returns_indent
+        ):
+            if any([line.lstrip().startswith(p) for p in returns]):
+                if start_idx is not None:
+                    indices2remove.extend(list(range(start_idx, idx)))
+                start_idx = idx
+            elif start_idx is not None:
+                indices2remove.extend(list(range(start_idx, idx)))
+                start_idx = None
+    if start_idx is not None:
+        indices2remove(list(range(start_idx, len(new_doc))))
+        new_doc.extend(["\n", parameters_indicator or returns_indicator])
+    new_doc = "\n".join(
+        [line for idx, line in enumerate(new_doc) if idx not in indices2remove]
+    )
+    return new_doc
 
 
 @contextmanager
@@ -1085,12 +1213,12 @@ def timeout(duration: float):
     if np.isinf(duration):
         duration = 0
     elif duration < 0:
-        raise ValueError("duration must be non-negative")
+        raise ValueError("`duration` must be non-negative")
     elif duration > 0:  # granularity is 1 second, so round up
         duration = max(1, int(duration))
 
     def timeout_handler(signum, frame):
-        raise TimeoutError(f"block timedout after {duration} seconds")
+        raise TimeoutError(f"block timedout after `{duration}` seconds")
 
     signal.signal(signal.SIGALRM, timeout_handler)
     signal.alarm(duration)
@@ -1100,7 +1228,6 @@ def timeout(duration: float):
 
 class Timer(ReprMixin):
     """
-
     Context manager to time the execution of a block of code.
 
     Usage
@@ -1119,9 +1246,8 @@ class Timer(ReprMixin):
 
     __name__ = "Timer"
 
-    def __init__(self, name: Optional[str] = None, verbose: int = 0) -> NoReturn:
+    def __init__(self, name: Optional[str] = None, verbose: int = 0) -> None:
         """
-
         Parameters
         ----------
         name: str, optional
@@ -1142,12 +1268,12 @@ class Timer(ReprMixin):
         self.levels = {self.name: 1}
         return self
 
-    def __exit__(self, *args) -> NoReturn:
+    def __exit__(self, *args) -> None:
         for k in self.timers:
             self.stop_timer(k)
             self.timers[k] = self.ends[k] - self.timers[k]
 
-    def add_timer(self, name: str, level: int = 1) -> NoReturn:
+    def add_timer(self, name: str, level: int = 1) -> None:
         """
         add a new timer for some subtask
 
@@ -1163,7 +1289,7 @@ class Timer(ReprMixin):
         self.ends[name] = 0
         self.levels[name] = level
 
-    def stop_timer(self, name: str) -> NoReturn:
+    def stop_timer(self, name: str) -> None:
         """
         stop a timer
 
@@ -1204,3 +1330,106 @@ class Timer(ReprMixin):
 
     def extra_repr_keys(self) -> List[str]:
         return ["name", "verbose"]
+
+
+def get_kwargs(func_or_cls: Callable, kwonly: bool = False) -> Dict[str, Any]:
+    """
+    get the kwargs of a function or class
+
+    Parameters
+    ----------
+    func_or_cls: Callable,
+        the function or class to get the kwargs of
+    kwonly: bool, default False,
+        whether to get the kwonly kwargs of the function or class
+
+    Returns
+    -------
+    kwargs: Dict[str, Any],
+        the kwargs of the function or class
+
+    """
+    fas = inspect.getfullargspec(func_or_cls)
+    kwargs = {}
+    if fas.kwonlydefaults is not None:
+        kwargs = deepcopy(fas.kwonlydefaults)
+    if not kwonly and fas.defaults is not None:
+        kwargs.update(
+            {k: v for k, v in zip(fas.args[-len(fas.defaults) :], fas.defaults)}
+        )
+    if len(kwargs) == 0:
+        # perhaps `inspect.getfullargspec` does not work
+        # we should use `inspect.signature` instead
+        # for example, the model init functions defined in
+        # https://github.com/pytorch/vision/blob/release/0.13/torchvision/models/resnet.py
+        # TODO: discard old code, and use only this block
+        signature = inspect.signature(func_or_cls)
+        valid_kinds = [inspect.Parameter.KEYWORD_ONLY]
+        if not kwonly:
+            valid_kinds.append(inspect.Parameter.POSITIONAL_OR_KEYWORD)
+        for k, v in signature.parameters.items():
+            if v.default is not inspect.Parameter.empty and v.kind in valid_kinds:
+                kwargs[k] = v.default
+    return kwargs
+
+
+def get_required_args(func_or_cls: Callable) -> List[str]:
+    """
+    get the required positional arguments of a function or class
+
+    Parameters
+    ----------
+    func_or_cls: Callable,
+        the function or class to get the required arguments of
+
+    Returns
+    -------
+    required_args: list of str,
+        the names of required arguments of the function or class
+
+    """
+    signature = inspect.signature(func_or_cls)
+    required_args = [
+        k
+        for k, v in signature.parameters.items()
+        if v.default is inspect.Parameter.empty
+        and v.kind
+        in [inspect.Parameter.POSITIONAL_OR_KEYWORD, inspect.Parameter.POSITIONAL_ONLY]
+    ]
+    return required_args
+
+
+def add_kwargs(func: Callable, **kwargs: Any) -> Callable:
+    """
+    add keyword arguments to a function,
+    in order to make it compatible with other functions
+
+    Parameters
+    ----------
+    func: Callable,
+        the function to be decorated
+    kwargs: Any,
+        the keyword arguments to be added
+
+    Returns
+    -------
+    Callable,
+        the decorated function
+
+    """
+    old_kwargs = get_kwargs(func)
+    full_kwargs = {**old_kwargs, **kwargs}
+
+    @wraps(func)
+    def wrapper(*args: Any, **kwargs_: Any) -> Any:
+        """ """
+        assert set(kwargs_).issubset(full_kwargs), (
+            "got unexpected keyword arguments: "
+            f"{list(set(kwargs_).difference(full_kwargs))}"
+        )
+        filtered_kwargs = {k: v for k, v in kwargs_.items() if k in old_kwargs}
+        return func(*args, **filtered_kwargs)
+
+        # TODO: update the signature of the function
+
+    return wrapper
