@@ -35,7 +35,6 @@ from .rpeaks import (
     gamboa_detect,
     gqrs_detect,
     hamilton_detect,
-    pantompkins_detect,
     ssf_detect,
     xqrs_detect,
 )
@@ -51,7 +50,6 @@ __all__ = [
 QRS_DETECTORS = {
     "xqrs": xqrs_detect,
     "gqrs": gqrs_detect,
-    "pantompkins": pantompkins_detect,
     "hamilton": hamilton_detect,
     "ssf": ssf_detect,
     "christov": christov_detect,
@@ -83,7 +81,6 @@ def preprocess_multi_lead_signal(
     verbose: int = 0,
 ) -> Dict[str, np.ndarray]:
     """
-
     perform preprocessing for multi-lead ecg signal (with units in mV),
     preprocessing may include median filter, bandpass filter, and rpeaks detection, etc.
 
@@ -167,7 +164,8 @@ def preprocess_multi_lead_signal(
             rpeak_fn=rpeak_fn,
             verbose=verbose,
         )
-    rpeaks = merge_rpeaks(rpeaks_candidates, raw_sig, fs, verbose)
+    else:
+        rpeaks = merge_rpeaks(rpeaks_candidates, raw_sig, fs, verbose)
     retval = CFG(
         {
             "filtered_ecg": filtered_ecg,
@@ -186,7 +184,6 @@ def preprocess_single_lead_signal(
     verbose: int = 0,
 ) -> Dict[str, np.ndarray]:
     """
-
     perform preprocessing for single lead ecg signal (with units in mV),
     preprocessing may include median filter, bandpass filter, and rpeaks detection, etc.
 
@@ -280,8 +277,7 @@ def rpeaks_detect_multi_leads(
     rpeak_fn: str = "xqrs",
     verbose: int = 0,
 ) -> np.ndarray:
-    """finished, NOT checked,
-
+    """
     detect rpeaks from the filtered multi-lead ecg signal (with units in mV)
 
     Parameters
@@ -327,7 +323,6 @@ def merge_rpeaks(
     rpeaks_candidates: List[np.ndarray], sig: np.ndarray, fs: Real, verbose: int = 0
 ) -> np.ndarray:
     """
-
     merge rpeaks that are detected from each lead of multi-lead signals (with units in mV),
     using certain criterion merging qrs masks from each lead
 
