@@ -6,6 +6,7 @@ challenge (e.g. CinC, CPSC series) specific metrics are not included.
 
 """
 
+import warnings
 from numbers import Real
 from typing import Dict, Optional, Sequence, Tuple, Union
 
@@ -54,15 +55,19 @@ def top_n_accuracy(
 
     Examples
     --------
+    ```python
     >>> from torch_ecg.cfg import DEFAULTS
     >>> labels, outputs = DEFAULTS.RNG_randint(0, 10, (100)), DEFAULTS.RNG.uniform(0, 1, (100, 10))  # 100 samples, 10 classes
     >>> top_n_accuracy(labels, outputs, 3)
     0.32
     >>> top_n_accuracy(labels, outputs, [1,3,5])
     {'top_1_acc': 0.12, 'top_3_acc': 0.32, 'top_5_acc': 0.52}
+    ```
 
     """
-    assert outputs.shape[0] == labels.shape[0]
+    assert (
+        outputs.shape[0] == labels.shape[0]
+    ), "outputs and labels must have the same batch size"
     labels, outputs = torch.as_tensor(labels), torch.as_tensor(outputs)
     batch_size, n_classes, *extra_dims = outputs.shape
     if isinstance(n, int):
@@ -114,9 +119,13 @@ def confusion_matrix(
 
     """
     labels, outputs = cls_to_bin(labels, outputs, num_classes)
-    assert np.shape(labels) == np.shape(outputs)
-    assert all(value in (0, 1) for value in np.unique(labels))
-    assert all(value in (0, 1) for value in np.unique(outputs))
+    assert np.shape(labels) == np.shape(
+        outputs
+    ), "labels and outputs must have the same shape"
+    assert all(value in (0, 1) for value in np.unique(labels)), "labels must be binary"
+    assert all(
+        value in (0, 1) for value in np.unique(outputs)
+    ), "outputs must be binary"
 
     num_samples, num_classes = np.shape(labels)
 
@@ -158,9 +167,13 @@ def one_vs_rest_confusion_matrix(
 
     """
     labels, outputs = cls_to_bin(labels, outputs, num_classes)
-    assert np.shape(labels) == np.shape(outputs)
-    assert all(value in (0, 1) for value in np.unique(labels))
-    assert all(value in (0, 1) for value in np.unique(outputs))
+    assert np.shape(labels) == np.shape(
+        outputs
+    ), "labels and outputs must have the same shape"
+    assert all(value in (0, 1) for value in np.unique(labels)), "labels must be binary"
+    assert all(
+        value in (0, 1) for value in np.unique(outputs)
+    ), "outputs must be binary"
 
     num_samples, num_classes = np.shape(labels)
 
@@ -189,10 +202,11 @@ _METRICS_FROM_CONFUSION_MATRIX_PARAMS = """
     Parameters
     ----------
     labels: np.ndarray or Tensor,
-        binary labels, of shape: (n_samples, n_classes)
+        binary labels, of shape: (n_samples, n_classes),
         or indices of each label class, of shape: (n_samples,)
     outputs: np.ndarray or Tensor,
-        binary outputs, of shape: (n_samples, n_classes)
+        probability outputs, of shape: (n_samples, n_classes),
+        or binary outputs, of shape: (n_samples, n_classes),
         or indices of each class predicted, of shape: (n_samples,)
     num_classes: int, optional,
         number of classes,
@@ -200,7 +214,10 @@ _METRICS_FROM_CONFUSION_MATRIX_PARAMS = """
         then `num_classes` must be specified.
     weights: np.ndarray or Tensor, optional,
         weights for each class, of shape: (n_classes,),
-        used to compute macro {metric},
+        used to compute macro {metric}.
+    thr: float, default: 0.5,
+        threshold for binary classification,
+        valid only if `outputs` is of shape (n_samples, n_classes).
 """
 
 
@@ -213,6 +230,7 @@ def metrics_from_confusion_matrix(
     outputs: Union[np.ndarray, Tensor],
     num_classes: Optional[int] = None,
     weights: Optional[Union[np.ndarray, Tensor]] = None,
+    thr: float = 0.5,
 ) -> Dict[str, Union[float, np.ndarray]]:
     """
     Returns
@@ -223,18 +241,42 @@ def metrics_from_confusion_matrix(
     Examples
     --------
     >>> from torch_ecg.cfg import DEFAULTS
-    >>> labels, outputs = DEFAULTS.RNG_randint(0,2,(100,10)), DEFAULTS.RNG_randint(0,2,(100,10))
+    >>> # binary labels (100 samples, 10 classes, multi-label)
+    >>> labels = DEFAULTS.RNG_randint(0, 1, (100, 10))
+    >>> # probability outputs (100 samples, 10 classes, multi-label)
+    >>> outputs = DEFAULTS.RNG.random((100, 10))
     >>> metrics = metrics_from_confusion_matrix(labels, outputs)
+    >>> # binarize outputs (100 samples, 10 classes, multi-label)
+    >>> outputs = DEFAULTS.RNG_randint(0, 1, (100, 10))
+    >>> # would raise
+    >>> # RuntimeWarning: `outputs` is probably binary, AUC may be incorrect
+    >>> metrics = metrics_from_confusion_matrix(labels, outputs)
+    >>> # categorical outputs (100 samples, 10 classes)
+    >>> outputs = DEFAULTS.RNG_randint(0, 9, (100,))
+    >>> # would raise
+    >>> # RuntimeWarning: `outputs` is probably binary, AUC may be incorrect
+    >>> metrics = metrics_from_confusion_matrix(labels, outputs)
+
 
     References
     ----------
     1. https://en.wikipedia.org/wiki/Precision_and_recall
 
     """
+    outputs_ndim = np.ndim(outputs)
     labels, outputs = cls_to_bin(labels, outputs, num_classes)
     num_samples, num_classes = np.shape(labels)
 
-    ovr_cm = ovr_confusion_matrix(labels, outputs)
+    # probability outputs to binary outputs
+    bin_outputs = np.zeros_like(outputs, dtype=int)
+    bin_outputs[outputs >= thr] = 1
+    bin_outputs[outputs < thr] = 0
+    if np.unique(outputs).size == 2:
+        warnings.warn(
+            "`outputs` is probably binary, AUC may be incorrect", RuntimeWarning
+        )
+
+    ovr_cm = ovr_confusion_matrix(labels, bin_outputs)
 
     # sens: sensitivity, recall, hit rate, or true positive rate
     # spec: specificity, selectivity or true negative rate
@@ -282,6 +324,10 @@ def metrics_from_confusion_matrix(
             (tp + fp) * (tp + fn) * (tn + fp) * (tn + fn)
         )
 
+        if outputs_ndim == 1:
+            auroc[k] = np.nan
+            auprc[k] = np.nan
+            continue
         thresholds = np.unique(outputs[:, k])
         thresholds = np.append(thresholds, thresholds[-1] + 1)
         thresholds = thresholds[::-1]
@@ -406,6 +452,7 @@ def f_measure(
     outputs: Union[np.ndarray, Tensor],
     num_classes: Optional[int] = None,
     weights: Optional[Union[np.ndarray, Tensor]] = None,
+    thr: float = 0.5,
 ) -> Tuple[float, np.ndarray]:
     """
     Returns
@@ -416,7 +463,7 @@ def f_measure(
         F1-measures for each class, of shape: (n_classes,)
 
     """
-    m = metrics_from_confusion_matrix(labels, outputs, num_classes, weights)
+    m = metrics_from_confusion_matrix(labels, outputs, num_classes, weights, thr)
 
     return m["macro_f1"], m["f1"]
 
@@ -432,6 +479,7 @@ def sensitivity(
     outputs: Union[np.ndarray, Tensor],
     num_classes: Optional[int] = None,
     weights: Optional[Union[np.ndarray, Tensor]] = None,
+    thr: float = 0.5,
 ) -> Tuple[float, np.ndarray]:
     """
     Returns
@@ -442,7 +490,7 @@ def sensitivity(
         sensitivities for each class, of shape: (n_classes,)
 
     """
-    m = metrics_from_confusion_matrix(labels, outputs, num_classes, weights)
+    m = metrics_from_confusion_matrix(labels, outputs, num_classes, weights, thr)
 
     return m["macro_sens"], m["sens"]
 
@@ -464,6 +512,7 @@ def precision(
     outputs: Union[np.ndarray, Tensor],
     num_classes: Optional[int] = None,
     weights: Optional[Union[np.ndarray, Tensor]] = None,
+    thr: float = 0.5,
 ) -> Tuple[float, np.ndarray]:
     """
     Returns
@@ -474,7 +523,7 @@ def precision(
         precisions for each class, of shape: (n_classes,)
 
     """
-    m = metrics_from_confusion_matrix(labels, outputs, num_classes, weights)
+    m = metrics_from_confusion_matrix(labels, outputs, num_classes, weights, thr)
 
     return m["macro_prec"], m["prec"]
 
@@ -494,6 +543,7 @@ def specificity(
     outputs: Union[np.ndarray, Tensor],
     num_classes: Optional[int] = None,
     weights: Optional[Union[np.ndarray, Tensor]] = None,
+    thr: float = 0.5,
 ) -> Tuple[float, np.ndarray]:
     """
     Returns
@@ -504,7 +554,7 @@ def specificity(
         specificities for each class, of shape: (n_classes,)
 
     """
-    m = metrics_from_confusion_matrix(labels, outputs, num_classes, weights)
+    m = metrics_from_confusion_matrix(labels, outputs, num_classes, weights, thr)
 
     return m["macro_spec"], m["spec"]
 
@@ -525,6 +575,7 @@ def auc(
     outputs: Union[np.ndarray, Tensor],
     num_classes: Optional[int] = None,
     weights: Optional[Union[np.ndarray, Tensor]] = None,
+    thr: float = 0.5,
 ) -> Tuple[float, float, np.ndarray, np.ndarray]:
     """
     Returns
@@ -539,7 +590,11 @@ def auc(
         AUPRCs for each class, of shape: (n_classes,)
 
     """
-    m = metrics_from_confusion_matrix(labels, outputs, num_classes, weights)
+    if outputs.ndim == 1:
+        raise ValueError(
+            "outputs must be of shape (n_samples, n_classes) to compute AUC"
+        )
+    m = metrics_from_confusion_matrix(labels, outputs, num_classes, weights, thr)
 
     return m["macro_auroc"], m["macro_auprc"], m["auroc"], m["auprc"]
 
@@ -555,6 +610,7 @@ def accuracy(
     outputs: Union[np.ndarray, Tensor],
     num_classes: Optional[int] = None,
     weights: Optional[Union[np.ndarray, Tensor]] = None,
+    thr: float = 0.5,
 ) -> float:
     """
     Returns
@@ -577,7 +633,6 @@ def QRS_score(
     thr: float = 0.075,
 ) -> float:
     """
-
     QRS accuracy score, proposed in CPSC2019.
 
     Parameters
@@ -590,7 +645,7 @@ def QRS_score(
         sampling frequency of ECG signal
     thr: float, default 0.075,
         threshold for a prediction to be truth positive,
-        with units in seconds,
+        with units in seconds
 
     Returns
     -------
@@ -650,13 +705,38 @@ def cls_to_bin(
     outputs: Union[np.ndarray, Tensor],
     num_classes: Optional[int] = None,
 ) -> Tuple[np.ndarray, np.ndarray]:
-    """ """
+    """
+    Convert categorical (of shape (n_samples,)) labels and outputs
+    to binary (of shape (n_samples, n_classes)) labels and outputs if applicable.
+
+    Parameters
+    ----------
+    labels: ndarray or Tensor,
+        categorical labels of shape (n_samples,)
+        or binary labels of shape (n_samples, n_classes)
+    outputs: ndarray or Tensor,
+        categorical outputs of shape (n_samples,)
+        or binary outputs of shape (n_samples, n_classes)
+    num_classes: int, optional,
+        number of classes,
+        required if both `labels` and `outputs` are categorical
+
+    Returns
+    -------
+    labels: ndarray,
+        binary labels of shape (n_samples, n_classes)
+    outputs: ndarray,
+        binary outputs of shape (n_samples, n_classes)
+
+    """
     if isinstance(labels, Tensor):
         labels = labels.cpu().numpy()
     if isinstance(outputs, Tensor):
         outputs = outputs.cpu().numpy()
     if labels.ndim == outputs.ndim == 1:
-        assert num_classes is not None
+        assert (
+            num_classes is not None
+        ), "num_classes is required if both labels and outputs are categorical"
         shape = (labels.shape[0], num_classes)
         labels = _cls_to_bin(labels, shape)
         outputs = _cls_to_bin(outputs, shape)
@@ -670,7 +750,22 @@ def cls_to_bin(
 
 
 def _cls_to_bin(cls: np.ndarray, shape: Tuple[int]) -> np.ndarray:
-    """ """
+    """
+    Convert categorical array to binary array.
+
+    Parameters
+    ----------
+    cls: ndarray,
+        categorical array of shape (n_samples,)
+    shape: tuple,
+        shape of binary array
+
+    Returns
+    -------
+    bin_: ndarray,
+        binarized array of `cls` of shape (n_samples, n_classes)
+
+    """
     bin_ = np.zeros(shape)
     for i in range(shape[0]):
         bin_[i, cls[i]] = 1
@@ -723,7 +818,9 @@ def compute_wave_delineation_metrics(
         sensitivity, precision, f1_score, mean_error, standard_deviation
 
     """
-    assert len(truth_masks) == len(pred_masks)
+    assert len(truth_masks) == len(
+        pred_masks
+    ), "length of truth_masks and pred_masks should be the same"
     truth_waveforms, pred_waveforms = [], []
     # compute for each element
     for tm, pm in zip(truth_masks, pred_masks):
