@@ -4,20 +4,20 @@ utilities for nn models
 """
 
 import re
+import warnings
 from copy import deepcopy
-from itertools import repeat
+from itertools import repeat, chain
 from math import floor
 from numbers import Real
-from typing import List, Optional, Sequence, Tuple, Union
+from pathlib import Path
+from typing import List, Optional, Sequence, Tuple, Union, Dict
 
 import numpy as np
 import torch
 from torch import Tensor, nn
 
-from ..cfg import DEFAULTS
-
-if DEFAULTS.torch_dtype == torch.float64:
-    torch.set_default_tensor_type(torch.DoubleTensor)
+from ..cfg import CFG, DEFAULTS
+from .utils_data import cls_to_bin
 
 
 __all__ = [
@@ -42,14 +42,13 @@ def extend_predictions(
     preds: Sequence, classes: List[str], extended_classes: List[str]
 ) -> np.ndarray:
     """
-
     extend the prediction arrays to prediction arrays in larger range of classes
 
     Parameters
     ----------
     preds: sequence,
-        sequence of predictions (scalar or binary),
-        of shape (n_records, n_classes), or (n_classes,),
+        sequence of predictions (scalar or binary) of shape (n_records, n_classes),
+        or categorical predictions of shape (n_classes,),
         where n_classes = `len(classes)`
     classes: list of str,
         classes of the predictions of `preds`
@@ -62,14 +61,40 @@ def extend_predictions(
         the extended array of predictions, with indices in `extended_classes`,
         of shape (n_records, n_classes), or (n_classes,)
 
+    Examples
+    --------
+    ```python
+    >>> n_records, n_classes = 10, 3
+    >>> classes = ["NSR", "AF", "PVC"]
+    >>> extended_classes = ["AF", "RBBB", "PVC", "NSR"]
+    >>> scalar_pred = torch.rand(n_records, n_classes)
+    >>> extended_pred = extend_predictions(scalar_pred, classes, extended_classes)
+    >>> bin_pred = torch.randint(0, 2, (n_records, n_classes))
+    >>> extended_pred = extend_predictions(bin_pred, classes, extended_classes)
+    >>> cate_pred = torch.randint(0, n_classes, (n_records,))
+    >>> extended_pred = extend_predictions(cate_pred, classes, extended_classes)
+    ```
+
     """
-    _preds = np.atleast_2d(preds)
+    assert len(set(classes) - set(extended_classes)) == 0, (
+        "`extended_classes` is not a superset of `classes`, "
+        f"with {set(classes)-set(extended_classes)} in `classes` but not in `extended_classes`"
+    )
+
+    if isinstance(preds, Tensor):
+        _preds = preds.numpy()
+    else:
+        _preds = np.array(preds)
+
+    if np.ndim(_preds) == 1:  # categorical predictions
+        extended_preds = cls_to_bin(_preds, len(classes))
+        extended_preds = extend_predictions(extended_preds, classes, extended_classes)
+        extended_preds = np.where(extended_preds == 1)[1]
+        return extended_preds
+
     assert _preds.shape[1] == len(
         classes
     ), f"`pred` indicates {_preds.shape[1]} classes, while `classes` has {len(classes)}"
-    assert (
-        len(set(classes) - set(extended_classes)) == 0
-    ), f"`extended_classes` is not a superset of `classes`, with {set(classes)-set(extended_classes)} in `classes` but not in `extended_classes`"
 
     extended_preds = np.zeros((_preds.shape[0], len(extended_classes)))
 
@@ -94,10 +119,10 @@ def compute_output_shape(
     output_padding: Union[Sequence[int], int] = 0,
     dilation: Union[Sequence[int], int] = 1,
     channel_last: bool = False,
+    asymmetric_padding: Union[Sequence[int], Sequence[Sequence[int]]] = None,
 ) -> Tuple[Union[int, None]]:
     """
-
-    compute the output shape of a (transpose) convolution/maxpool/avgpool layer
+    Compute the output shape of a (transpose) convolution/maxpool/avgpool layer
 
     Parameters
     ----------
@@ -114,7 +139,7 @@ def compute_output_shape(
         stride (down-sampling length) of the layer, should be compatible with `input_shape`
     padding: int, or sequence of int, default 0,
         padding length(s) of the layer, should be compatible with `input_shape`
-    out_padding: int, or sequence of int, default 0,
+    output_padding: int, or sequence of int, default 0,
         additional size added to one side of the output shape,
         used only for transpose convolution
     dilation: int, or sequence of int, default 1,
@@ -122,6 +147,8 @@ def compute_output_shape(
     channel_last: bool, default False,
         channel dimension is the last dimension,
         or the second dimension (the first is the batch dimension by convention)
+    asymmetric_padding: (2-)sequence of int or sequence of (2-)sequence of int,
+        asymmetric paddings for all dimensions or for each dimension
 
     Returns
     -------
@@ -133,6 +160,7 @@ def compute_output_shape(
     [1] https://discuss.pytorch.org/t/utility-function-for-calculating-the-shape-of-a-conv-output/11173/5
 
     """
+    # check validity of arguments
     __TYPES__ = [
         "conv",
         "convolution",
@@ -148,7 +176,48 @@ def compute_output_shape(
         "averagepooling",
     ]
     lt = "".join(layer_type.lower().split("_"))
-    assert lt in __TYPES__
+    assert (
+        lt in __TYPES__
+    ), f"Unknown layer type `{layer_type}`, should be one of: {__TYPES__}"
+
+    def assert_positive_integer(num):
+        return isinstance(num, int) and num > 0
+
+    def assert_non_negative_integer(num):
+        return isinstance(num, int) and num >= 0
+
+    dim = len(input_shape) - 2
+    assert dim > 0, (
+        "`input_shape` should be a sequence of length at least 3, "
+        "to be a valid (with batch and channel) shape of a non-degenerate Tensor"
+    )
+    assert all(
+        s is None or assert_positive_integer(s) for s in input_shape
+    ), "`input_shape` should be a sequence containing only `None` and positive integers"
+
+    if num_filters is not None:
+        assert assert_positive_integer(
+            num_filters
+        ), "`num_filters` should be `None` or positive integer"
+    assert all(
+        assert_positive_integer(num)
+        for num in np.asarray(kernel_size).flatten().tolist()
+    ), "`kernel_size` should contain only positive integers"
+    assert all(
+        assert_positive_integer(num) for num in np.asarray(stride).flatten().tolist()
+    ), "`stride` should contain only positive integers"
+    assert all(
+        assert_non_negative_integer(num)
+        for num in np.asarray(padding).flatten().tolist()
+    ), "`padding` should contain only non-negative integers"
+    assert all(
+        assert_non_negative_integer(num)
+        for num in np.asarray(output_padding).flatten().tolist()
+    ), "`output_padding` should contain only non-negative integers"
+    assert all(
+        assert_positive_integer(num) for num in np.asarray(dilation).flatten().tolist()
+    ), "`dilation` should contain only positive integers"
+
     if lt in [
         "conv",
         "convolution",
@@ -185,10 +254,12 @@ def compute_output_shape(
         "transposeconvolution",
     ]:
         out_channels = num_filters
-    dim = len(input_shape) - 2
-    assert (
-        dim > 0
-    ), "input_shape should be a sequence of length at least 3, to be a valid (with batch and channel) shape of a non-degenerate Tensor"
+
+    def check_output_validity(shape):
+        assert all(
+            p is None or p > 0 for p in shape
+        ), f"output shape `{shape}` is illegal, please check input arguments"
+        return shape
 
     # none_dim_msg = "only batch and channel dimension can be `None`"
     # if channel_last:
@@ -203,7 +274,7 @@ def compute_output_shape(
                     "out channel dimension and spatial dimensions are all `None`"
                 )
             output_shape = tuple(list(input_shape[:-1]) + [out_channels])
-            return output_shape
+            return check_output_validity(output_shape)
         elif any([n is None for n in input_shape[1:-1]]):
             raise ValueError(none_dim_msg)
     else:
@@ -213,7 +284,7 @@ def compute_output_shape(
                     "out channel dimension and spatial dimensions are all `None`"
                 )
             output_shape = tuple([input_shape[0], out_channels] + list(input_shape[2:]))
-            return output_shape
+            return check_output_validity(output_shape)
         elif any([n is None for n in input_shape[2:]]):
             raise ValueError(none_dim_msg)
 
@@ -223,7 +294,8 @@ def compute_output_shape(
         _kernel_size = kernel_size
     else:
         raise ValueError(
-            f"input has {dim} dimensions, while kernel has {len(kernel_size)} dimensions, both not including the channel dimension"
+            f"input has {dim} dimensions, while kernel has {len(kernel_size)} dimensions, "
+            "both not including the channel dimension"
         )
 
     if isinstance(stride, int):
@@ -232,24 +304,46 @@ def compute_output_shape(
         _stride = stride
     else:
         raise ValueError(
-            f"input has {dim} dimensions, while kernel has {len(stride)} dimensions, both not including the channel dimension"
+            f"input has {dim} dimensions, while `kernel` has {len(stride)} dimensions, "
+            "both not including the channel dimension"
         )
 
+    # NOTE: asymmetric padding along one spatial dimension
+    # seems not supported yet by PyTorch's builtin Module classes
     if isinstance(padding, int):
         _padding = list(repeat(list(repeat(padding, 2)), dim))
-    elif len(padding) == 2 and isinstance(padding[0], int):
-        _padding = list(repeat(padding, dim))
-    elif (
-        len(padding) == dim
-        and all([isinstance(p, Sequence) for p in padding])
-        and all([len(p) == 2 for p in padding])
-    ):
-        _padding = padding
+    # elif len(padding) == 2 and isinstance(padding[0], int):
+    #     _padding = list(repeat(padding, dim))
+    # elif (
+    #     len(padding) == dim
+    #     and all([isinstance(p, Sequence) for p in padding])
+    #     and all([len(p) == 2 for p in padding])
+    # ):
+    elif len(padding) == dim:
+        _padding = [list(repeat(p, 2)) for p in padding]
     else:
-        # raise ValueError(
-        #     f"input has {dim} dimensions, while kernel has {len(padding)} dimensions, both not including the channel dimension"
-        # )
-        raise ValueError("Invalid padding")
+        raise ValueError(
+            f"input has {dim} dimensions, while `padding` has {len(padding)} dimensions, "
+            "both not including the channel dimension"
+        )
+        # raise ValueError("Invalid `padding`")
+
+    if asymmetric_padding is not None:
+        assert hasattr(asymmetric_padding, "__len__"), "Invalid `asymmetric_padding`"
+        if isinstance(asymmetric_padding[0], int):
+            assert len(asymmetric_padding) == 2 and isinstance(
+                asymmetric_padding[1], int
+            ), "Invalid `asymmetric_padding`"
+            _asymmetric_padding = list(repeat(asymmetric_padding, dim))
+        else:
+            assert len(asymmetric_padding) == dim and all(
+                len(ap) == 2 and all(isinstance(p, int) for p in ap)
+                for ap in asymmetric_padding
+            ), "Invalid `asymmetric_padding`"
+            _asymmetric_padding = asymmetric_padding
+        for idx in range(dim):
+            _padding[idx][0] += _asymmetric_padding[idx][0]
+            _padding[idx][1] += _asymmetric_padding[idx][1]
 
     if isinstance(output_padding, int):
         _output_padding = list(repeat(output_padding, dim))
@@ -257,7 +351,8 @@ def compute_output_shape(
         _output_padding = output_padding
     else:
         raise ValueError(
-            f"input has {dim} dimensions, while kernel has {len(output_padding)} dimensions, both not including the channel dimension"
+            f"input has {dim} dimensions, while `output_padding` has {len(output_padding)} dimensions, "
+            "both not including the channel dimension"
         )
 
     if isinstance(dilation, int):
@@ -266,7 +361,8 @@ def compute_output_shape(
         _dilation = dilation
     else:
         raise ValueError(
-            f"input has {dim} dimensions, while kernel has {len(dilation)} dimensions, both not including the channel dimension"
+            f"input has {dim} dimensions, while `dilation` has {len(dilation)} dimensions, "
+            "both not including the channel dimension"
         )
 
     if channel_last:
@@ -303,7 +399,7 @@ def compute_output_shape(
     else:
         output_shape = tuple([input_shape[0], out_channels] + output_shape)
 
-    return output_shape
+    return check_output_validity(output_shape)
 
 
 def compute_conv_output_shape(
@@ -314,11 +410,13 @@ def compute_conv_output_shape(
     padding: Union[Sequence[int], int] = 0,
     dilation: Union[Sequence[int], int] = 1,
     channel_last: bool = False,
+    asymmetric_padding: Union[Sequence[int], Sequence[Sequence[int]]] = None,
 ) -> Tuple[Union[int, None]]:
-    """finished, cheched,
-
-    compute the output shape of a convolution/maxpool/avgpool layer
+    """
+    Compute the output shape of a convolution/maxpool/avgpool layer
 
+    Parameters
+    ----------
     input_shape: sequence of int or None,
         shape of an input Tensor,
         the first dimension is the batch dimension, which is allowed to be `None`
@@ -335,6 +433,8 @@ def compute_conv_output_shape(
     channel_last: bool, default False,
         channel dimension is the last dimension,
         or the second dimension (the first is the batch dimension by convention)
+    asymmetric_padding: (2-)sequence of int or sequence of (2-)sequence of int,
+        asymmetric paddings for all dimensions or for each dimension
 
     Returns
     -------
@@ -352,6 +452,7 @@ def compute_conv_output_shape(
         0,
         dilation,
         channel_last,
+        asymmetric_padding,
     )
     return output_shape
 
@@ -364,10 +465,11 @@ def compute_maxpool_output_shape(
     dilation: Union[Sequence[int], int] = 1,
     channel_last: bool = False,
 ) -> Tuple[Union[int, None]]:
-    """finished, cheched,
-
+    """
     compute the output shape of a maxpool layer
 
+    Parameters
+    ----------
     input_shape: sequence of int or None,
         shape of an input Tensor,
         the first dimension is the batch dimension, which is allowed to be `None`
@@ -410,10 +512,11 @@ def compute_avgpool_output_shape(
     padding: Union[Sequence[int], int] = 0,
     channel_last: bool = False,
 ) -> Tuple[Union[int, None]]:
-    """finished, cheched,
-
-    compute the output shape of a avgpool layer
+    """
+    Compute the output shape of a avgpool layer
 
+    Parameters
+    ----------
     input_shape: sequence of int or None,
         shape of an input Tensor,
         the first dimension is the batch dimension, which is allowed to be `None`
@@ -456,10 +559,10 @@ def compute_deconv_output_shape(
     output_padding: Union[Sequence[int], int] = 0,
     dilation: Union[Sequence[int], int] = 1,
     channel_last: bool = False,
+    asymmetric_padding: Union[Sequence[int], Sequence[Sequence[int]]] = None,
 ) -> Tuple[Union[int, None]]:
     """
-
-    compute the output shape of a transpose convolution layer
+    Compute the output shape of a transpose convolution layer
 
     Parameters
     ----------
@@ -482,6 +585,8 @@ def compute_deconv_output_shape(
     channel_last: bool, default False,
         channel dimension is the last dimension,
         or the second dimension (the first is the batch dimension by convention)
+    asymmetric_padding: (2-)sequence of int or sequence of (2-)sequence of int,
+        asymmetric paddings for all dimensions or for each dimension
 
     Returns
     -------
@@ -499,6 +604,7 @@ def compute_deconv_output_shape(
         output_padding,
         dilation,
         channel_last,
+        asymmetric_padding,
     )
     return output_shape
 
@@ -526,13 +632,14 @@ def compute_sequential_output_shape(
     batch_size: Optional[int] = None,
 ) -> Sequence[Union[int, None]]:
     """
-
-    compute the output shape of a sequential model
+    Compute the output shape of a sequential model
 
     Parameters
     ----------
     model: nn.Sequential,
-        the sequential model
+        the sequential model,
+        each `module` of the `model` should have
+        a `compute_output_shape` method
     seq_len: int,
         length of the 1d sequence
     batch_size: int, optional,
@@ -555,41 +662,94 @@ def compute_sequential_output_shape(
 
 
 def compute_module_size(
-    module: nn.Module, human: bool = False, dtype: str = "float32"
+    module: nn.Module,
+    requires_grad: bool = True,
+    include_buffers: bool = False,
+    human: bool = False,
 ) -> Union[int, str]:
     """
-
     compute the size (number of parameters) of a module
 
     Parameters
     ----------
     module: Module,
         a torch Module
+    requires_grad: bool, default True,
+        whether to only count the parameters that require gradients
+    include_buffers: bool, default False,
+        whether to include the buffers,
+        if `requires_grad` is True, `include_buffers` will be ignored
     human: bool, default False,
         return size in a way that is easy to read by a human,
-        by appending a suffix corresponding to the unit (K, M, G, T, P)
-    dtype: str, default "float32",
-        data type of the module parameters, one of "float16", "float32", "float64"
+        by appending a suffix corresponding to the unit (B, K, M, G, T, P)
 
     Returns
     -------
     n_params: int,
         size (number of parameters) of this torch module
 
+    Examples
+    --------
+    ```python
+    >>> import torch
+    >>> class Model(torch.nn.Sequential):
+            def __init__(self):
+                super().__init__()
+                self.add_module("linear", torch.nn.Linear(10, 20, dtype=torch.float16))
+                self.register_buffer("hehe", torch.ones(20, 2, dtype=torch.float64))
+    >>> model = Model()
+    >>> model.linear.weight.requires_grad_(False)
+    >>> compute_module_size(model)
+    20
+    >>> compute_module_size(model, requires_grad=False)
+    220
+    >>> compute_module_size(model, requires_grad=False, include_buffers=True)
+    260
+    >>> compute_module_size(model, requires_grad=False, include_buffers=True, human=True)
+    '0.7K'
+    >>> compute_module_size(model, requires_grad=False, include_buffers=False, human=True)
+    '0.4K'
+    >>> compute_module_size(model, human=True)
+    '40.0B'
+    ```
+
     """
-    module_parameters = filter(lambda p: p.requires_grad, module.parameters())
-    n_params = sum([np.prod(p.size()) for p in module_parameters])
+    if requires_grad:
+        tensor_containers = filter(lambda p: p.requires_grad, module.parameters())
+        if include_buffers:
+            warnings.warn(
+                "`include_buffers` is ignored when `requires_grad` is True",
+                RuntimeWarning,
+            )
+    elif include_buffers:
+        tensor_containers = chain(module.parameters(), module.buffers())
+    else:
+        tensor_containers = module.parameters()
     if human:
-        n_params = (
-            n_params * {"float16": 2, "float32": 4, "float64": 8}[dtype.lower()] / 1024
+        size_dict = {
+            "torch.float16": 2,
+            "torch.float32": 4,
+            "torch.float64": 8,
+            "torch.int8": 1,
+            "torch.int16": 2,
+            "torch.int32": 4,
+            "torch.int64": 8,
+            "torch.uint8": 1,
+        }
+        n_params = sum(
+            [
+                np.prod(item.size()) * size_dict[str(item.dtype)]
+                for item in tensor_containers
+            ]
         )
         div_count = 0
-        while n_params >= 1024:
+        while n_params >= 1024 * 0.1:
             n_params /= 1024
             div_count += 1
-        # cvt_dict = {0:"K", 1:"M", 2:"G", 3:"T", 4:"P"}
-        cvt_dict = {c: u for c, u in enumerate(list("KMGTP"))}
+        cvt_dict = {c: u for c, u in enumerate(list("BKMGTP"))}
         n_params = f"""{n_params:.1f}{cvt_dict[div_count]}"""
+    else:
+        n_params = int(sum([np.prod(item.size()) for item in tensor_containers]))
     return n_params
 
 
@@ -600,7 +760,6 @@ def compute_receptive_field(
     input_len: Optional[int] = None,
 ) -> Union[int, float]:
     r"""
-
     computes the (generic) receptive field of feature map of certain channel,
     from certain flow (if not merged, different flows, e.g. shortcut, must be computed separately),
     for convolutions, (non-global) poolings.
@@ -641,12 +800,14 @@ def compute_receptive_field(
 
     Examples
     --------
+    ```python
     >>> compute_receptive_field([11,2,7,7,2,5,5,5,2],[1,2,1,1,2,1,1,1,2])
     90
     >>> compute_receptive_field([11,2,7,7,2,5,5,5,2],[1,2,1,1,2,1,1,1,2],[2,1,2,4,1,8,8,8,1])
     484
     >>> compute_receptive_field([11,2,7,7,2,5,5,5,2],[1,2,1,1,2,1,1,1,2],[4,1,4,8,1,16,32,64,1])
     1984
+    ```
 
     this is the receptive fields of the output feature maps
     of the 3 branches of the multi-scopic net, using its original hyper-parameters,
@@ -675,9 +836,38 @@ def compute_receptive_field(
     return receptive_field
 
 
-def default_collate_fn(batch: Sequence[Tuple[np.ndarray, ...]]) -> Tuple[Tensor, ...]:
+def default_collate_fn(
+    batch: Sequence[Union[Tuple[np.ndarray, ...], Dict[str, np.ndarray]]]
+) -> Union[Tuple[Tensor, ...], Dict[str, Tensor]]:
     """
+    collate functions for model training
 
+    the data generator (`Dataset`) should generate (`__getitem__`) n-tuples `signals, labels, ...`,
+    or dictionaries of tensors
+
+    Parameters
+    ----------
+    batch: sequence,
+        sequence of n-tuples,
+        in which the first element is the signal, the second is the label, ...;
+        or sequence of dictionaries of tensors
+
+    Returns
+    -------
+    tuple or dict of Tensor,
+        the concatenated values to feed into neural networks
+
+    """
+    if isinstance(batch[0], dict):
+        keys = batch[0].keys()
+        collated = _default_collate_fn([tuple(b[k] for k in keys) for b in batch])
+        return {k: collated[i] for i, k in enumerate(keys)}
+    else:
+        return _default_collate_fn(batch)
+
+
+def _default_collate_fn(batch: Sequence[Tuple[np.ndarray, ...]]) -> Tuple[Tensor, ...]:
+    """
     collate functions for model training
 
     the data generator (`Dataset`) should generate (`__getitem__`) n-tuples `signals, labels, ...`
@@ -701,13 +891,13 @@ def default_collate_fn(batch: Sequence[Tuple[np.ndarray, ...]]) -> Tuple[Tensor,
     ret = []
     for i in range(n_fields):
         values = [[item[i]] for item in batch]
-        values = np.concatenate(values, axis=0).astype(DEFAULTS.np_dtype)
+        values = np.concatenate(values, axis=0)
         values = torch.from_numpy(values)
         ret.append(values)
     return tuple(ret)
 
 
-if torch.__version__ >= "1.15.0":
+if torch.__version__ >= "1.5.0":
 
     def _true_divide(dividend, divisor):
         return torch.true_divide(dividend, divisor)
@@ -722,22 +912,22 @@ def _adjust_cnn_filter_lengths(
     config: dict,
     fs: int,
     ensure_odd: bool = True,
-    pattern: str = "filter_length|filt_size",
+    pattern: str = "filter_length|filter_size",
 ) -> dict:
     """
-
     adjust the filter lengths (kernel sizes) in the config for convolutional neural networks,
     according to the new sampling frequency
 
     Parameters
     ----------
     config: dict,
-        the config dictionary
+        the config dictionary,
+        this dict is NOT modified
     fs: int,
         the new sampling frequency
     ensure_odd: bool, default True,
         if True, the new filter lengths are ensured to be odd
-    pattern: str, default "filter_length|filt_size",
+    pattern: str, default "filter_length|filter_size",
         the pattern to search for in the config items related to filter lengths
 
     Returns
@@ -750,10 +940,14 @@ def _adjust_cnn_filter_lengths(
     config = deepcopy(config)
     for k, v in config.items():
         if isinstance(v, dict):
-            tmp_config = v
+            tmp_config = deepcopy(v)
+            original_fs = tmp_config.get("fs", None)
             tmp_config.update({"fs": config["fs"]})
             config[k] = _adjust_cnn_filter_lengths(tmp_config, fs, ensure_odd, pattern)
-            config[k].pop("fs", None)
+            if original_fs is None:
+                config[k].pop("fs", None)
+            else:
+                config[k]["fs"] = fs
         elif re.findall(pattern, k):
             if isinstance(v, (Sequence, np.ndarray)):  # DO NOT use `Iterable`
                 config[k] = [
@@ -783,22 +977,22 @@ def adjust_cnn_filter_lengths(
     config: dict,
     fs: int,
     ensure_odd: bool = True,
-    pattern: str = "filter_length|filt_size",
+    pattern: str = "filter_length|filter_size",
 ) -> dict:
     """
-
     adjust the filter lengths in the config for convolutional neural networks,
     according to the new sampling frequency
 
     Parameters
     ----------
     config: dict,
-        the config dictionary
+        the config dictionary,
+        this dict is NOT modified
     fs: int,
         the new sampling frequency
     ensure_odd: bool, default True,
         if True, the new filter lengths are ensured to be odd
-    pattern: str, default "filter_length|filt_size",
+    pattern: str, default "filter_length|filter_size",
         the pattern to search for in the config items related to filter lengths
 
     Returns
@@ -813,25 +1007,26 @@ def adjust_cnn_filter_lengths(
 
 
 class SizeMixin(object):
-    """
-
-    mixin class for size related methods
-
-    """
+    """Mixin class for size related methods"""
 
     @property
     def module_size(self) -> int:
-        return compute_module_size(
-            self,
-            dtype=self.dtype_,
-        )
+        return compute_module_size(self)
 
     @property
     def module_size_(self) -> str:
+        return compute_module_size(self, human=True)
+
+    @property
+    def sizeof(self) -> int:
         return compute_module_size(
-            self,
-            human=True,
-            dtype=self.dtype_,
+            self, requires_grad=False, include_buffers=True, human=False
+        )
+
+    @property
+    def sizeof_(self) -> str:
+        return compute_module_size(
+            self, requires_grad=False, include_buffers=True, human=True
         )
 
     @property
@@ -862,22 +1057,18 @@ class SizeMixin(object):
 
 
 class CkptMixin(object):
-    """
-
-    mixin class for loading from checkpoint class methods
-    """
+    """Mixin class for loading from checkpoint class methods"""
 
     @classmethod
     def from_checkpoint(
-        cls, path: str, device: Optional[torch.device] = None
+        cls, path: Union[str, Path], device: Optional[torch.device] = None
     ) -> Tuple[nn.Module, dict]:
         """
-
         load a model from a checkpoint
 
         Parameters
         ----------
-        path: str,
+        path: str or Path,
             path of the checkpoint
         device: torch.device, optional,
             map location of the model parameters,
@@ -898,11 +1089,37 @@ class CkptMixin(object):
             aux_config is not None
         ), "input checkpoint has no sufficient data to recover a model"
         kwargs = dict(
-            classes=aux_config["classes"],
             config=ckpt["model_config"],
         )
+        if "classes" in aux_config:
+            kwargs["classes"] = aux_config["classes"]
         if "n_leads" in aux_config:
             kwargs["n_leads"] = aux_config["n_leads"]
         model = cls(**kwargs)
         model.load_state_dict(ckpt["model_state_dict"])
         return model, aux_config
+
+    def save(self, path: Union[str, Path], train_config: CFG) -> None:
+        """
+        Save the model to `path`
+
+        Parameters
+        ----------
+        path: str or Path,
+            path to save the model
+        train_config: CFG,
+            config for training the model,
+            used when one restores the model
+
+        """
+        path = Path(path)
+        if not path.parent.exists():
+            path.parent.mkdir(parents=True)
+        torch.save(
+            {
+                "model_state_dict": self.state_dict(),
+                "model_config": self.config,
+                "train_config": train_config,
+            },
+            path,
+        )
